
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "CommonSettings.h"
#include "Utilities.h"
#include "PropertyValues.h"

using namespace OgreMax;


//Static initialization-----------------------------------------------------------------------
const TCHAR* CommonSettings::Values::LOD_REDUCTION_TYPES[] = {_T("percent"), _T("vertex")};
const int CommonSettings::Values::LOD_REDUCTION_TYPE_COUNT = COUNT_OF(CommonSettings::Values::LOD_REDUCTION_TYPES);


//Implementation----------------------------------------------------------------
void UserDataSettings::ClearDefaults()
{
    if (Utilities::IsSameNoCase(this->isUserDataCDATA, PropertyValues::TRUE_TEXT))
        this->isUserDataCDATA = _T("");
}

void LodSettings::ClearDefaults()
{    
    if (Utilities::IsSameNoCase(this->lodDistance, _T("0")))
        this->lodDistance = _T("");
}

void HardwareBufferUsage::ClearDefaults()
{
    if (Utilities::IsSameNoCase(this->usage, PropertyValues::HardwareBufferUsage::STATIC))
        this->usage = _T("");
    if (Utilities::IsSameNoCase(this->useShadow, PropertyValues::TRUE_TEXT))
        this->useShadow = _T("");
}

bool CommonSettings::Values::IsUserDataCDATA(const TSTR& value)
{
    return 
        Utilities::IsNull(value) || 
        Utilities::ToBool(value);
}

bool CommonSettings::Values::IsGenerateLodEnabled(const TSTR& text)
{
    return Utilities::ToBool(text);
}

void CommonSettings::Write(TSTR& result, const TSTR& key, const std::vector<ItemAllow>& items, const TSTR& newline)
{
    for (size_t i = 0; i < items.size(); i++)
    {
        result += key;
        result += _T(" ");
        result += items[i].name;
        result += _T("=");
        result += items[i].allow;
        result += newline;
    }
}

void CommonSettings::WriteNames(TSTR& result, const std::vector<ItemAllow>& items, const TSTR& newline)
{
    for (size_t i = 0; i < items.size(); i++)
    {
        result += items[i].name;
        result += newline;
    }
}

void CommonSettings::WriteNames(TSTR& result, const std::vector<ItemAllow>& items, bool allowValue, const TSTR& newline)
{
    for (size_t i = 0; i < items.size(); i++)
    {
        if (Utilities::ToBool(items[i].allow) == allowValue)
        {
            result += items[i].name;
            result += newline;
        }
    }
}

void CommonSettings::Write(TSTR& result, const std::vector<Rename>& items, const TSTR& newline)
{
    for (size_t i = 0; i < items.size(); i++)
    {
        result += items[i].from;
        result += _T("=");
        result += items[i].to;
        result += newline;
    }
}

void CommonSettings::WriteValues(TSTR& result, const std::vector<NumberedValue>& items, const TSTR& newline)
{
    for (size_t i = 0; i < items.size(); i++)
    {
        result += items[i].value;
        result += newline;
    }
}

void CommonSettings::Write(TSTR& result, const std::vector<RemoveBoneTranslation>& items, const TSTR& newline)
{
    for (size_t i = 0; i < items.size(); i++)
    {
        result += items[i].boneName;
        result += _T("=");
        result += items[i].xyz;
        result += newline;
    }
}