
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "DialogBase.h"
#include "PluginGlobals.h"

using namespace OgreMax;


//Implementation---------------------------------------------------------------
DialogBase::DialogBase()
{
    this->hDlg = NULL;
    this->oldUserData = 0;
    this->oldWndProc = NULL;    
}

DialogBase::~DialogBase()
{
}

INT_PTR DialogBase::DoModal(int dialogID, HWND hWndParent)
{
#ifdef MAX_VERSION
    if (hWndParent == NULL)
        hWndParent = GetCOREInterface()->GetMAXHWnd();
#endif

    return DialogBoxParam
        (
        PluginGlobals::GetInstanceHandle(), 
        MAKEINTRESOURCE(dialogID), 
        hWndParent,
        DialogProcCallback,
        (LPARAM)this
        );
}

void DialogBase::Attach(HWND hDlg)
{
    this->hDlg = hDlg;

    //Get old window user data and procedure
    this->oldUserData = GetWindowLongPtr(this->hDlg, GWLP_USERDATA);
    this->oldWndProc = (WNDPROC)GetWindowLongPtr(this->hDlg, GWLP_WNDPROC);

    //Replace user data and procedure
    SetWindowLongPtr(hDlg, GWLP_USERDATA, (LONG_PTR)this);
    SetWindowLongPtr(hDlg, GWLP_WNDPROC, (LONG_PTR)DialogProcCallback);    
}

void DialogBase::GetWindowText(TSTR& text)
{
    GetWindowText(this->hDlg, text);
}

void DialogBase::SetWindowText(const TSTR& text)
{
    ::SetWindowText(this->hDlg, text.data());
}

INT_PTR CALLBACK DialogBase::DialogProcCallback(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    DialogBase* dialog;

    if (message == WM_INITDIALOG)
    {
        dialog = (DialogBase*)lParam;
        dialog->hDlg = hDlg;
        SetWindowLongPtr(hDlg, GWLP_USERDATA, (LONG_PTR)lParam);
    }
    else
    {
        dialog = (DialogBase*)GetWindowLongPtr(hDlg, GWLP_USERDATA);
        if (dialog == NULL)
            return FALSE;
    }

    bool handled = false;
    INT_PTR result = dialog->DialogProc(message, wParam, lParam, handled);

    if (!handled && dialog->oldWndProc != NULL)
    {
        //Restore old window user data and procedure
        SetWindowLongPtr(dialog->hDlg, GWLP_USERDATA, (LONG_PTR)dialog->oldUserData);
        SetWindowLongPtr(dialog->hDlg, GWLP_WNDPROC, (LONG_PTR)dialog->oldWndProc);    
 
        //Call old window procedure
        result = dialog->oldWndProc(dialog->hDlg, message, wParam, lParam);
        
        //Replace user data and procedure
        SetWindowLongPtr(dialog->hDlg, GWLP_USERDATA, (LONG_PTR)dialog);
        SetWindowLongPtr(dialog->hDlg, GWLP_WNDPROC, (LONG_PTR)DialogProcCallback);    
    }

    return result;
}

void DialogBase::CheckRadioButton(int idFirstButton, int idLastButton, int idCheckButton)
{
    ::CheckRadioButton(this->hDlg, idFirstButton, idLastButton, idCheckButton);
}

HWND DialogBase::GetDlgItem(int idDlgItem)
{
    return ::GetDlgItem(this->hDlg, idDlgItem);
}

void DialogBase::CheckDlgButton(int idButton, UINT check)
{
    ::CheckDlgButton(this->hDlg, idButton, check);
}

bool DialogBase::IsDlgButtonChecked(int idButton)
{
    return ::IsDlgButtonChecked(this->hDlg, idButton) ? true : false;
}

const TSTR& DialogBase::IsDlgButtonCheckedText(int idButton)
{
    static const TSTR& trueText("true");
    static const TSTR& falseText("false");

    return ::IsDlgButtonChecked(this->hDlg, idButton) ? trueText : falseText;
}

void DialogBase::GetDlgItemText(int idControl, TSTR& text)
{
    GetWindowText(GetDlgItem(idControl), text);    
}

void DialogBase::SetDlgItemText(int idControl, const TCHAR* text)
{
    ::SetDlgItemText(this->hDlg, idControl, text);
}

void DialogBase::SetDlgItemText(int idControl, const TSTR& text)
{
    ::SetDlgItemText(this->hDlg, idControl, text.data());
}

void DialogBase::GetWindowText(HWND hwnd, TSTR& text)
{
    int length = ::GetWindowTextLength(hwnd);
    text.Resize(length + 1);    
    ::GetWindowText(hwnd, text.data(), length + 1);
}

void DialogBase::CenterWindow()
{
    ::CenterWindow(this->hDlg, GetParent(this->hDlg));
}

void DialogBase::AppendWindowText(const TSTR& s1, const TSTR& s2)
{
    TSTR title;
    GetWindowText(title);
    title += s1;
    title += s2;
    SetWindowText(title);
}