
#ifndef OgreMax_DialogBase_INCLUDED
#define OgreMax_DialogBase_INCLUDED


//Classes----------------------------------------------------------------------
namespace OgreMax
{    
    /** Base class for dealing with dialog windows */
    class DialogBase
    {
    public:
        DialogBase();
        virtual ~DialogBase();

        /** Creates a new dialog and runs it as a model dialog */
        virtual INT_PTR DoModal(int dialogID, HWND hWndParent = NULL);

        /** Attaches to an existing dialog */
        virtual void Attach(HWND hDlg);

        /** Detaches from the existing dialog. Should only be called if Attach() was called */
        virtual void Detach() {}

        /** Gets the data from the GUI and stores it in the internal settinsgs */
        virtual void GetGUIData()
        {
        }

        void GetWindowText(TSTR& text);
        void SetWindowText(const TSTR& text);

        void CheckRadioButton(int idFirstButton, int idLastButton, int idCheckButton);

        HWND GetDlgItem(int idDlgItem);

        void CheckDlgButton(int idButton, UINT check);
        bool IsDlgButtonChecked(int idButton);
        const TSTR& IsDlgButtonCheckedText(int idButton);

        void GetDlgItemText(int idControl, TSTR& text);
        void SetDlgItemText(int idControl, const TSTR& text);
        void SetDlgItemText(int idControl, const TCHAR* text);

        void CenterWindow();

        void AppendWindowText(const TSTR& s1, const TSTR& s2);

    protected:
        /** The dialog procedure to be overridden in derived classes */
        virtual INT_PTR DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled) = 0;

    private:
        /** The internal dialog procedure */
        static INT_PTR CALLBACK DialogProcCallback(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

        static void GetWindowText(HWND hwnd, TSTR& text);

    protected:
        HWND hDlg;

        LONG_PTR oldUserData;
        WNDPROC oldWndProc;    
    };
}

#endif

