#ifndef OgreMax_ExportSettings_INCLUDED
#define OgreMax_ExportSettings_INCLUDED


//Includes---------------------------------------------------------------------
#include "KeyedValues.h"


//Classes----------------------------------------------------------------------
namespace OgreMax 
{
    enum VertexColorFormat
    {
        DIRECT3D,
        OPENGL
    };

    enum Tex2D
    {
        UV, 
        VW, 
        WU
    };

    enum UpAxis
    {
        UP_AXIS_Y,
        UP_AXIS_Z
    };

    enum LodReductionType
    {
        LOD_PERCENT,
        LOD_VERTEX
    };

    enum PhysicsExportType
    {
        NO_PHYSICS,
        OPAL_PHYSICS
    };

    /**
     * Class for managing exporter settings
     */ 
    class ExportSettings
    {
    public:
        ExportSettings();
        virtual ~ExportSettings();

        void Initialize(const KeyedValues& properties);

        void MakeExportPaths(const TSTR& exportPath, const TSTR& exportFileName);

        //General exporter settings
        bool exportMeshes;                      //Indicates whether meshes should be exported at all
        bool exportMaterials;                   //Indicates whether materials should be exported
        PhysicsExportType physicsExportType;    //Indicates what type of physics export should be performed
        bool exportVertexColors;                //Indicates whether vertex colors should be exported
        UpAxis upAxis;                          //Indicates whether the up axis is Y. Default is 3DS Max's Z up
        float scale;                            //Scale to use when exporting
        float scaledWorldUnitsPerMeter;         //The number of world units per meter, relative to the scaling factor
        Tex2D texCoord;                         //interpretation of 2D texture coordinates (UV, VW, WU)
        TSTR noMaterialName;                    //Name of material to use when no material is applied to a mesh
        
        //This can be modified to turn on/off sorting for the output bone assignments
        //Sorting makes it easier to visually inspect the output, no sorting is faster
        bool sortBoneAssignments;

        //Binary mesh settings
        bool exportTextMesh;                  //Enable binary mesh export. False indicates just XML meshes are exported        
        bool exportBinaryMesh;                  //Enable binary mesh export. False indicates just XML meshes are exported        
        bool generateTangents;                  //Indicates whether tangents should be generated
        bool generateEdgeLists;                 //Indicates whether edge lists should be generated (used in shadow volumes)
        bool generateLod;                       //Indicates whether LoD meshes should be generated        
        unsigned int numLodLevels;              //Number of LoD meshes to generate
        float lodDistance;                      //Distance increment per LoD level        
        LodReductionType lodReductionType;      //Indicates the type of lod reduction performed
        float lodPercentReduction;              //LoD percentage reduction
        size_t lodVertexReduction;              //LoD fixed reduction
        VertexColorFormat vertexColorFormat;    //Direct3D or OpenGL
        Ogre::Serializer::Endian endian;        //Byte ordering

        //Paths        
        TSTR exportPath;                        //Path to directory that contains the exported files
        TSTR meshPath;                          //Path to directory that contains the exported meshes
        TSTR materialPath;                      //Path to directory that contains the exported materials
        TSTR modelPath;                         //Path to directory that contains the exported models
        TSTR logPath;                           //Path to directory that will contain log files        
        TSTR materialFileName;                  //Exported material filename        
    };
}


#endif