
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "KeyedValues.h"
#include "Utilities.h"

using namespace OgreMax;


//Implementation---------------------------------------------------------------
static void PrepareKey(TSTR& key)
{
    key.toLower();
}

KeyedValues::KeyedValues()
{
}

KeyedValues::KeyedValues(const TSTR& text)
{
    Parse(text);
}

void KeyedValues::Parse(const TSTR& text)
{
    //Parse the input text by looking for a key=value pair at the start of each line
    int index = 0;
    int nextIndex;
    int length = text.length();
    TSTR key, value;
    int line = 0;
    
    while (index < length)
    {
        if (Utilities::ParseKeyValueLine(text, index, nextIndex, key, value, false))
            SetValue(key, value, line);
        
        index = nextIndex;
        line++;
    }
}

bool KeyedValues::HasValue(const TSTR& key) const
{
    TSTR preparedKey = key;
    PrepareKey(preparedKey);

    for (size_t i = 0; i < this->lookup.size(); i++)
    {
        if (this->lookup[i].first == preparedKey)
            return true;
    }
    return false;
}

bool KeyedValues::GetValue(const TSTR& key, TSTR& value) const
{
    TSTR preparedKey = key;
    PrepareKey(preparedKey);

    for (size_t i = 0; i < this->lookup.size(); i++)
    {
        if (this->lookup[i].first == preparedKey)
        {
            value = this->lookup[i].second.value;
            return true;
        }
    }

    return false;
}

void KeyedValues::SetValue(const TSTR& key, const TSTR& value, int line)
{
    //Prepare the value that goes into the map
    MapValue mapValue;
    mapValue.originalKey = key;
    mapValue.value = value;
    mapValue.line = line;

    //Prepare the key used to put the value into the map
    TSTR preparedKey = key;
    PrepareKey(preparedKey);

    for (size_t i = 0; i < this->lookup.size(); i++)
    {
        if (this->lookup[i].first == preparedKey)
        {
            this->lookup[i].second.value = value;
            return;
        }
    }

    //Put the value in the map
    VectorItem item;
    item.first = preparedKey;
    item.second = mapValue;
    this->lookup.push_back(item);
}

void KeyedValues::Clear()
{
    this->lookup.clear();
}

void KeyedValues::Load(const TSTR& fileName)
{
    stdifstream inFile;

    const std::streamsize MAX_LINE_LENGTH = 1024; //The maximum length of a line, including null terminator
    TCHAR line[MAX_LINE_LENGTH];  //A single line from the file
    int nextIndex;   //This is ignored. Needed to parse the line
    TSTR key, value; //A parsed key and value parsed from a line
    
    inFile.open(fileName.data(), std::ios::in);
    if (inFile.is_open())
    { 
        while (!inFile.eof())
        {
            inFile.getline(line, MAX_LINE_LENGTH);
            if (Utilities::ParseKeyValueLine(line, 0, nextIndex, key, value))
                SetValue(key, value);
        }

        inFile.close();
    }
}

void KeyedValues::Save(const TSTR& fileName)
{
    stdofstream outFile;

    outFile.open(fileName.data(), std::ios::out);
    if (outFile.is_open())
    {
        for (Lookup::iterator item = this->lookup.begin(); item != this->lookup.end(); ++item)
        {
            outFile 
                << item->second.originalKey.data() 
                << _T("=") 
                << item->second.value.data()
                << std::endl;
        }

        outFile.close();
    }
}
