#ifndef OgreMax_KeyedValues_INCLUDED
#define OgreMax_KeyedValues_INCLUDED


//Classes----------------------------------------------------------------------
namespace OgreMax
{
    /**
     * A collection of string key/value pairs
     */
    class KeyedValues
    {
    public:
        KeyedValues();
        KeyedValues(const TSTR& text);
        virtual ~KeyedValues() {}

        /**
         * Parses all the keys and values from the specified text.
         * The parsed values are merged with the current settings
         */
        void Parse(const TSTR& text);

        /** 
         * Determines if there is a value for the specified key
         * @param key [in] - The key to test. The key is case insensitive.
         * @return Returns true if the specified key was found, false otherwise
         */
        bool HasValue(const TSTR& key) const;

        /** 
         * Gets the value for the specified key
         * @param key [in] - The key for which the value is retrieved. The key
         * is case insensitive.
         * @param value [out] - If the key was found, this contains the corresponding value
         * @return Returns true if the specified key was found, false otherwise
         */
        bool GetValue(const TSTR& key, TSTR& value) const;

        /**
         * Sets the value for the specified key.
         */
        void SetValue(const TSTR& key, const TSTR& value, int line = -1);

        /** Clears all the key=value pairs. */
        void Clear();

        /** 
         * Loads the configuration settings from the specified file.
         * The values in the file are merged with the current settings
         */
        void Load(const TSTR& fileName);

        /** Saves the configuration settings to the specified file */
        void Save(const TSTR& fileName);        

        int GetSize() const 
        {
            return (int)this->lookup.size();
        }

        struct MapValue
        {
            TSTR originalKey;
            TSTR value;
            int line;
        };

        typedef std::pair<TSTR, MapValue> VectorItem;
        typedef std::vector<VectorItem> Lookup;
        typedef Lookup::iterator Iterator;

        Iterator GetIterator() 
        {
            return this->lookup.begin();
        }

        Iterator GetIteratorEnd()
        {
            return this->lookup.end();
        }

    protected:
        Lookup lookup;
    };
}


#endif