
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "MaterialExporter.h"
#include "Utilities.h"
#include <stdarg.h>
#include <direct.h>
#include "generate_shaders.hpp"
#include <boost/filesystem.hpp>
#include <string>
//Special Rules for 3dsmax materials
//Filter texture is NOT USED for anything. If you must set a texture for FILTER_COLOR then please use a copy of the glow map texture since it modifies the glow map
//Filter color DIVIDES the emission color, increasing it above and beyond 1.  It can be up to 100x greater than 1 if necessary...divide by 0 is special and is sun-like color
//If you wish to have a glow-only item without diffuse map, max requires you still specify the diffuse map to be the same as glow so it looks "right"
//but for my purposes go ahead and set the diffuse COLOR (in the blinn/phong parameter set) to black to prevent a per-light pass from being generated. and of course bake in any alpha you may have into the glow map instead of diffuse map
bool similars(std::string a, std::string b) {
	if (stricmp(a.c_str(),b.c_str())==0) return true;
	return false;
}
bool find_file( const boost::filesystem::path & dir_path,         // in this directory,
                const std::string & file_name, // search for this name,
                boost::filesystem::path & path_found, int toplevel, bool materials=false )            // placing path here if found
{
  using namespace boost::filesystem;

  if ( !exists( dir_path ) ) {
	  //std::string tmp=dir_path.file_string();
	  //MessageBox(NULL,tmp.c_str(),"DNE",MB_OK);
	  return false;
  } 

  directory_iterator end_itr; // default construction yields past-the-end
  for ( directory_iterator itr( dir_path );
        itr != end_itr;
        ++itr )
  {
    if ( is_directory(itr->status()))
    {
		if (materials==false||itr->leaf()!="textures") {
			/*std::string s=itr->path().string();
			std::string s1=itr->leaf()+"::::"+s;
			std::string mat="false";
			if (materials)mat="true";
			MessageBox(NULL,s1.c_str(),mat.c_str(),MB_OK|MB_ICONEXCLAMATION);
			*/
			if ( find_file( itr->path(), file_name, path_found, toplevel-1,itr->leaf()=="materials" ) ) {
				//std::string tmp=dir_path.file_string();
				//MessageBox(NULL,tmp.c_str(),"E",MB_OK);
				return true;
			}
		}else {
			//std::string s=itr->path().string();
			//MessageBox(NULL,"IGNORED",s.c_str(),MB_OK|MB_ICONEXCLAMATION);
		}
	}
    else if ( similars(itr->leaf(),file_name) ) // see below
    {
      path_found = itr->path();
	  //std::string tmp=path_found.file_string();
	  //MessageBox(NULL,tmp.c_str(),"EE",MB_OK);
			
      return true;
    }
  }
  return false;
}
bool doAbort=false;
#define ERRMSG(LEN,X) {char data[LEN]; sprintf X; if (AssertDialog(data)==IDCANCEL) doAbort=true;}

using namespace OgreMax;
static const Class_ID GNORMAL_CLASS_ID(0x243e22c6, 0x63f6a014);
static const int BUMP_IDENTIFIER=16384;
static const Class_ID TEXMAP_MULTIPLY_CLASS_ID(656, 0);
static const Class_ID TEXMAP_OVER_CLASS_ID(640, 0);

//Implementation---------------------------------------------------------------
MaterialExporter::MaterialExporter
    (
    const KeyedValues& sceneProps,
    const ExportSettings& settings,
    IGameScene* gameInterface,
    Interface* maxInterface
    ) :
    ExporterBase(settings, gameInterface, maxInterface),
    sceneProperties(sceneProps)
{
}

MaterialExporter::~MaterialExporter()
{
}

void MaterialExporter::ExportMaterials(const MaterialMap& materials)
{
    //Export the materials into a single script
    TSTR materialScript;
    ExportMaterials(materials, materialScript);

    //Save the material script
    stdofstream materialFile;
	_mkdir("materials");
	_mkdir("materials/scripts");
	std::string filename=this->exportSettings.materialFileName.data();
	std::string::size_type lastdir=filename.find_last_of("\\/");
	if (lastdir!=std::string::npos) {
		filename=filename.substr(0,lastdir)+"/materials/scripts"+filename.substr(lastdir);
	}else
		filename="materials/scripts/"+filename;
    materialFile.open(filename.c_str(), std::ios::out);
    if (materialFile.is_open())
    {
        materialFile << materialScript.data();
        materialFile.close();
	}else {
		MessageBox(NULL,filename.c_str(),"Could not open",MB_OK);
	}
}

bool MaterialExporter::ExportMaterials(const MaterialMap& materialMap, TSTR& output)
{
	stdstringstream out;
	out.precision(8);
	out << std::fixed;

	if (WriteMaterials(materialMap, out))
    {
        stdstring outputString = out.str();

        output = outputString.c_str();

        return true;
    }
	else
    {
        output = _T("");
		return false;
    }
}
int AssertDialog(const char * data) {
	return MessageBox(NULL,data,"Export Warning",MB_CANCELTRYCONTINUE|MB_ICONEXCLAMATION);
}
bool MaterialExporter::ExportMaterial(IGameMaterial* material, const TSTR& materialName, TSTR& output)
{
	stdstringstream out;


	if (material)
    {
		int numSubMtl = material->GetSubMaterialCount();
		if (numSubMtl > 0)
        {
			for (int i = 0; i < numSubMtl; i++) {
				if (i!=0)
					AssertDialog("More than one submaterial disallowed! Please combine your materials into a single material and use RGB Multiply and Combine to layer on textures.");
				else
					WritePass(out, materialName, material->GetSubMaterial(i));			
			}
		}
		else
			WritePass(out, materialName, material);
	}
	else
    	WritePass(out, materialName, material);



    stdstring outputString = out.str();
    output = outputString.c_str();

	return true;
}
enum Map3dsTypes {
		MAP_AMBIENT=ID_AM,
		MAP_DIFFUSE=ID_DI,
		MAP_SPECULAR=ID_SP,
		MAP_SPECLEVEL=ID_SH,
		MAP_GLOSSINESS=ID_SS,
		MAP_GLOW=ID_SI,
		MAP_OPACITY=ID_OP,
		MAP_FILTER=ID_FI,
		MAP_BUMP=ID_BU,
		MAP_REFLECTION=ID_RL,
		MAP_REFRACTION=ID_RR,
		MAP_DISPLACEMENT=ID_DP,
		NUM_MAPS=NTEXMAPS
};
static std::string nickname3ds[NUM_MAPS]={"Ambient","Diffuse","Specular","SpecLevel","Glossiness","Glow","Opacity","Filter","Normal","Reflection","Refraction","Displacement"};

struct TextureEntry {
	std::string filename;
	enum BLENDMODE{NONE,OVER,MULTIPLY,OVERLAY} combiner;
	int texture_set;
	TextureEntry() {
		texture_set=0;
		combiner=NONE;
	}
};
struct TextureTree {
	std::vector<TextureEntry> entries;
	TextureEntry bump;
	float weight;
};
const char * cgfilename[NUM_MAPS]={
	"Ambient","Diffuse","Specular","","Glossiness","","","","Normal","","",""};
const char * cgambientfilename[NUM_MAPS]={
	"Ambient","Diffuse","Specular","","Glossiness","Glow","","","Normal","Reflection","",""};
typedef std::pair<std::string,int> TEXTUREINT;
int myabs(int a) {
if (a<0) return -a;
return a;
}
static bool texmapequal(Texmap*a, Texmap*b) {
	if (a==NULL||b==NULL)
		return (a==NULL&&b==NULL);
	if (a->ClassID()!=b->ClassID()) {
		return false;
	}
	if (a->ClassID()==Class_ID(BMTEX_CLASS_ID,0)) {
		std::string aname= ((BitmapTex*)a)->GetMapName();
		std::string bname= ((BitmapTex*)b)->GetMapName();
		return aname==bname;
	}
	int numasubmaps=a->NumSubTexmaps();
	int numbsubmaps=b->NumSubTexmaps();
	if (numasubmaps!=numbsubmaps)
		return false;
	bool retval=true;
	for (int i=0;i<numasubmaps;++i) {
		retval=retval&&texmapequal(a->GetSubTexmap(i),b->GetSubTexmap(i));
	}
	return retval;
}
class ExportMat {
public:
	Point4 colors[NUM_MAPS];
	int texture[NUM_MAPS];
	TextureTree shadeTree[NUM_MAPS];
	std::vector<std::vector<TEXTUREINT> > textures;
	bool two_sided;
	bool translucent;//if true it can be semitransparent instead of either opaque or invis
	float specularLevel;
	float glossiness;
	float soften;
	//float normalbump_level;Stored in the weight value
	float bump_level;
	float normal_level;
	std::string material_name;
	FILE* findTextureA(std::string filename) {
		FILE *fp=fopen(filename.c_str(),"rb");
		if (fp) {
			//MessageBox(NULL,filename.c_str(),"TexFound",MB_OK|MB_ICONEXCLAMATION);
			return fp;
		}
		std::string::size_type where=filename.find_first_of("\\/");
		if (where!=std::string::npos) {
			fp=findTextureA(filename.substr(where+1));
			if (fp) return fp;
		}
		filename="../"+filename;
		fp=fopen(filename.c_str(),"rb");
		//if (fp) {
		//		MessageBox(NULL,filename.c_str(),"TexFound",MB_OK|MB_ICONEXCLAMATION);}
		return fp;
	}
	FILE* findTexture(std::string filename) {
		static std::string look_here;
		std::string tmp=look_here+"/"+filename;
		FILE *fp=fopen(tmp.c_str(),"rb");
		if (fp) {		
			//MessageBox(NULL,tmp.c_str(),"TexFound",MB_OK|MB_ICONEXCLAMATION);
			return fp;
		}
		fp=findTextureA(filename);
		if (fp!=NULL) return fp;
		std::string::size_type where=filename.find_last_of("\\/");
		if (where!=std::string::npos)
			filename=filename.substr(where+1);
		std::string starterstring="../";
		boost::filesystem::path start(starterstring);
		boost::filesystem::path found;
		for (int i=0;i<4;++i) {
			try {
			if (find_file(start,filename,found,i)) {
				std::string what= found.file_string();
				where=what.find_last_of("\\/");
				if (where!=std::string::npos) {
					look_here=what.substr(0,where);
				}
				//MessageBox(NULL,what.c_str(),"TexFound",MB_OK|MB_ICONEXCLAMATION);
				return fopen(what.c_str(),"rb");
			}
			}catch(...) {
				static bool tmp=false;
				if (!tmp) {
					tmp=true;
					MessageBox(NULL,"Boost find failed","Permission denied",MB_OK|MB_ICONEXCLAMATION);
				}
			}
			starterstring+="../";
			start=boost::filesystem::path(starterstring);
		}
		return NULL;
	}
	void copyTexture(std::string texturename, const KeyedValues& sceneProperties) {
		if (texturename.length()==0)
			return;
		FILE * fp=findTexture(texturename);
		if (fp) { 
			TSTR fn=TSTR("materials/textures/")+Utilities::GetRenamedBitmapFileName(Utilities::GetFileName(texturename.c_str()),sceneProperties);
			fseek(fp,0,SEEK_END);
			size_t siz=ftell(fp);
			fseek(fp,0,SEEK_SET);
			char *dat=(char*)malloc(siz);
			fread(dat,siz,1,fp);
			fclose(fp);
			fp=fopen(fn.data(),"rb");
			if (fp==NULL) {
				fp=fopen(fn.data(),"wb");
				if (fp) {
					fwrite(dat,siz,1,fp);
					fclose(fp);
				}else {
					fn=TSTR("Texture File Unwriteable ")+fn;
					static bool tmp=false;
					if (!tmp) {
						tmp=true;
						MessageBox(NULL,fn.data(),"Permission denied",MB_OK|MB_ICONEXCLAMATION);
					}
				}
			}else fclose(fp);
		}else {
			texturename=std::string("Texture File Unreadable ")+texturename;
			MessageBox(NULL,texturename.c_str(),"Permission denied",MB_OK|MB_ICONEXCLAMATION);

		}
	}
       void gatherTextures(std::vector<std::string>&tex) {
           for (int i=0;i<NUM_MAPS;++i) {
               std::string val=this->shadeTree[i].bump.filename;
               if (val.length()) tex.push_back(val);
               for (unsigned int j=0;j<this->shadeTree[i].entries.size();++j) {
                   val=this->shadeTree[i].entries[j].filename;
                   if (val.length()) tex.push_back(val);
               }
           }           
       }
    static void stolower(std::string&s) {
        for (size_t i=0;i<s.length();++i) {
            s[i]=tolower(s[i]);
        }
    }
       void renameLowerTextures() {
           for (int i=0;i<NUM_MAPS;++i) {
               stolower(this->shadeTree[i].bump.filename);
               for (unsigned int j=0;j<this->shadeTree[i].entries.size();++j) {
                   stolower(this->shadeTree[i].entries[j].filename);
               }
           }           
       }
       void renameTextures(const std::map<std::string,std::string>&tex) {
           for (int i=0;i<NUM_MAPS;++i) {
               std::map<std::string,std::string>::const_iterator where=tex.find(this->shadeTree[i].bump.filename);
               if (where!=tex.end()) {
                   this->shadeTree[i].bump.filename=where->second;
               }
               for (unsigned int j=0;j<this->shadeTree[i].entries.size();++j) {
                   where=tex.find(this->shadeTree[i].entries[j].filename);
                   if (where!=tex.end()) {
                       this->shadeTree[i].entries[j].filename=where->second;
                   }
               }
           }           
       }
       bool xcase_better(const std::string&a, const std::string&b) {
		   //FILE * fp=fopen("c:\\temp\\log.txt","a ");
		   //fprintf (fp,"Comparing %s and %s\n",a.c_str(),b.c_str());

		   if (a.length()!=b.length()) {
			   //fclose(fp);
			   return false;
		   }
           int uppercount=0;
           int defuppercount=0;
           bool onelower=false;
           for (size_t i=0;i<a.length();++i) {
               if (tolower(a[i])!=a[i]) uppercount++;
               if (tolower(b[i])!=b[i]) defuppercount++;
               if (toupper(a[i])!=a[i]) onelower=true;
               if (tolower(a[i])!=tolower(b[i])) {
				   //fprintf (fp,"Notlower %d %c and %c\n",i,a[i],b[i]);
				   //fclose(fp);
                   return false;
               }
           }
		    //fprintf (fp,"Aucount %d Bucount %d\n",uppercount,defuppercount);
			if (defuppercount<uppercount) {/*fclose(fp);*/return true;}
			if (defuppercount>uppercount) {/*fclose(fp);*/return false;}
			//fprintf (fp,"default return %d",(int)(a<b));
           return a<b;
       }
	   bool case_better(std::string a,std::string b) {
		   std::string::size_type wb=b.find_last_of("\\/");
		   std::string::size_type wa=a.find_last_of("\\/");
		   if (wb!=std::string::npos) {
				b=b.substr(wb+1);
		   }
		   if (wa!=std::string::npos) {
				a=a.substr(wa+1);
		   }
		   return xcase_better(a,b);
	   }
       void findRenameTextures(const std::vector<std::string>&tex,std::map<std::string,std::string> &ret) {
           for (std::vector<std::string>::const_iterator ti=tex.begin(),tie=tex.end();ti!=tie;++ti) {
               ret[*ti]=*ti;
               for (std::vector<std::string>::const_iterator tj=tex.begin(),tje=tex.end();tj!=tje;++tj) {
                   if (case_better(*tj,*ti)) {
                       ret[*ti]=*tj;                       
                   }
               }
           }
       }
    
	void copyTextures( const KeyedValues& sceneProperties) {
		_mkdir("materials");
		_mkdir("materials/textures");
		for (int i=0;i<NUM_MAPS;++i) {
			copyTexture(this->shadeTree[i].bump.filename,sceneProperties);
			for (unsigned int j=0;j<this->shadeTree[i].entries.size();++j) {
				copyTexture(this->shadeTree[i].entries[j].filename,sceneProperties);
			}
		}
	}
	ExportMat(std::string name){
		material_name=name;
		for (int i=0;i<NUM_MAPS;++i) {
			colors[i].x=colors[i].y=colors[i].z=colors[i].w=1.0f;
			texture[i]=-1;
		}
		colors[MAP_GLOW].x=colors[MAP_GLOW].y=colors[MAP_GLOW].z=0;
		glossiness=10;
		soften=.10;
		specularLevel=0;
		normal_level=1;
		bump_level=1;
		two_sided=false;
		translucent=false;
	}
	///Function gets the combiner of a complex set of mixing or combining or RGB multiply functions. Modifies tex if it needs to be moved to a subcomponent
	TextureEntry::BLENDMODE getCombinerMode(unsigned int which,Texmap **tex, bool &use_ao_alpha) {
		use_ao_alpha=false;
		if (tex[0]->ClassID()==Class_ID(BMTEX_CLASS_ID,0))
			return TextureEntry::NONE;
		if (tex[0]->ClassID()==TEXMAP_MULTIPLY_CLASS_ID)
			return TextureEntry::MULTIPLY;
		if (tex[0]->ClassID()==TEXMAP_OVER_CLASS_ID)
			return TextureEntry::OVER;
		if (tex[0]->ClassID()==Class_ID(MIX_CLASS_ID,0)) {
			//could be any of the 3!
			if (tex[0]->NumSubTexmaps()<3||tex[0]->GetSubTexmap(2)==NULL)
				return TextureEntry::OVERLAY;
			if (tex[0]->GetSubTexmap(0)==NULL||tex[0]->GetSubTexmap(1)==NULL){
				ERRMSG(1024,("Material %s, Error: Mixing with unselected inputs to the mix for texture %s is unsupported.",material_name.c_str(),nickname3ds[which].c_str()));
				return TextureEntry::OVER;
			}
            if (which==MAP_BUMP) {
                if (tex[0]->GetSubTexmap(2)->ClassID()==TEXMAP_MULTIPLY_CLASS_ID) {
                    int numchild=tex[0]->GetSubTexmap(1)->NumSubTexmaps();
                    if (numchild<=1||tex[0]->GetSubTexmap(1)->GetSubTexmap(0)==NULL||tex[0]->GetSubTexmap(1)->GetSubTexmap(1)==NULL) {
                        return TextureEntry::MULTIPLY;//respects mix alpha by multiplying it with .5
                    }
                }
            }else if (tex[0]->GetSubTexmap(1)->ClassID()==TEXMAP_MULTIPLY_CLASS_ID) {
				int numchild=tex[0]->GetSubTexmap(1)->NumSubTexmaps();
				if (numchild>1) {
					if (texmapequal(tex[0]->GetSubTexmap(1)->GetSubTexmap(1),tex[0]->GetSubTexmap(0))||
						texmapequal(tex[0]->GetSubTexmap(1)->GetSubTexmap(0),tex[0]->GetSubTexmap(0))) {
						//possibly a candidate for multiplication since its blending to be same as parent
						*tex=tex[0]->GetSubTexmap(1);
						if (tex[0]->GetSubTexmap(2)->ClassID()==Class_ID(BMTEX_CLASS_ID,0)) {
							if (std::string(((BitmapTex*)tex[0]->GetSubTexmap(2))->GetMapName())==this->shadeTree[MAP_AMBIENT].entries[0].filename) {
								use_ao_alpha=true;
								return TextureEntry::MULTIPLY;
							}else {
								ERRMSG(1024,("Material %s Error in texture %s: Mixing a multiply with an alpha that is not from ambient occlusion",material_name.c_str(),nickname3ds[which].c_str()));
							}
						}else {
							ERRMSG(1024,("Material %s, Error in texture %s: Mixing a multiply with an alpha that is not just ambient occlusion",material_name.c_str(),nickname3ds[which].c_str()));
						}
						return TextureEntry::MULTIPLY;
					}
					//not equal-- probably an over operator, dont munge tex
				}
			}
			use_ao_alpha=true;
			return TextureEntry::OVER;
		}else {
			ERRMSG(1024,(data,"Material %s, For Texture %s, don't know how to combine Class type %x %x\n",material_name.c_str(),nickname3ds[which].c_str(),tex[0]->ClassID().PartA(),tex[0]->ClassID().PartB()));
		}
		return TextureEntry::OVERLAY;
	}

	void LoadSingleTexture(TextureEntry&te, Texmap *tex){
		te.filename=(TCHAR*)((BitmapTex*)tex)->GetMapName();
		te.texture_set=tex->GetMapChannel();
	}
	TextureEntry LoadSingleTexture(Texmap *tex) {
		TextureEntry retval;
		LoadSingleTexture(retval,tex);
		return retval;
	}
    enum WhichDXT{wDXT1,wDXT3,wDXT5,wNONE,wUNK};
#define DLEN 1024
	const char * ambOccName() {
		if (shadeTree[MAP_AMBIENT].entries.size()!=0)
			return shadeTree[MAP_AMBIENT].entries[0].filename.c_str();
		return "";
	}
	static char*findsub(char dat[DLEN], char *str,unsigned int strl) {
		for (int i=0;i+strl<DLEN;++i) {
			if (memcmp(dat+i,str,strl)==0) return dat+i;
		}
		return NULL;
	}
    WhichDXT whichDXT(char dat[DLEN]) {
        if (findsub(dat,"DXT1",4))return wDXT1;
        if (findsub(dat,"DXT3",4))return wDXT3;
        if (findsub(dat,"DXT5",4))return wDXT5;
        return wNONE;
    }
    bool checkDXT1() {
        if (shadeTree[MAP_AMBIENT].entries.size()!=0) {
			//ERRMSG(1024,(data,"Checking %s for dds",shadeTree[MAP_AMBIENT].entries[0].filename.c_str()));
            std::string filename=shadeTree[MAP_AMBIENT].entries[0].filename;
            FILE *fp=findTexture(filename);
            if (fp) {
                char dat[DLEN]={0};
				size_t howmuch=0;
                while (0!=(howmuch=fread(dat,1,DLEN,fp))) {
                    WhichDXT x = whichDXT(dat);
                    if (x!=wNONE)  {
                        fclose(fp);
                        fp=NULL;
						if (x==wDXT1) {
							//ERRMSG(1024,(data,"DXT1 %s for dds",shadeTree[MAP_AMBIENT].entries[0].filename.c_str()));
							return true;
						}
						if (x==wDXT5) {
							//ERRMSG(1024,(data,"DXT5 %s for dds",shadeTree[MAP_AMBIENT].entries[0].filename.c_str()));
							return false;
						}
                        if (x==wDXT3) return false;
                        break;
                    }
					if (howmuch!=DLEN){
						ERRMSG(1024,(data,"Failed to FIND dds %s",shadeTree[MAP_AMBIENT].entries[0].filename.c_str()));
						break;
					}
					if (0!=fseek(fp,-4,SEEK_CUR)){//move backwards in case we missed a DXTx
						//ERRMSG(1024,(data,"Failed to seek in file %s",shadeTree[MAP_AMBIENT].entries[0].filename.c_str()));
						break;
					}
                }               
                if (fp!=NULL)
                    fclose(fp);
			}else {
				ERRMSG(1024,(data,"Failed %s for dds",shadeTree[MAP_AMBIENT].entries[0].filename.c_str()));

                return false;
            }
            return false;
        }
		return true;
    }
	bool LoadShadeTree(int whichTexture, Texmap *tex) {
		if (!tex) return true;
		if (tex->ClassID()==TEXMAP_OVER_CLASS_ID&&tex->NumSubTexmaps()>2&&tex->GetSubTexmap(0)&&tex->GetSubTexmap(1)&&tex->GetSubTexmap(2)) {
			if (tex->NumSubTexmaps()>3) {
				ERRMSG(1024,(data,"Material %s, Map %s combines more than 3 bitmaps",material_name.c_str(),nickname3ds[whichTexture].c_str()));
				if (doAbort) return false;
			}
			if (tex->GetSubTexmap(0)&&tex->GetSubTexmap(0)->ClassID()==Class_ID(BMTEX_CLASS_ID,0)&&tex->GetSubTexmap(1)&&tex->GetSubTexmap(1)->ClassID()==Class_ID(BMTEX_CLASS_ID,0)&&tex->GetSubTexmap(2)&&tex->GetSubTexmap(2)->ClassID()==Class_ID(BMTEX_CLASS_ID,0)) {
				shadeTree[whichTexture].entries.push_back(LoadSingleTexture(tex->GetSubTexmap(0)));
				shadeTree[whichTexture].entries.back().combiner=TextureEntry::OVER;
				shadeTree[whichTexture].entries.push_back(LoadSingleTexture(tex->GetSubTexmap(1)));
				shadeTree[whichTexture].entries.back().combiner=TextureEntry::OVER;
				shadeTree[whichTexture].entries.push_back(LoadSingleTexture(tex->GetSubTexmap(2)));
				shadeTree[whichTexture].entries.back().combiner=TextureEntry::OVER;
				if (whichTexture==MAP_BUMP) {
					ERRMSG(1024,(data,"Material %s Warning: Normal map not allowed to combine more than 2 textures, ignoring last texture.",material_name.c_str()));	
				}else if (!checkDXT1()) {
					ERRMSG(1024,(data,"Material %s Warning: 3rd combined %s texture will be modulated by alpha from ambient occlusion map, despite intention. If you don't want this behavior then set ambient occlusion map %s to type DXT1 so no alpha channel is exposed.",material_name.c_str(),nickname3ds[whichTexture].c_str(),ambOccName()));
				}
			}else {
				ERRMSG(1024,(data,"Material %s, Map %s combines exactly 3 textures but some of them have subtextures.",material_name.c_str(),nickname3ds[whichTexture].c_str()));
			}
		}else {//all our objects have 2 items
			bool t_use_ao_alpha=false,a_use_ao_alpha=false,b_use_ao_alpha=false;
			TextureEntry::BLENDMODE ttype=getCombinerMode(whichTexture,&tex,t_use_ao_alpha);
			if (ttype==TextureEntry::NONE) {
				shadeTree[whichTexture].entries.push_back(LoadSingleTexture(tex));			
			}else {
				if (tex->NumSubTexmaps()<2||tex->GetSubTexmap(0)==NULL||tex->GetSubTexmap(1)==NULL) {
					ERRMSG(1024,(data,"Material %s, Map %s combines textures that are unspecified.",material_name.c_str(),nickname3ds[whichTexture].c_str()));
				}else {
					Texmap *atex=tex->GetSubTexmap(0);
					Texmap *btex=tex->GetSubTexmap(1);
					TextureEntry::BLENDMODE atype=getCombinerMode(whichTexture,&atex,a_use_ao_alpha);
					TextureEntry::BLENDMODE btype=getCombinerMode(whichTexture,&btex,b_use_ao_alpha);
					if (atype==TextureEntry::NONE&&btype==TextureEntry::NONE) {
						//single level combiner
						shadeTree[whichTexture].entries.push_back(LoadSingleTexture(atex));
						shadeTree[whichTexture].entries.back().combiner=TextureEntry::OVER;
						shadeTree[whichTexture].entries.push_back(LoadSingleTexture(btex));
						shadeTree[whichTexture].entries.back().combiner=ttype;
                        bool DXT1=checkDXT1();
						if ((t_use_ao_alpha==false&&whichTexture!=MAP_BUMP/*bump maps are allowed to have it both ways*/&&!DXT1)) {
							ERRMSG(1024,(data,"Material %s Map %s layer 2 assumed to blend with alpha from ambient occlusion map (if available). If you don't want this behavior then set ambient occlusion map %s to type DXT1 so no alpha channel is exposed.",material_name.c_str(),nickname3ds[whichTexture].c_str(),ambOccName()));
						}
					}else if (atype==TextureEntry::NONE) {//no bump maps allowed past this point
						if (whichTexture==MAP_BUMP) {
							ERRMSG(1024,(data,"Material %s Map Normal only allowed to have 2 layers.",material_name.c_str()));
						}
						//combine then combine
						if (btex->NumSubTexmaps()<2||btex->GetSubTexmap(0)==NULL||btex->GetSubTexmap(1)==NULL) {
							ERRMSG(1024,(data,"Material %s Map %s layer 2 combines textures that are unspecified.",material_name.c_str(),nickname3ds[whichTexture].c_str()));
						}else {
							Texmap *ctex=btex->GetSubTexmap(1);
							btex=btex->GetSubTexmap(0);
							if (atex->ClassID()!=Class_ID(BMTEX_CLASS_ID,0)||btex->ClassID()!=Class_ID(BMTEX_CLASS_ID,0)||ctex->ClassID()!=Class_ID(BMTEX_CLASS_ID,0)) {
								ERRMSG(1024,(data,"Material %s Warning: Map %s combines textures that are not bitmaps: unsupported.",material_name.c_str(),nickname3ds[whichTexture].c_str()));
							}else {
								if (ttype==TextureEntry::OVER&&(btype==TextureEntry::MULTIPLY||btype==TextureEntry::OVERLAY)) {
									ERRMSG(1024,(data,"Material %s Warning: Map %s combines a texture with a multipliction of textures: not supported, will reverse order of operations to be combine, then multiply.",material_name.c_str(),nickname3ds[whichTexture].c_str()));
								}
								shadeTree[whichTexture].entries.push_back(LoadSingleTexture(atex));
								shadeTree[whichTexture].entries.back().combiner=TextureEntry::OVER;
								shadeTree[whichTexture].entries.push_back(LoadSingleTexture(btex));
								shadeTree[whichTexture].entries.back().combiner=ttype;
								shadeTree[whichTexture].entries.push_back(LoadSingleTexture(ctex));
								shadeTree[whichTexture].entries.back().combiner=btype;
								if (b_use_ao_alpha==false&&t_use_ao_alpha==false) {
                                    if (!checkDXT1()) {
                                        ERRMSG(1024,(data,"Material %s Map %s layer 2 assumed to blend with alpha from ambient occlusion map (if available). If you don't want this behavior then set ambient occlusion map %s to type DXT1 so no alpha channel is exposed.",material_name.c_str(),nickname3ds[whichTexture].c_str(),ambOccName()));
                                    }
								}
								if (b_use_ao_alpha==true) {
									ERRMSG(1024,(data,"Material %s Map %s layer 3 cannot blend with ambient occlusion alpha.",material_name.c_str(),nickname3ds[whichTexture].c_str()));
								}
							}
						}
					}else if (btype==TextureEntry::NONE) {
						if (whichTexture==MAP_BUMP) {
							ERRMSG(1024,(data,"Material %s Map Normal only allowed to have 2 layers.",material_name.c_str()));
						}
						if (atex==NULL||btex==NULL||atex->NumSubTexmaps()<2||atex->GetSubTexmap(0)==NULL||atex->GetSubTexmap(1)==NULL) {
							ERRMSG(1024,(data,"Material %s Map layer 2 %s combines textures that are unspecified.",material_name.c_str(),nickname3ds[whichTexture].c_str()));
						}else if (btex!=NULL) {
							Texmap*ctex=btex;
							btex=atex->GetSubTexmap(1);
							atex=atex->GetSubTexmap(0);
 							if (atex->ClassID()!=Class_ID(BMTEX_CLASS_ID,0)||btex->ClassID()!=Class_ID(BMTEX_CLASS_ID,0)||ctex->ClassID()!=Class_ID(BMTEX_CLASS_ID,0)) {
								ERRMSG(1024,(data,"Material %s Warning: Map %s combines textures that are not bitmaps: unsupported.",material_name.c_str(),nickname3ds[whichTexture].c_str()));
							}else {
								shadeTree[whichTexture].entries.push_back(LoadSingleTexture(atex));
								shadeTree[whichTexture].entries.back().combiner=TextureEntry::OVER;
								shadeTree[whichTexture].entries.push_back(LoadSingleTexture(btex));
								shadeTree[whichTexture].entries.back().combiner=atype;
								shadeTree[whichTexture].entries.push_back(LoadSingleTexture(ctex));
								shadeTree[whichTexture].entries.back().combiner=ttype;
								if (t_use_ao_alpha==false&&a_use_ao_alpha==false) {
                                    if (!checkDXT1()) {
                                        ERRMSG(1024,(data,"Material %s Map %s layer 2 assumed to blend with alpha from ambient occlusion map (if available). If you don't want this behavior set ambient occlusion map %s to type DXT1 so no alpha channel is exposed.",material_name.c_str(),nickname3ds[whichTexture].c_str(),ambOccName()));
                                    }
								}
								if (t_use_ao_alpha==true) {
									ERRMSG(1024,(data,"Material %s Map %s layer 3 cannot blend with ambient occlusion alpha.",material_name.c_str(),nickname3ds[whichTexture].c_str()));
								}
							}
						}
					}else {
						ERRMSG(1024,(data,"Material %s Map %s combines 4 textures into 2 groups each having a subgroup. This is not supported.",material_name.c_str(),nickname3ds[whichTexture].c_str()));
					}
				}
			}
		}
		return doAbort;
	}
	void LoadShadeTree(int whichTexture, float weight, Texmap*tex) {
		TextureTree* tree=&shadeTree[whichTexture];
		tree->weight=weight;
		if (whichTexture==MAP_DIFFUSE && tex->ClassID()==TEXMAP_MULTIPLY_CLASS_ID&&tex->NumSubTexmaps()>0&&tex->GetSubTexmap(0)&&tex->GetSubTexmap(0)->GetMapChannel()==1) {
			int numsubmaps=tex->NumSubTexmaps();
			LoadShadeTree(MAP_AMBIENT,tex->GetSubTexmap(0));
			if (numsubmaps>1&&tex->GetSubTexmap(1)) {
				LoadShadeTree(whichTexture,tex->GetSubTexmap(1));
			}
		}else if (whichTexture==MAP_BUMP) {
			if (tex->ClassID()==GNORMAL_CLASS_ID) {
				if (0) for (unsigned int i=0;i<1024;++i) {
					
					char * temp=(char*)tex;
					if (*(float*)(temp+i)==0.125) {
						ERRMSG(1024,(data,"0.125f located at %d offset",i));
					}
					if (*(double*)(temp+i)==0.125) {
						ERRMSG(1024,(data,"0.125 located at %d offset",i));
					}
					if (*(float*)(temp+i)==0.25f) {
						ERRMSG(1024,(data,"0.25f located at %d offset",i));
					}
					if (*(double*)(temp+i)==0.25) {
						ERRMSG(1024,(data,"0.25 located at %d offset",i));
					}
					
				}
				this->normal_level=*(float*)(((char*)tex)+92);//offsets measured by experiment
				this->bump_level=*(float*)(((char*)tex)+96);//offsets measured by experiment
				int numsubmaps=tex->NumSubTexmaps();
				if (numsubmaps>0&&tex->GetSubTexmap(0)) {
					LoadShadeTree(whichTexture,tex->GetSubTexmap(0));
				}
				if (numsubmaps>1&&tex->GetSubTexmap(1)) {
					if(tex->GetSubTexmap(1)->ClassID()==Class_ID(BMTEX_CLASS_ID,0)) {
						LoadSingleTexture(tree->bump,tex->GetSubTexmap(1));
					}else {
						ERRMSG(1024,(data,"Material %s Bump maps must be Bitmap textures and may not be combined or multiplied or mixed",material_name.c_str()));
					}
				}
			}else if (tex->ClassID()==Class_ID(BMTEX_CLASS_ID,0)) {
				ERRMSG(1024,(data,"Material %s Bump map without normal map unsupported",material_name.c_str()));
			}else {
				ERRMSG(1024,(data,"Material %s Bump Map Needs to be a NormalBump type",material_name.c_str()));
			}
		}else {
			LoadShadeTree(whichTexture,tex);
		}
	}
	void writecolor(stdostream&out, float level, Point4 color) {
		out<<color.x*level << _T(" ") << color.y*level << _T(" ") << color.z*level << _T(" ") << color.w*level;

	}
	void writemultcolor(stdostream&out, float level, Point4 color) {
		out<<(color.x*(1-level)+level) << _T(" ") << (color.y*(1-level)+level) << _T(" ") << (color.z*(1-level)+level) << _T(" ") << color.w;
	}
    std::string writetex(int name,int offset,const char *texmapname,const KeyedValues& sceneProperties) {
        std::string retval;
		TextureEntry *tex=&this->shadeTree[name].bump;
		if (offset>=0&&offset<this->shadeTree[name].entries.size()) {
			tex=&this->shadeTree[name].entries[offset];
		}else if (offset>=0) {
			tex=NULL;
		}
		if ((tex==NULL||tex->filename.length()==0)&&offset<0&&name==MAP_BUMP) {//bump map
			if (this->shadeTree[MAP_OPACITY].entries.size()) {//no bump map specified, therefore should put opacity map in bump map
				tex=&this->shadeTree[MAP_OPACITY].entries[0];
				bool diffuse_safe=false;
				if (this->shadeTree[MAP_DIFFUSE].entries.size()&&tex->filename==this->shadeTree[MAP_DIFFUSE].entries[0].filename&&tex->texture_set==this->shadeTree[MAP_DIFFUSE].entries[0].texture_set) {//if the first opacity map ==diffuse
					if (this->shadeTree[MAP_OPACITY].entries.size()>1) {//either take the second map if it exists
						tex=&this->shadeTree[MAP_OPACITY].entries[1];
						diffuse_safe=true;
					}else {
						tex=NULL;//or else blank it out--no reason to double multiply diffuse
					}
				}
				if (tex&&tex->texture_set!=1) {
					ERRMSG(1024,(data,"Warning opacity texture %s is not in the first UV set (%d instead) and does not match diffuse texture %s",tex->filename.c_str(),tex->texture_set,this->shadeTree[MAP_DIFFUSE].entries.size()?this->shadeTree[MAP_DIFFUSE].entries[0].filename.c_str():""));
					tex=NULL;
				}//else if (diffuse_safe==false&&(this->shadeTree[MAP_DIFFUSE].entries.size()&&(this->shadeTree[MAP_OPACITY].entries.size()<=1))) {
				//	ERRMSG(1024,(data,"Warning opacity texture %s is not in the first UV set and does not match diffuse texture %s",tex->filename.c_str(),this->shadeTree[MAP_DIFFUSE].entries.size()?this->shadeTree[MAP_DIFFUSE].entries[0].filename.c_str():""));										
				//} //someday shoudl check for diffuse opacity texture matching...but not THIS day


			}
		}
		if (tex!=NULL&&tex->filename.length()) {
            retval+="\t\t\t\ttexture ";
            retval+=Utilities::GetRenamedBitmapFileName(Utilities::GetFileName(tex->filename.c_str()),sceneProperties);
            retval+="\n\t\t\t\ttex_coord_set ";
            retval+=(char)('0'+tex->texture_set-1);
            retval+='\n';
		}
        return retval;
	}
	void writentex(stdostream&out, int name, const char *normal_name, const char*dirtnormal_name, const char* bump_name,bool only_write_second) {
		if (name>=0&&name<textures.size()&&textures[name].size()) {
			char *tab3="\t\t\t";
			char *tab4="\t\t\t\t";
			for (int i=((int)textures[name].size())-1;i>=0;--i) {
				if (only_write_second==false||textures[name][i].second==BUMP_IDENTIFIER) {
					if (textures[name][i].first.length()) {
						out << tab3<< "texture_unit ";
						if (textures[name][i].second==BUMP_IDENTIFIER) {
							out << bump_name;
						}else if (i==0) {
							out << normal_name;
						}else {
							out << dirtnormal_name;
						}
						out << std::endl;
						out << tab3<< "{"<<std::endl;
						out << tab4<<"texture "<<textures[name][i].first<<std::endl;
						if (0) {//no printing tex_coord_set...the wildcard inheritance takes care of that for us
							if (textures[name][i].second==BUMP_IDENTIFIER) {
								out << tab4<<"tex_coord_set 0"<<std::endl;
							}else {
								out << tab4<<"tex_coord_set "<<(textures[name][i].second<0?-textures[name][i].second:textures[name][i].second)-1<<std::endl;
							}
						}
						out << tab4<<"tex_address_mode wrap"<<std::endl;
						out << tab3<< "}"<<std::endl;
					}
				}
			}
		}
	}
	std::string processDirt(std::vector<TEXTUREINT>&combined) {
		std::string retval;
		std::vector<std::string> dirtterms;
		dirtterms.push_back("");
		dirtterms.push_back("");
		dirtterms.push_back("Dirt");
		dirtterms.push_back("Cruft");

		for (size_t j=1/*offset since 0th is covered above*/;j<combined.size();++j) {
			if (combined[j].first.length()) {
				int channel=combined[j].second>0?combined[j].second:-combined[j].second;
				if (channel<dirtterms.size()) {
					retval+=dirtterms[channel];
					if( combined[j].second<0) {
						retval+='M';
					}
				}
			}
		}
		return retval;
	}
	int getMapCapability(int map,int offset) {
		if (this->shadeTree[map].entries.size()>offset) {
			TextureEntry::BLENDMODE comb=this->shadeTree[map].entries[offset].combiner;
			int texcoord=this->shadeTree[map].entries[offset].texture_set;
			int blendmode=1;
			if (comb==TextureEntry::OVER||comb==TextureEntry::NONE) 
				blendmode=0;
			if (comb==TextureEntry::OVERLAY)
				blendmode=2;
			return blendmode*4+texcoord;
		}else return 0;
	}
	static std::string FixNickname(std::string input) {
		if (input=="Cruftmal") return "NormalOfDirt";
		if (input=="Occl") return "AmbientOcclusion";
		return input;
	}
	void write(stdostream&out, const KeyedValues& sceneProperties) {
		bool skinning=false;//FIXME
		bool subsurface=false;//FIXME
		using namespace ProgramDefines;
		copyTextures(sceneProperties);
		int caps[NUM_NICK]={0};
		caps[NUL_LIGHTS_NICK]=8;
		caps[SKINNING_NICK]=skinning?1:0;
		caps[AMBIENT_OCCLUSION_NICK]=getMapCapability(MAP_AMBIENT,0);
		caps[DIFFUSE_NICK]=getMapCapability(MAP_DIFFUSE,0);
		caps[DIFFUSE_DIRT_NICK]=getMapCapability(MAP_DIFFUSE,1); 
		caps[DIFFUSE_CRUFT_NICK]=getMapCapability(MAP_DIFFUSE,2);
		caps[AMBIENT_NICK]=(caps[DIFFUSE_NICK]==0&&caps[DIFFUSE_DIRT_NICK]==0&&caps[DIFFUSE_CRUFT_NICK]==0)?0:1;

		caps[SPECULAR_NICK]=getMapCapability(MAP_SPECULAR,0);
		caps[SPECULAR_DIRT_NICK]=getMapCapability(MAP_SPECULAR,1);
		caps[SPECULAR_CRUFT_NICK]=getMapCapability(MAP_SPECULAR,2);
		caps[TWO_SIDED_NICK]=two_sided?1:0;
		caps[GLOW_NICK]=getMapCapability(MAP_GLOW,0);
		caps[GLOW_DIRT_NICK]=getMapCapability(MAP_GLOW,1);
		caps[GLOW_CRUFT_NICK]=getMapCapability(MAP_GLOW,2);
		
		caps[BUMP_NICK]=this->shadeTree[MAP_BUMP].bump.filename.length()?this->shadeTree[MAP_BUMP].bump.texture_set:0;
		caps[NORMAL_NICK]=getMapCapability(MAP_BUMP,0);
		caps[DIRT_NORMAL_NICK]=getMapCapability(MAP_BUMP,1);
		
		caps[REFLECTION_NICK]=getMapCapability(MAP_REFLECTION,0);
		caps[SUBSURFACE_NICK]=(skinning&&subsurface)?1:0;
		caps[AMBIENT_ENVIRONMENT_NICK]=(skinning&&subsurface)?1:0;
		if (!capCompat(NUM_NICK, caps)) {
			std::string datstr;
			for (int i=0;i<NUL_LIGHTS_NICK;++i) {
				if (caps[i]&&i!=AMBIENT_NICK) {
					if (datstr.length())
						datstr+=", and ";
					datstr+=FixNickname(nickname[i]);
				}
			}
			ERRMSG(1024,(data,"You are using too many textures since you are using %s",datstr.c_str()));
		}
		std::string foundName;
		int found_caps[NUM_NICK];
		int score=findMatch(caps,foundName,found_caps, true);
		std::string materialName=Utilities::MakeMaterialName(material_name.c_str());
		if (score!=0) {
			std::string inp,outp;
			inp=capName(caps,true);
			outp=capName(found_caps,true);
			std::string errmsg;
			for (int i=0;i<ProgramDefines::NUM_NICK;++i) {
				if (caps[i]!=found_caps[i]) {
					errmsg+=" Map "+FixNickname(ProgramDefines::nickname[i]);
					if (found_caps[i]&&!caps[i]) {
						errmsg+=" included as blank texture: may as well put something there.";
					}else {
						if (found_caps[i]%4!=caps[i]%4) {
							errmsg +=" wanted UV channel ";
							errmsg+=('0'+caps[i]%4);
							errmsg+=" got UV channel ";
							errmsg+=('0'+found_caps[i]%4);
						}
						if (found_caps[i]/4!=caps[i]/4) {
							errmsg+=" wanted to ";
							if (caps[i]/4==0)
								errmsg+="Composite";
							else if (caps[i]/4==1&&i!=DIRT_NORMAL_NICK)
								errmsg+="Multiply";
							else if (caps[i]/4==2)
								errmsg+="Overlay";
							errmsg+=" actually ";
							if (found_caps[i]/4==0)
								errmsg+="Composite";
							else if (found_caps[i]/4==1&&i!=DIRT_NORMAL_NICK)
								errmsg+="Multiply";
							else if (found_caps[i]/4==2)
								errmsg+="Overlay";
						}
						errmsg+='.';
					}
				}
			}
			//FILE * fp=fopen("c:\\temp\\log.txt","a");
			//fprintf(fp,"%s not %s\n",inp.c_str(),outp.c_str());
			//fclose(fp);
			ERRMSG(4096,(data,"Material %s not matched perfectly.%s",material_name.c_str(),errmsg.c_str()));
		}
		bool use_alpha=shadeTree[MAP_OPACITY].entries.size()!=0;
		if (use_alpha) {
			if (shadeTree[MAP_DIFFUSE].entries.size()) {
				if (shadeTree[MAP_DIFFUSE].entries[0].filename!=shadeTree[MAP_OPACITY].entries[0].filename) {
					if ((shadeTree[MAP_OPACITY].entries[0].texture_set!=1&&shadeTree[MAP_BUMP].bump.filename.size()==0)||(shadeTree[MAP_BUMP].bump.filename.size()&&(shadeTree[MAP_BUMP].bump.filename!=shadeTree[MAP_OPACITY].entries[0].filename||shadeTree[MAP_BUMP].bump.texture_set!=shadeTree[MAP_OPACITY].entries[0].texture_set))) {
						ERRMSG(1024,(data,"Material %s has specified opacity %s (UV set %d) but does not match diffuse map %s 's alpha or bump map %s 's alpha (UV set %d)",material_name.c_str(),shadeTree[MAP_OPACITY].entries[0].filename.c_str(),shadeTree[MAP_OPACITY].entries[0].texture_set,shadeTree[MAP_DIFFUSE].entries[0].filename.c_str(),shadeTree[MAP_BUMP].bump.filename.c_str(),shadeTree[MAP_BUMP].bump.texture_set==0?1:shadeTree[MAP_BUMP].bump.texture_set));				
					}
				}
				if (shadeTree[MAP_OPACITY].entries.size()>1) {
					if (shadeTree[MAP_DIFFUSE].entries[0].filename!=shadeTree[MAP_OPACITY].entries[1].filename) {
						if ((shadeTree[MAP_OPACITY].entries[1].texture_set!=1&&shadeTree[MAP_BUMP].bump.filename.size()==0)||( shadeTree[MAP_BUMP].bump.filename.size()&&(shadeTree[MAP_BUMP].bump.filename!=shadeTree[MAP_OPACITY].entries[1].filename||shadeTree[MAP_BUMP].bump.texture_set!=shadeTree[MAP_OPACITY].entries[1].texture_set))) {
							ERRMSG(1024,(data,"Material %s has specified opacity %s (UV set %d) but does not match diffuse map %s 's alpha or bump map %s 's alpha (UV set %d)",material_name.c_str(),shadeTree[MAP_OPACITY].entries[1].filename.c_str(),shadeTree[MAP_OPACITY].entries[1].texture_set,shadeTree[MAP_DIFFUSE].entries[0].filename.c_str(),shadeTree[MAP_BUMP].bump.filename.c_str(),shadeTree[MAP_BUMP].bump.texture_set==0?1:shadeTree[MAP_BUMP].bump.texture_set));				
						}
					}
				}
			}else if (shadeTree[MAP_GLOW].entries.size()) {
				if (shadeTree[MAP_GLOW].entries[0].filename!=shadeTree[MAP_OPACITY].entries[0].filename) {
					if ((shadeTree[MAP_OPACITY].entries[0].texture_set!=1&&shadeTree[MAP_BUMP].bump.filename.size()==0)||( shadeTree[MAP_BUMP].bump.filename.size()&&(shadeTree[MAP_BUMP].bump.filename!=shadeTree[MAP_OPACITY].entries[0].filename||shadeTree[MAP_BUMP].bump.texture_set!=shadeTree[MAP_OPACITY].entries[0].texture_set))) {
						ERRMSG(1024,(data,"Material %s has specified opacity %s (UV set %d) but does not match glow map %s 's alpha or bump map %s 's alpha (UV set %d)",material_name.c_str(),shadeTree[MAP_OPACITY].entries[0].filename.c_str(),shadeTree[MAP_OPACITY].entries[0].texture_set,shadeTree[MAP_GLOW].entries[0].filename.c_str(),shadeTree[MAP_BUMP].bump.filename.c_str(),shadeTree[MAP_BUMP].bump.texture_set==0?1:shadeTree[MAP_BUMP].bump.texture_set));
					}
				}
				if (shadeTree[MAP_OPACITY].entries.size()>1) {
					if (shadeTree[MAP_GLOW].entries[0].filename!=shadeTree[MAP_OPACITY].entries[1].filename) {
						if ((shadeTree[MAP_OPACITY].entries[1].texture_set!=1&&shadeTree[MAP_BUMP].bump.filename.size()==0)||( shadeTree[MAP_BUMP].bump.filename.size()&&(shadeTree[MAP_BUMP].bump.filename!=shadeTree[MAP_OPACITY].entries[1].filename||shadeTree[MAP_BUMP].bump.texture_set!=shadeTree[MAP_OPACITY].entries[1].texture_set))) {
							ERRMSG(1024,(data,"Material %s has specified opacity %s (UV set %d) but does not match glow map %s 's alpha or bump map %s 's alpha (UV set %d) ",material_name.c_str(),shadeTree[MAP_OPACITY].entries[1].filename.c_str(),shadeTree[MAP_OPACITY].entries[1].texture_set,shadeTree[MAP_GLOW].entries[0].filename.c_str(),shadeTree[MAP_BUMP].bump.filename.c_str(),shadeTree[MAP_BUMP].bump.texture_set==0?1:shadeTree[MAP_BUMP].bump.texture_set));
						}
					}
				}
			}else {
				ERRMSG(1024,(data,"Material %s has specified opacity but no diffuse or glow map",material_name.c_str()));
			}
		}
		std::string alphaName=materialName+"__Blend";
        out << _T("import * from ")<<foundName<<_T(".material")<<std::endl;;
		out << _T("material ")<<(use_alpha?alphaName:materialName)<<" : "<<foundName<<std::endl;
		out << _T("{") <<std::endl;
		out << _T("\ttechnique *")<<std::endl;
		out << _T("\t{")<<std::endl;
		out << _T("\t\tpass *") << std::endl;
		out << _T("\t\t{")<<std::endl;
		if (this->two_sided) {
			out << _T("\t\t\tcull_hardware none")<<std::endl;
		}
		out << _T("\t\t\tfragment_program_ref *")<<std::endl;
		out << _T("\t\t\t{")<<std::endl;
		float offset_scale=this->bump_level*shadeTree[MAP_BUMP].weight/500;
		//float offset_bias=-offset_scale/2;
		float offset_bias = this->bump_level * 20.0;
		float normal_scale=this->normal_level*shadeTree[MAP_BUMP].weight*3;
		out << _T("\t\t\t\tparam_named surfaceSpecularEmissive float4 ")<<this->specularLevel<<_T(" ")<<this->glossiness<<_T(" ")<<this->soften<<_T(" ");
		float glow_power=this->shadeTree[MAP_GLOW].weight;
		if (this->shadeTree[MAP_FILTER].weight==0)
			glow_power*=4000;
		else
			glow_power/=shadeTree[MAP_FILTER].weight;
		out<< glow_power<<std::endl;
		if (found_caps[NORMAL_NICK]||found_caps[BUMP_NICK]) {
			out << _T("\t\t\t\tparam_named bumpScaleBiasNormScale float4 ")<<offset_scale<<_T(" ")<<offset_bias<<_T(" ")<<normal_scale<<_T(" ")<< normal_scale <<std::endl;
		}
		out << _T("\t\t\t}")<<std::endl;
        std::string texunits[NUM_NICK];
		texunits[AMBIENT_OCCLUSION_NICK]=writetex(MAP_AMBIENT,0,"AmbientOcclusion",sceneProperties);

		texunits[DIFFUSE_NICK]=writetex(MAP_DIFFUSE,0,"Diffuse",sceneProperties);
		texunits[DIFFUSE_DIRT_NICK]=writetex(MAP_DIFFUSE,1,"DiffuseDirt",sceneProperties);
		texunits[DIFFUSE_CRUFT_NICK]=writetex(MAP_DIFFUSE,2,"DiffuseGrunge",sceneProperties);
        
		texunits[SPECULAR_NICK]=writetex(MAP_SPECULAR,0,"Specular",sceneProperties);
		texunits[SPECULAR_DIRT_NICK]=writetex(MAP_SPECULAR,1,"SpecularDirt",sceneProperties);        
		texunits[SPECULAR_CRUFT_NICK]=writetex(MAP_SPECULAR,2,"SpecularGrunge",sceneProperties);

		texunits[GLOW_NICK]=writetex(MAP_GLOW,0,"Glow",sceneProperties);
		texunits[GLOW_DIRT_NICK]=writetex(MAP_GLOW,1,"GlowDirt",sceneProperties);
		texunits[GLOW_CRUFT_NICK]=writetex(MAP_GLOW,2,"GlowGrunge",sceneProperties);

		texunits[REFLECTION_NICK]=writetex(MAP_REFLECTION,0,"Reflection",sceneProperties);//hmm need to process into cubemap

		texunits[NORMAL_NICK]=writetex(MAP_BUMP,0,"Normal",sceneProperties);
		texunits[DIRT_NORMAL_NICK]=writetex(MAP_BUMP,1,"NormalDirt",sceneProperties);
		
		texunits[BUMP_NICK]=writetex(MAP_BUMP,-1,"Bump",sceneProperties);
        out << setTextureUnits(found_caps,false,texunits);
		/*for (int i=0;i<NUM_MAPS;++i) {
			out << "Entry "<<i<<" With Weight "<<shadeTree[i].weight;
			for (unsigned int j=0;j<shadeTree[i].entries.size();++j) {
				out << ", "<<shadeTree[i].entries[j].filename<<":"<<shadeTree[i].entries[j].texture_set;
				if (j!=0)
					out << "@"<<shadeTree[i].entries[j].combiner;
				
			}
			out << std::endl;
		}*/
		out << _T("\t\t}") << std::endl;
		out << _T("\t}") << std::endl;
		out << _T("}") << std::endl;
		if (use_alpha) {
			out << _T("material ")<<materialName<<" : "<<alphaName<<std::endl;
			out << _T("{") <<std::endl;
			out << _T("\ttechnique *")<<std::endl;
			out << _T("\t{")<<std::endl;
			out << _T("\t\tpass Only*") << std::endl;
			out << _T("\t\t{")<<std::endl;
			if (this->translucent) {
	            out << _T("\t\t\tdepth_write off")<<std::endl;
	            out << _T("\t\t\talpha_rejection greater 0")<<std::endl;
			}else {
	            out << _T("\t\t\talpha_to_coverage on")<<std::endl;
		        out << _T("\t\t\talpha_rejection greater 128")<<std::endl;			
			}
			out << _T("\t\t\tscene_blend alpha_blend")<<std::endl;
			out << _T("\t\t}") << std::endl;
			out << _T("\t\tpass Ambient*") << std::endl;
			out << _T("\t\t{")<<std::endl;
			if (this->translucent) {
	            out << _T("\t\t\tdepth_write off")<<std::endl;
	            out << _T("\t\t\talpha_rejection greater 0")<<std::endl;
			}else {
	            out << _T("\t\t\talpha_to_coverage on")<<std::endl;
		        out << _T("\t\t\talpha_rejection greater 128")<<std::endl;			
			}
			out << _T("\t\t\tscene_blend alpha_blend")<<std::endl;
			out << _T("\t\t}") << std::endl;
			out << _T("\t\tpass Per*") << std::endl;
			out << _T("\t\t{")<<std::endl;
			if (this->translucent) {
	            out << _T("\t\t\tdepth_write off")<<std::endl;
	            out << _T("\t\t\talpha_rejection greater 0")<<std::endl;
			}else {
	            out << _T("\t\t\talpha_to_coverage on")<<std::endl;
		        out << _T("\t\t\talpha_rejection greater 128")<<std::endl;			
			}
			out << _T("\t\t\tscene_blend src_alpha one")<<std::endl;
			out << _T("\t\t}") << std::endl;
			out << _T("\t}") << std::endl;
			out << _T("}") << std::endl;
		}
	}
};
//std::ostream *kout=NULL;
void MaterialExporter::AddMultitexture(ExportMat&emat, Texmap * mtmap, int suboffset, bool lastmult) {
	if (mtmap->ClassID() == Class_ID(BMTEX_CLASS_ID,0)) {
		TSTR bmap = ((BitmapTex*)mtmap)->GetMapName();		
		bmap = Utilities::GetFileName(bmap);
		bmap = Utilities::GetRenamedBitmapFileName(bmap, this->sceneProperties);

		std::string tmp=bmap;
		emat.textures.back().push_back(TEXTUREINT(tmp,mtmap->GetMapChannel()));
//		if (kout)(*kout) << "//AddingBare "<<tmp<<std::endl;
	} else if (mtmap->ClassID() == Class_ID(TEXMAP_CONTAINER_CLASS_ID, 0)||mtmap->ClassID()==Class_ID(MIX_CLASS_ID,0)||mtmap->ClassID()==TEXMAP_OVER_CLASS_ID||mtmap->ClassID()==TEXMAP_MULTIPLY_CLASS_ID) {
		bool mult=(mtmap->ClassID()==TEXMAP_MULTIPLY_CLASS_ID)?true:false;
		int numsubmaps=mtmap->NumSubTexmaps();
		if (mtmap->ClassID()==Class_ID(MIX_CLASS_ID,0)&&numsubmaps>2)
			numsubmaps=2;//dont allow specification of mix function--thats either blend50% or the alpha of the diffuse map for now
		for (int i=suboffset;i<numsubmaps;++i) {
			Texmap*texMap;
			texMap = mtmap->GetSubTexmap(i);
			if (texMap&&texMap->ClassID() == Class_ID(BMTEX_CLASS_ID, 0)) {
				std::string tmp;
				emat.textures.back().push_back(TEXTUREINT(tmp=Utilities::GetRenamedBitmapFileName(Utilities::GetFileName((TCHAR*)((BitmapTex*)texMap)->GetMapName()),this->sceneProperties),(i==0?lastmult:mult)?-texMap->GetMapChannel():texMap->GetMapChannel()));
//				if (kout)(*kout) << "//Adding "<<tmp<<std::endl;
			}else if (texMap){
				AddMultitexture(emat,texMap,0,mult);
			}
		}
	}else  {
//		if (kout)(*kout) << "//UnknownX " << mtmap->ClassID().PartA()<< "::"<<mtmap->ClassID().PartB()<<std::endl;
		emat.textures.back().push_back(TEXTUREINT());
	}
}
bool MaterialExporter::WritePass(stdostream& out, const TSTR& materialName, IGameMaterial* material)
{

	Point4 val4;
	Point3 val3;
	PropType pt;
	ExportMat emat(materialName.data());
	IGameProperty* prop;

    prop = (material) ? material->GetAmbientData() : 0;
	if (prop)
    {
		pt = prop->GetType();

		if (pt == IGAME_POINT3_PROP)
        {
			prop->GetPropertyValue(val3);
			emat.colors[MAP_AMBIENT].x=val3.x;
			emat.colors[MAP_AMBIENT].y=val3.y;
			emat.colors[MAP_AMBIENT].z=val3.z;
		}

		if (pt == IGAME_POINT4_PROP)
        {
			prop->GetPropertyValue(val4);
			emat.colors[MAP_AMBIENT]=val4;
		}
	}

    prop = (material) ? material->GetDiffuseData() : 0;
	if (prop)
    {
		pt = prop->GetType();

		if (pt == IGAME_POINT3_PROP)
        {
			prop->GetPropertyValue(val3);
			emat.colors[MAP_DIFFUSE].x=val3.x;
			emat.colors[MAP_DIFFUSE].y=val3.y;
			emat.colors[MAP_DIFFUSE].z=val3.z;
		}
		if (pt == IGAME_POINT4_PROP)
        {
			prop->GetPropertyValue(val4);
			emat.colors[MAP_DIFFUSE]=val4;
		}
	}

    prop = (material) ? material->GetSpecularData() : 0;
	if (prop)
    {
		pt = prop->GetType();

		if (pt == IGAME_POINT3_PROP)
        {
            float alpha = 1;
			prop->GetPropertyValue(val3);
			emat.colors[MAP_SPECULAR].x=val3.x;
			emat.colors[MAP_SPECULAR].y=val3.y;
			emat.colors[MAP_SPECULAR].z=val3.z;
		}

		if (pt == IGAME_POINT4_PROP)
        {
			prop->GetPropertyValue(val4);
			emat.colors[MAP_SPECULAR]=val4;//fixme overwrite alpha with shininess
		}
	}

    prop = (material) ? material->GetEmissiveData() : 0;
	if (prop)
    {
		pt = prop->GetType();

		if (pt == IGAME_POINT3_PROP)
        {
			prop->GetPropertyValue(val3);
			emat.colors[MAP_GLOW].x=val3.x;
			emat.colors[MAP_GLOW].y=val3.y;
			emat.colors[MAP_GLOW].z=val3.z;
		}

		if (pt == IGAME_POINT4_PROP)
        {
			prop->GetPropertyValue(val4);
			emat.colors[MAP_GLOW]=val4;
		}
	}

    int numTexMaps = (material) ? material->GetNumberOfTextureMaps() : 0;
	int texcounter=0;
	std::string ambient_occlusion;
	int ambient_occlusion_channel=0;
	StdMat2 * tmat=dynamic_cast<StdMat2*>(material?material->GetMaxMaterial():NULL);
	if (material&&tmat) {
		emat.two_sided=tmat->GetTwoSided()?true:false;
		emat.translucent=tmat->GetOpacity(0.0)<1.0;
		Shader * shad=tmat->GetShader();
		if (shad) {
                     emat.glossiness=shad->GetGlossiness(0)*100.;
			emat.soften=shad->GetSoftenLevel(0);
			emat.specularLevel=shad->GetSpecularLevel(0);
		}
		int texcounter=0;
		for (int i=0;i<NUM_MAPS;++i) {
			long chan=tmat->StdIDToChannel(i);
			float weight = tmat->GetTexmapAmt(chan,0);
			emat.shadeTree[i].weight=weight;
			if (tmat->MapEnabled(chan)) {
				if (texcounter<numTexMaps) {
					IGameTextureMap* tmap = material->GetIGameTextureMap(texcounter);
					if (tmap ) {
						Texmap * maxtmap=tmap->GetMaxTexmap();
						if (maxtmap) {
							emat.LoadShadeTree(i,weight,maxtmap);
							if (doAbort)
								return false;
						}
					}
				}else {
					ERRMSG(1024,(data,"Error: Texture for channel # %d does not exist in standard material",texcounter));
					if (doAbort)
						return false;
				}
				++texcounter;
			}
		}
	}
       std::vector<std::string> texvec;
       std::map<std::string,std::string> texmap;
       //emat.gatherTextures(texvec); 
       //emat.findRenameTextures(texvec,texmap);
       emat.renameLowerTextures();//texmap);
	emat.write(out,this->sceneProperties);

	return true;
}

bool MaterialExporter::WriteMaterials(const MaterialMap& materialMap, stdostream& out)
{
	doAbort=false;
    for (MaterialMap::const_iterator it = materialMap.begin();
	    it != materialMap.end();
        ++it)
    {
        const TSTR& materialName = it->first;
		IGameMaterial* material = it->second;
		if (material)
        {
			int numSubMtl = material->GetSubMaterialCount();
			if (numSubMtl > 0)
            {
				for (int i = 0; i < numSubMtl; i++) {
					if (i!=0) {
						AssertDialog("More than one submaterial disallowed! Please combine your materials into a single material and use RGB Multiply and Combine to layer on textures.");						
						if (doAbort) return false;
					}else {
                		WritePass(out, materialName, material->GetSubMaterial(i));
						if (doAbort) return false;
					}
				}
			}
			else
				WritePass(out, materialName, material);
			if (doAbort) return false;
		}
		else
        	WritePass(out, materialName, material);
		if (doAbort) return false;
	}

	return true;
}
