
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "MeshAnimationSettingsSerializer.h"
#include "Utilities.h"
#include "StringSplitter.h"
#include "PropertyKeys.h"
#include "PropertyValues.h"
#include "PluginGlobals.h"

using namespace OgreMax;


//Implementation---------------------------------------------------------------
MeshAnimationSettingsSerializer::MeshAnimationSettingsSerializer()
{
    this->includeMarker = false;

    this->track = NONE;
    this->changed = false;    
}

void MeshAnimationSettingsSerializer::ParseProperty
    (
    const TSTR& key, 
    const TSTR& value, 
    int line, 
    std::vector<int>& handledLines
    )
{
    if (Utilities::IsSameNoCase(key, PropertyKeys::ANIMATION_NAME))
    {
        handledLines.push_back(line);
        this->settings.name = value;
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::SCALE))
    {
        handledLines.push_back(line);
        this->settings.scale = value;
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::LENGTH))
    {
        handledLines.push_back(line);
        this->settings.length = value;
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::COPY_FIRST_KEY_TO_LAST))
    {
        handledLines.push_back(line);
        this->settings.copyFirstKeyToLast = value;
    }
    else if (Utilities::StartsWithNoCase(key, PropertyKeys::EXPORT_BONE_TREE))
    {
        TSTR keyValueLine = key + _T("=") + value;
        
        BoneAllow exportBone;
        exportBone.tree = PropertyValues::TRUE_TEXT;
        if (Utilities::ParsePrefixedKeyValueLine(keyValueLine, exportBone.name, exportBone.allow))
        {
            this->settings.exportBones.push_back(exportBone);
            handledLines.push_back(line);
        }
    }
    else if (Utilities::StartsWithNoCase(key, PropertyKeys::EXPORT_BONE))
    {
        TSTR keyValueLine = key + _T("=") + value;
        
        BoneAllow exportBone;
        if (Utilities::ParsePrefixedKeyValueLine(keyValueLine, exportBone.name, exportBone.allow))
        {
            this->settings.exportBones.push_back(exportBone);
            handledLines.push_back(line);
        }
    }
    else if (Utilities::StartsWithNoCase(key, PropertyKeys::REMOVE_BONE_TRANSLATION))
    {
        TSTR keyValueLine = key + _T("=") + value;
        
        RemoveBoneTranslation remove;
        if (Utilities::ParsePrefixedKeyValueLine(keyValueLine, remove.boneName, remove.xyz))
        {
            this->settings.removeBoneTranslations.push_back(remove);
            handledLines.push_back(line);
        }
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::SAMPLE_INTERVAL))
    {
        handledLines.push_back(line);
        this->settings.sampleInterval = value;
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::SAMPLE_RATE))
    {
        handledLines.push_back(line);
        this->settings.sampleRate = value;
    }
}

void MeshAnimationSettingsSerializer::GetPropertiesText(TSTR& text, const TSTR& newline)
{
    text = _T("");

    if (!Utilities::IsNull(this->settings.name))
        Utilities::AppendKeyAndValue(text, PropertyKeys::ANIMATION_NAME, this->settings.name, newline);
    if (!Utilities::IsNull(this->settings.scale))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SCALE, this->settings.scale, newline);
    if (!Utilities::IsNull(this->settings.length))
        Utilities::AppendKeyAndValue(text, PropertyKeys::LENGTH, this->settings.length, newline);
    if (!Utilities::IsNull(this->settings.copyFirstKeyToLast))
        Utilities::AppendKeyAndValue(text, PropertyKeys::COPY_FIRST_KEY_TO_LAST, this->settings.copyFirstKeyToLast, newline);
    for (size_t i = 0; i < this->settings.exportBones.size(); i++)
    {
        if (Utilities::ToBool(this->settings.exportBones[i].tree))
            Utilities::AppendPrefixedKeyAndValue(text, PropertyKeys::EXPORT_BONE_TREE, this->settings.exportBones[i].name, this->settings.exportBones[i].allow, newline);
        else
            Utilities::AppendPrefixedKeyAndValue(text, PropertyKeys::EXPORT_BONE, this->settings.exportBones[i].name, this->settings.exportBones[i].allow, newline);
    }
    for (size_t i = 0; i < this->settings.removeBoneTranslations.size(); i++)
        Utilities::AppendPrefixedKeyAndValue(text, PropertyKeys::REMOVE_BONE_TRANSLATION, this->settings.removeBoneTranslations[i].boneName, this->settings.removeBoneTranslations[i].xyz, newline);
    if (!Utilities::IsNull(this->settings.sampleInterval))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SAMPLE_INTERVAL, this->settings.sampleInterval, newline);
    if (!Utilities::IsNull(this->settings.sampleRate))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SAMPLE_RATE, this->settings.sampleRate, newline);
}

AllMeshAnimationSettingsSerializer::AllMeshAnimationSettingsSerializer()
{
    this->node = 0;
}

void AllMeshAnimationSettingsSerializer::Read(INode* node)
{
    this->node = node;

    static MeshAnimationSettingsSerializer::Track tracks[MODIFIER_TRACK_COUNT] = 
        {
        MeshAnimationSettingsSerializer::MORPHER,
        MeshAnimationSettingsSerializer::PHYSIQUE,
        MeshAnimationSettingsSerializer::SKIN,
        MeshAnimationSettingsSerializer::OBJECT
        };

    //Get all the animation note tracks
    this->noteTracks[0] = (DefNoteTrack*)Utilities::GetAnimationNoteTrack(Utilities::FindMorpherModifier(node));
    this->noteTracks[1] = (DefNoteTrack*)Utilities::GetAnimationNoteTrack(Utilities::FindPhysiqueModifier(node));    
    this->noteTracks[2] = (DefNoteTrack*)Utilities::GetAnimationNoteTrack(Utilities::FindSkinModifier(node));
    this->noteTracks[3] = (DefNoteTrack*)Utilities::GetAnimationNoteTrack(node->GetObjectRef());

    //Count the number of animations
    int animationCount = 0;
    for (int i = 0; i < MODIFIER_TRACK_COUNT; i++)
    {
        if (this->noteTracks[i] != NULL)
            animationCount += this->noteTracks[i]->NumKeys() / 2;        
    }
    this->settings.resize(animationCount);

    //Get the animations
    animationCount = 0;
    for (int i = 0; i < MODIFIER_TRACK_COUNT; i++)
    {
        if (this->noteTracks[i] != NULL)
        {
            for (int keyIndex = 0; keyIndex < this->noteTracks[i]->NumKeys(); keyIndex += 2)
            {
                this->settings[animationCount].track = tracks[i];

                this->settings[animationCount].timeInterval.SetStart(this->noteTracks[i]->GetKeyTime(keyIndex));
                this->settings[animationCount].timeInterval.SetEnd(this->noteTracks[i]->GetKeyTime(keyIndex + 1));

                this->settings[animationCount].noteKey = this->noteTracks[i]->keys[keyIndex];

                this->settings[animationCount].Read(this->noteTracks[i]->keys[keyIndex]->note);
                
                animationCount++;
            }
        }
    }
}

void AllMeshAnimationSettingsSerializer::Write()
{
    for (size_t i = 0; i < this->settings.size(); i++)
    {
        TSTR animationText;
        this->settings[i].settings.ClearDefaults();
        this->settings[i].Write(animationText);
        this->settings[i].noteKey->note = animationText;
    }
}