
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "MeshNormals.h"

using namespace OgreMax;


//Implementation---------------------------------------------------------------

//VNormal implementation---------------------------
void VNormal::AddNormal(Point3& n, DWORD smoothingGroups) 
{
	if (!(smoothingGroups & this->smooth) && this->initialized) 
	{
		//Smoothing groups don't overlap. Add a new normal
	    if (this->next) 
            this->next->AddNormal(n, smoothingGroups);
		else
			this->next = new VNormal(n, smoothingGroups);
	} 
	else 
	{
        //Smoothing groups overlap or this is an initialization
		this->norm += n;
		this->smooth |= smoothingGroups;
		this->initialized = true;
	}
}

const Point3& VNormal::GetNormal(DWORD smoothingGroups) const
{
	//Retrieves a normal if the smoothing groups overlap or there is 
	//only one in the list

	if (this->smooth & smoothingGroups || !this->next) 
		return this->norm;
	else 
		return this->next->GetNormal(smoothingGroups);	
}

void VNormal::Normalize() 
{
	//Normalize each normal in the list

	VNormal* ptr = next;
    VNormal* prev = this;

	while (ptr) 
	{
		if (ptr->smooth & this->smooth) 
		{
			this->norm += ptr->norm;
			prev->next = ptr->next;
			delete ptr;
			ptr = prev->next;
		} 
		else 
		{
			prev = ptr;
			ptr = ptr->next;
		}
	}

	this->norm = ::Normalize(this->norm);
	
    if (this->next) 
		this->next->Normalize();
}

//MeshNormals implementation---------------------------
MeshNormals::MeshNormals()
{
}

MeshNormals::MeshNormals(Mesh& mesh)
{
    Create(mesh);
}

void MeshNormals::Create(Mesh& mesh)
{
    Face* face = mesh.faces;	
	Point3* verts = mesh.verts;
	Point3 v0, v1, v2;	
	
    //Initialize arrays
    this->faceNormals.SetCount(mesh.numFaces);
    this->faceSmoothingGroups.SetCount(mesh.numFaces);
    this->vertexNormals.SetCount(mesh.numVerts);
    for (int vertexIndex = 0; vertexIndex < mesh.numVerts; vertexIndex++) 
	    this->vertexNormals[vertexIndex] = VNormal();
        	
    //Compute face and vertex normals
    for (int faceIndex = 0; faceIndex < mesh.numFaces; faceIndex++, face++) 
    {
	    //Face smoothing groups
        this->faceSmoothingGroups[faceIndex] = face->smGroup;

        //Face normal
	    v0 = verts[face->v[0]];
	    v1 = verts[face->v[1]];
	    v2 = verts[face->v[2]];
	    this->faceNormals[faceIndex] = (v1-v0) ^ (v2-v1);
        this->faceNormals[faceIndex] = Normalize(this->faceNormals[faceIndex]);
		
        //Add the face normal to all the vertex normals
        for (int vertexIndex = 0; vertexIndex < 3; vertexIndex++) 
            this->vertexNormals[face->v[vertexIndex]].AddNormal(this->faceNormals[faceIndex], face->smGroup);
    }
	
    //Normalize vertex normals
    for (int vertexIndex = 0; vertexIndex < mesh.numVerts; vertexIndex++) 
	    this->vertexNormals[vertexIndex].Normalize();
}

Point3 MeshNormals::GetNormal(int vertexIndex, int faceIndex, DWORD smoothingGroups)
{
    if (this->faceSmoothingGroups[faceIndex] != 0)
        return this->vertexNormals[vertexIndex].GetNormal(smoothingGroups);
    else
        return this->faceNormals[faceIndex];
}