#ifndef OgreMax_MeshNormals_INCLUDED
#define OgreMax_MeshNormals_INCLUDED


//Classes----------------------------------------------------------------------
namespace OgreMax
{
    /** A mesh vertex normal, used by the MeshNormals class */
    class VNormal 
    {
    public:
	    VNormal() 
        {
            this->norm = Point3(0,0,0);
            this->smooth = 0;
            this->next = NULL;
            this->initialized = false;
        }
	    
        VNormal(const Point3& n, DWORD s) 
        {
            this->norm = n;
            this->smooth = s;
            this->next = NULL;
            this->initialized = true;
        }
	    
        ~VNormal() 
        {
            delete this->next;
        }

	    void AddNormal(Point3& n, DWORD smoothingGroups);

	    const Point3& GetNormal(DWORD smoothingGroups) const;

	    void Normalize();

    public:
        Point3 norm;
	    DWORD smooth;
	    VNormal* next;
	    bool initialized;
    };

    /**
     * The MeshNormals class assists in generating vertex normals for a
     * mesh, while taking smoothing groups into account.
     */
    class MeshNormals
    {
    public:
        MeshNormals();
	    MeshNormals(Mesh& mesh);

        void Create(Mesh& mesh);
	    
        Point3 GetNormal(int vertexIndex, int faceIndex, DWORD smoothingGroups);

    public:
	    Tab<VNormal> vertexNormals;
        Tab<Point3> faceNormals;
        Tab<DWORD> faceSmoothingGroups;
    };
}


#endif