
#ifndef OgreMax_ObjectSettings_INCLUDED
#define OgreMax_ObjectSettings_INCLUDED


//Includes---------------------------------------------------------------------
#include "CommonSettings.h"


//Classes----------------------------------------------------------------------
namespace OgreMax 
{
    struct ObjectSettings : UserDataSettings, LodSettings
    {
        TSTR type;
        TSTR visibility;
        TSTR targetType;
        TSTR isModel;
        TSTR modelName;
        TSTR animationName;
        TSTR animationEnabled;
        TSTR animationLooped;
        std::vector<NumberedValue> noteTrackNames;
        TSTR renderQueue;

        //Entity/plane settings
        TSTR isStatic;
        HardwareBufferUsage vertexBufferUsage;
        HardwareBufferUsage indexBufferUsage;
        
        //Entity settings
        TSTR meshName;
        TSTR isSimple;
        
        TSTR skeletonName;
        TSTR vertexAnimationType;
        TSTR skeletonAnimationSampleInterval;
        TSTR skeletonAnimationSampleRate;
        TSTR vertexAnimationSampleInterval;
        TSTR vertexAnimationSampleRate;
        TSTR meshAnimationSampleInterval;
        TSTR meshAnimationSampleRate;        
        TSTR removeBoneTranslation;

        //External/particle system/terrain settings
        TSTR fileName;

        //External settings
        TSTR externalItemType;
        
        //Sky settings
        TSTR enableSky;
        TSTR drawSkyFirst;

        //Sky dome settings
        TSTR scale;

        //Sky dome/plane settings
        TSTR curvature;

        //Sky plane settings
        TSTR bow;

        //Billboard set settings
        TSTR billboardType;
        TSTR billboardOrigin;
        TSTR billboardCommonDirection;
        TSTR billboardCommonUpVector;
        TSTR billboardRotationType;

        //Billboard settings
        TSTR billboardColor;

        //Plane settings
        TSTR generateNormals;

        TSTR ignoreChildren;
        TSTR childSorting;
        TSTR childOrder;

        void ClearDefaults();

        struct Values
        {
            static TSTR GetRenderQueue(const TSTR& renderQueueText);
            static TSTR GetRenderQueueAdd(const TSTR& renderQueueText);

            static bool IgnoreChildren(const TSTR& text);
            
            static bool IsModel(const TSTR& text);

            static bool IsNodeMainAnimationEnabled(const TSTR& text);
            static bool IsNodeMainAnimationLooped(const TSTR& text);

            static bool IsStatic(const TSTR& text);
            static bool IsSimple(const TSTR& text);

            static bool BufferUsageUseShadow(const TSTR& text);

            static bool GenerateNormals(const TSTR& text);

            static bool IsEnabled(const TSTR& text);
            static bool DrawFirst(const TSTR& text);

            static bool CopyFirstKeyToLast(const TSTR& text);
        };
    };  
}


#endif