
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "ObjectSettingsDialog.h"
#include "Utilities.h"
#include "resource.h"

using namespace OgreMax;


//Globals----------------------------------------------------------------------
static const int tabNameIDs[] = 
    {
    IDS_OBJECT_SETTINGS_GENERAL_TAB, 
    IDS_SETTINGS_USER_DATA_TAB, 
    IDS_OBJECT_SETTINGS_BILLBOARD_TAB,
    IDS_OBJECT_SETTINGS_BILLBOARD_SET_TAB,                
    IDS_OBJECT_SETTINGS_ENTITY_TAB,
    IDS_OBJECT_SETTINGS_EXTERNAL_TAB,
    IDS_OBJECT_SETTINGS_MODEL_INSTANCE_TAB,
    IDS_OBJECT_SETTINGS_PARTICLE_SYSTEM_TAB,
    IDS_OBJECT_SETTINGS_PLANE_TAB,
    IDS_OBJECT_SETTINGS_SKY_BOX_TAB,
    IDS_OBJECT_SETTINGS_SKY_DOME_TAB,
    IDS_OBJECT_SETTINGS_SKY_PLANE_TAB,
    IDS_OBJECT_SETTINGS_TERRAIN_TAB
    };
static const int tabPageIDs[] = 
    {
    IDD_OBJECT_SETTINGS_GENERAL_PAGE, 
    IDD_SETTINGS_USER_DATA_PAGE, 
    IDD_OBJECT_SETTINGS_BILLBOARD_PAGE,
    IDD_OBJECT_SETTINGS_BILLBOARD_SET_PAGE,                
    IDD_OBJECT_SETTINGS_ENTITY_PAGE,
    IDD_OBJECT_SETTINGS_EXTERNAL_PAGE,
    IDD_OBJECT_SETTINGS_MODEL_INSTANCE_PAGE,
    IDD_OBJECT_SETTINGS_PARTICLE_SYSTEM_PAGE,
    IDD_OBJECT_SETTINGS_PLANE_PAGE,
    IDD_OBJECT_SETTINGS_SKY_BOX_PAGE,
    IDD_OBJECT_SETTINGS_SKY_DOME_PAGE,
    IDD_OBJECT_SETTINGS_SKY_PLANE_PAGE,
    IDD_OBJECT_SETTINGS_TERRAIN_PAGE
    };

//The number of non-object tab pages that are inserted before object-related tab pages
static const int nonObjectPageCount = 2;

static const ObjectTypeDetector::Type tabPageObjectTypes[] = 
    {
    ObjectTypeDetector::BILLBOARD,
    ObjectTypeDetector::BILLBOARD_SET,                
    ObjectTypeDetector::ENTITY,
    ObjectTypeDetector::EXTERNAL,
    ObjectTypeDetector::MODEL_INSTANCE,
    ObjectTypeDetector::PARTICLE_SYSTEM,
    ObjectTypeDetector::PLANE,
    ObjectTypeDetector::SKY_BOX,
    ObjectTypeDetector::SKY_DOME,
    ObjectTypeDetector::SKY_PLANE,
    ObjectTypeDetector::TERRAIN
    };


//Local functions--------------------------------------------------------------
static int ObjectTypeToTabPageIndex(ObjectTypeDetector::Type& objectType)
{
    for (int i = 0; i < COUNT_OF(tabPageObjectTypes); i++)
    {
        if (tabPageObjectTypes[i] == objectType)
            return i + nonObjectPageCount;
    }

    return -1;
}


//Implementation---------------------------------------------------------------
ObjectSettingsDialog::ObjectSettingsDialog(ObjectSettings& objectSettings, INode* node) : 
    settings(objectSettings),
    generalTabPage(this, node, objectSettings, objectType),
    userDataTabPage(objectSettings),
    billboardTabPage(objectSettings),
    billboardSetTabPage(objectSettings),
    entityTabPage(objectSettings, node),
    externalTabPage(objectSettings),
    modelTabPage(objectSettings),
    particleSystemTabPage(objectSettings),
    planeTabPage(objectSettings),
    boxTabPage(objectSettings),
    skyDomeTabPage(objectSettings),
    skyPlaneTabPage(objectSettings),
    terrainTabPage(objectSettings)
{
    this->node = node;
    this->objectType.Detect(this->node, &this->settings.type);

    this->tabPages.reserve(COUNT_OF(tabNameIDs));
    this->tabPages.push_back(&this->generalTabPage);
    this->tabPages.push_back(&this->userDataTabPage);
    this->tabPages.push_back(&this->billboardTabPage);
    this->tabPages.push_back(&this->billboardSetTabPage);
    this->tabPages.push_back(&this->entityTabPage);
    this->tabPages.push_back(&this->externalTabPage);
    this->tabPages.push_back(&this->modelTabPage);
    this->tabPages.push_back(&this->particleSystemTabPage);
    this->tabPages.push_back(&this->planeTabPage);
    this->tabPages.push_back(&this->boxTabPage);
    this->tabPages.push_back(&this->skyDomeTabPage);
    this->tabPages.push_back(&this->skyPlaneTabPage);
    this->tabPages.push_back(&this->terrainTabPage);
}

ObjectSettingsDialog::~ObjectSettingsDialog()
{    
}

int ObjectSettingsDialog::DoModal()
{
    return DialogBase::DoModal(IDD_OBJECT_SETTINGS_DIALOG);
}

void ObjectSettingsDialog::SelectedObjectType(ObjectTypeDetector::Type objectType)
{
    //Hide existing page
    this->tabControl.HidePageTabRange(nonObjectPageCount);

    //Show the new page
    int pageIndex = ObjectTypeToTabPageIndex(objectType);
    if (pageIndex != -1)
        this->tabControl.ShowPageTab(pageIndex);
}

INT_PTR ObjectSettingsDialog::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    switch (message)
    {
        case WM_INITDIALOG:
        {
            //Set window title
            AppendWindowText(_T(" - "), this->node->GetName());

            //Center in parent window
            CenterWindow();

            //Create tab control
            this->tabControl.Create(GetDlgItem(IDC_OBJECT_SETTINGS_TAB), true);

            //Add the non-object tab pages
            for (size_t i = 0; i < nonObjectPageCount; i++)
                this->tabPages[i]->Attach(this->tabControl.AddPage(tabNameIDs[i], tabPageIDs[i]));
            
            //Add the object-related tab pages
            for (size_t i = nonObjectPageCount; i < this->tabPages.size(); i++)
            {
                bool visible = this->objectType.type == tabPageObjectTypes[i - nonObjectPageCount];
                this->tabPages[i]->Attach(this->tabControl.AddPage(tabNameIDs[i], tabPageIDs[i], -1, visible));
            }

            handled = true;

            break;
        }
        case WM_DESTROY:
        {
            for (size_t i = 0; i < this->tabPages.size(); i++)
                this->tabPages[i]->Detach();

            handled = true;

            break;
        }
        case WM_COMMAND:
        {
            int id = LOWORD(wParam);
            int code = HIWORD(wParam);

            switch (id)
            {
                case IDOK:
                {
                    //Figure out which object type is currently selected
                    int objectTypePage = this->tabControl.GetNearestVisiblePage(nonObjectPageCount, true);
                    
                    //Get the GUI data
                    for (size_t i = 0; i < nonObjectPageCount; i++)
                        this->tabPages[i]->GetGUIData();
                    if (objectTypePage >= nonObjectPageCount)
                        this->tabPages[objectTypePage]->GetGUIData();

                    this->settings.ClearDefaults();

                    handled = true;
                    EndDialog(this->hDlg, TRUE);
                    return TRUE;
                }
                case IDCANCEL:
                {
                    handled = true;
                    EndDialog(this->hDlg, FALSE);
                    return FALSE;
                }
            }

            break;
        }
        case WM_NOTIFY:
        {
            NMHDR* pnm = (NMHDR*)lParam;
            UINT controlID = (UINT)wParam;
            switch (controlID)
            {
                case IDC_OBJECT_SETTINGS_TAB: 
                {
                    this->tabControl.Notify(pnm);
                    handled = true;
                    return FALSE;
                }
            }
        }
    }

    return 0;
}