
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "ObjectSettingsSerializer.h"
#include "Utilities.h"
#include "StringSplitter.h"
#include "PropertyKeys.h"
#include "PluginGlobals.h"

using namespace OgreMax;


//Implementation---------------------------------------------------------------
ObjectSettingsSerializer::ObjectSettingsSerializer()
{
}

void ObjectSettingsSerializer::ParseProperty
    (
    const TSTR& key, 
    const TSTR& value, 
    int line, 
    std::vector<int>& handledLines
    )
{
    if (Utilities::IsSameNoCase(key, PropertyKeys::ID))
    {
        handledLines.push_back(line);
        this->settings.id = value;
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::USER_DATA_REFERENCE))
    {
        handledLines.push_back(line);
        this->settings.userDataReference = value;
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::USER_DATA_MARKER))
    {
        this->settings.userDataMarker = value;
        handledLines.push_back(line);

        //Get the user data
        int firstLine = this->textLines.FindLine(this->settings.userDataMarker);
        if (firstLine != -1)
        {
            int lastLine = this->textLines.FindLine(this->settings.userDataMarker, firstLine + 1);
            if (lastLine != -1)
            {
                //Mark all of the user data lines for removal
                for (int i = firstLine; i <= lastLine; i++)
                    handledLines.push_back(i);

                //Extract user data
                this->settings.userData = this->textLines.Join(PluginGlobals::CONTROL_NEWLINE, firstLine + 1, lastLine - firstLine - 1);
                Utilities::TrimWhitespace(this->settings.userData);
            }
        }
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::IS_USER_DATA_CDATA))
    {
        this->settings.isUserDataCDATA = value;
        handledLines.push_back(line);
    }    

    else if (Utilities::IsSameNoCase(key, PropertyKeys::GENERATE_LOD))
    {
        this->settings.generateLod = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::NUM_LOD_LEVELS))
    {
        this->settings.numLodLevels = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::LOD_DISTANCE))
    {
        this->settings.lodDistance = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::LOD_REDUCTION_TYPE))
    {
        this->settings.lodReductionType = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::LOD_PERCENT_REDUCTION))
    {
        this->settings.lodReductionAmount = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::LOD_VERTEX_REDUCTION))
    {
        this->settings.lodReductionAmount = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::LOD_REDUCTION_AMOUNT))
    {
        this->settings.lodReductionAmount = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::TYPE))
    {
        this->settings.type = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::VISIBILITY))
    {
        this->settings.visibility = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::TARGET_TYPE))
    {
        this->settings.targetType = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::IS_MODEL))
    {
        this->settings.isModel = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::MODEL_NAME))
    {
        this->settings.modelName = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::ANIMATION_NAME))
    {
        this->settings.animationName = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::ANIMATION_ENABLED))
    {
        this->settings.animationEnabled = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::ANIMATION_LOOPED))
    {
        this->settings.animationLooped = value;
        handledLines.push_back(line);
    }
    else if (Utilities::StartsWithNoCase(key, PropertyKeys::NOTE_TRACK_NAME))
    {
        NumberedValue numberedValue;
        numberedValue.number = Utilities::ParseTrailingInt(key) - 1;
        numberedValue.value = value;
        this->settings.noteTrackNames.push_back(numberedValue);
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::ANIMATION_LOOPED))
    {
        this->settings.animationLooped = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::RENDER_QUEUE))
    {
        this->settings.renderQueue = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::IS_STATIC))
    {
        this->settings.isStatic = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::VERTEX_BUFFER_USAGE))
    {
        this->settings.vertexBufferUsage.usage = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::VERTEX_BUFFER_USE_SHADOW))
    {
        this->settings.vertexBufferUsage.useShadow = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::INDEX_BUFFER_USAGE))
    {
        this->settings.indexBufferUsage.usage = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::INDEX_BUFFER_USE_SHADOW))
    {
        this->settings.indexBufferUsage.useShadow = value;
        handledLines.push_back(line);
    }
        
    else if (Utilities::IsSameNoCase(key, PropertyKeys::MESH_NAME))
    {
        this->settings.meshName = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::SIMPLE_MESH))
    {
        this->settings.isSimple = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::SKELETON_NAME))
    {
        this->settings.skeletonName = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::VERTEX_ANIMATION_TYPE))
    {
        this->settings.vertexAnimationType = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::SKELETON_ANIMATION_SAMPLE_INTERVAL))
    {
        this->settings.skeletonAnimationSampleInterval = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::SKELETON_ANIMATION_SAMPLE_RATE))
    {
        this->settings.skeletonAnimationSampleRate = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::VERTEX_ANIMATION_SAMPLE_INTERVAL))
    {
        this->settings.vertexAnimationSampleInterval = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::VERTEX_ANIMATION_SAMPLE_RATE))
    {
        this->settings.vertexAnimationSampleRate = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::MESH_ANIMATION_SAMPLE_INTERVAL))
    {
        this->settings.meshAnimationSampleInterval = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::MESH_ANIMATION_SAMPLE_RATE))
    {
        this->settings.meshAnimationSampleRate = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::REMOVE_BONE_TRANSLATION))
    {
        this->settings.removeBoneTranslation = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::FILE))
    {
        this->settings.fileName = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::ITEM_TYPE))
    {
        this->settings.externalItemType = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::ENABLE))
    {
        this->settings.enableSky = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::DRAW_FIRST))
    {
        this->settings.drawSkyFirst = value;
        handledLines.push_back(line);
    }        

    else if (Utilities::IsSameNoCase(key, PropertyKeys::CURVATURE))
    {
        this->settings.curvature = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::SCALE))
    {
        this->settings.scale = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::BOW))
    {
        this->settings.bow = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::BILLBOARD_SET_TYPE))
    {
        this->settings.billboardType = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::ORIGIN))
    {
        this->settings.billboardOrigin = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::COMMON_DIRECTION))
    {
        this->settings.billboardCommonDirection = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::COMMON_UP_VECTOR))
    {
        this->settings.billboardCommonUpVector = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::ROTATION_TYPE))
    {
        this->settings.billboardRotationType = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::DIFFUSE_COLOR))
    {
        this->settings.billboardColor = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::NORMALS))
    {
        this->settings.generateNormals = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::IGNORE_CHILDREN))
    {
        this->settings.ignoreChildren = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::CHILD_SORTING))
    {
        this->settings.childSorting = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::CHILD_ORDER))
    {
        this->settings.childOrder = value;
        handledLines.push_back(line);
    }   
}

void ObjectSettingsSerializer::GetPropertiesText(TSTR& text, const TSTR& newline)
{
    text = _T("");

    if (!Utilities::IsNull(this->settings.id))
        Utilities::AppendKeyAndValue(text, PropertyKeys::ID, this->settings.id, newline);
    if (!Utilities::IsNull(this->settings.userDataReference))
        Utilities::AppendKeyAndValue(text, PropertyKeys::USER_DATA_REFERENCE, this->settings.userDataReference, newline);
    if (!Utilities::IsNull(this->settings.userDataMarker) && !Utilities::IsNull(this->settings.userData))
    {
        Utilities::AppendKeyAndValue(text, PropertyKeys::USER_DATA_MARKER, this->settings.userDataMarker, newline);
        Utilities::AppendMarkedText(text, this->settings.userDataMarker, this->settings.userData, newline);
    }
    if (!Utilities::IsNull(this->settings.isUserDataCDATA))
        Utilities::AppendKeyAndValue(text, PropertyKeys::IS_USER_DATA_CDATA, this->settings.isUserDataCDATA, newline);    

    if (!Utilities::IsNull(this->settings.generateLod))
        Utilities::AppendKeyAndValue(text, PropertyKeys::GENERATE_LOD, this->settings.generateLod, newline);    
    if (!Utilities::IsNull(this->settings.numLodLevels))
        Utilities::AppendKeyAndValue(text, PropertyKeys::NUM_LOD_LEVELS, this->settings.numLodLevels, newline);    
    if (!Utilities::IsNull(this->settings.lodDistance))
        Utilities::AppendKeyAndValue(text, PropertyKeys::LOD_DISTANCE, this->settings.lodDistance, newline);    
    if (!Utilities::IsNull(this->settings.lodReductionType))
        Utilities::AppendKeyAndValue(text, PropertyKeys::LOD_REDUCTION_TYPE, this->settings.lodReductionType, newline);    
    if (!Utilities::IsNull(this->settings.lodReductionAmount))
        Utilities::AppendKeyAndValue(text, PropertyKeys::LOD_REDUCTION_AMOUNT, this->settings.lodReductionAmount, newline);    
    
    if (!Utilities::IsNull(this->settings.type))
        Utilities::AppendKeyAndValue(text, PropertyKeys::TYPE, this->settings.type, newline);
    if (!Utilities::IsNull(this->settings.visibility))
        Utilities::AppendKeyAndValue(text, PropertyKeys::VISIBILITY, this->settings.visibility, newline);
    if (!Utilities::IsNull(this->settings.targetType))
        Utilities::AppendKeyAndValue(text, PropertyKeys::TARGET_TYPE, this->settings.targetType, newline);
    if (!Utilities::IsNull(this->settings.isModel))
        Utilities::AppendKeyAndValue(text, PropertyKeys::IS_MODEL, this->settings.isModel, newline);
    if (!Utilities::IsNull(this->settings.modelName))
        Utilities::AppendKeyAndValue(text, PropertyKeys::MODEL_NAME, this->settings.modelName, newline);
    if (!Utilities::IsNull(this->settings.animationName))
        Utilities::AppendKeyAndValue(text, PropertyKeys::ANIMATION_NAME, this->settings.animationName, newline);
    if (!Utilities::IsNull(this->settings.animationEnabled))
        Utilities::AppendKeyAndValue(text, PropertyKeys::ANIMATION_ENABLED, this->settings.animationEnabled, newline);
    if (!Utilities::IsNull(this->settings.animationLooped))
        Utilities::AppendKeyAndValue(text, PropertyKeys::ANIMATION_LOOPED, this->settings.animationLooped, newline);
    for (size_t i = 0; i < this->settings.noteTrackNames.size(); i++)
        Utilities::AppendNumberedItem(text, PropertyKeys::NOTE_TRACK_NAME, this->settings.noteTrackNames[i].number + 1, this->settings.noteTrackNames[i].value, newline);
    if (!Utilities::IsNull(this->settings.renderQueue))
        Utilities::AppendKeyAndValue(text, PropertyKeys::RENDER_QUEUE, this->settings.renderQueue, newline);

    if (!Utilities::IsNull(this->settings.isStatic))
        Utilities::AppendKeyAndValue(text, PropertyKeys::IS_STATIC, this->settings.isStatic, newline);
    if (!Utilities::IsNull(this->settings.vertexBufferUsage.usage))
        Utilities::AppendKeyAndValue(text, PropertyKeys::VERTEX_BUFFER_USAGE, this->settings.vertexBufferUsage.usage, newline);
    if (!Utilities::IsNull(this->settings.vertexBufferUsage.useShadow))
        Utilities::AppendKeyAndValue(text, PropertyKeys::VERTEX_BUFFER_USE_SHADOW, this->settings.vertexBufferUsage.useShadow, newline);
    if (!Utilities::IsNull(this->settings.indexBufferUsage.usage))
        Utilities::AppendKeyAndValue(text, PropertyKeys::INDEX_BUFFER_USAGE, this->settings.indexBufferUsage.usage, newline);
    if (!Utilities::IsNull(this->settings.indexBufferUsage.useShadow))
        Utilities::AppendKeyAndValue(text, PropertyKeys::INDEX_BUFFER_USE_SHADOW, this->settings.indexBufferUsage.useShadow, newline);
            
    if (!Utilities::IsNull(this->settings.meshName))
        Utilities::AppendKeyAndValue(text, PropertyKeys::MESH_NAME, this->settings.meshName, newline);
    if (!Utilities::IsNull(this->settings.isSimple))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SIMPLE_MESH, this->settings.isSimple, newline);

    if (!Utilities::IsNull(this->settings.skeletonName))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SKELETON_NAME, this->settings.skeletonName, newline);
    if (!Utilities::IsNull(this->settings.vertexAnimationType))
        Utilities::AppendKeyAndValue(text, PropertyKeys::VERTEX_ANIMATION_TYPE, this->settings.vertexAnimationType, newline);
    if (!Utilities::IsNull(this->settings.skeletonAnimationSampleInterval))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SKELETON_ANIMATION_SAMPLE_INTERVAL, this->settings.skeletonAnimationSampleInterval, newline);
    if (!Utilities::IsNull(this->settings.skeletonAnimationSampleRate))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SKELETON_ANIMATION_SAMPLE_RATE, this->settings.skeletonAnimationSampleRate, newline);
    if (!Utilities::IsNull(this->settings.vertexAnimationSampleInterval))
        Utilities::AppendKeyAndValue(text, PropertyKeys::VERTEX_ANIMATION_SAMPLE_INTERVAL, this->settings.vertexAnimationSampleInterval, newline);
    if (!Utilities::IsNull(this->settings.vertexAnimationSampleRate))
        Utilities::AppendKeyAndValue(text, PropertyKeys::VERTEX_ANIMATION_SAMPLE_RATE, this->settings.vertexAnimationSampleRate, newline);
    if (!Utilities::IsNull(this->settings.meshAnimationSampleInterval))
        Utilities::AppendKeyAndValue(text, PropertyKeys::MESH_ANIMATION_SAMPLE_INTERVAL, this->settings.meshAnimationSampleInterval, newline);
    if (!Utilities::IsNull(this->settings.meshAnimationSampleRate))
        Utilities::AppendKeyAndValue(text, PropertyKeys::MESH_ANIMATION_SAMPLE_RATE, this->settings.meshAnimationSampleRate, newline);
    if (!Utilities::IsNull(this->settings.removeBoneTranslation))
        Utilities::AppendKeyAndValue(text, PropertyKeys::REMOVE_BONE_TRANSLATION, this->settings.removeBoneTranslation, newline);
    
    if (!Utilities::IsNull(this->settings.fileName))
        Utilities::AppendKeyAndValue(text, PropertyKeys::FILE, this->settings.fileName, newline);

    if (!Utilities::IsNull(this->settings.externalItemType))
        Utilities::AppendKeyAndValue(text, PropertyKeys::ITEM_TYPE, this->settings.externalItemType, newline);

    if (!Utilities::IsNull(this->settings.enableSky))
        Utilities::AppendKeyAndValue(text, PropertyKeys::ENABLE, this->settings.enableSky, newline);
    if (!Utilities::IsNull(this->settings.drawSkyFirst))
        Utilities::AppendKeyAndValue(text, PropertyKeys::DRAW_FIRST, this->settings.drawSkyFirst, newline);

    if (!Utilities::IsNull(this->settings.curvature))
        Utilities::AppendKeyAndValue(text, PropertyKeys::CURVATURE, this->settings.curvature, newline);

    if (!Utilities::IsNull(this->settings.scale))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SCALE, this->settings.scale, newline);
    if (!Utilities::IsNull(this->settings.bow))
        Utilities::AppendKeyAndValue(text, PropertyKeys::BOW, this->settings.bow, newline);

    if (!Utilities::IsNull(this->settings.billboardType))
        Utilities::AppendKeyAndValue(text, PropertyKeys::BILLBOARD_SET_TYPE, this->settings.billboardType, newline);
    if (!Utilities::IsNull(this->settings.billboardOrigin))
        Utilities::AppendKeyAndValue(text, PropertyKeys::ORIGIN, this->settings.billboardOrigin, newline);
    if (!Utilities::IsNull(this->settings.billboardCommonDirection))
        Utilities::AppendKeyAndValue(text, PropertyKeys::COMMON_DIRECTION, this->settings.billboardCommonDirection, newline);
    if (!Utilities::IsNull(this->settings.billboardCommonUpVector))
        Utilities::AppendKeyAndValue(text, PropertyKeys::COMMON_UP_VECTOR, this->settings.billboardCommonUpVector, newline);
    if (!Utilities::IsNull(this->settings.billboardRotationType))
        Utilities::AppendKeyAndValue(text, PropertyKeys::ROTATION_TYPE, this->settings.billboardRotationType, newline);

    if (!Utilities::IsNull(this->settings.billboardColor))
        Utilities::AppendKeyAndValue(text, PropertyKeys::DIFFUSE_COLOR, this->settings.billboardColor, newline);

    if (!Utilities::IsNull(this->settings.generateNormals))
        Utilities::AppendKeyAndValue(text, PropertyKeys::NORMALS, this->settings.generateNormals, newline);

    if (!Utilities::IsNull(this->settings.ignoreChildren))
        Utilities::AppendKeyAndValue(text, PropertyKeys::IGNORE_CHILDREN, this->settings.ignoreChildren, newline);
    if (!Utilities::IsNull(this->settings.childSorting))
        Utilities::AppendKeyAndValue(text, PropertyKeys::CHILD_SORTING, this->settings.childSorting, newline);
    if (!Utilities::IsNull(this->settings.childOrder))
        Utilities::AppendKeyAndValue(text, PropertyKeys::CHILD_ORDER, this->settings.childOrder, newline);
}