
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "ObjectSettingsTabPages.h"
#include "ObjectSettingsDialog.h"
#include "MeshAnimationSettingsDialog.h"
#include "PluginGlobals.h"
#include "Utilities.h"
#include "StringSplitter.h"
#include "PropertyKeys.h"
#include "PropertyValues.h"
#include "resource.h"

using namespace OgreMax;


//Globals-----------------------------------------------------------------------
static const int visibilityComboStringIDs[] = {IDS_DEFAULT, IDS_VISIBLE, IDS_HIDDEN, IDS_TREE_VISIBLE, IDS_TREE_HIDDEN};
static const TCHAR* VISIBILITIES[] = {_T(""), _T("visible"), _T("hidden"), _T("treeVisible"), _T("treeHidden")};
static const int VISIBILITY_COUNT = COUNT_OF(VISIBILITIES);

static const int renderQueueComboStringIDs[] = {IDS_BACKGROUND, IDS_SKIES_EARLY, IDS_QUEUE1, IDS_QUEUE2, IDS_WORLD_GEOMETRY1, IDS_QUEUE3, IDS_QUEUE4, IDS_MAIN, IDS_QUEUE6, IDS_QUEUE7, IDS_WORLD_GEOMETRY2, IDS_QUEUE8, IDS_QUEUE9, IDS_SKIES_LATE, IDS_OVERLAY, IDS_MAX};
static const TCHAR* RENDER_QUEUES[] = {_T("background"), _T("skiesEarly"), _T("queue1"), _T("queue2"), _T("worldGeometry1"), _T("queue3"), _T("queue4"), _T("main"), _T("queue6"), _T("queue7"), _T("worldGeometry2"), _T("queue8"), _T("queue9"), _T("skiesLate"), _T("overlay"), _T("max")};
static const int RENDER_QUEUE_COUNT = COUNT_OF(RENDER_QUEUES);
static const int MAIN_RENDER_QUEUE_INDEX = 7;

static const int sortChildrenComboStringIDs[] = {IDS_NONE, IDS_NAME, IDS_ID, IDS_ID_NUMBER, IDS_CHILD_ORDER};
static const TCHAR* SORT_CHILDREN[] = {_T(""), _T("name"), _T("id"), _T("idNumber"), _T("childOrder")};
static const int SORT_CHILDREN_COUNT = COUNT_OF(SORT_CHILDREN);

static const int billboardTypeComboStringIDs[] = {IDS_POINT, IDS_ORIENTED_COMMON, IDS_ORIENTED_SELF, IDS_PERPENDICULAR_COMMON, IDS_PERPENDICULAR_SELF};
static const TCHAR* BILLBOARD_TYPES[] = {_T("point"), _T("orientedCommon"), _T("orientedSelf"), _T("perpendicularCommon"), _T("perpendicularSelf")};
static const int BILLBOARD_TYPE_COUNT = COUNT_OF(BILLBOARD_TYPES);

static const int billboardOriginComboStringIDs[] = {IDS_CENTER, IDS_TOP_LEFT, IDS_TOP_CENTER, IDS_TOP_RIGHT, IDS_CENTER_RIGHT, IDS_BOTTOM_RIGHT, IDS_BOTTOM_CENTER, IDS_BOTTOM_LEFT, IDS_CENTER_LEFT};
static const TCHAR* BILLBOARD_ORIGINS[] = {_T("center"), _T("topLeft"), _T("topCenter"), _T("topRight"), _T("centerRight"), _T("bottomRight"), _T("bottomCenter"), _T("bottomLeft"), _T("centerLeft")};
static const int BILLBOARD_ORIGIN_COUNT = COUNT_OF(BILLBOARD_ORIGINS);

static const int billboardRotationComboStringIDs[] = {IDS_VERTEX, IDS_TEX_COORD};
static const TCHAR* BILLBOARD_ROTATIONS[] = {_T("vertex"), _T("texCoord")};
static const int BILLBOARD_ROTATION_COUNT = COUNT_OF(BILLBOARD_ROTATIONS);

static const int bufferUsageComboStringIDs[] = {IDS_STATIC, IDS_DYNAMIC, IDS_WRITE_ONLY, IDS_STATIC_WRITE_ONLY, IDS_DYNAMIC_WRITE_ONLY};
static const TCHAR* BUFFER_USAGES[] = {_T("static"), _T("dynamic"), _T("writeOnly"), _T("staticWriteOnly"), _T("dynamicWriteOnly")};
static const int BUFFER_USAGE_COUNT = COUNT_OF(BUFFER_USAGES);

static const int sampleTypeComboStringIDs[] = {IDS_NONE, IDS_SECONDS_BETWEEN_SAMPLES, IDS_SAMPLES_PER_SECOND};
enum {NO_SAMPLE_RATE_INDEX, SAMPLE_INTERVAL_INDEX, SAMPLE_RATE_INDEX, SAMPLE_TYPE_COUNT};

static const int generateLodStringIDs[] = {IDS_USE_SCENE_SETTING, IDS_YES, IDS_NO};
static const int GENERATE_LOD_COUNT = COUNT_OF(generateLodStringIDs);
static const TCHAR* GENERATE_LOD[] = {_T(""), _T("true"), _T("false")};

static const int lodReductionTypeComboStringIDs[] = {IDS_USE_SCENE_SETTING, IDS_PERCENT, IDS_VERTEX};

static const int meshAnimationTrackStringIDs[] = {IDS_EMPTY_STRING_WITH_SPACE, IDS_OBJECT, IDS_MORPHER, IDS_PHYSIQUE, IDS_SKIN};

static const int commonDirectionEditIDs[] = {IDC_COMMON_DIRECTION_X_EDIT, IDC_COMMON_DIRECTION_Y_EDIT, IDC_COMMON_DIRECTION_Z_EDIT};
static const int commonUpVectorEditIDs[] = {IDC_COMMON_UP_VECTOR_X_EDIT, IDC_COMMON_UP_VECTOR_Y_EDIT, IDC_COMMON_UP_VECTOR_Z_EDIT};


//Local functions--------------------------------------------------------------
static int GetObjectTypeValueComboStringID(ObjectTypeDetector::Type type)
{
    switch (type)
    {
        case ObjectTypeDetector::DEFAULT: return IDS_DEFAULT;
        case ObjectTypeDetector::IGNORE_OBJECT: return IDS_IGNORE;
        case ObjectTypeDetector::EMPTY: return IDS_EMPTY;
        case ObjectTypeDetector::MODEL_INSTANCE: return IDS_MODEL_INSTANCE;
        case ObjectTypeDetector::ENTITY: return IDS_ENTITY;
        case ObjectTypeDetector::LIGHT: return IDS_LIGHT;
        case ObjectTypeDetector::CAMERA: return IDS_CAMERA;
        case ObjectTypeDetector::EXTERNAL: return IDS_EXTERNAL;
        case ObjectTypeDetector::TERRAIN: return IDS_TERRAIN;
        case ObjectTypeDetector::SKY_BOX: return IDS_SKY_BOX;
        case ObjectTypeDetector::SKY_DOME: return IDS_SKY_DOME;
        case ObjectTypeDetector::SKY_PLANE: return IDS_SKY_PLANE;
        case ObjectTypeDetector::PARTICLE_SYSTEM: return IDS_PARTICLE_SYSTEM;
        case ObjectTypeDetector::BILLBOARD_SET: return IDS_BILLBOARD_SET;
        case ObjectTypeDetector::BILLBOARD: return IDS_BILLBOARD;
        case ObjectTypeDetector::PLANE: return IDS_PLANE;
        default: return IDS_DEFAULT;
    }    
}

static const TSTR& GetObjectTypeComboValue(int i)
{
    static const TSTR values[] = 
        {
        _T(""),
        PropertyValues::ObjectType::IGNORE_OBJECT,
        PropertyValues::ObjectType::EMPTY,
        PropertyValues::ObjectType::MODEL_INSTANCE,
        PropertyValues::ObjectType::ENTITY,
        PropertyValues::ObjectType::LIGHT,
        PropertyValues::ObjectType::CAMERA,
        PropertyValues::ObjectType::EXTERNAL,
        PropertyValues::ObjectType::TERRAIN,
        PropertyValues::ObjectType::SKY_BOX,
        PropertyValues::ObjectType::SKY_DOME,
        PropertyValues::ObjectType::SKY_PLANE,
        PropertyValues::ObjectType::PARTICLE_SYSTEM,
        PropertyValues::ObjectType::BILLBOARD_SET,
        PropertyValues::ObjectType::BILLBOARD,
        PropertyValues::ObjectType::PLANE
        };    
    return values[i];
}

static const TSTR& GetMeshAnimationTypeComboValue(int i)
{
    static const TSTR values[] = 
        {
        _T(""), //None
        PropertyValues::MeshAnimationType::POSE, //Pose only
        PropertyValues::MeshAnimationType::MORPH_ONLY, //Morph only
        _T(""), //Skeleton only
        PropertyValues::MeshAnimationType::POSE, //Skeleton and pose
        PropertyValues::MeshAnimationType::MORPH //Skeleton and morph
        };
    return values[i];
}

static const TSTR& GetVisibilityComboValue(int i)
{
    static const TSTR values[] = 
        {
        _T(""),
        PropertyValues::Visibility::VISIBLE,
        PropertyValues::Visibility::HIDDEN,
        PropertyValues::Visibility::TREE_VISIBLE,
        PropertyValues::Visibility::TREE_HIDDEN
        };
    return values[i];
}

static const TSTR& GetRenderQueueComboValue(int i)
{
    static const TSTR values[] =
        {
        PropertyValues::RenderQueue::BACKGROUND,
        PropertyValues::RenderQueue::SKIES_EARLY,
        PropertyValues::RenderQueue::QUEUE1,
        PropertyValues::RenderQueue::QUEUE2,
        PropertyValues::RenderQueue::WORLD_GEOMETRY1,
        PropertyValues::RenderQueue::QUEUE3,
        PropertyValues::RenderQueue::QUEUE4,
        PropertyValues::RenderQueue::MAIN,
        PropertyValues::RenderQueue::QUEUE6,
        PropertyValues::RenderQueue::QUEUE7,
        PropertyValues::RenderQueue::WORLD_GEOMETRY2,
        PropertyValues::RenderQueue::QUEUE8,
        PropertyValues::RenderQueue::QUEUE9,
        PropertyValues::RenderQueue::SKIES_LATE,
        PropertyValues::RenderQueue::OVERLAY,
        PropertyValues::RenderQueue::MAX
        };
    return values[i];
}

static const TSTR& GetChildSortingComboValue(int i)
{
    static const TSTR values[] =
        {
        _T(""),
        PropertyValues::ChildSorting::NAME,
        PropertyValues::ChildSorting::ID,
        PropertyValues::ChildSorting::ID_NUMBER,
        PropertyValues::ChildSorting::CHILD_ORDER
        };
    return values[i];
}

static int GetObjectTypeValueComboStringIndex(const ObjectTypeDetector& objectType)
{
    for (int i = 0; i < (int)objectType.allowedTypes.size(); i++)
    {
        if (objectType.type == objectType.allowedTypes[i])
        {
            //Add 1 because the first one is "default"
            return i + 1;
        }
    }

    return 0;    
}

static const TSTR& GetBillboardSetTypeComboValue(int i)
{
    static const TSTR values[] =
        {
        PropertyValues::BillboardSetType::POINT,
        PropertyValues::BillboardSetType::ORIENTED_COMMON,
        PropertyValues::BillboardSetType::ORIENTED_SELF,
        PropertyValues::BillboardSetType::PERPENDICULAR_COMMON,
        PropertyValues::BillboardSetType::PERPENDICULAR_SELF
        };
    return values[i];
}

static const TSTR& GetBillboardOriginComboValue(int i)
{
    static const TSTR values[] =
        {
        PropertyValues::BillboardOrigin::CENTER,
        PropertyValues::BillboardOrigin::TOP_LEFT,
        PropertyValues::BillboardOrigin::TOP_CENTER,
        PropertyValues::BillboardOrigin::TOP_RIGHT,
        PropertyValues::BillboardOrigin::CENTER_RIGHT,
        PropertyValues::BillboardOrigin::BOTTOM_RIGHT,
        PropertyValues::BillboardOrigin::BOTTOM_CENTER,
        PropertyValues::BillboardOrigin::BOTTOM_LEFT,
        PropertyValues::BillboardOrigin::CENTER_LEFT
        };
    return values[i];
}

static const TSTR& GetBillboardRotationTypeComboValue(int i)
{
    static const TSTR values[] =
        {
        PropertyValues::BillboardRotationType::VERTEX,
        PropertyValues::BillboardRotationType::TEX_COORD
        };
    return values[i];
}

static const TSTR& GetBufferUsageComboValue(int i)
{
    static const TSTR values[] =
        {
        PropertyValues::HardwareBufferUsage::STATIC,
        PropertyValues::HardwareBufferUsage::DYNAMIC,
        PropertyValues::HardwareBufferUsage::WRITE_ONLY,
        PropertyValues::HardwareBufferUsage::STATIC_WRITE_ONLY,
        PropertyValues::HardwareBufferUsage::DYNAMIC_WRITE_ONLY
        };
    return values[i];
}

static const TSTR& GetGenerateLod(int i)
{
    static const TSTR values[] =
        {
        _T(""),
        PropertyValues::TRUE_TEXT,
        PropertyValues::FALSE_TEXT
        };
    return values[i];
}

static const TSTR& GetLODReductionTypeComboValue(int i)
{
    static const TSTR values[] =
        {
        _T(""),
        PropertyValues::LODReductionType::PERCENT,
        PropertyValues::LODReductionType::VERTEX
        };
    return values[i];
}

static int GetVisibilityValueComboStringIndex(const TSTR& value)
{
    return Utilities::GetValueIndex(value, VISIBILITY_COUNT, VISIBILITIES);    
}

static int GetRenderQueueValueComboStringIndex(const TSTR& value)
{
    TSTR renderQueue = ObjectSettings::Values::GetRenderQueue(value);
    return Utilities::GetValueIndex(renderQueue, RENDER_QUEUE_COUNT, RENDER_QUEUES, MAIN_RENDER_QUEUE_INDEX);    
}

static int GetSortChildrenValueComboStringIndex(const TSTR& value)
{
    return Utilities::GetValueIndex(value, SORT_CHILDREN_COUNT, SORT_CHILDREN);    
}

static int GetBillboardTypeValueComboStringIndex(const TSTR& value)
{
    return Utilities::GetValueIndex(value, BILLBOARD_TYPE_COUNT, BILLBOARD_TYPES);    
}

static int GetBillboardOriginValueComboStringIndex(const TSTR& value)
{
    return Utilities::GetValueIndex(value, BILLBOARD_ORIGIN_COUNT, BILLBOARD_ORIGINS);    
}

static int GetBillboardRotationValueComboStringIndex(const TSTR& value)
{
    return Utilities::GetValueIndex(value, BILLBOARD_ROTATION_COUNT, BILLBOARD_ROTATIONS);    
}

static int GetBufferUsageValueComboStringIndex(const TSTR& value)
{
    return Utilities::GetValueIndex(value, BUFFER_USAGE_COUNT, BUFFER_USAGES);    
}

static int GetMeshAnimationTypeComboStringID(MeshAnimationTypeDetector::Type type)
{
    switch (type)
    {
        case MeshAnimationTypeDetector::POSE_ONLY: return IDS_POSE;
        case MeshAnimationTypeDetector::MORPH_ONLY: return IDS_MORPH;
        case MeshAnimationTypeDetector::SKELETON_ONLY: return IDS_SKELETON;
        case MeshAnimationTypeDetector::SKELETON_AND_POSE: return IDS_SKELETON_AND_POSE;
        case MeshAnimationTypeDetector::SKELETON_AND_MORPH: return IDS_SKELETON_AND_MORPH;
        default: return IDS_DEFAULT;
    }    
}

static int GetMeshAnimationTypeValueComboStringID(MeshAnimationTypeDetector::Type type)
{
    switch (type)
    {
        case MeshAnimationTypeDetector::DEFAULT: return IDS_DEFAULT;
        case MeshAnimationTypeDetector::POSE_ONLY: return IDS_POSE;
        case MeshAnimationTypeDetector::MORPH_ONLY: return IDS_MORPH;
        case MeshAnimationTypeDetector::SKELETON_ONLY: return IDS_SKELETON;
        case MeshAnimationTypeDetector::SKELETON_AND_POSE: return IDS_SKELETON_AND_POSE;
        case MeshAnimationTypeDetector::SKELETON_AND_MORPH: return IDS_SKELETON_AND_MORPH;
        default: return IDS_DEFAULT;
    }
}

static int GetMeshAnimationTypeValueComboStringIndex(const MeshAnimationTypeDetector& meshAnimationType)
{
    for (int i = 0; i < (int)meshAnimationType.allowedTypes.size(); i++)
    {
        if (meshAnimationType.type == meshAnimationType.allowedTypes[i])
        {
            //Add 1 because the first one is "default"
            return i + 1;
        }
    }

    return 0;    
}

static int GetAnimationSampleTypeValueComboStringIndex
    (
    const TSTR& intervalValue, 
    const TSTR& rateValue, 
    const TSTR& defaultIntervalValue, 
    const TSTR& defaultRateValue
    )
{
    if (Utilities::IsNull(intervalValue) && 
        Utilities::IsNull(rateValue) && 
        Utilities::IsNull(defaultIntervalValue) && 
        Utilities::IsNull(defaultRateValue))
    {
        return NO_SAMPLE_RATE_INDEX;
    }
    else if (!Utilities::IsNull(intervalValue) || !Utilities::IsNull(defaultIntervalValue))
        return SAMPLE_INTERVAL_INDEX;
    else
        return SAMPLE_RATE_INDEX;
}

static const TSTR& ChooseAnimationSampleValue
    (
    const TSTR& intervalValue, 
    const TSTR& rateValue, 
    const TSTR& defaultIntervalValue, 
    const TSTR& defaultRateValue
    )
{
    if (!Utilities::IsNull(intervalValue))
        return intervalValue;
    else if (!Utilities::IsNull(defaultIntervalValue))
        return defaultIntervalValue;
    else if (!Utilities::IsNull(defaultIntervalValue))
        return defaultIntervalValue;
    else
        return defaultRateValue;
}

static int GetGenerateLodValueComboStringIndex(const TSTR& value)
{
    return Utilities::GetValueIndex(value, GENERATE_LOD_COUNT, GENERATE_LOD);
}

static int GetLODReductionTypeValueComboStringIndex(const TSTR& value)
{
    if (Utilities::IsNull(value))
        return 0;
    else
        return Utilities::GetValueIndex(value, CommonSettings::Values::LOD_REDUCTION_TYPE_COUNT, CommonSettings::Values::LOD_REDUCTION_TYPES) + 1;
}

static void SetSampleValue(TSTR& sampleInterval, TSTR& sampleRate, int sampleTypeIndex, const TSTR& sampleValue)
{
    sampleInterval = sampleRate = _T("");

    if (sampleTypeIndex == SAMPLE_INTERVAL_INDEX)
        sampleInterval = sampleValue;
    else if (sampleTypeIndex == SAMPLE_RATE_INDEX)
        sampleRate = sampleValue;
}

static bool MeshAnimationSamplingComboIndexRequiresTimeValue(int index)
{
    return index != 0;
}


//Implementation----------------------------------------------------------------

//ObjectSettingsGeneralTabPage-------------------------
ObjectSettingsGeneralTabPage::ObjectSettingsGeneralTabPage
    (
    ObjectSettingsDialog* ownerDialog,
    INode* node,
    ObjectSettings& objectSettings, 
    ObjectTypeDetector& objectTypeDetector
    ) : 
    settings(objectSettings),
    objectType(objectTypeDetector)
{    
    this->ownerDialog = ownerDialog;
    this->node = node;
}

void ObjectSettingsGeneralTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void ObjectSettingsGeneralTabPage::GetGUIData()
{   
    //Type combo    
    HWND typeComboHwnd = GetDlgItem(IDC_OBJECT_TYPE_COMBO);
    int selectedTypeIndex = ComboBox_GetCurSel(typeComboHwnd);
    ObjectTypeDetector::Type objectType = (ObjectTypeDetector::Type)ComboBox_GetItemData(typeComboHwnd, selectedTypeIndex);
    if (objectType != this->defaultObjectType.type)
        this->settings.type = GetObjectTypeComboValue((int)objectType);
    else
        this->settings.type = _T("");
                    
    //Visibility combo
    HWND visibilityComboHwnd = GetDlgItem(IDC_VISIBILITY_COMBO);
    int selectedVisibilityIndex = ComboBox_GetCurSel(visibilityComboHwnd);
    this->settings.visibility = GetVisibilityComboValue(selectedVisibilityIndex);
        
    //Render queue combo
    HWND renderQueueComboHwnd = GetDlgItem(IDC_RENDER_QUEUE_COMBO);
    int selectedRenderQueueIndex = ComboBox_GetCurSel(renderQueueComboHwnd);
    this->settings.renderQueue = GetRenderQueueComboValue(selectedRenderQueueIndex);
        
    //Render queue addition edit
    if (!this->settings.renderQueue.isNull())
    {
        TSTR renderQueueAddText;
        GetDlgItemText(IDC_RENDER_QUEUE_ADD_EDIT, renderQueueAddText);        
        Utilities::TrimWhitespace(renderQueueAddText);
        if (!renderQueueAddText.isNull())
        {
            this->settings.renderQueue += _T("+");
            this->settings.renderQueue += renderQueueAddText;            
        }
    }

    //Ignore children checkbox
    this->settings.ignoreChildren = IsDlgButtonCheckedText(IDC_IGNORE_CHILDREN_CHECK);

    //Sort children combo
    HWND sortChildrenComboHwnd = GetDlgItem(IDC_SORT_CHILDREN_COMBO);
    int selectedSortChildrenIndex = ComboBox_GetCurSel(sortChildrenComboHwnd);
    this->settings.childSorting = GetChildSortingComboValue(selectedSortChildrenIndex);
    
    //Child order edit
    GetDlgItemText(IDC_CHILD_ORDER_EDIT, this->settings.childOrder);

    //Export to model file
    this->settings.isModel = IsDlgButtonCheckedText(IDC_IS_MODEL_CHECK);

    //Model name edit
    GetDlgItemText(IDC_MODEL_NAME_EDIT, this->settings.modelName);

    //Animation name edit
    GetDlgItemText(IDC_NODE_ANIMATION_NAME, this->settings.animationName);

    //Animation enabled checkbox
    this->settings.animationEnabled = IsDlgButtonCheckedText(IDC_NODE_ANIMATION_ENABLE_CHECK);

    //Animation looped checkbox
    this->settings.animationLooped = IsDlgButtonCheckedText(IDC_NODE_ANIMATION_LOOP_CHECK);

    //Remove bone translations checkboxes
    this->settings.removeBoneTranslation = _T("");
    if (IsDlgButtonChecked(IDC_NODE_REMOVE_BONE_TRANSLATION_X_CHECK))
        this->settings.removeBoneTranslation += _T("x");
    if (IsDlgButtonChecked(IDC_NODE_REMOVE_BONE_TRANSLATION_Y_CHECK))
        this->settings.removeBoneTranslation += _T("y");
    if (IsDlgButtonChecked(IDC_NODE_REMOVE_BONE_TRANSLATION_Z_CHECK))
        this->settings.removeBoneTranslation += _T("z");
    
    //Note track names
    TSTR noteTrackNamesText;
    GetDlgItemText(IDC_NOTE_TRACK_NAMES_EDIT, noteTrackNamesText);
    StringSplitter noteTrackNames(noteTrackNamesText, _T('\n'));
    this->settings.noteTrackNames.clear();
    this->settings.noteTrackNames.reserve(noteTrackNames.size());
    NumberedValue noteTrack;
    for (size_t i = 0; i < noteTrackNames.size(); i++)
    {
        Utilities::TrimWhitespace(noteTrackNames[i]);
        if (!noteTrackNames[i].isNull())
        {         
            noteTrack.value = noteTrackNames[i];
            this->settings.noteTrackNames.push_back(noteTrack);
            noteTrack.number++;
        }
    }
}

void ObjectSettingsGeneralTabPage::SetGUIData()
{
    //Type combo    
    HWND typeComboHwnd = GetDlgItem(IDC_OBJECT_TYPE_COMBO);
    ComboBox_AddString(typeComboHwnd, PluginGlobals::GetString(IDS_DEFAULT));
    this->defaultObjectType.Detect(this->node, NULL, false);
    ComboBox_SetItemData(typeComboHwnd, 0, this->defaultObjectType.type);
    for (int i = 0; i < this->objectType.allowedTypes.size(); i++)
    {
        ComboBox_AddString(typeComboHwnd, PluginGlobals::GetString(GetObjectTypeValueComboStringID(this->objectType.allowedTypes[i])));
        ComboBox_SetItemData(typeComboHwnd, i + 1, this->objectType.allowedTypes[i]);
    }
    ComboBox_SetCurSel(typeComboHwnd, GetObjectTypeValueComboStringIndex(this->objectType));
    UpdateObjectTypeDefaultText();
    
    //Visibility combo
    HWND visibilityComboHwnd = GetDlgItem(IDC_VISIBILITY_COMBO);
    for (int i = 0; i < VISIBILITY_COUNT; i++)
        ComboBox_AddString(visibilityComboHwnd, PluginGlobals::GetString(visibilityComboStringIDs[i]));
    ComboBox_SetCurSel(visibilityComboHwnd, GetVisibilityValueComboStringIndex(this->settings.visibility));    

    //Render queue combo
    HWND renderQueueComboHwnd = GetDlgItem(IDC_RENDER_QUEUE_COMBO);
    for (int i = 0; i < RENDER_QUEUE_COUNT; i++)
        ComboBox_AddString(renderQueueComboHwnd, PluginGlobals::GetString(renderQueueComboStringIDs[i]));
    ComboBox_SetCurSel(renderQueueComboHwnd, GetRenderQueueValueComboStringIndex(this->settings.renderQueue));    

    //Render queue addition edit
    TSTR renderQueueAddText = ObjectSettings::Values::GetRenderQueueAdd(this->settings.renderQueue);
    SetDlgItemText(IDC_RENDER_QUEUE_ADD_EDIT, renderQueueAddText);

    //Ignore children checkbox
    CheckDlgButton(IDC_IGNORE_CHILDREN_CHECK, ObjectSettings::Values::IgnoreChildren(this->settings.ignoreChildren));

    //Sort children combo
    HWND sortChildrenComboHwnd = GetDlgItem(IDC_SORT_CHILDREN_COMBO);
    for (int i = 0; i < SORT_CHILDREN_COUNT; i++)
        ComboBox_AddString(sortChildrenComboHwnd, PluginGlobals::GetString(sortChildrenComboStringIDs[i]));
    ComboBox_SetCurSel(sortChildrenComboHwnd, GetSortChildrenValueComboStringIndex(this->settings.childSorting));    

    //Child order edit
    SetDlgItemText(IDC_CHILD_ORDER_EDIT, this->settings.childOrder);

    //Export to model file
    CheckDlgButton(IDC_IS_MODEL_CHECK, ObjectSettings::Values::IsModel(this->settings.isModel));

    //Model name edit
    SetDlgItemText(IDC_MODEL_NAME_EDIT, this->settings.modelName);

    //Animation name edit
    SetDlgItemText(IDC_NODE_ANIMATION_NAME, this->settings.animationName);

    //Animation enabled checkbox
    CheckDlgButton(IDC_NODE_ANIMATION_ENABLE_CHECK, ObjectSettings::Values::IsNodeMainAnimationEnabled(this->settings.animationEnabled));

    //Animation looped checkbox
    CheckDlgButton(IDC_NODE_ANIMATION_LOOP_CHECK, ObjectSettings::Values::IsNodeMainAnimationLooped(this->settings.animationLooped));

    //Remove bone translations checkboxes
    CheckDlgButton(IDC_NODE_REMOVE_BONE_TRANSLATION_X_CHECK, Utilities::FindCharNoCase(this->settings.removeBoneTranslation, _T('x')) != -1);
    CheckDlgButton(IDC_NODE_REMOVE_BONE_TRANSLATION_Y_CHECK, Utilities::FindCharNoCase(this->settings.removeBoneTranslation, _T('y')) != -1);
    CheckDlgButton(IDC_NODE_REMOVE_BONE_TRANSLATION_Z_CHECK, Utilities::FindCharNoCase(this->settings.removeBoneTranslation, _T('z')) != -1);

    //Note track names
    TSTR noteTracksText;
    CommonSettings::WriteValues(noteTracksText, this->settings.noteTrackNames, PluginGlobals::CONTROL_NEWLINE);
    SetDlgItemText(IDC_NOTE_TRACK_NAMES_EDIT, noteTracksText);
}

INT_PTR ObjectSettingsGeneralTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    switch (message)
    {
        case WM_COMMAND:
        {
            int id = LOWORD(wParam);
            int code = HIWORD(wParam);

            switch (id)
            {
                case IDC_OBJECT_TYPE_COMBO:
                {
                    switch (code)
                    {
                        case CBN_SELCHANGE:
                        {
                            ObjectTypeDetector::Type objectType = UpdateObjectTypeDefaultText();
                            
                            //Notify owner dialog that the selected object type changed
                            this->ownerDialog->SelectedObjectType(objectType);

                            handled = true;
                            
                            break;
                        }
                    }
                    break;
                }
            }

            break;
        }        
    }

    return 0;
}

ObjectTypeDetector::Type ObjectSettingsGeneralTabPage::UpdateObjectTypeDefaultText()
{
    //Get selected index for object type combo box 
    int objectTypeIndex = ComboBox_GetCurSel(GetDlgItem(IDC_OBJECT_TYPE_COMBO));
    
    //Turn index into an object type
    ObjectTypeDetector::Type objectType;
    if (objectTypeIndex == 0)
        objectType = (ObjectTypeDetector::Type)ComboBox_GetItemData(GetDlgItem(IDC_OBJECT_TYPE_COMBO), objectTypeIndex);
    else
        objectType = this->objectType.allowedTypes[objectTypeIndex - 1];

    //Show default text
    TSTR objectTypeDefaultText;
    if (objectTypeIndex == 0 && objectType != ObjectTypeDetector::DEFAULT)
    {
        int objectStringID = GetObjectTypeValueComboStringID(objectType);
        TSTR objectTypeText = PluginGlobals::GetString(objectStringID);

        TSTR defaultFormat = PluginGlobals::GetString(IDS_DEFAULT_FORMAT);
        
        objectTypeDefaultText.printf(defaultFormat, objectTypeText.data());
    }
    SetDlgItemText(IDC_OBJECT_TYPE_TEXT, objectTypeDefaultText);

    return objectType;
}


//ObjectSettingsBillboardTabPage-------------------------
ObjectSettingsBillboardTabPage::ObjectSettingsBillboardTabPage(ObjectSettings& objectSettings) : settings(objectSettings)
{
    this->colorSwatch = NULL;
}

void ObjectSettingsBillboardTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void ObjectSettingsBillboardTabPage::Detach()
{
    ReleaseIColorSwatch(this->colorSwatch);
}

void ObjectSettingsBillboardTabPage::GetGUIData()
{    
    //Color
    Color color(this->colorSwatch->GetColor());
    this->settings.billboardColor = Utilities::ToString(color);    
}

void ObjectSettingsBillboardTabPage::SetGUIData()
{
    //Color
    this->colorSwatch = GetIColorSwatch(GetDlgItem(IDC_BILLBOARD_COLOR_SWATCH));
    Point3 color3 = Utilities::ToPoint3(this->settings.billboardColor);
    Color color(color3.x, color3.y, color3.z);
    this->colorSwatch->SetColor(color, FALSE);
}

INT_PTR ObjectSettingsBillboardTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    return 0;
}

//ObjectSettingsBillboardSetTabPage-------------------------
ObjectSettingsBillboardSetTabPage::ObjectSettingsBillboardSetTabPage(ObjectSettings& objectSettings) : settings(objectSettings)
{
}

void ObjectSettingsBillboardSetTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void ObjectSettingsBillboardSetTabPage::GetGUIData()
{    
    //Billboard type combo
    HWND billboardTypeComboHwnd = GetDlgItem(IDC_BILLBOARD_TYPE_COMBO);
    int selectedBillboardTypeIndex = ComboBox_GetCurSel(billboardTypeComboHwnd);    
    this->settings.billboardType = GetBillboardSetTypeComboValue(selectedBillboardTypeIndex);

    //Billboard origin combo
    HWND billboardOriginComboHwnd = GetDlgItem(IDC_BILLBOARD_ORIGIN_COMBO);
    int selectedBillboardOriginIndex = ComboBox_GetCurSel(billboardOriginComboHwnd);
    this->settings.billboardOrigin = GetBillboardOriginComboValue(selectedBillboardOriginIndex);

    //Billboard rotation combo
    HWND billboardRotationComboHwnd = GetDlgItem(IDC_BILLBOARD_ROTATION_COMBO);
    int selectedBillboardRotationIndex = ComboBox_GetCurSel(billboardRotationComboHwnd);
    this->settings.billboardRotationType = GetBillboardRotationTypeComboValue(selectedBillboardRotationIndex);

    //Common direction edits
    this->settings.billboardCommonDirection = _T("");
    TSTR commonDirectionPart;
    for (int i = 0; i < COUNT_OF(commonDirectionEditIDs); i++)
    {
        GetDlgItemText(commonDirectionEditIDs[i], commonDirectionPart);
        this->settings.billboardCommonDirection += commonDirectionPart.isNull() ? _T("0") : commonDirectionPart;
        if (i < COUNT_OF(commonDirectionEditIDs) - 1)
            this->settings.billboardCommonDirection += _T(",");
    }
    
    //Common up vector edits
    this->settings.billboardCommonUpVector = _T("");
    TSTR commonUpVectorPart;
    for (int i = 0; i < COUNT_OF(commonUpVectorEditIDs); i++)
    {
        GetDlgItemText(commonUpVectorEditIDs[i], commonUpVectorPart);
        this->settings.billboardCommonUpVector += commonUpVectorPart.isNull() ? _T("0") : commonUpVectorPart;
        if (i < COUNT_OF(commonUpVectorEditIDs) - 1)
            this->settings.billboardCommonUpVector += _T(",");
    }
}

void ObjectSettingsBillboardSetTabPage::SetGUIData()
{
    //Billboard type combo
    HWND billboardTypeComboHwnd = GetDlgItem(IDC_BILLBOARD_TYPE_COMBO);
    for (int i = 0; i < BILLBOARD_TYPE_COUNT; i++)
        ComboBox_AddString(billboardTypeComboHwnd, PluginGlobals::GetString(billboardTypeComboStringIDs[i]));
    ComboBox_SetCurSel(billboardTypeComboHwnd, GetBillboardTypeValueComboStringIndex(this->settings.billboardType));    

    //Billboard origin combo
    HWND billboardOriginComboHwnd = GetDlgItem(IDC_BILLBOARD_ORIGIN_COMBO);
    for (int i = 0; i < BILLBOARD_ORIGIN_COUNT; i++)
        ComboBox_AddString(billboardOriginComboHwnd, PluginGlobals::GetString(billboardOriginComboStringIDs[i]));
    ComboBox_SetCurSel(billboardOriginComboHwnd, GetBillboardOriginValueComboStringIndex(this->settings.billboardOrigin));    

    //Billboard rotation combo
    HWND billboardRotationComboHwnd = GetDlgItem(IDC_BILLBOARD_ROTATION_COMBO);
    for (int i = 0; i < BILLBOARD_ROTATION_COUNT; i++)
        ComboBox_AddString(billboardRotationComboHwnd, PluginGlobals::GetString(billboardRotationComboStringIDs[i]));
    ComboBox_SetCurSel(billboardRotationComboHwnd, GetBillboardRotationValueComboStringIndex(this->settings.billboardRotationType));    

    //Common direction edits
    StringSplitter commonDirection(this->settings.billboardCommonDirection, _T(','));
    for (int i = 0; i < std::min(3, (int)commonDirection.size()); i++)
    {
        Utilities::TrimWhitespace(commonDirection[i]);
        SetDlgItemText(commonDirectionEditIDs[i], commonDirection[i]);
    }

    //Common up vector edits
    StringSplitter commonUpVector(this->settings.billboardCommonUpVector, _T(','));
    for (int i = 0; i < std::min(3, (int)commonUpVector.size()); i++)
    {
        Utilities::TrimWhitespace(commonUpVector[i]);
        SetDlgItemText(commonUpVectorEditIDs[i], commonUpVector[i]);
    }
}

INT_PTR ObjectSettingsBillboardSetTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    return 0;
}

//ObjectSettingsEntityTabPage-------------------------
ObjectSettingsEntityTabPage::ObjectSettingsEntityTabPage(ObjectSettings& objectSettings, INode* node) : settings(objectSettings)
{
    this->node = node;
    this->lodDistanceSpin = NULL;
}

void ObjectSettingsEntityTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void ObjectSettingsEntityTabPage::Detach()
{
    ReleaseISpinner(this->lodDistanceSpin);    
}

void ObjectSettingsEntityTabPage::GetGUIData()
{
    //Mesh name edit
    GetDlgItemText(IDC_MESH_NAME_EDIT, this->settings.meshName);

    //Is static check
    this->settings.isStatic = IsDlgButtonCheckedText(IDC_STATIC_CHECK);

    //Is simple check
    this->settings.isSimple = IsDlgButtonCheckedText(IDC_SIMPLE_MESH_CHECK);

    //Vertex buffer usage combo
    HWND vertexBufferUsageComboHwnd = GetDlgItem(IDC_VERTEX_BUFFER_USAGE_COMBO);
    int selectedVertexBufferUsageIndex = ComboBox_GetCurSel(vertexBufferUsageComboHwnd);    
    this->settings.vertexBufferUsage.usage = GetBufferUsageComboValue(selectedVertexBufferUsageIndex);

    //Vertex buffer shadow check
    this->settings.vertexBufferUsage.useShadow = IsDlgButtonCheckedText(IDC_VERTEX_BUFFER_USE_SHADOW_CHECK);

    //Vertex buffer usage combo
    HWND indexBufferUsageComboHwnd = GetDlgItem(IDC_INDEX_BUFFER_USAGE_COMBO);
    int selectedIndexBufferUsageIndex = ComboBox_GetCurSel(indexBufferUsageComboHwnd);    
    this->settings.indexBufferUsage.usage = GetBufferUsageComboValue(selectedIndexBufferUsageIndex);

    //Index buffer shadow check
    this->settings.indexBufferUsage.useShadow = IsDlgButtonCheckedText(IDC_INDEX_BUFFER_USE_SHADOW_CHECK);

    //Skeleton name edit
    GetDlgItemText(IDC_SKELETON_NAME_EDIT, this->settings.skeletonName);

    //Mesh animation type combo    
    HWND meshAnimationTypeComboHwnd = GetDlgItem(IDC_MESH_ANIMATION_TYPE_COMBO);
    int selectedTypeIndex = ComboBox_GetCurSel(meshAnimationTypeComboHwnd);
    MeshAnimationTypeDetector::Type meshAnimationType = (MeshAnimationTypeDetector::Type)ComboBox_GetItemData(meshAnimationTypeComboHwnd, selectedTypeIndex);
    if (meshAnimationType != this->defaultMeshAnimationType.type)
        this->settings.vertexAnimationType = GetMeshAnimationTypeComboValue((int)meshAnimationType);
    else
        this->settings.vertexAnimationType = _T("");
        
    this->settings.meshAnimationSampleInterval = _T("");
    this->settings.meshAnimationSampleRate = _T("");

    //Skeleton animation sampling combo
    HWND skeletonAnimationSamplingComboHwnd = GetDlgItem(IDC_SKELETON_ANIMATION_SAMPLING_COMBO);
    int selectedSkeletonAnimationSampleTypeIndex = ComboBox_GetCurSel(skeletonAnimationSamplingComboHwnd);
    
    //Skeleton animation sampling value
    TSTR skeletonAnimationSampling;
    GetDlgItemText(IDC_SKELETON_ANIMATION_SAMPLING_EDIT, skeletonAnimationSampling);    
    SetSampleValue(this->settings.skeletonAnimationSampleInterval, this->settings.skeletonAnimationSampleRate, selectedSkeletonAnimationSampleTypeIndex, skeletonAnimationSampling);
    
    //Vertex animation sampling combo
    HWND vertexAnimationSamplingComboHwnd = GetDlgItem(IDC_VERTEX_ANIMATION_SAMPLING_COMBO);
    int selectedVertexAnimationSampleTypeIndex = ComboBox_GetCurSel(vertexAnimationSamplingComboHwnd);
    
    //Vertex animation sampling value
    TSTR vertexAnimationSampling;
    GetDlgItemText(IDC_VERTEX_ANIMATION_SAMPLING_EDIT, vertexAnimationSampling);
    SetSampleValue(this->settings.vertexAnimationSampleInterval, this->settings.vertexAnimationSampleRate, selectedSkeletonAnimationSampleTypeIndex, vertexAnimationSampling);

    //Generate LOD combo
    HWND generateLodComboHwnd = GetDlgItem(IDC_GENERATE_LOD_COMBO);
    int selectedGenerateLodIndex = ComboBox_GetCurSel(generateLodComboHwnd);
    this->settings.generateLod = GetGenerateLod(selectedGenerateLodIndex);

    //Number of LOD levels edit
    GetDlgItemText(IDC_NUM_LOD_LEVELS_EDIT, this->settings.numLodLevels);
    
    //LOD distance edit/spin
    this->settings.lodDistance = Utilities::ToString(this->lodDistanceSpin->GetFVal());

    //LOD reduction type combo
    HWND lodReductionTypeComboHwnd = GetDlgItem(IDC_LOD_REDUCE_TYPE_COMBO);
    int selectedLodReductionTypeIndex = ComboBox_GetCurSel(lodReductionTypeComboHwnd);    
    this->settings.lodReductionType = GetLODReductionTypeComboValue(selectedLodReductionTypeIndex);    

    //LOD reduction amount edit
    if (!this->settings.lodReductionType.isNull())
        GetDlgItemText(IDC_LOD_REDUCE_AMOUNT_EDIT, this->settings.lodReductionAmount);
    else
        this->settings.lodReductionAmount = _T("");

    //Mesh animations
    this->allMeshAnimationSettingsSerializer.Write();
}

void ObjectSettingsEntityTabPage::SetGUIData()
{
    //Mesh name edit
    SetDlgItemText(IDC_MESH_NAME_EDIT, this->settings.meshName);

    //Is static check
    CheckDlgButton(IDC_STATIC_CHECK, ObjectSettings::Values::IsStatic(this->settings.isStatic));

    //Is simple check
    CheckDlgButton(IDC_SIMPLE_MESH_CHECK, ObjectSettings::Values::IsSimple(this->settings.isSimple));

    //Vertex buffer usage combo
    HWND vertexBufferUsageComboHwnd = GetDlgItem(IDC_VERTEX_BUFFER_USAGE_COMBO);
    for (int i = 0; i < BUFFER_USAGE_COUNT; i++)
        ComboBox_AddString(vertexBufferUsageComboHwnd, PluginGlobals::GetString(bufferUsageComboStringIDs[i]));
    ComboBox_SetCurSel(vertexBufferUsageComboHwnd, GetBufferUsageValueComboStringIndex(this->settings.vertexBufferUsage.usage));    

    //Vertex buffer shadow check
    CheckDlgButton(IDC_VERTEX_BUFFER_USE_SHADOW_CHECK, ObjectSettings::Values::BufferUsageUseShadow(this->settings.vertexBufferUsage.useShadow));

    //Vertex buffer usage combo
    HWND indexBufferUsageComboHwnd = GetDlgItem(IDC_INDEX_BUFFER_USAGE_COMBO);
    for (int i = 0; i < BUFFER_USAGE_COUNT; i++)
        ComboBox_AddString(indexBufferUsageComboHwnd, PluginGlobals::GetString(bufferUsageComboStringIDs[i]));
    ComboBox_SetCurSel(indexBufferUsageComboHwnd, GetBufferUsageValueComboStringIndex(this->settings.indexBufferUsage.usage));    

    //Index buffer shadow check
    CheckDlgButton(IDC_INDEX_BUFFER_USE_SHADOW_CHECK, ObjectSettings::Values::BufferUsageUseShadow(this->settings.indexBufferUsage.useShadow));

    //Skeleton name edit
    SetDlgItemText(IDC_SKELETON_NAME_EDIT, this->settings.skeletonName);

    //Mesh animation type combo    
    this->meshAnimationType.Detect(this->node, &this->settings.vertexAnimationType);
    HWND meshAnimationTypeComboHwnd = GetDlgItem(IDC_MESH_ANIMATION_TYPE_COMBO);
    ComboBox_AddString(meshAnimationTypeComboHwnd, PluginGlobals::GetString(IDS_DEFAULT));
    this->defaultMeshAnimationType.Detect(this->node, NULL, false);
    ComboBox_SetItemData(meshAnimationTypeComboHwnd, 0, this->defaultMeshAnimationType.type);
    for (int i = 0; i < this->meshAnimationType.allowedTypes.size(); i++)
    {
        ComboBox_AddString(meshAnimationTypeComboHwnd, PluginGlobals::GetString(GetMeshAnimationTypeComboStringID(this->meshAnimationType.allowedTypes[i])));
        ComboBox_SetItemData(meshAnimationTypeComboHwnd, i + 1, this->meshAnimationType.allowedTypes[i]);
    }
    ComboBox_SetCurSel(meshAnimationTypeComboHwnd, GetMeshAnimationTypeValueComboStringIndex(this->meshAnimationType));    
    UpdateMeshAnimationTypeDefaultText();

    //Skeleton animation sampling value
    const TSTR& skeletonAnimationSampling = ChooseAnimationSampleValue(this->settings.skeletonAnimationSampleInterval, this->settings.skeletonAnimationSampleRate, this->settings.meshAnimationSampleInterval, this->settings.meshAnimationSampleRate);
    SetDlgItemText(IDC_SKELETON_ANIMATION_SAMPLING_EDIT, skeletonAnimationSampling.data());

    //Skeleton animation sampling combo
    HWND skeletonAnimationSamplingComboHwnd = GetDlgItem(IDC_SKELETON_ANIMATION_SAMPLING_COMBO);
    for (int i = 0; i < SAMPLE_TYPE_COUNT; i++)
        ComboBox_AddString(skeletonAnimationSamplingComboHwnd, PluginGlobals::GetString(sampleTypeComboStringIDs[i]));
    ComboBox_SetCurSel(skeletonAnimationSamplingComboHwnd, GetAnimationSampleTypeValueComboStringIndex(this->settings.skeletonAnimationSampleInterval, this->settings.skeletonAnimationSampleRate, this->settings.meshAnimationSampleInterval, this->settings.meshAnimationSampleRate));    

    //Vertex animation sampling value
    const TSTR& vertexAnimationSampling = ChooseAnimationSampleValue(this->settings.vertexAnimationSampleInterval, this->settings.vertexAnimationSampleRate, this->settings.meshAnimationSampleInterval, this->settings.meshAnimationSampleRate);
    SetDlgItemText(IDC_VERTEX_ANIMATION_SAMPLING_EDIT, vertexAnimationSampling.data());

    //Vertex animation sampling combo
    HWND vertexAnimationSamplingComboHwnd = GetDlgItem(IDC_VERTEX_ANIMATION_SAMPLING_COMBO);
    for (int i = 0; i < SAMPLE_TYPE_COUNT; i++)
        ComboBox_AddString(vertexAnimationSamplingComboHwnd, PluginGlobals::GetString(sampleTypeComboStringIDs[i]));
    ComboBox_SetCurSel(vertexAnimationSamplingComboHwnd, GetAnimationSampleTypeValueComboStringIndex(this->settings.vertexAnimationSampleInterval, this->settings.vertexAnimationSampleRate, this->settings.meshAnimationSampleInterval, this->settings.meshAnimationSampleRate));    

    //Generate LOD combo
    HWND generateLodComboHwnd = GetDlgItem(IDC_GENERATE_LOD_COMBO);
    for (int i = 0; i < GENERATE_LOD_COUNT; i++)
        ComboBox_AddString(generateLodComboHwnd, PluginGlobals::GetString(generateLodStringIDs[i]));
    ComboBox_SetCurSel(generateLodComboHwnd, GetGenerateLodValueComboStringIndex(this->settings.generateLod));

    //Number of LOD levels edit
    SetDlgItemText(IDC_NUM_LOD_LEVELS_EDIT, this->settings.numLodLevels);
    
    //LOD distance edit/spin
    this->lodDistanceSpin = SetupUniverseSpinner(this->hDlg, IDC_LOD_DISTANCE_SPIN, IDC_LOD_DISTANCE_EDIT, 0, 100000, Utilities::ToFloat(this->settings.lodDistance));
        
    //LOD reduction amount edit
    SetDlgItemText(IDC_LOD_REDUCE_AMOUNT_EDIT, this->settings.lodReductionAmount);

    //LOD reduction type combo
    HWND lodReductionTypeComboHwnd = GetDlgItem(IDC_LOD_REDUCE_TYPE_COMBO);
    for (int i = 0; i < COUNT_OF(lodReductionTypeComboStringIDs); i++)
        ComboBox_AddString(lodReductionTypeComboHwnd, PluginGlobals::GetString(lodReductionTypeComboStringIDs[i]));
    ComboBox_SetCurSel(lodReductionTypeComboHwnd, GetLODReductionTypeValueComboStringIndex(this->settings.lodReductionType));    

    //Mesh animations
    this->meshAnimationsListControl.Create(GetDlgItem(IDC_MESH_ANIMATIONS_LIST));
    this->meshAnimationsListControl.InsertColumn(IDS_NAME, 0, 120);
    this->meshAnimationsListControl.InsertColumn(IDS_TRACK, 1, 60);
    this->meshAnimationsListControl.InsertColumn(IDS_FRAMES, 2, 55);
    this->meshAnimationsListControl.InsertColumn(IDS_LENGTH, 3, 55);
    this->meshAnimationsListControl.InsertColumn(IDS_SAMPLE_RATE_INTERVAL, 4, 128);
    
    this->allMeshAnimationSettingsSerializer.Read(this->node);
    for (int i = 0; i < (int)this->allMeshAnimationSettingsSerializer.settings.size(); i++)
    {
        MeshAnimationSettingsSerializer& anim = this->allMeshAnimationSettingsSerializer.settings[i];
        this->meshAnimationsListControl.InsertItem(i, 0, anim.settings.name, (LPARAM)&anim);        
        UpdateMeshAnimationGUIData(i);
    }

    //Final updates
    UpdateSkeletonAnimationSamplingTypeEdit();
    UpdateVertexAnimationSamplingTypeEdit();
    UpdateLODReductionAmountEdit();
}

void ObjectSettingsEntityTabPage::UpdateSkeletonAnimationSamplingTypeEdit()
{
    HWND comboHwnd = GetDlgItem(IDC_SKELETON_ANIMATION_SAMPLING_COMBO);
    int selectedIndex = ComboBox_GetCurSel(comboHwnd);
    bool show = MeshAnimationSamplingComboIndexRequiresTimeValue(selectedIndex);
    ShowWindow(GetDlgItem(IDC_SKELETON_ANIMATION_SAMPLING_EDIT), show ? TRUE : FALSE);
}

void ObjectSettingsEntityTabPage::UpdateVertexAnimationSamplingTypeEdit()
{
    HWND comboHwnd = GetDlgItem(IDC_VERTEX_ANIMATION_SAMPLING_COMBO);
    int selectedIndex = ComboBox_GetCurSel(comboHwnd);
    bool show = MeshAnimationSamplingComboIndexRequiresTimeValue(selectedIndex);
    ShowWindow(GetDlgItem(IDC_VERTEX_ANIMATION_SAMPLING_EDIT), show ? TRUE : FALSE);
}

void ObjectSettingsEntityTabPage::UpdateLODReductionAmountEdit()
{
    HWND comboHwnd = GetDlgItem(IDC_LOD_REDUCE_TYPE_COMBO);
    int selectedIndex = ComboBox_GetCurSel(comboHwnd);
    bool show = selectedIndex > 0;
    ShowWindow(GetDlgItem(IDC_LOD_REDUCE_AMOUNT_EDIT), show ? TRUE : FALSE);
}

INT_PTR ObjectSettingsEntityTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    switch (message)
    {
        case WM_COMMAND:
        {
            int id = LOWORD(wParam);
            int code = HIWORD(wParam);

            switch (id)
            {
                case IDC_MESH_ANIMATION_TYPE_COMBO:
                {
                    switch (code)
                    {
                        case CBN_SELCHANGE:
                        {
                            UpdateMeshAnimationTypeDefaultText();                            
                            handled = true;                            
                            break;
                        }
                    }
                    break;
                }         
                case IDC_MESH_ANIMATION_EDIT_BUTTON:
                {
                    switch (code)
                    {
                        case BN_CLICKED:
                        {
                            int selectedAnimationIndex = this->meshAnimationsListControl.GetCurSel();
                            if (selectedAnimationIndex != -1)
                            {
                                MeshAnimationSettingsSerializer* meshAnimation = (MeshAnimationSettingsSerializer*)this->meshAnimationsListControl.GetData(selectedAnimationIndex);

                                MeshAnimationSettingsDialog dialog(meshAnimation->settings);
                                if (dialog.DoModal(this->hDlg))
                                {                                    
                                    //Show new data in user interface
                                    meshAnimation->changed = true;
                                    UpdateMeshAnimationGUIData(selectedAnimationIndex);
                                }                                
                            }

                            handled = true;
                            break;
                        }
                    }
                }
                case IDC_SKELETON_ANIMATION_SAMPLING_COMBO:
                {
                    switch (code)
                    {
                        case CBN_SELCHANGE:
                        {
                            UpdateSkeletonAnimationSamplingTypeEdit();
                            handled = true;                            
                            break;
                        }
                    }
                    break;
                }
                case IDC_VERTEX_ANIMATION_SAMPLING_COMBO:
                {
                    switch (code)
                    {
                        case CBN_SELCHANGE:
                        {
                            UpdateVertexAnimationSamplingTypeEdit();
                            handled = true;                            
                            break;
                        }
                    }
                    break;
                }
                case IDC_LOD_REDUCE_TYPE_COMBO:
                {
                    switch (code)
                    {
                        case CBN_SELCHANGE:
                        {
                            UpdateLODReductionAmountEdit();
                            handled = true;
                            break;
                        }
                    }
                }
            }

            break;
        }        
    }

    return 0;
}

void ObjectSettingsEntityTabPage::UpdateMeshAnimationGUIData(int listItemIndex)
{
    MeshAnimationSettingsSerializer* meshAnimation = (MeshAnimationSettingsSerializer*)this->meshAnimationsListControl.GetData(listItemIndex);
    MeshAnimationSettingsSerializer& anim = *meshAnimation;
    Interval& interval = anim.timeInterval;
    
    TSTR framesText;
    framesText.printf("%d-%d", Utilities::TicksToFrames(interval.Start()), Utilities::TicksToFrames(interval.End()));

    float length = TicksToSec(interval.Duration()) * Utilities::GetAnimationScale(anim.settings.scale, anim.settings.length, interval);
    TSTR lengthText;
    lengthText.printf("%.2fs", length);

    TSTR sampling;
    if (!anim.settings.sampleInterval.isNull())
        sampling.printf("%ss", anim.settings.sampleInterval.data());
    else if (!anim.settings.sampleRate.isNull())
        sampling.printf("%s/s", anim.settings.sampleRate.data());
    
    this->meshAnimationsListControl.SetItem(listItemIndex, 0, anim.settings.name);
    this->meshAnimationsListControl.SetItem(listItemIndex, 1, PluginGlobals::GetString(meshAnimationTrackStringIDs[(int)anim.track]));
    this->meshAnimationsListControl.SetItem(listItemIndex, 2, framesText);        
    this->meshAnimationsListControl.SetItem(listItemIndex, 3, lengthText);
    this->meshAnimationsListControl.SetItem(listItemIndex, 4, sampling);
}

MeshAnimationTypeDetector::Type ObjectSettingsEntityTabPage::UpdateMeshAnimationTypeDefaultText()
{
    //Get selected index for object type combo box 
    int animationTypeIndex = ComboBox_GetCurSel(GetDlgItem(IDC_MESH_ANIMATION_TYPE_COMBO));
    
    //Turn index into an object type
    MeshAnimationTypeDetector::Type animationType;
    if (animationTypeIndex == 0)
        animationType = (MeshAnimationTypeDetector::Type)ComboBox_GetItemData(GetDlgItem(IDC_MESH_ANIMATION_TYPE_COMBO), animationTypeIndex);
    else
        animationType = this->meshAnimationType.allowedTypes[animationTypeIndex - 1];

    //Show default text
    TSTR animationTypeDefaultText;
    if (animationTypeIndex == 0 && animationType != MeshAnimationTypeDetector::DEFAULT)
    {
        int animationTypeStringID = GetMeshAnimationTypeValueComboStringID(animationType);
        TSTR animationTypeText = PluginGlobals::GetString(animationTypeStringID);

        TSTR defaultFormat = PluginGlobals::GetString(IDS_DEFAULT_FORMAT);
        
        animationTypeDefaultText.printf(defaultFormat, animationTypeText.data());
    }
    SetDlgItemText(IDC_MESH_ANIMATION_TYPE_TEXT, animationTypeDefaultText);

    return animationType;
}

//ObjectSettingsExternalTabPage-------------------------
ObjectSettingsExternalTabPage::ObjectSettingsExternalTabPage(ObjectSettings& objectSettings) : settings(objectSettings)
{
}

void ObjectSettingsExternalTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void ObjectSettingsExternalTabPage::GetGUIData()
{    
    //Item type edit
    GetDlgItemText(IDC_ITEM_TYPE_EDIT, this->settings.externalItemType);

    //File edit
    GetDlgItemText(IDC_FILE_EDIT, this->settings.fileName);
}

void ObjectSettingsExternalTabPage::SetGUIData()
{
    //Item type edit
    SetDlgItemText(IDC_ITEM_TYPE_EDIT, this->settings.externalItemType);

    //File edit
    SetDlgItemText(IDC_FILE_EDIT, this->settings.fileName);
}

INT_PTR ObjectSettingsExternalTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    return 0;
}

//ObjectSettingsModelInstanceTabPage-------------------------
ObjectSettingsModelInstanceTabPage::ObjectSettingsModelInstanceTabPage(ObjectSettings& objectSettings) : settings(objectSettings)
{
}

void ObjectSettingsModelInstanceTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void ObjectSettingsModelInstanceTabPage::GetGUIData()
{    
    //Model name edit
    GetDlgItemText(IDC_MODEL_NAME_EDIT, this->settings.modelName);
}

void ObjectSettingsModelInstanceTabPage::SetGUIData()
{
    //Model name edit
    SetDlgItemText(IDC_MODEL_NAME_EDIT, this->settings.modelName);
}

INT_PTR ObjectSettingsModelInstanceTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    return 0;
}

//ObjectSettingsParticleSystemTabPage-------------------------
ObjectSettingsParticleSystemTabPage::ObjectSettingsParticleSystemTabPage(ObjectSettings& objectSettings) : settings(objectSettings)
{
}

void ObjectSettingsParticleSystemTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void ObjectSettingsParticleSystemTabPage::GetGUIData()
{    
    //File edit
    GetDlgItemText(IDC_FILE_EDIT, this->settings.fileName);
}

void ObjectSettingsParticleSystemTabPage::SetGUIData()
{
    //File edit
    SetDlgItemText(IDC_FILE_EDIT, this->settings.fileName);
}

INT_PTR ObjectSettingsParticleSystemTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    return 0;
}

//ObjectSettingsPlaneTabPage-------------------------
ObjectSettingsPlaneTabPage::ObjectSettingsPlaneTabPage(ObjectSettings& objectSettings) : settings(objectSettings)
{
}

void ObjectSettingsPlaneTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void ObjectSettingsPlaneTabPage::GetGUIData()
{    
    //Is static check
    this->settings.isStatic = IsDlgButtonCheckedText(IDC_STATIC_CHECK);

    //Generate normals check
    this->settings.generateNormals = IsDlgButtonCheckedText(IDC_GENERATE_NORMALS_CHECK);

    //Vertex buffer usage combo
    HWND vertexBufferUsageComboHwnd = GetDlgItem(IDC_VERTEX_BUFFER_USAGE_COMBO);
    int selectedVertexBufferUsageIndex = ComboBox_GetCurSel(vertexBufferUsageComboHwnd);    
    this->settings.vertexBufferUsage.usage = GetBufferUsageComboValue(selectedVertexBufferUsageIndex);

    //Vertex buffer shadow check
    this->settings.vertexBufferUsage.useShadow = IsDlgButtonCheckedText(IDC_VERTEX_BUFFER_USE_SHADOW_CHECK);

    //Vertex buffer usage combo
    HWND indexBufferUsageComboHwnd = GetDlgItem(IDC_INDEX_BUFFER_USAGE_COMBO);
    int selectedIndexBufferUsageIndex = ComboBox_GetCurSel(indexBufferUsageComboHwnd);    
    this->settings.indexBufferUsage.usage = GetBufferUsageComboValue(selectedIndexBufferUsageIndex);

    //Index buffer shadow check
    this->settings.indexBufferUsage.useShadow = IsDlgButtonCheckedText(IDC_INDEX_BUFFER_USE_SHADOW_CHECK);
}

void ObjectSettingsPlaneTabPage::SetGUIData()
{
    //Is static check
    CheckDlgButton(IDC_STATIC_CHECK, ObjectSettings::Values::IsStatic(this->settings.isStatic));

    //Generate normals check
    CheckDlgButton(IDC_GENERATE_NORMALS_CHECK, ObjectSettings::Values::GenerateNormals(this->settings.generateNormals));

    //Vertex buffer usage combo
    HWND vertexBufferUsageComboHwnd = GetDlgItem(IDC_VERTEX_BUFFER_USAGE_COMBO);
    for (int i = 0; i < BUFFER_USAGE_COUNT; i++)
        ComboBox_AddString(vertexBufferUsageComboHwnd, PluginGlobals::GetString(bufferUsageComboStringIDs[i]));
    ComboBox_SetCurSel(vertexBufferUsageComboHwnd, GetBufferUsageValueComboStringIndex(this->settings.vertexBufferUsage.usage));    

    //Vertex buffer shadow check
    CheckDlgButton(IDC_VERTEX_BUFFER_USE_SHADOW_CHECK, ObjectSettings::Values::BufferUsageUseShadow(this->settings.vertexBufferUsage.useShadow));

    //Vertex buffer usage combo
    HWND indexBufferUsageComboHwnd = GetDlgItem(IDC_INDEX_BUFFER_USAGE_COMBO);
    for (int i = 0; i < BUFFER_USAGE_COUNT; i++)
        ComboBox_AddString(indexBufferUsageComboHwnd, PluginGlobals::GetString(bufferUsageComboStringIDs[i]));
    ComboBox_SetCurSel(indexBufferUsageComboHwnd, GetBufferUsageValueComboStringIndex(this->settings.indexBufferUsage.usage));    

    //Index buffer shadow check
    CheckDlgButton(IDC_INDEX_BUFFER_USE_SHADOW_CHECK, ObjectSettings::Values::BufferUsageUseShadow(this->settings.indexBufferUsage.useShadow));
}

INT_PTR ObjectSettingsPlaneTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    return 0;
}

//ObjectSettingsSkyBoxTabPage-------------------------
ObjectSettingsSkyBoxTabPage::ObjectSettingsSkyBoxTabPage(ObjectSettings& objectSettings) : settings(objectSettings)
{
}

void ObjectSettingsSkyBoxTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void ObjectSettingsSkyBoxTabPage::GetGUIData()
{    
    //Enable check
    this->settings.enableSky = IsDlgButtonCheckedText(IDC_ENABLE_CHECK);

    //Draw first check
    this->settings.drawSkyFirst = IsDlgButtonCheckedText(IDC_DRAW_FIRST_CHECK);
}

void ObjectSettingsSkyBoxTabPage::SetGUIData()
{
    //Enable check
    CheckDlgButton(IDC_ENABLE_CHECK, ObjectSettings::Values::IsEnabled(this->settings.enableSky));

    //Draw first check
    CheckDlgButton(IDC_DRAW_FIRST_CHECK, ObjectSettings::Values::DrawFirst(this->settings.drawSkyFirst));
}

INT_PTR ObjectSettingsSkyBoxTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    return 0;
}

//ObjectSettingsSkyDomeTabPage-------------------------
ObjectSettingsSkyDomeTabPage::ObjectSettingsSkyDomeTabPage(ObjectSettings& objectSettings) : settings(objectSettings)
{
}

void ObjectSettingsSkyDomeTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void ObjectSettingsSkyDomeTabPage::GetGUIData()
{    
    //Enable check
    this->settings.enableSky = IsDlgButtonCheckedText(IDC_ENABLE_CHECK);

    //Draw first check
    this->settings.drawSkyFirst = IsDlgButtonCheckedText(IDC_DRAW_FIRST_CHECK);

    //Curvature edit
    GetDlgItemText(IDC_CURVATURE_EDIT, this->settings.curvature);
}

void ObjectSettingsSkyDomeTabPage::SetGUIData()
{
    //Enable check
    CheckDlgButton(IDC_ENABLE_CHECK, ObjectSettings::Values::IsEnabled(this->settings.enableSky));

    //Draw first check
    CheckDlgButton(IDC_DRAW_FIRST_CHECK, ObjectSettings::Values::DrawFirst(this->settings.drawSkyFirst));

    //Curvature edit
    SetDlgItemText(IDC_CURVATURE_EDIT, this->settings.curvature);
}

INT_PTR ObjectSettingsSkyDomeTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    return 0;
}

//ObjectSettingsSkyPlaneTabPage-------------------------
ObjectSettingsSkyPlaneTabPage::ObjectSettingsSkyPlaneTabPage(ObjectSettings& objectSettings) : settings(objectSettings)
{
}

void ObjectSettingsSkyPlaneTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void ObjectSettingsSkyPlaneTabPage::GetGUIData()
{    
    //Enable check
    this->settings.enableSky = IsDlgButtonCheckedText(IDC_ENABLE_CHECK);

    //Draw first check
    this->settings.drawSkyFirst = IsDlgButtonCheckedText(IDC_DRAW_FIRST_CHECK);

    //Scale edit
    GetDlgItemText(IDC_SCALE_EDIT, this->settings.scale);

    //Bow edit
    GetDlgItemText(IDC_BOW_EDIT, this->settings.bow);
}

void ObjectSettingsSkyPlaneTabPage::SetGUIData()
{
    //Enable check
    CheckDlgButton(IDC_ENABLE_CHECK, ObjectSettings::Values::IsEnabled(this->settings.enableSky));

    //Draw first check
    CheckDlgButton(IDC_DRAW_FIRST_CHECK, ObjectSettings::Values::DrawFirst(this->settings.drawSkyFirst));

    //Scale edit
    SetDlgItemText(IDC_SCALE_EDIT, this->settings.scale);

    //Bow edit
    SetDlgItemText(IDC_BOW_EDIT, this->settings.bow);
}

INT_PTR ObjectSettingsSkyPlaneTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    return 0;
}

//ObjectSettingsTerrainTabPage-------------------------
ObjectSettingsTerrainTabPage::ObjectSettingsTerrainTabPage(ObjectSettings& objectSettings) : settings(objectSettings)
{
}

void ObjectSettingsTerrainTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void ObjectSettingsTerrainTabPage::GetGUIData()
{    
    //File edit
    GetDlgItemText(IDC_FILE_EDIT, this->settings.fileName);
}

void ObjectSettingsTerrainTabPage::SetGUIData()
{
    //File edit
    SetDlgItemText(IDC_FILE_EDIT, this->settings.fileName);
}

INT_PTR ObjectSettingsTerrainTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    return 0;
}

