#ifndef OgreMax_ObjectSettingsTabPages_INCLUDED
#define OgreMax_ObjectSettingsTabPages_INCLUDED


//Includes---------------------------------------------------------------------
#include "DialogBase.h"
#include "ObjectSettings.h"
#include "MeshAnimationSettingsSerializer.h"
#include "TypeDetectors.h"
#include "ListControl.h"


//Classes----------------------------------------------------------------------
namespace OgreMax
{    
    class ObjectSettingsDialog;


    /** Object settings "General" tab page */
    class ObjectSettingsGeneralTabPage : public DialogBase
    {
    public:
        ObjectSettingsGeneralTabPage
            (
            ObjectSettingsDialog* ownerDialog, 
            INode* node,
            ObjectSettings& settings, 
            ObjectTypeDetector& objectType
            );

        void Attach(HWND hDlg);

        void GetGUIData();

    protected:
        void SetGUIData();

        INT_PTR DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled);

        ObjectTypeDetector::Type UpdateObjectTypeDefaultText();

    private:
        ObjectSettingsDialog* ownerDialog;
        ObjectSettings& settings;
        ObjectTypeDetector& objectType;
        ObjectTypeDetector defaultObjectType;
        INode* node;
    };

    /** Object settings "Billboard" tab page */
    class ObjectSettingsBillboardTabPage : public DialogBase
    {
    public:
        ObjectSettingsBillboardTabPage(ObjectSettings& settings);

        void Attach(HWND hDlg);
        void Detach();

        void GetGUIData();

    protected:
        void SetGUIData();

        INT_PTR DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled);

    private:
        ObjectSettings& settings;

        IColorSwatch* colorSwatch;
    };

    /** Object settings "Billboard Set" tab page */
    class ObjectSettingsBillboardSetTabPage : public DialogBase
    {
    public:
        ObjectSettingsBillboardSetTabPage(ObjectSettings& settings);

        void Attach(HWND hDlg);

        void GetGUIData();

    protected:
        void SetGUIData();

        INT_PTR DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled);

    private:
        ObjectSettings& settings;
    };

    /** Object settings "Entity" tab page */
    class ObjectSettingsEntityTabPage : public DialogBase
    {
    public:
        ObjectSettingsEntityTabPage(ObjectSettings& settings, INode* node);

        void Attach(HWND hDlg);
        void Detach();

        void GetGUIData();

    protected:
        void SetGUIData();

        void UpdateSkeletonAnimationSamplingTypeEdit();
        void UpdateVertexAnimationSamplingTypeEdit();
        void UpdateLODReductionAmountEdit();

        INT_PTR DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled);

        void UpdateMeshAnimationGUIData(int listItemIndex);

        MeshAnimationTypeDetector::Type UpdateMeshAnimationTypeDefaultText();

    private:
        ObjectSettings& settings;
        INode* node;
        ISpinnerControl* lodDistanceSpin;
        MeshAnimationTypeDetector meshAnimationType;
        MeshAnimationTypeDetector defaultMeshAnimationType;
        ListControl meshAnimationsListControl;
        AllMeshAnimationSettingsSerializer allMeshAnimationSettingsSerializer;
    };

    /** Object settings "External" tab page */
    class ObjectSettingsExternalTabPage : public DialogBase
    {
    public:
        ObjectSettingsExternalTabPage(ObjectSettings& settings);

        void Attach(HWND hDlg);

        void GetGUIData();

    protected:
        void SetGUIData();

        INT_PTR DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled);

    private:
        ObjectSettings& settings;
    };

    /** Object settings "Model Instance" tab page */
    class ObjectSettingsModelInstanceTabPage : public DialogBase
    {
    public:
        ObjectSettingsModelInstanceTabPage(ObjectSettings& settings);

        void Attach(HWND hDlg);

        void GetGUIData();

    protected:
        void SetGUIData();

        INT_PTR DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled);

    private:
        ObjectSettings& settings;
    };

    /** Object settings "Particle System" tab page */
    class ObjectSettingsParticleSystemTabPage : public DialogBase
    {
    public:
        ObjectSettingsParticleSystemTabPage(ObjectSettings& settings);

        void Attach(HWND hDlg);

        void GetGUIData();

    protected:
        void SetGUIData();

        INT_PTR DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled);

    private:
        ObjectSettings& settings;
    };

    /** Object settings "Plane" tab page */
    class ObjectSettingsPlaneTabPage : public DialogBase
    {
    public:
        ObjectSettingsPlaneTabPage(ObjectSettings& settings);

        void Attach(HWND hDlg);

        void GetGUIData();

    protected:
        void SetGUIData();

        INT_PTR DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled);

    private:
        ObjectSettings& settings;
    };

    /** Object settings "Sky Box" tab page */
    class ObjectSettingsSkyBoxTabPage : public DialogBase
    {
    public:
        ObjectSettingsSkyBoxTabPage(ObjectSettings& settings);

        void Attach(HWND hDlg);

        void GetGUIData();

    protected:
        void SetGUIData();

        INT_PTR DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled);

    private:
        ObjectSettings& settings;
    };

    /** Object settings "Sky Dome" tab page */
    class ObjectSettingsSkyDomeTabPage : public DialogBase
    {
    public:
        ObjectSettingsSkyDomeTabPage(ObjectSettings& settings);

        void Attach(HWND hDlg);

        void GetGUIData();

    protected:
        void SetGUIData();

        INT_PTR DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled);

    private:        
        ObjectSettings& settings;
    };

    /** Object settings "Sky Plane" tab page */
    class ObjectSettingsSkyPlaneTabPage : public DialogBase
    {
    public:
        ObjectSettingsSkyPlaneTabPage(ObjectSettings& settings);

        void Attach(HWND hDlg);

        void GetGUIData();

    protected:
        void SetGUIData();

        INT_PTR DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled);

    private:        
        ObjectSettings& settings;
    };

    /** Object settings "Terrain" tab page */
    class ObjectSettingsTerrainTabPage : public DialogBase
    {
    public:
        ObjectSettingsTerrainTabPage(ObjectSettings& settings);

        void Attach(HWND hDlg);

        void GetGUIData();

    protected:
        void SetGUIData();

        INT_PTR DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled);

    private:        
        ObjectSettings& settings;
    };
}

#endif
