
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "OgreMaxGlobalUtilities.h"
#include "OgreMaxSceneExporter.h"
#include "resource.h"
#include "PluginGlobals.h"
#include "SceneSettingsDialog.h"
#include "ObjectSettingsDialog.h"
#include "SceneSettingsSerializer.h"
#include "ObjectSettingsSerializer.h"

using namespace OgreMax;


//Static initialization--------------------------------------------------------
OgreMaxGlobalUtilities OgreMaxGlobalUtilities::instance;


//Local classes-----------------------------------------------------------------
class GlobalMenuClassDescType : public ClassDesc2 
{
public:
    int IsPublic()                      {return 1;}
    void* Create(BOOL loading = FALSE)  {return &OgreMaxGlobalUtilities::instance;}
    const TCHAR* ClassName()            {return PluginGlobals::GetString(IDS_GLOBAL_UTILITIES_CLASS_NAME);}
    SClass_ID SuperClassID()            {return GUP_CLASS_ID;}
    Class_ID ClassID()                  {return Class_ID(0x4162415e, 0x43eb28b4);}
    const TCHAR* Category()             {return PluginGlobals::GetString(IDS_PLUGINS_CATEGORY);}
    const TCHAR* InternalName()         {return _T("OgreMaxGlobalUtilities");}
    HINSTANCE HInstance()               {return PluginGlobals::GetInstanceHandle();}

public:
    static GlobalMenuClassDescType instance;
};
GlobalMenuClassDescType GlobalMenuClassDescType::instance;

enum 
{
    fnId_ExportScene,
    fnId_ExportSceneSelected,

    fnId_SceneSettings,
    fnId_ObjectSettings,

    fnId_About
};

class GlobalMenuActions : public FPStaticInterface
{
public:
	DECLARE_DESCRIPTOR(GlobalMenuActions)

	BEGIN_FUNCTION_MAP
		FN_ACTION(fnId_ExportScene, ExportScene);
		FN_ACTION(fnId_ExportSceneSelected, ExportSceneSelected);
        FN_ACTION(fnId_SceneSettings, SceneSettings);
        FN_ACTION(fnId_ObjectSettings, ObjectSettings);
        FN_ACTION(fnId_About, About);
	END_FUNCTION_MAP

    FPStatus ExportScene() 
    { 
        OgreMaxGlobalUtilities::GetInstance().OnExportScene(); 
        return FPS_OK; 
    }

	FPStatus ExportSceneSelected() 
    { 
        OgreMaxGlobalUtilities::GetInstance().OnExportSceneSelected(); 
        return FPS_OK; 
    }

    FPStatus SceneSettings() 
    { 
        OgreMaxGlobalUtilities::GetInstance().OnSceneSettings(); 
        return FPS_OK; 
    }

    FPStatus ObjectSettings() 
    { 
        OgreMaxGlobalUtilities::GetInstance().OnObjectSettings(); 
        return FPS_OK; 
    }

    FPStatus About() 
    { 
        OgreMaxGlobalUtilities::GetInstance().OnAbout(); 
        return FPS_OK; 
    }
};

static GlobalMenuActions globalMenuActions
    (
    Interface_ID(0x3bef7f06, 0x45b3aa9), _T("Global_Interface"), 0, &GlobalMenuClassDescType::instance, FP_ACTIONS,

	kActionMainUIContext,
/*
	fnId_ExportScene, _T("Export Scene"), IDS_EXPORT_SCENE_ACTION_DESCRIPTION, 0,
		f_category, _T("OgreMax Actions"), IDS_PLUGINS_CATEGORY,
		f_menuText,	IDS_EXPORT_SCENE_MENU_TEXT,
		end,
*/
	fnId_ExportSceneSelected, _T("Export Selected"), IDS_EXPORT_SCENE_SELECTION_ACTION_DESCRIPTION, 0,
		f_category, _T("OgreMax Actions"), IDS_PLUGINS_CATEGORY,
		f_menuText,	IDS_EXPORT_SCENE_SELECTION_MENU_TEXT,
		end,

    fnId_SceneSettings, _T("Scene Settings"), IDS_SCENE_SETTINGS_ACTION_DESCRIPTION, 0,
		f_category, _T("OgreMax Actions"), IDS_PLUGINS_CATEGORY,
		f_menuText,	IDS_SCENE_SETTINGS_MENU_TEXT,
		end,

    fnId_ObjectSettings, _T("Object Settings"), IDS_OBJECT_SETTINGS_ACTION_DESCRIPTION, 0,
		f_category, _T("OgreMax Actions"), IDS_PLUGINS_CATEGORY,
		f_menuText,	IDS_OBJECT_SETTINGS_MENU_TEXT,
		end,

    fnId_About, _T("About"), IDS_ABOUT_ACTION_DESCRIPTION, 0,
		f_category, _T("OgreMax Actions"), IDS_PLUGINS_CATEGORY,
		f_menuText,	IDS_ABOUT_MENU_TEXT,
		end,

	end
    );

//OgreMaxGlobalUtilities implementation----------------------------------------
OgreMaxGlobalUtilities::OgreMaxGlobalUtilities() 
{    
}
        
OgreMaxGlobalUtilities::~OgreMaxGlobalUtilities()
{    
}

DWORD OgreMaxGlobalUtilities::Start()
{
    this->exportedFileName.Resize(MAX_PATH + 1);

    //Register a callback so that we get notified of a color change so that we can re-register the menu
	RegisterNotification(InterfaceChanged, this, NOTIFY_COLOR_CHANGE);

	//When done, unregister the callback
	CreateOgreMaxMenu();	
	return GUPRESULT_KEEP;
}

void OgreMaxGlobalUtilities::Stop()
{
    UnRegisterNotification(InterfaceChanged, this, NOTIFY_COLOR_CHANGE);
}

DWORD OgreMaxGlobalUtilities::Control(DWORD parameter)
{
	return GUPRESULT_KEEP;
}

FPInterfaceDesc* OgreMaxGlobalUtilities::GetDesc()
{ 
    return &globalMenuActions; 
}

void OgreMaxGlobalUtilities::CreateOgreMaxMenu()
{
    TSTR menuName = PluginGlobals::GetString(IDS_MENU_NAME);

    //Remove existing OgreMax menu(s)
    IMenuManager* menuManager = Max()->GetMenuManager();
    IMenu* oldMenu;
    while (oldMenu = menuManager->FindMenu(menuName))
        menuManager->UnRegisterMenu(oldMenu);
    menuManager->UpdateMenuBar();
    
    //Get menu bar
    IMenu* mainMenu = menuManager->GetMainMenuBar();
	if (mainMenu == NULL) 
    	return;

    //Create a new sub-menu
	IMenu* ogreMaxMenu = GetIMenu();
	ogreMaxMenu->SetTitle(menuName);
	
	//Register the new menu with the system
	menuManager->RegisterMenu(ogreMaxMenu, 0);
	
    //Add "export scene" menu item
	IMenuItem* exportSceneMenuItem = GetIMenuItem();
	exportSceneMenuItem->SetActionItem(globalMenuActions.action_table->GetAction(fnId_ExportScene));
    ogreMaxMenu->AddItem(exportSceneMenuItem);
    	
    //Add "export selected " menu item
    IMenuItem* exportSceneSelectedMenuItem = GetIMenuItem();
	exportSceneSelectedMenuItem->SetActionItem(globalMenuActions.action_table->GetAction(fnId_ExportSceneSelected));
    ogreMaxMenu->AddItem(exportSceneSelectedMenuItem);	

    //Add separator menu item
    IMenuItem* separatorMenuItem;
    separatorMenuItem = GetIMenuItem();
    separatorMenuItem->ActAsSeparator();
    ogreMaxMenu->AddItem(separatorMenuItem);

    //Add "scene settings" menu item
    IMenuItem* sceneSettingsMenuItem = GetIMenuItem();
	sceneSettingsMenuItem->SetActionItem(globalMenuActions.action_table->GetAction(fnId_SceneSettings));
    ogreMaxMenu->AddItem(sceneSettingsMenuItem);

    //Add "object settings" menu item
    IMenuItem* objectSettingsMenuItem = GetIMenuItem();
	objectSettingsMenuItem->SetActionItem(globalMenuActions.action_table->GetAction(fnId_ObjectSettings));
    ogreMaxMenu->AddItem(objectSettingsMenuItem);

    //Add separator menu item
    separatorMenuItem = GetIMenuItem();
    separatorMenuItem->ActAsSeparator();
    ogreMaxMenu->AddItem(separatorMenuItem);

    //Add "about" menu item
    IMenuItem* aboutMenuItem = GetIMenuItem();
	aboutMenuItem->SetActionItem(globalMenuActions.action_table->GetAction(fnId_About));
    ogreMaxMenu->AddItem(aboutMenuItem);	

	 //Create a new menu item to hold the main OgreMax menu
	IMenuItem* ogreMaxMenuItem = GetIMenuItem();
	ogreMaxMenuItem->SetSubMenu(ogreMaxMenu);
	
	//Add the sub-menu item to the main menu bar
	mainMenu->AddItem(ogreMaxMenuItem, mainMenu->NumItems() - 1);

	menuManager->UpdateMenuBar();
}

void OgreMaxGlobalUtilities::InterfaceChanged(void* param, NotifyInfo* info)
{
    instance.CreateOgreMaxMenu();
}

void OgreMaxGlobalUtilities::OnExportScene()
{
    DoExport(false);
}

void OgreMaxGlobalUtilities::OnExportSceneSelected()
{   
    DoExport(true);    
}

void OgreMaxGlobalUtilities::OnSceneSettings()
{
    //Get summary info text
    TSTR sceneInfo = Utilities::GetSceneSummaryInfo();

    //Parse scene settings
    SceneSettingsSerializer serializer;
    serializer.Read(sceneInfo);

    //Show settings dialog
    SceneSettingsDialog dialog(serializer.settings);
    if (dialog.DoModal())
    {
        //Write the settings to text
        TSTR text;
        serializer.Write(text);

        //Set summary info text
        Utilities::SetSceneSummaryInfo(text);        
    }
}

void OgreMaxGlobalUtilities::OnObjectSettings()
{
    int selectedNodeCount = Max()->GetSelNodeCount();
    if (selectedNodeCount == 0)
    {
        Utilities::ShowMessage(IDS_OBJECT_SETTINGS_ERROR_NO_SELECTED_NODE, IDS_OBJECT_SETTINGS_ERROR_CAPTION, MB_OK | MB_ICONINFORMATION);
    }
    else if (selectedNodeCount > 1)
    {
        Utilities::ShowMessage(IDS_OBJECT_SETTINGS_ERROR_TOO_MANY_SELECTED_NODES, IDS_OBJECT_SETTINGS_ERROR_CAPTION, MB_OK | MB_ICONINFORMATION);
    }
    else
    {
        //Get user property buffer
        INode* node = Max()->GetSelNode(0);
        TSTR objectInfo;
        node->GetUserPropBuffer(objectInfo);

        //Parse object settings
        ObjectSettingsSerializer serializer;
        serializer.Read(objectInfo);

        //Show settings dialog
        ObjectSettingsDialog dialog(serializer.settings, node);
        if (dialog.DoModal())
        {
            //Write the settings to text
            TSTR text;
            serializer.Write(text);

            //Set user property buffer
            node->SetUserPropBuffer(text);
        }
    }
}

void OgreMaxGlobalUtilities::OnAbout()
{    
    Utilities::ShowAbout();
}

void OgreMaxGlobalUtilities::DoExport(bool selected)
{
    OPENFILENAME ofn;
    ZeroMemory(&ofn, sizeof(ofn));
    ofn.lStructSize = sizeof(ofn);
    
    ofn.hwndOwner = Max()->GetMAXHWnd();
    ofn.hInstance = PluginGlobals::GetInstanceHandle();
    ofn.lpstrFilter = _T("OgreMax Scene (*.SCENE)\0*.SCENE\0\0");
    ofn.nFilterIndex = 1;
    ofn.lpstrFile = this->exportedFileName.data();
    ofn.nMaxFile = MAX_PATH;
    ofn.lpstrTitle = PluginGlobals::GetString(IDS_EXPORT_DIALOG_TITLE);
    ofn.Flags = OFN_DONTADDTORECENT | OFN_ENABLESIZING;
    ofn.nFileOffset = Utilities::GetFileNameOffset(this->exportedFileName);

    if (GetSaveFileName(&ofn))
    {
        Max()->ExportToFile
            (
            this->exportedFileName, 
            TRUE, 
            selected ? SCENE_EXPORT_SELECTED : 0, 
            &OgreMaxSceneExporter::GetClassDesc()->ClassID()
            );
    }
}

ClassDesc* OgreMaxGlobalUtilities::GetClassDesc()
{
    return &GlobalMenuClassDescType::instance;
}