//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "PluginGlobals.h"
#include "OgreMaxMaterial.h"
#include "OgreMaxMaterialDlg.h"
#include "resource.h"

using namespace OgreMax;


//Local classes-----------------------------------------------------------------
class OgreMaxMaterialClassDesc : public ClassDesc
{
public:
	int IsPublic()              {return 1;}
	void* Create(BOOL loading)  {return new OgreMaxMaterial(loading);}
	const TCHAR* ClassName()    {return PluginGlobals::GetString(IDS_MATERIAL_CLASS_NAME);}
	SClass_ID SuperClassID()    {return MATERIAL_CLASS_ID;}
    Class_ID ClassID()          {return OgreMaxMaterial::GetClassClassID();}
	const TCHAR* Category()     {return PluginGlobals::GetString(IDS_PLUGINS_CATEGORY);}
	
public:
    static OgreMaxMaterialClassDesc instance;
};
OgreMaxMaterialClassDesc OgreMaxMaterialClassDesc::instance;


//Implementation---------------------------------------------------------------
OgreMaxMaterial::OgreMaxMaterial(BOOL loading)
{
    this->paramDlg = NULL;

    this->flags = RECEIVES_SHADOWS | ROLLUP_OPEN;
	this->rollScroll = 0;

    this->techniques = NULL;
    this->viewRenderMaterial = NULL;

    if (!loading)
		Reset();
}

RefResult OgreMaxMaterial::NotifyRefChanged
    (
    Interval changeInt, 
    RefTargetHandle hTarget, 
    PartID& partID, 
    RefMessage message
    )
{
    return REF_SUCCEED;
}

void OgreMaxMaterial::Update(TimeValue t, Interval& valid)
{
    Point3 p;
	if (!this->ivalid.InInterval(t)) 
    {
		this->ivalid.SetInfinite();

        if (this->techniques != NULL)
        {
    	    for (int i = 0; i < OgreMaxTechniques::MAX_TECHNIQUES; i++) 	
            {
                if (GetTechnique(i)->IsOn())
				    GetTechnique(i)->Update(t, this->ivalid);
		    }
        }

        if (this->viewRenderMaterial != NULL)
            this->viewRenderMaterial->Update(t, valid);
	}
	valid &= ivalid;
}

RefTargetHandle OgreMaxMaterial::Clone(RemapDir& remap)
{
    OgreMaxMaterial* newMaterial = new OgreMaxMaterial();

    //Copy superclass stuff
    *static_cast<MtlBase*>(newMaterial) = *this;
	
    //Copy everything else
	newMaterial->ReplaceReference(TECHNIQUES_REF, remap.CloneRef(this->techniques));
	newMaterial->ReplaceReference(VIEW_RENDER_MATERIAL_REF, remap.CloneRef(this->viewRenderMaterial));    
	newMaterial->flags = flags;
	
    BaseClone(this, newMaterial, remap);
	
    return newMaterial;
}

void OgreMaxMaterial::Reset()
{
    SuspendSetKeyMode();

    this->ivalid.SetEmpty();

    this->flags = RECEIVES_SHADOWS | ROLLUP_OPEN;
	
    ReplaceReference(TECHNIQUES_REF, new OgreMaxTechniques(this));	
	ReplaceReference(VIEW_RENDER_MATERIAL_REF, NULL);
	
	ResumeSetKeyMode();
}

Interval OgreMaxMaterial::Validity(TimeValue t)
{
    Interval v;
	Update(t, v);
	return this->ivalid;
}

ParamDlg* OgreMaxMaterial::CreateParamDlg(HWND hWndMaterialEdit, IMtlParams* materialParams)
{
    Interval v;
	Update(materialParams->GetTime(), v);
    OgreMaxMaterialDlg* dlg = new OgreMaxMaterialDlg(hWndMaterialEdit, materialParams, this);    
    dlg->BuildDialog();
    dlg->LoadDialog();	 
    SetParamDlg(dlg);    
	return dlg;
}

Color OgreMaxMaterial::GetAmbient(int mtlNum, BOOL backFace)
{
    if (this->viewRenderMaterial != NULL)
        return this->viewRenderMaterial->GetAmbient(mtlNum, backFace);
    else
        return Color(0, 0, 0);
}

Color OgreMaxMaterial::GetDiffuse(int mtlNum, BOOL backFace)
{
    if (this->viewRenderMaterial != NULL)
        return this->viewRenderMaterial->GetDiffuse(mtlNum, backFace);
    else
        return Color(0, 0, 0);
}

Color OgreMaxMaterial::GetSpecular(int mtlNum, BOOL backFace)
{
    if (this->viewRenderMaterial != NULL)
        return this->viewRenderMaterial->GetSpecular(mtlNum, backFace);
    else
        return Color(0, 0, 0);
}

float OgreMaxMaterial::GetShininess(int mtlNum, BOOL backFace)
{
    if (this->viewRenderMaterial != NULL)
        return this->viewRenderMaterial->GetShininess(mtlNum, backFace);
    else
        return 0;
}

float OgreMaxMaterial::GetShinStr(int mtlNum, BOOL backFace)
{
    if (this->viewRenderMaterial != NULL)
        return this->viewRenderMaterial->GetShinStr(mtlNum, backFace);
    else
        return 0;
}

float OgreMaxMaterial::GetXParency(int mtlNum, BOOL backFace)
{
    if (this->viewRenderMaterial != NULL)
        return this->viewRenderMaterial->GetXParency(mtlNum, backFace);
    else
        return 0;
}
  
void OgreMaxMaterial::SetAmbient(Color c, TimeValue t)
{
    if (this->viewRenderMaterial != NULL)
        this->viewRenderMaterial->SetAmbient(c, t);    
}

void OgreMaxMaterial::SetDiffuse(Color c, TimeValue t)
{
    if (this->viewRenderMaterial != NULL)
        this->viewRenderMaterial->SetDiffuse(c, t);
}

void OgreMaxMaterial::SetSpecular(Color c, TimeValue t)
{
    if (this->viewRenderMaterial != NULL)
        this->viewRenderMaterial->SetSpecular(c, t);
}

void OgreMaxMaterial::SetShininess(float s, TimeValue t)
{
    if (this->viewRenderMaterial != NULL)
        this->viewRenderMaterial->SetShininess(s, t);
}

void OgreMaxMaterial::Shade(ShadeContext& sc)
{
    if (this->viewRenderMaterial != NULL)
        this->viewRenderMaterial->Shade(sc);
}

void OgreMaxMaterial::SetParamDlg(OgreMaxMaterialDlg* paramDlg)
{
    this->paramDlg = paramDlg;
}

ULONG OgreMaxMaterial::Requirements(int subMtlNum)
{
    if (this->viewRenderMaterial != NULL)
        return this->viewRenderMaterial->Requirements(subMtlNum);
    else
        return 0;
}

int OgreMaxMaterial::NumSubTexmaps() 
{ 
    return OgreMaxTechniques::MAX_TECHNIQUES; 
}

Texmap* OgreMaxMaterial::GetSubTexmap(int i)
{
    if (this->techniques != NULL)
        return GetTechnique(i)->GetMap();
    else
        return NULL;
}

int OgreMaxMaterial::MapSlotType(int i) 
{ 
    return MAPSLOT_TEXTURE; 
}

void OgreMaxMaterial::SetSubTexmap(int i, Texmap* m)
{
    assert(i < OgreMaxTechniques::MAX_TECHNIQUES);
	
    if (this->techniques != NULL)
    {
        this->techniques->ReplaceReference(2*i+1, m);
	    if (m != NULL) 
        {
		    if (GetTechnique(i)->GetDistanceControl() == NULL)
            {			
                Control* newController = NewDefaultFloatController();
			    this->techniques->ReplaceReference(2*i, newController);

                float value = GetTechnique(i)->GetDistance();
			    newController->SetValue(0, &value);
		    }
	    }
	    else 
        {
		    if (GetTechnique(i)->GetDistanceControl() != NULL)			
			    this->techniques->DeleteReference(2*i);		
	    }
    }

	if (this->paramDlg)
		this->paramDlg->UpdateTechniqueDisplay(i);
}

TSTR OgreMaxMaterial::GetSubTexmapSlotName(int i)
{
    return PluginGlobals::GetString(IDS_PASS_SLOT_NAME);
}

int OgreMaxMaterial::SubTexmapOn(int i) 
{ 
    return this->techniques->GetTechnique(i)->IsOn();
} 

int OgreMaxMaterial::NumSubMtls()
{
    return 1;
}

Mtl* OgreMaxMaterial::GetSubMtl(int i)
{
    return this->viewRenderMaterial;
}

void OgreMaxMaterial::SetSubMtl(int i, Mtl* mtl)
{
    ReplaceReference(1, mtl);

    if (this->paramDlg)
		this->paramDlg->UpdateViewRenderMaterialDisplay();
}

int OgreMaxMaterial::VPDisplaySubMtl()
{
    return 0;
}

TSTR OgreMaxMaterial::GetSubMtlSlotName(int i)
{
    return PluginGlobals::GetString(IDS_VIEWPORT_RENDER_MATERIAL);
}

TSTR OgreMaxMaterial::GetSubMtlTVName(int i)
{
    return PluginGlobals::GetString(IDS_VIEWPORT_RENDER_MATERIAL);
}

ClassDesc* OgreMaxMaterial::GetClassDesc()
{
    return &OgreMaxMaterialClassDesc::instance;
}

Class_ID OgreMaxMaterial::GetClassClassID()
{
    return Class_ID(0x37308ed8, 0x4d6eb329);
}

Class_ID OgreMaxMaterial::ClassID()
{
    return GetClassClassID();
}

SClass_ID OgreMaxMaterial::SuperClassID()
{ 
    return MATERIAL_CLASS_ID; 
}

void OgreMaxMaterial::GetClassName(TSTR& s)
{
    s = PluginGlobals::GetString(IDS_MATERIAL_CLASS_NAME); 
}  

void OgreMaxMaterial::DeleteThis() 
{
	delete this;
}

TSTR OgreMaxMaterial::SubAnimName(int i) 
{ 
    TSTR name;

    switch (i)
    {
        case TECHNIQUES_REF:
        {
            name = PluginGlobals::GetString(IDS_TECHNIQUES); 
            break;
        }
        case VIEW_RENDER_MATERIAL_REF: 
        {
            name = PluginGlobals::GetString(IDS_VIEWPORT_RENDER_MATERIAL); 
            break;
        }
    }

    return name;
}		

int OgreMaxMaterial::NumSubs()
{
    return NUM_REFS; 
}  

Animatable* OgreMaxMaterial::SubAnim(int i) 
{
	switch (i) 
    {
		case TECHNIQUES_REF: return this->techniques;
		case VIEW_RENDER_MATERIAL_REF: return this->viewRenderMaterial;
		default: 
        {
            assert(0); 
            return NULL;
        }
	}
}

int OgreMaxMaterial::NumRefs()
{
    return NUM_REFS; 
}

RefTargetHandle OgreMaxMaterial::GetReference(int i) 
{
	switch (i) 
    {
		case TECHNIQUES_REF: return this->techniques;
		case VIEW_RENDER_MATERIAL_REF: return this->viewRenderMaterial;
		default: 
        {
            assert(0); 
            return NULL;
        }
	}
}

void OgreMaxMaterial::SetReference(int i, RefTargetHandle refTarget) 
{
	switch (i) 
    {
		case TECHNIQUES_REF:	
        {
            this->techniques = reinterpret_cast<OgreMaxTechniques*>(refTarget);
            if (this->techniques != NULL)
                this->techniques->SetClientPtr(this);
            
            break;
        }
		case VIEW_RENDER_MATERIAL_REF:
        {
            this->viewRenderMaterial = reinterpret_cast<Mtl*>(refTarget);

			break;
		}
		default: assert(0);
	}
}

ULONG OgreMaxMaterial::GetFlags()
{
    return this->flags;
}

void OgreMaxMaterial::SetFlag(ULONG f, ULONG value) 
{
	if (value) 
    {
        //Add to the flags
        this->flags |= f; 
    }
	else 
    {
        //Remove the flags
        this->flags &= ~f;
    }
}

BOOL OgreMaxMaterial::IsTechniqueEnabled(int index)
{
    if (this->techniques != NULL)
        return GetTechnique(index)->IsOn();
    else
        return FALSE;
}

void OgreMaxMaterial::EnableTechnique(int index, ULONG enable)
{
    if (this->techniques != NULL)
        GetTechnique(index)->SetOn(enable);
}

float OgreMaxMaterial::GetDistance(int index)
{
    if (this->techniques != NULL)
        return GetTechnique(index)->GetDistance();
    else
        return 0;
}

void OgreMaxMaterial::SetDistance(int index, float distance)
{
    if (this->techniques != NULL)
        GetTechnique(index)->SetDistance(distance);    
}

TSTR OgreMaxMaterial::GetTechniqueName(int index)
{
    TSTR name;
    if (this->techniques != NULL && GetTechnique(index)->GetMap() != NULL)
    {
        Texmap* texmap = this->techniques->GetTechnique(index)->GetMap();
        name = texmap->GetName();
    }
    else
        name = PluginGlobals::GetString(IDS_NONE);
    return name;
}

TechniqueSlot* OgreMaxMaterial::GetTechnique(int index)
{
    assert(this->techniques);
    return this->techniques->GetTechnique(index);
}

Mtl* OgreMaxMaterial::GetViewRenderMaterial()
{
    return this->viewRenderMaterial;
}

#define MATERIAL_HEADER_CHUNK 0x4000
#define MATERIAL_FLAGS_CHUNK 0x5000
#define MATERIAL_VERSION_CHUNK 0x5300

#define MATERIAL_VERSION 1

IOResult OgreMaxMaterial::Save(ISave* isave)
{
    ULONG byteCount;
    IOResult result;

	isave->BeginChunk(MATERIAL_HEADER_CHUNK);
	result = MtlBase::Save(isave);
	if (result != IO_OK) 
        return result;
	isave->EndChunk();

    isave->BeginChunk(MATERIAL_VERSION_CHUNK);
	int version = MATERIAL_VERSION;
	isave->Write(&version, sizeof(version), &byteCount);			
	isave->EndChunk();

	isave->BeginChunk(MATERIAL_FLAGS_CHUNK);
	isave->Write(&this->flags, sizeof(this->flags), &byteCount);			
	isave->EndChunk();

	return IO_OK;
}

IOResult OgreMaxMaterial::Load(ILoad* iload)
{
    ULONG byteCount;
	int id;
	int version = 0;
	IOResult result;

	while ((result = iload->OpenChunk()) == IO_OK) 
    {
		switch (id = iload->CurChunkID())  
        {
			case MATERIAL_HEADER_CHUNK:
            {
				result = MtlBase::Load(iload);
				this->ivalid.SetEmpty();
				break;
            }
            case MATERIAL_VERSION_CHUNK:
            {
				result = iload->Read(&version, sizeof(version), &byteCount);
				break;
            }
			case MATERIAL_FLAGS_CHUNK:
            {
				result = iload->Read(&this->flags, sizeof(this->flags), &byteCount);
				break;
            }
		}
		iload->CloseChunk();
		if (result != IO_OK) 
			return result;
	}
	
	return IO_OK;
}
