#ifndef OgreMax_OgreMaxMaterial_INCLUDED
#define OgreMax_OgreMaxMaterial_INCLUDED


//Includes---------------------------------------------------------------------
#include <stdmat.h>
#include "OgreMaxTechniques.h"


//Classes----------------------------------------------------------------------
namespace OgreMax
{
    class OgreMaxMaterialDlg;

    class OgreMaxMaterial : public Mtl
    {
    public:
        enum 
        {
            DONT_EXPORT_MATERIAL = 1<<0,
            RECEIVES_SHADOWS = 1<<1,
            TRANSPARENCY_CASTS_SHADOWS = 1<<2,
            TECHNIQUES_ARE_LODS = 1<<3,            
            
            ROLLUP_OPEN                 = 1<<28
        };

        OgreMaxMaterial(BOOL loading = FALSE);

        RefResult NotifyRefChanged
            (
            Interval changeInt, 
            RefTargetHandle hTarget, 
	        PartID& partID, 
            RefMessage message
            );

        void Update(TimeValue t, Interval& valid);

        RefTargetHandle Clone(RemapDir& remap);
        void Reset();

        Interval Validity(TimeValue t);

        ParamDlg* CreateParamDlg(HWND hWndMaterialEdit, IMtlParams* materialParams);

        Color GetAmbient(int mtlNum = 0, BOOL backFace = FALSE);		
        Color GetDiffuse(int mtlNum = 0, BOOL backFace = FALSE);		
	    Color GetSpecular(int mtlNum = 0, BOOL backFace = FALSE);
	    float GetShininess(int mtlNum = 0, BOOL backFace = FALSE);
	    float GetShinStr(int mtlNum = 0, BOOL backFace = FALSE);
	    float GetXParency(int mtlNum = 0, BOOL backFace = FALSE);
          
        void SetAmbient(Color c, TimeValue t);
        void SetDiffuse(Color c, TimeValue t);
        void SetSpecular(Color c, TimeValue t);
        void SetShininess(float s, TimeValue t);

        void Shade(ShadeContext& sc);

        void SetParamDlg(OgreMaxMaterialDlg* paramDlg);

        ULONG Requirements(int subMtlNum);

        int NumSubTexmaps();
	    Texmap* GetSubTexmap(int i);
	    int MapSlotType(int i);
	    void SetSubTexmap(int i, Texmap* m);
	    TSTR GetSubTexmapSlotName(int i);
	    int SubTexmapOn(int i);

        int NumSubMtls();
		Mtl* GetSubMtl(int i);
		void SetSubMtl(int i, Mtl* mtl);
		int VPDisplaySubMtl();
		TSTR GetSubMtlSlotName(int i);
		TSTR GetSubMtlTVName(int i);
		
        static ClassDesc* GetClassDesc();
        static Class_ID GetClassClassID();
        Class_ID ClassID();
	    SClass_ID SuperClassID();
	    void GetClassName(TSTR& s);
        void DeleteThis();

        TSTR SubAnimName(int i);
        int NumSubs();
        Animatable* SubAnim(int i);
        int NumRefs();
        RefTargetHandle GetReference(int i);
        void SetReference(int i, RefTargetHandle rtarg);
        
        ULONG GetFlags();
        void SetFlag(ULONG f, ULONG value);

        BOOL IsTechniqueEnabled(int index);
        void EnableTechnique(int index, ULONG enable);

        float GetDistance(int index);
        void SetDistance(int index, float distance);

        TSTR GetTechniqueName(int index);

        TechniqueSlot* GetTechnique(int index);
        Mtl* GetViewRenderMaterial();

        IOResult Save(ISave* isave);
        IOResult Load(ILoad* iload);

    public:
        OgreMaxMaterialDlg* paramDlg;

        ULONG flags;
	    int rollScroll;

        enum 
        {
            TECHNIQUES_REF = 0,
            VIEW_RENDER_MATERIAL_REF = 1,
            NUM_REFS = 2
        };

        /** Reference 0. */
	    OgreMaxTechniques* techniques;

        /** Reference 1. */
        Mtl* viewRenderMaterial;

        /** Validity interval. */
        Interval ivalid;
    };
}


#endif
