
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "OgreMaxMaterialDlg.h"
#include "OgreMaxMaterial.h"
#include "PluginGlobals.h"
#include "resource.h"

using namespace OgreMax;


//Local functions--------------------------------------------------------------
enum {NUM_DISPLAY_TECHNIQUES = 4};

static int IndexToCheckBoxID(int i)
{
    static int checkboxes[NUM_DISPLAY_TECHNIQUES] = 
        {
        IDC_TECHNIQUE_ENABLE_CHECK1,
        IDC_TECHNIQUE_ENABLE_CHECK2,
        IDC_TECHNIQUE_ENABLE_CHECK3,
        IDC_TECHNIQUE_ENABLE_CHECK4
        };

    return checkboxes[i];
}

static int IndexToButtonID(int i)
{
    static int buttons[NUM_DISPLAY_TECHNIQUES] = 
        {
        IDC_TECHNIQUE_BUTTON1,
        IDC_TECHNIQUE_BUTTON2,
        IDC_TECHNIQUE_BUTTON3,
        IDC_TECHNIQUE_BUTTON4
        };
    
    return buttons[i];
}

static int IndexToDistanceSpinner(int i)
{
    static int spinners[NUM_DISPLAY_TECHNIQUES] = 
        {
        IDC_TECHNIQUE_LOD_DISTANCE_SPINNER1,
        IDC_TECHNIQUE_LOD_DISTANCE_SPINNER2,
        IDC_TECHNIQUE_LOD_DISTANCE_SPINNER3,
        IDC_TECHNIQUE_LOD_DISTANCE_SPINNER4
        };

    return spinners[i];
}

static int IndexToDistanceEdit(int i)
{
    static int edits[NUM_DISPLAY_TECHNIQUES] = 
        {
        IDC_TECHNIQUE_LOD_DISTANCE_EDIT1,
        IDC_TECHNIQUE_LOD_DISTANCE_EDIT2,
        IDC_TECHNIQUE_LOD_DISTANCE_EDIT3,
        IDC_TECHNIQUE_LOD_DISTANCE_EDIT4
        };

    return edits[i];
}

static int SpinnerIDToIndex(int id)
{
    switch (id) 
    {				
		case IDC_TECHNIQUE_LOD_DISTANCE_SPINNER1: return 0;
        case IDC_TECHNIQUE_LOD_DISTANCE_SPINNER2: return 1;
        case IDC_TECHNIQUE_LOD_DISTANCE_SPINNER3: return 2;
        case IDC_TECHNIQUE_LOD_DISTANCE_SPINNER4: return 3;
	}

    return -1;
}

static int ButtonIDToIndex(int id)
{
    switch (id)
    {
        case IDC_TECHNIQUE_BUTTON1: return 0;
        case IDC_TECHNIQUE_BUTTON2: return 1;
        case IDC_TECHNIQUE_BUTTON3: return 2;
        case IDC_TECHNIQUE_BUTTON4: return 3;
    }

    return -1;
}

static int CheckBoxIDToIndex(int id)
{
    switch (id)
    {
        case IDC_TECHNIQUE_ENABLE_CHECK1: return 0;
        case IDC_TECHNIQUE_ENABLE_CHECK2: return 1;
        case IDC_TECHNIQUE_ENABLE_CHECK3: return 2;
        case IDC_TECHNIQUE_ENABLE_CHECK4: return 3;
    }

    return -1;
}


//Implementation---------------------------------------------------------------
OgreMaxMaterialDlg::OgreMaxMaterialDlg
    (
    HWND hWndMaterialEdit, 
    IMtlParams* materialParams, 
    OgreMaxMaterial* material
    )
{
    this->hWndMaterialEdit = hWndMaterialEdit;
    this->materialParams = materialParams;
    this->material = material;

    this->mainPanel = NULL;

    this->dragAndDropManager.Init(this);

    for (int i = 0; i < NUM_DISPLAY_TECHNIQUES; i++) 
    {
        this->techniqueButton[i] = NULL;
        this->distanceSpin[i] = NULL;
    }

    this->viewRenderMaterialButton = NULL;

    this->currentTime = 0;

    this->techniqueScrollOffset = 0;

    this->initialized = false;
}

OgreMaxMaterialDlg::~OgreMaxMaterialDlg()
{
    //Capture some final rollup values
    this->material->SetFlag
        (
        OgreMaxMaterial::ROLLUP_OPEN, 
        this->materialParams->IsRollupPanelOpen(this->mainPanel)
        );
	this->material->rollScroll = this->materialParams->GetRollupScrollPos();

    //Remove the parameter dialog
	this->material->SetParamDlg(NULL);

    //Release the custom buttons and spinners used for the techniques
	for (int i = 0; i < NUM_DISPLAY_TECHNIQUES; i++) 
    {
		ReleaseISpinner(this->distanceSpin[i]);
		ReleaseICustButton(this->techniqueButton[i]);
    }

    //Release the custom button used for the view/render material
    ReleaseICustButton(this->viewRenderMaterialButton);
    
    //Remove the user data that we previously set
	SetWindowLongPtr(this->mainPanel, GWLP_USERDATA, NULL);	
}

Class_ID OgreMaxMaterialDlg::ClassID() 
{
    return OgreMaxMaterial::GetClassClassID();  
}

void OgreMaxMaterialDlg::SetThing(ReferenceTarget* refTarget)
{
    assert(refTarget->SuperClassID() == MATERIAL_CLASS_ID);
    assert(refTarget->ClassID() == OgreMaxMaterial::GetClassClassID());

	if (this->material) 
		this->material->SetParamDlg(NULL);
	
    this->material = static_cast<OgreMaxMaterial*>(refTarget);
    if (this->material)
	    this->material->SetParamDlg(this);
	
    LoadDialog();	
}

ReferenceTarget* OgreMaxMaterialDlg::GetThing() 
{ 
    return this->material; 
}

void OgreMaxMaterialDlg::DeleteThis() 
{ 
    delete this;  
}	

void OgreMaxMaterialDlg::SetTime(TimeValue t)
{
    if (t != this->currentTime) 
    {
		this->currentTime = t;
		Interval v;
        this->material->Update(this->materialParams->GetTime(), v);
		LoadDialog();
	}
}

int OgreMaxMaterialDlg::FindSubTexFromHWND(HWND hWnd)
{
    for (int i = 0; i < NUM_DISPLAY_TECHNIQUES; i++) 
    {
		if (this->techniqueButton[i]->GetHwnd() == hWnd) 
			return i;
	}	
	
	return -1;
}

int OgreMaxMaterialDlg::FindSubMtlFromHWND(HWND hWnd)
{
    if (this->viewRenderMaterialButton->GetHwnd() == hWnd)
        return 0;

    return -1;
}

void OgreMaxMaterialDlg::ReloadDialog()
{
    Interval v;
    this->material->Update(this->materialParams->GetTime(), v);
    LoadDialog();        
}

void OgreMaxMaterialDlg::ActivateDlg(BOOL activate)
{
}

void OgreMaxMaterialDlg::LoadDialog()
{
    if (this->material != NULL && this->mainPanel != NULL)
    {
		Interval v;

        ULONG flags = this->material->GetFlags();
        SetCheckBox(this->mainPanel, IDC_DONT_EXPORT_MATERIAL_CHECK, (flags & OgreMaxMaterial::DONT_EXPORT_MATERIAL) ? TRUE : FALSE);
        SetCheckBox(this->mainPanel, IDC_RECEIVES_SHADOWS_CHECK, (flags & OgreMaxMaterial::RECEIVES_SHADOWS) ? TRUE : FALSE);
        SetCheckBox(this->mainPanel, IDC_TRANSPARENCY_CASTS_SHADOWS_CHECK, (flags & OgreMaxMaterial::TRANSPARENCY_CASTS_SHADOWS) ? TRUE : FALSE);
        SetCheckBox(this->mainPanel, IDC_TECHNIQUES_ARE_LODS_CHECK, (flags & OgreMaxMaterial::TECHNIQUES_ARE_LODS) ? TRUE : FALSE);
        		
        for (int i = 0; i < NUM_DISPLAY_TECHNIQUES; i++)
            UpdateTechniqueDisplay(i);

        UpdateViewRenderMaterialDisplay();        
	}
}

void OgreMaxMaterialDlg::BuildDialog() 
{
    this->mainPanel = 
	    this->materialParams->AddRollupPage
            ( 
            PluginGlobals::GetInstanceHandle(),
		    MAKEINTRESOURCE(IDD_MATERIAL_MAIN_ROLLUP),
		    MainPanelDlgProc, 
		    PluginGlobals::GetString(IDS_MATERIAL_PARAMETERS), 
		    reinterpret_cast<LPARAM>(this),
            this->material->flags & OgreMaxMaterial::ROLLUP_OPEN ? 0 : APPENDROLL_CLOSED
		    );

    this->materialParams->SetRollupScrollPos(this->material->rollScroll);
}

void OgreMaxMaterialDlg::UpdateTechniqueDisplay(int i) 
{
    if (this->material != NULL && this->material->techniques != NULL)
    {
        int materialTechniqueIndex = i + this->techniqueScrollOffset;

        //Set spinner value
        this->distanceSpin[i]->SetValue(this->material->GetDistance(materialTechniqueIndex), FALSE);

        //Set checkbox
        SetCheckBox
            (
            this->mainPanel, 
            IndexToCheckBoxID(i), 
            this->material->IsTechniqueEnabled(materialTechniqueIndex)
            );

        //Set button text
        TSTR name = this->material->GetTechniqueName(materialTechniqueIndex);
        this->techniqueButton[i]->SetText(name.data());	
    }
}

void OgreMaxMaterialDlg::UpdateViewRenderMaterialDisplay()
{
    if (this->material != NULL)
    {
        Mtl* viewRenderMaterial = this->material->GetViewRenderMaterial();
        if (viewRenderMaterial != NULL)
            this->viewRenderMaterialButton->SetText((TCHAR*)viewRenderMaterial->GetName());
    }
}

BOOL OgreMaxMaterialDlg::MainPanelProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    int id = LOWORD(wParam);
	int code = HIWORD(wParam);
    switch (message)
    {
		case WM_INITDIALOG:
		{
            for (int i = 0; i < NUM_DISPLAY_TECHNIQUES; i++)
            {
                this->techniqueButton[i] = GetICustButton(GetDlgItem(hDlg, IndexToButtonID(i)));
                this->techniqueButton[i]->SetDADMgr(&this->dragAndDropManager);
                this->distanceSpin[i] = SetupIntSpinner(hDlg, IndexToDistanceSpinner(i), IndexToDistanceEdit(i), 0, 100000, 0);
            }

			this->viewRenderMaterialButton = GetICustButton(GetDlgItem(hDlg, IDC_MAX_VIEWPORT_RENDER_MATERIAL_BUTTON));
            
			return TRUE;
		}			
		case WM_COMMAND: 
		{ 
            int buttonIndex = ButtonIDToIndex(id);
            int checkboxIndex = CheckBoxIDToIndex(id);
            if (buttonIndex != -1)
                PostMessage(this->hWndMaterialEdit, WM_TEXMAP_BUTTON, buttonIndex, (LPARAM)this->material);
            else if (checkboxIndex != -1)
                this->material->EnableTechnique(checkboxIndex + this->techniqueScrollOffset, GetCheckBox(hDlg, id));
            else
            {
                switch (id) 
                {
                    case IDC_MAX_VIEWPORT_RENDER_MATERIAL_BUTTON:
                    {
                        PostMessage(this->hWndMaterialEdit, WM_SUB_MTL_BUTTON, 0, (LPARAM)this->material);                    
                        break;
                    }
                    case IDC_DONT_EXPORT_MATERIAL_CHECK:
                    {
                        this->material->SetFlag(OgreMaxMaterial::DONT_EXPORT_MATERIAL, GetCheckBox(hDlg, IDC_DONT_EXPORT_MATERIAL_CHECK));
                        break;
                    }
                    case IDC_RECEIVES_SHADOWS_CHECK:
                    {
                        this->material->SetFlag(OgreMaxMaterial::RECEIVES_SHADOWS, GetCheckBox(hDlg, IDC_RECEIVES_SHADOWS_CHECK));
                        break;
                    }
                    case IDC_TRANSPARENCY_CASTS_SHADOWS_CHECK:
                    {
                        this->material->SetFlag(OgreMaxMaterial::TRANSPARENCY_CASTS_SHADOWS, GetCheckBox(hDlg, IDC_TRANSPARENCY_CASTS_SHADOWS_CHECK));
                        break;
                    }
                    case IDC_TECHNIQUES_ARE_LODS_CHECK:
                    {
                        this->material->SetFlag(OgreMaxMaterial::TECHNIQUES_ARE_LODS, GetCheckBox(hDlg, IDC_TECHNIQUES_ARE_LODS_CHECK));
                        break;
                    }		                
			    }
            }

			return TRUE;
        }
		case CC_SPINNER_CHANGE: 
        {
			if (!theHold.Holding()) 
                theHold.Begin();

            int controlIndex  = SpinnerIDToIndex(id);
            if (controlIndex != -1)
            {
                int techniqueIndex = controlIndex + this->techniqueScrollOffset;
                this->material->SetDistance(techniqueIndex, this->distanceSpin[controlIndex]->GetIVal());
            }

			break;
        }
		case CC_SPINNER_BUTTONDOWN:
        {
			theHold.Begin();
			break;
        }
		case WM_CUSTEDIT_ENTER:
		case CC_SPINNER_BUTTONUP: 
        {
			if (HIWORD(wParam) || message == WM_CUSTEDIT_ENTER) 
				theHold.Accept(PluginGlobals::GetString(IDS_PARAMETER_CHANGE));
			else 
				theHold.Cancel();

			break;
        }
        case WM_PAINT:
        {
            if (!this->initialized)
            {
				this->initialized = true;
				ReloadDialog();
			}

			return FALSE;
        }
    }

	return FALSE;
}

INT_PTR CALLBACK OgreMaxMaterialDlg::MainPanelDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) 
{
    OgreMaxMaterialDlg* dialog = NULL;
	if (message == WM_INITDIALOG) 
    {
		dialog = reinterpret_cast<OgreMaxMaterialDlg*>(lParam);
		dialog->mainPanel = hWnd;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, lParam);
	}
	else 
    {
        dialog = reinterpret_cast<OgreMaxMaterialDlg*>(GetWindowLongPtr(hWnd, GWLP_USERDATA));
	    if (dialog == NULL)
			return FALSE; 
	}
	
    return dialog->MainPanelProc(hWnd, message, wParam, lParam);
}
