#ifndef OgreMax_OgreMaxMaterialDlg_INCLUDED
#define OgreMax_OgreMaxMaterialDlg_INCLUDED


//Includes---------------------------------------------------------------------
#include "OgreMaxTechniques.h"


//Classes----------------------------------------------------------------------
namespace OgreMax
{
    class OgreMaxMaterial;

    /**
     * Manages the rollup(s) for an OgreMaxMaterial in the 3DS Max material editor.
     */
    class OgreMaxMaterialDlg : public ParamDlg 
    {
    public:
        OgreMaxMaterialDlg(HWND hWndMaterialEdit, IMtlParams* materialParams, OgreMaxMaterial* material); 
	    ~OgreMaxMaterialDlg();

	    Class_ID ClassID();
        void SetThing(ReferenceTarget* refTarget);
	    ReferenceTarget* GetThing();
	    void DeleteThis();
	    void SetTime(TimeValue t);
	    int FindSubTexFromHWND(HWND hWnd);
        int FindSubMtlFromHWND(HWND hWnd);

        /** Updates material and puts parameters into dialog. */
        void ReloadDialog();

        /** Called to notify the dialog it's being activated/deactiviated. */
        void ActivateDlg(BOOL activate);

        /**
         * Puts parameters into dialog.
         */
        void LoadDialog();

        /**
         * Loads the dialog resources and puts them into the dialog.
         */
        void BuildDialog();

        /**
         * Updates the information for the specified technique in the UI.
         * @param techniqueIndex [in] - The index of the technique to update
         * in the UI.
         */
        void UpdateTechniqueDisplay(int techniqueIndex);

        /**
         * Updates the view/render material in the UI.
         */
        void UpdateViewRenderMaterialDisplay();

    private:
        BOOL MainPanelProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

        static INT_PTR CALLBACK MainPanelDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

    private:
        HWND hWndMaterialEdit;
        IMtlParams* materialParams;
        OgreMaxMaterial* material;

        HWND mainPanel;

        TexDADMgr dragAndDropManager;	
        ISpinnerControl* distanceSpin[OgreMaxTechniques::MAX_TECHNIQUES];	
	    ICustButton* techniqueButton[OgreMaxTechniques::MAX_TECHNIQUES];	    

        ICustButton* viewRenderMaterialButton;

        TimeValue currentTime;

        int techniqueScrollOffset;

        bool initialized;
    };
}


#endif
