//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "PluginGlobals.h"
#include "OgreMaxPass.h"
#include "OgreMaxPassDlg.h"
#include "resource.h"

using namespace OgreMax;


//Local classes-----------------------------------------------------------------
class OgreMaxPassClassDesc : public ClassDesc
{
public:
	int IsPublic()              {return 1;}
	void* Create(BOOL loading)  {return new OgreMaxPass(loading);}
	const TCHAR* ClassName()    {return PluginGlobals::GetString(IDS_PASS_CLASS_NAME);}
	SClass_ID SuperClassID()    {return TEXMAP_CLASS_ID;}
    Class_ID ClassID()          {return OgreMaxPass::GetClassClassID();}
	const TCHAR* Category()     {return PluginGlobals::GetString(IDS_PLUGINS_CATEGORY);}

public:
    static OgreMaxPassClassDesc instance;
};
OgreMaxPassClassDesc OgreMaxPassClassDesc::instance;


//Globals----------------------------------------------------------------------

//Parameter block descriptor
static ParamBlockDescID ogreMaxPassParamBlock[] = 
{
    {TYPE_RGBA, NULL, FALSE, OgreMaxPass::PB_AMBIENT},
	{TYPE_RGBA, NULL, FALSE, OgreMaxPass::PB_DIFFUSE},
	{TYPE_RGBA, NULL, FALSE, OgreMaxPass::PB_SPECULAR},
	{TYPE_FLOAT, NULL, FALSE, OgreMaxPass::PB_GLOSSINESS},
	{TYPE_FLOAT, NULL, FALSE, OgreMaxPass::PB_OPACITY},
    {TYPE_RGBA, NULL, FALSE, OgreMaxPass::PB_EMISSIVE},
	{TYPE_INT, NULL, FALSE, OgreMaxPass::PB_SOURCE_BLEND_FACTOR},
	{TYPE_INT, NULL, FALSE, OgreMaxPass::PB_DESTINATION_BLEND_FACTOR},
	{TYPE_INT, NULL, FALSE, OgreMaxPass::PB_SHADING},
	{TYPE_BOOL, NULL, FALSE, OgreMaxPass::PB_ENABLE_DEPTH_CHECK},
    {TYPE_BOOL, NULL, FALSE, OgreMaxPass::PB_ENABLE_DEPTH_WRITE},
    {TYPE_INT, NULL, FALSE, OgreMaxPass::PB_DEPTH_FUNCTION},
    {TYPE_INT, NULL, FALSE, OgreMaxPass::PB_DEPTH_BIAS},
    {TYPE_INT, NULL, FALSE, OgreMaxPass::PB_ALPHA_FUNCTION},
    {TYPE_INT, NULL, FALSE, OgreMaxPass::PB_ALPHA_VALUE},
    {TYPE_BOOL, NULL, FALSE, OgreMaxPass::PB_ENABLE_LIGHTING},
    {TYPE_BOOL, NULL, FALSE, OgreMaxPass::PB_ENABLE_COLOR_WRITE},
    {TYPE_INT, NULL, FALSE, OgreMaxPass::PB_MAX_LIGHTS},
    {TYPE_BOOL, NULL, FALSE, OgreMaxPass::PB_ONCE_PER_LIGHT},
    {TYPE_INT, NULL, FALSE, OgreMaxPass::PB_SUPPORTED_LIGHT_TYPE},
    {TYPE_INT, NULL, FALSE, OgreMaxPass::PB_HARDWARE_CULL},
    {TYPE_INT, NULL, FALSE, OgreMaxPass::PB_SOFTWARE_CULL},
    {TYPE_INT, NULL, FALSE, OgreMaxPass::PB_FOG_FUNCTION},
    {TYPE_RGBA, NULL, FALSE, OgreMaxPass::PB_FOG_COLOR},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxPass::PB_FOG_START},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxPass::PB_FOG_END},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxPass::PB_FOG_DENSITY}
};

#define PARAM_BLOCK_VERSION 1


//Implementation---------------------------------------------------------------
OgreMaxPass::OgreMaxPass(BOOL loading)
{
    this->paramDlg = NULL;

    this->flags = MAIN_ROLLUP_OPEN | SHADER_ROLLUP_OPEN;
	this->rollScroll = 0;

    this->textureUnits = NULL;
    this->paramBlock = NULL;
    
    if (!loading)
		Reset();
}

Color OgreMaxPass::GetAmbientColor(TimeValue t)
{
    return this->paramBlock->GetColor(PB_AMBIENT, t);
}

Color OgreMaxPass::GetDiffuseColor(TimeValue t)
{
    return this->paramBlock->GetColor(PB_DIFFUSE, t);
}

Color OgreMaxPass::GetSpecularColor(TimeValue t)
{
    return this->paramBlock->GetColor(PB_SPECULAR, t);
}

float OgreMaxPass::GetGlossiness(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_GLOSSINESS, t);
}

float OgreMaxPass::GetOpacity(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_OPACITY, t);    
}

Color OgreMaxPass::GetEmissiveColor(TimeValue t)
{
    return this->paramBlock->GetColor(PB_EMISSIVE, t);
}

int OgreMaxPass::GetSourceBlend(TimeValue t)
{
    return this->paramBlock->GetInt(PB_SOURCE_BLEND_FACTOR, t);
}

int OgreMaxPass::GetDestinationBlend(TimeValue t)
{
    return this->paramBlock->GetInt(PB_DESTINATION_BLEND_FACTOR, t);
}

int OgreMaxPass::GetShading(TimeValue t)
{
    return this->paramBlock->GetInt(PB_SHADING, t);
}

BOOL OgreMaxPass::GetEnableDepthCheck(TimeValue t)
{
    return this->paramBlock->GetInt(PB_ENABLE_DEPTH_CHECK, t);
}

BOOL OgreMaxPass::GetEnableDepthWrite(TimeValue t)
{
    return this->paramBlock->GetInt(PB_ENABLE_DEPTH_WRITE, t);
}

int OgreMaxPass::GetDepthFunction(TimeValue t)
{
    return this->paramBlock->GetInt(PB_DEPTH_FUNCTION, t);
}

int OgreMaxPass::GetDepthBias(TimeValue t)
{
    return this->paramBlock->GetInt(PB_DEPTH_BIAS, t);
}

int OgreMaxPass::GetAlphaFunction(TimeValue t)
{
    return this->paramBlock->GetInt(PB_ALPHA_FUNCTION, t);
}

int OgreMaxPass::GetAlphaValue(TimeValue t)
{
    return this->paramBlock->GetInt(PB_ALPHA_VALUE, t);
}

BOOL OgreMaxPass::GetEnableLighting(TimeValue t)
{
    return this->paramBlock->GetInt(PB_ENABLE_LIGHTING, t);
}

BOOL OgreMaxPass::GetEnableColorWrite(TimeValue t)
{
    return this->paramBlock->GetInt(PB_ENABLE_COLOR_WRITE, t);
}

int OgreMaxPass::GetMaxLights(TimeValue t)
{
    return this->paramBlock->GetInt(PB_MAX_LIGHTS, t);
}

BOOL OgreMaxPass::GetOncePerLight(TimeValue t)
{
    return this->paramBlock->GetInt(PB_ONCE_PER_LIGHT, t);    
}

int OgreMaxPass::GetSupportedLightType(TimeValue t)
{
    return this->paramBlock->GetInt(PB_SUPPORTED_LIGHT_TYPE, t);
}

int OgreMaxPass::GetHardwareCull(TimeValue t)
{
    return this->paramBlock->GetInt(PB_HARDWARE_CULL, t);    
}

int OgreMaxPass::GetSoftwareCull(TimeValue t)
{
    return this->paramBlock->GetInt(PB_SOFTWARE_CULL, t);    
}

int OgreMaxPass::GetFogFunction(TimeValue t)
{
    return this->paramBlock->GetInt(PB_FOG_FUNCTION, t);
}

Color OgreMaxPass::GetFogColor(TimeValue t)
{
    return this->paramBlock->GetColor(PB_FOG_COLOR, t);
}

float OgreMaxPass::GetFogStart(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_FOG_START, t);
}

float OgreMaxPass::GetFogEnd(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_FOG_END, t);
}

float OgreMaxPass::GetFogDensity(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_FOG_DENSITY, t);
}

void OgreMaxPass::SetAmbientColor(Color value, TimeValue t)
{
    this->paramBlock->SetValue(PB_AMBIENT, t, value);
}

void OgreMaxPass::SetDiffuseColor(Color value, TimeValue t)
{
    this->paramBlock->SetValue(PB_DIFFUSE, t, value);
}

void OgreMaxPass::SetSpecularColor(Color value, TimeValue t)
{
    this->paramBlock->SetValue(PB_SPECULAR, t, value);
}

void OgreMaxPass::SetGlossiness(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_GLOSSINESS, t, value);
}

void OgreMaxPass::SetOpacity(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_OPACITY, t, value);
}

void OgreMaxPass::SetEmissiveColor(Color value, TimeValue t)
{
    this->paramBlock->SetValue(PB_EMISSIVE, t, value);
}

void OgreMaxPass::SetSourceBlend(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_SOURCE_BLEND_FACTOR, t, value);
}

void OgreMaxPass::SetDestinationBlend(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_DESTINATION_BLEND_FACTOR, t, value);
}

void OgreMaxPass::SetShading(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_SHADING, t, value);
}

void OgreMaxPass::SetEnableDepthCheck(BOOL value, TimeValue t)
{
    this->paramBlock->SetValue(PB_ENABLE_DEPTH_CHECK, t, value);
}

void OgreMaxPass::SetEnableDepthWrite(BOOL value, TimeValue t)
{
    this->paramBlock->SetValue(PB_ENABLE_DEPTH_WRITE, t, value);
}

void OgreMaxPass::SetDepthFunction(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_DEPTH_FUNCTION, t, value);
}

void OgreMaxPass::SetDepthBias(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_DEPTH_BIAS, t, value);
}

void OgreMaxPass::SetAlphaFunction(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_ALPHA_FUNCTION, t, value);
}

void OgreMaxPass::SetAlphaValue(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_ALPHA_VALUE, t, value);
}

void OgreMaxPass::SetEnableLighting(BOOL value, TimeValue t)
{
    this->paramBlock->SetValue(PB_ENABLE_LIGHTING, t, value);
}

void OgreMaxPass::SetEnableColorWrite(BOOL value, TimeValue t)
{
    this->paramBlock->SetValue(PB_ENABLE_COLOR_WRITE, t, value);
}

void OgreMaxPass::SetMaxLights(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_MAX_LIGHTS, t, value);
}

void OgreMaxPass::SetOncePerLight(BOOL value, TimeValue t)
{
    this->paramBlock->SetValue(PB_ONCE_PER_LIGHT, t, value);
}

void OgreMaxPass::SetSupportedLightType(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_SUPPORTED_LIGHT_TYPE, t, value);
}

void OgreMaxPass::SetHardwareCull(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_HARDWARE_CULL, t, value);
}

void OgreMaxPass::SetSoftwareCull(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_SOFTWARE_CULL, t, value);
}

void OgreMaxPass::SetFogFunction(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_FOG_FUNCTION, t, value);
}

void OgreMaxPass::SetFogColor(Color value, TimeValue t)
{
    this->paramBlock->SetValue(PB_FOG_COLOR, t, value);
}

void OgreMaxPass::SetFogStart(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_FOG_START, t, value);
}

void OgreMaxPass::SetFogEnd(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_FOG_END, t, value);
}

void OgreMaxPass::SetFogDensity(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_FOG_DENSITY, t, value);
}

BOOL OgreMaxPass::IsVertexShaderEnabled()
{
    return TestFlag(VERTEX_SHADER_ACTIVE);
}

BOOL OgreMaxPass::IsPixelShaderEnabled()
{
    return TestFlag(PIXEL_SHADER_ACTIVE);
}

void OgreMaxPass::EnableVertexShader(BOOL enable)
{
    SetFlag(VERTEX_SHADER_ACTIVE, enable);
}

void OgreMaxPass::EnablePixelShader(BOOL enable)
{
    SetFlag(PIXEL_SHADER_ACTIVE, enable);
}

AColor OgreMaxPass::EvalColor(ShadeContext& sc)
{
    return AColor(0, 0, 0);
}

Point3 OgreMaxPass::EvalNormalPerturb(ShadeContext& sc)
{
    return Point3(0, 0, 0);
}

RefResult OgreMaxPass::NotifyRefChanged
    (
    Interval changeInt, 
    RefTargetHandle hTarget, 
    PartID& partID, 
    RefMessage message
    )
{
    switch (message) 
    {
		case REFMSG_WANT_SHOWPARAMLEVEL:
		{
			BOOL* result = reinterpret_cast<BOOL*>(partID);
			*result = TRUE;
			return REF_STOP;
		}
		case REFMSG_CHANGE:
        {
			this->ivalid.SetEmpty();
			if (this->paramDlg != NULL)
            	this->paramDlg->Invalidate();
            break;
        }
		case REFMSG_GET_PARAM_DIM: 
        {
			GetParamDim* gpd = reinterpret_cast<GetParamDim*>(partID);
			switch (gpd->index) 
            {
                case PB_AMBIENT: gpd->dim = stdColor255Dim; break;
                case PB_DIFFUSE: gpd->dim = stdColor255Dim; break;
                case PB_SPECULAR: gpd->dim = stdColor255Dim; break;
                case PB_GLOSSINESS: gpd->dim = stdPercentDim; break;
                case PB_OPACITY: gpd->dim = stdPercentDim; break;
                case PB_EMISSIVE: gpd->dim = stdColor255Dim; break;
                case PB_SOURCE_BLEND_FACTOR: gpd->dim = defaultDim; break;
                case PB_DESTINATION_BLEND_FACTOR: gpd->dim = defaultDim; break;
                case PB_SHADING: gpd->dim = defaultDim; break;
                case PB_ENABLE_DEPTH_CHECK: gpd->dim = defaultDim; break;
                case PB_ENABLE_DEPTH_WRITE: gpd->dim = defaultDim; break;
                case PB_DEPTH_FUNCTION: gpd->dim = defaultDim; break;
                case PB_DEPTH_BIAS: gpd->dim = defaultDim; break;
                case PB_ALPHA_FUNCTION: gpd->dim = defaultDim; break;
                case PB_ALPHA_VALUE: gpd->dim = defaultDim; break;
                case PB_ENABLE_LIGHTING: gpd->dim = defaultDim; break;
                case PB_ENABLE_COLOR_WRITE: gpd->dim = defaultDim; break;
                case PB_MAX_LIGHTS: gpd->dim = defaultDim; break;
                case PB_ONCE_PER_LIGHT: gpd->dim = defaultDim; break;
                case PB_SUPPORTED_LIGHT_TYPE: gpd->dim = defaultDim; break;
                case PB_HARDWARE_CULL: gpd->dim = defaultDim; break;
                case PB_SOFTWARE_CULL: gpd->dim = defaultDim; break;
                case PB_FOG_FUNCTION: gpd->dim = defaultDim; break;
                case PB_FOG_COLOR: gpd->dim = stdColor255Dim; break;
                case PB_FOG_START: gpd->dim = defaultDim; break;
                case PB_FOG_END: gpd->dim = defaultDim; break;
                case PB_FOG_DENSITY: gpd->dim = defaultDim; break;				
			}
			return REF_STOP; 
		}
		case REFMSG_GET_PARAM_NAME: 
        {
			GetParamName* gpn = reinterpret_cast<GetParamName*>(partID);			
            switch (gpn->index) 
            {
                case PB_AMBIENT: gpn->name = PluginGlobals::GetString(IDS_AMBIENT); break;
                case PB_DIFFUSE: gpn->name = PluginGlobals::GetString(IDS_DIFFUSE); break;
                case PB_SPECULAR: gpn->name = PluginGlobals::GetString(IDS_SPECULAR); break;
                case PB_GLOSSINESS: gpn->name = PluginGlobals::GetString(IDS_GLOSSINESS); break;
                case PB_OPACITY: gpn->name = PluginGlobals::GetString(IDS_OPACITY); break;
                case PB_EMISSIVE: gpn->name = PluginGlobals::GetString(IDS_EMISSIVE); break;
                case PB_SOURCE_BLEND_FACTOR: gpn->name = PluginGlobals::GetString(IDS_SOURCE_BLEND); break;
                case PB_DESTINATION_BLEND_FACTOR: gpn->name = PluginGlobals::GetString(IDS_DESTINATION_BLEND); break;
                case PB_SHADING: gpn->name = PluginGlobals::GetString(IDS_SHADING); break;
                case PB_ENABLE_DEPTH_CHECK: gpn->name = PluginGlobals::GetString(IDS_ENABLE_DEPTH_CHECK); break;
                case PB_ENABLE_DEPTH_WRITE: gpn->name = PluginGlobals::GetString(IDS_ENABLE_DEPTH_WRITE); break;
                case PB_DEPTH_FUNCTION: gpn->name = PluginGlobals::GetString(IDS_DEPTH_FUNCTION); break;
                case PB_DEPTH_BIAS: gpn->name = PluginGlobals::GetString(IDS_DEPTH_BIAS); break;
                case PB_ALPHA_FUNCTION: gpn->name = PluginGlobals::GetString(IDS_ALPHA_FUNCTION); break;
                case PB_ALPHA_VALUE: gpn->name = PluginGlobals::GetString(IDS_ALPHA_VALUE); break;
                case PB_ENABLE_LIGHTING: gpn->name = PluginGlobals::GetString(IDS_ENABLE_LIGHTING); break;
                case PB_ENABLE_COLOR_WRITE: gpn->name = PluginGlobals::GetString(IDS_ENABLE_COLOR_WRITE); break;
                case PB_MAX_LIGHTS: gpn->name = PluginGlobals::GetString(IDS_MAX_LIGHTS); break;
                case PB_ONCE_PER_LIGHT: gpn->name = PluginGlobals::GetString(IDS_ONCE_PER_LIGHT); break;
                case PB_SUPPORTED_LIGHT_TYPE: gpn->name = PluginGlobals::GetString(IDS_SUPPORTED_LIGHT_TYPE); break;
                case PB_HARDWARE_CULL: gpn->name = PluginGlobals::GetString(IDS_HARDWARE_CULL); break;
                case PB_SOFTWARE_CULL: gpn->name = PluginGlobals::GetString(IDS_SOFTWARE_CULL); break;
                case PB_FOG_FUNCTION: gpn->name = PluginGlobals::GetString(IDS_FOG_FUNCTION); break;
                case PB_FOG_COLOR: gpn->name = PluginGlobals::GetString(IDS_FOG_COLOR); break;
                case PB_FOG_START: gpn->name = PluginGlobals::GetString(IDS_FOG_START); break;
                case PB_FOG_END: gpn->name = PluginGlobals::GetString(IDS_FOG_END); break;
                case PB_FOG_DENSITY: gpn->name = PluginGlobals::GetString(IDS_FOG_DENSITY); break;
			}
			return REF_STOP; 
		}
	}

    return REF_SUCCEED;
}

void OgreMaxPass::Update(TimeValue t, Interval& valid)
{
    Point3 p;
	if (!this->ivalid.InInterval(t)) 
    {
		this->ivalid.SetInfinite();

        if (this->textureUnits != NULL)
        {
    	    for (int i = 0; i < OgreMaxTextureUnits::MAX_TEXTURE_UNITS; i++) 	
            {
                if (GetTextureUnit(i)->IsOn())
				    GetTextureUnit(i)->Update(t, this->ivalid);
		    }
        }
	}
	valid &= ivalid;
}

RefTargetHandle OgreMaxPass::Clone(RemapDir& remap)
{
    OgreMaxPass* newTechnique = new OgreMaxPass();

    //Copy superclass stuff
    *static_cast<MtlBase*>(newTechnique) = *this;
	
    //Copy everything elsemm
	newTechnique->ReplaceReference(TEXTURE_UNITS_REF, remap.CloneRef(this->textureUnits));	
    newTechnique->ReplaceReference(PARAM_BLOCK_REF, remap.CloneRef(this->paramBlock));	
    
	newTechnique->flags = flags;
	
    BaseClone(this, newTechnique, remap);
	
    return newTechnique;
}

void OgreMaxPass::Reset()
{
    SuspendSetKeyMode();

    this->ivalid.SetEmpty();

    this->flags = MAIN_ROLLUP_OPEN | SHADER_ROLLUP_OPEN;
	
    ReplaceReference(TEXTURE_UNITS_REF, new OgreMaxTextureUnits(this));	
	ReplaceReference
        (
        PARAM_BLOCK_REF, 
        CreateParameterBlock(ogreMaxPassParamBlock, NUM_PB_ENTRIES, PARAM_BLOCK_VERSION)
        );	

    //Set the default values    
    SetAmbientColor(Color(RGB(178, 178, 178)));
    SetDiffuseColor(Color(RGB(178, 178, 178)));
    SetSpecularColor(Color(RGB(229, 229, 229)));
    SetGlossiness(20.0f);
    SetOpacity(100.0f);
    SetEmissiveColor(Color(0.0f, 0.0f, 0.0f));
    SetSourceBlend(BLEND_FACTOR_ONE);
    SetDestinationBlend(BLEND_FACTOR_ZERO);
    SetShading(SHADING_GOURAUD);
    SetEnableDepthCheck(TRUE);
    SetEnableDepthWrite(TRUE);
    SetDepthFunction(DEPTH_FUNCTION_LESS_EQUAL);
    SetDepthBias(0);
    SetAlphaFunction(ALPHA_FUNCTION_ALWAYS_PASS);
    SetAlphaValue(0);
    SetEnableLighting(TRUE);
    SetEnableColorWrite(TRUE);
    SetMaxLights(8);
    SetOncePerLight(FALSE);
    SetSupportedLightType(LIGHT_TYPE_ALL);
    SetHardwareCull(HARDWARE_CULL_CLOCKWISE);
    SetSoftwareCull(SOFTWARE_CULL_BACK);
    SetFogFunction(FOG_FUNCTION_NONE);
    SetFogColor(Color(RGB(178, 178, 178)));
    SetFogStart(100.0f);
    SetFogEnd(10000.0f);
    SetFogDensity(.002f); 

	ResumeSetKeyMode();
}

Interval OgreMaxPass::Validity(TimeValue t)
{
    Interval v;
	Update(t, v);
	return this->ivalid;
}

ParamDlg* OgreMaxPass::CreateParamDlg(HWND hWndMaterialEdit, IMtlParams* materialParams)
{
    Interval v;
	Update(materialParams->GetTime(), v);
	OgreMaxPassDlg* dlg = new OgreMaxPassDlg(hWndMaterialEdit, materialParams, this);    
	dlg->BuildDialog();
	dlg->LoadDialog();	 
	SetParamDlg(dlg);
	return dlg;
}

void OgreMaxPass::SetParamDlg(OgreMaxPassDlg* paramDlg)
{
    this->paramDlg = paramDlg;
}

int OgreMaxPass::NumSubTexmaps() 
{ 
    return OgreMaxTextureUnits::MAX_TEXTURE_UNITS; 
}

Texmap* OgreMaxPass::GetSubTexmap(int i)
{
    if (this->textureUnits != NULL)
        return GetTextureUnit(i)->GetMap();
    else
        return NULL;
}

int OgreMaxPass::MapSlotType(int i) 
{ 
    return MAPSLOT_TEXTURE; 
}

void OgreMaxPass::SetSubTexmap(int i, Texmap* m)
{
    assert(i < OgreMaxTextureUnits::MAX_TEXTURE_UNITS);
	
    if (this->textureUnits != NULL)
        this->textureUnits->ReplaceReference(i, m);	    
    
	if (this->paramDlg)
		this->paramDlg->UpdateTextureUnitDisplay(i);
}

TSTR OgreMaxPass::GetSubTexmapSlotName(int i)
{
    return PluginGlobals::GetString(IDS_PASS_SLOT_NAME);
}

int OgreMaxPass::SubTexmapOn(int i) 
{ 
    return this->textureUnits->GetTextureUnit(i)->IsOn();
} 

ClassDesc* OgreMaxPass::GetClassDesc()
{
    return &OgreMaxPassClassDesc::instance;
}

Class_ID OgreMaxPass::GetClassClassID()
{
    return Class_ID(0xfc89cdc, 0x8d7b4538);
}

Class_ID OgreMaxPass::ClassID()
{
    return GetClassClassID();
}

SClass_ID OgreMaxPass::SuperClassID()
{ 
    return TEXMAP_CLASS_ID; 
}

void OgreMaxPass::GetClassName(TSTR& s)
{
    s = PluginGlobals::GetString(IDS_PASS_CLASS_NAME); 
}  

void OgreMaxPass::DeleteThis() 
{
	delete this;
}

TSTR OgreMaxPass::SubAnimName(int i) 
{ 
    TSTR name;

    switch (i)
    {
        case TEXTURE_UNITS_REF:
        {
            name = PluginGlobals::GetString(IDS_TEXTURE_UNITS); 
            break;
        }        
        case PARAM_BLOCK_REF:
        {
            name = PluginGlobals::GetString(IDS_PARAMETERS);
            break;
        }
    }

    return name;
}		

int OgreMaxPass::NumSubs()
{
    return NUM_REFS; 
}  

Animatable* OgreMaxPass::SubAnim(int i) 
{
	switch (i) 
    {
		case TEXTURE_UNITS_REF: return this->textureUnits;
        case PARAM_BLOCK_REF: return this->paramBlock;
		default: 
        {
            assert(0); 
            return NULL;
        }
	}
}

int OgreMaxPass::NumRefs()
{
    return NUM_REFS; 
}

RefTargetHandle OgreMaxPass::GetReference(int i) 
{
	switch (i) 
    {
		case TEXTURE_UNITS_REF: return this->textureUnits;
        case PARAM_BLOCK_REF: return this->paramBlock;
		default: 
        {
            assert(0); 
            return NULL;
        }
	}
}

void OgreMaxPass::SetReference(int i, RefTargetHandle refTarget) 
{
	switch (i) 
    {
		case TEXTURE_UNITS_REF:	
        {
            this->textureUnits = reinterpret_cast<OgreMaxTextureUnits*>(refTarget);
            if (this->textureUnits != NULL)
                this->textureUnits->SetClientPtr(this);
            
            break;
        }
        case PARAM_BLOCK_REF:
        {
            this->paramBlock = reinterpret_cast<IParamBlock*>(refTarget);
            break;
        }
		default: assert(0);
	}
}

ULONG OgreMaxPass::GetFlags()
{
    return this->flags;
}

BOOL OgreMaxPass::TestFlag(ULONG f)
{
    return (this->flags & f) != 0 ? TRUE : FALSE;
}

void OgreMaxPass::SetFlag(ULONG f, ULONG value) 
{
	if (value) 
    {
        //Add to the flags
        this->flags |= f; 
    }
	else 
    {
        //Remove the flags
        this->flags &= ~f;
    }
}

BOOL OgreMaxPass::IsTextureUnitEnabled(int index)
{
    if (this->textureUnits != NULL)
        return GetTextureUnit(index)->IsOn();
    else
        return FALSE;
}

void OgreMaxPass::EnableTextureUnit(int index, ULONG enable)
{
    if (this->textureUnits != NULL)
        GetTextureUnit(index)->SetOn(enable);
}

TSTR OgreMaxPass::GetTextureUnitName(int index)
{
    TSTR name;
    if (this->textureUnits != NULL && GetTextureUnit(index)->GetMap() != NULL)
    {
        Texmap* texmap = this->textureUnits->GetTextureUnit(index)->GetMap();
        name = texmap->GetName();
    }
    else
        name = PluginGlobals::GetString(IDS_NONE);
    return name;
}

TextureUnitSlot* OgreMaxPass::GetTextureUnit(int index)
{
    assert(this->textureUnits);
    return this->textureUnits->GetTextureUnit(index);
}

#define PASS_HEADER_CHUNK 0x4000
#define PASS_FLAGS_CHUNK 0x5000
#define PASS_VERSION_CHUNK 0x5300

#define PASS_VERSION 1

IOResult OgreMaxPass::Save(ISave* isave)
{
    ULONG byteCount;
    IOResult result;

	isave->BeginChunk(PASS_HEADER_CHUNK);
	result = MtlBase::Save(isave);
	if (result != IO_OK) 
        return result;
	isave->EndChunk();

    isave->BeginChunk(PASS_VERSION_CHUNK);
	int version = PASS_VERSION;
	isave->Write(&version, sizeof(version), &byteCount);			
	isave->EndChunk();

	isave->BeginChunk(PASS_FLAGS_CHUNK);
	isave->Write(&this->flags, sizeof(this->flags), &byteCount);			
	isave->EndChunk();

	return IO_OK;
}

IOResult OgreMaxPass::Load(ILoad* iload)
{
    ULONG byteCount;
	int id;
	int version = 0;
	IOResult result;

	while ((result = iload->OpenChunk()) == IO_OK) 
    {
		switch (id = iload->CurChunkID())  
        {
			case PASS_HEADER_CHUNK:
            {
				result = MtlBase::Load(iload);
				this->ivalid.SetEmpty();
				break;
            }
            case PASS_VERSION_CHUNK:
            {
				result = iload->Read(&version, sizeof(version), &byteCount);
				break;
            }
			case PASS_FLAGS_CHUNK:
            {
				result = iload->Read(&this->flags, sizeof(this->flags), &byteCount);
				break;
            }
		}
		iload->CloseChunk();
		if (result != IO_OK) 
			return result;
	}
	
	return IO_OK;
}

