#ifndef OgreMax_OgreMaxPass_INCLUDED
#define OgreMax_OgreMaxPass_INCLUDED


//Includes---------------------------------------------------------------------
#include "OgreMaxTextureUnits.h"


//Classes----------------------------------------------------------------------
namespace OgreMax
{
    class OgreMaxPassDlg;

    class OgreMaxPass : public Texmap
    {
    public:
        enum 
        {
            VERTEX_SHADER_ACTIVE             = 1<<0,
            PIXEL_SHADER_ACTIVE              = 1<<1,

            MAIN_ROLLUP_OPEN                 = 1<<28,
            SHADER_ROLLUP_OPEN               = 1<<29
        };

        enum
        {
            BLEND_FACTOR_ONE,
            BLEND_FACTOR_ZERO,
            BLEND_FACTOR_DESTINATION_COLOR,
            BLEND_FACTOR_COLOR,
            BLEND_FACTOR_INVERSE_DESTINATION_COLOR,
            BLEND_FACTOR_INVERSE_COLOR,
            BLEND_FACTOR_DESTINATION_ALPHA,
            BLEND_FACTOR_ALPHA,
            BLEND_FACTOR_INVERSE_DESTINATION_ALPHA,
            BLEND_FACTOR_INVERSE_ALPHA
        };

        enum
        {
            SHADING_FLAT,
            SHADING_GOURAUD,
            SHADING_PHONG
        };

        enum
        {
            DEPTH_FUNCTION_ALWAYS_FAIL,
            DEPTH_FUNCTION_ALWAYS_PASS,
            DEPTH_FUNCTION_LESS,
            DEPTH_FUNCTION_LESS_EQUAL,
            DEPTH_FUNCTION_EQUAL,
            DEPTH_FUNCTION_NOT_EQUAL,
            DEPTH_FUNCTION_GREATER_EQUAL,
            DEPTH_FUNCTION_GREATER
        };

        enum
        {
            ALPHA_FUNCTION_ALWAYS_FAIL,
            ALPHA_FUNCTION_ALWAYS_PASS,
            ALPHA_FUNCTION_LESS,
            ALPHA_FUNCTION_LESS_EQUAL,
            ALPHA_FUNCTION_EQUAL,
            ALPHA_FUNCTION_NOT_EQUAL,
            ALPHA_FUNCTION_GREATER_EQUAL,
            ALPHA_FUNCTION_GREATER
        };

        enum
        {
            LIGHT_TYPE_POINT,
            LIGHT_TYPE_DIRECTIONAL,
            LIGHT_TYPE_SPOT,
            LIGHT_TYPE_ALL
        };

        enum
        {
            HARDWARE_CULL_NONE,
            HARDWARE_CULL_CLOCKWISE,
            HARDWARE_CULL_COUNTERCLOCKWISE
        };

        enum
        {
            SOFTWARE_CULL_NONE,
            SOFTWARE_CULL_BACK,
            SOFTWARE_CULL_FRONT
        };

        enum
        {
            FOG_FUNCTION_NONE,
            FOG_FUNCTION_EXP,
            FOG_FUNCTION_EXP2,
            FOG_FUNCTION_LINEAR
        };

        OgreMaxPass(BOOL loading = FALSE);

        Color GetAmbientColor(TimeValue t = 0);
        Color GetDiffuseColor(TimeValue t = 0);
        Color GetSpecularColor(TimeValue t = 0);
        float GetGlossiness(TimeValue t = 0);
        float GetOpacity(TimeValue t = 0);
        Color GetEmissiveColor(TimeValue t = 0);
        int GetSourceBlend(TimeValue t = 0);
        int GetDestinationBlend(TimeValue t = 0);
        int GetShading(TimeValue t = 0);
        BOOL GetEnableDepthCheck(TimeValue t = 0);
        BOOL GetEnableDepthWrite(TimeValue t = 0);
        int GetDepthFunction(TimeValue t = 0);
        int GetDepthBias(TimeValue t = 0);
        int GetAlphaFunction(TimeValue t = 0);
        int GetAlphaValue(TimeValue t = 0);
        BOOL GetEnableLighting(TimeValue t = 0);
        BOOL GetEnableColorWrite(TimeValue t = 0);
        int GetMaxLights(TimeValue t = 0);
        BOOL GetOncePerLight(TimeValue t = 0);
        int GetSupportedLightType(TimeValue t = 0);
        int GetHardwareCull(TimeValue t = 0);
        int GetSoftwareCull(TimeValue t = 0);
        int GetFogFunction(TimeValue t = 0);
        Color GetFogColor(TimeValue t = 0);
        float GetFogStart(TimeValue t = 0);
        float GetFogEnd(TimeValue t = 0);
        float GetFogDensity(TimeValue t = 0);

        void SetAmbientColor(Color value, TimeValue t = 0);
        void SetDiffuseColor(Color value, TimeValue t = 0);
        void SetSpecularColor(Color value, TimeValue t = 0);
        void SetGlossiness(float value, TimeValue t = 0);
        void SetOpacity(float value, TimeValue t = 0);
        void SetEmissiveColor(Color value, TimeValue t = 0);
        void SetSourceBlend(int value, TimeValue t = 0);
        void SetDestinationBlend(int value, TimeValue t = 0);
        void SetShading(int value, TimeValue t = 0);
        void SetEnableDepthCheck(BOOL value, TimeValue t = 0);
        void SetEnableDepthWrite(BOOL value, TimeValue t = 0);
        void SetDepthFunction(int value, TimeValue t = 0);
        void SetDepthBias(int value, TimeValue t = 0);
        void SetAlphaFunction(int value, TimeValue t = 0);
        void SetAlphaValue(int value, TimeValue t = 0);
        void SetEnableLighting(BOOL value, TimeValue t = 0);
        void SetEnableColorWrite(BOOL value, TimeValue t = 0);
        void SetMaxLights(int value, TimeValue t = 0);
        void SetOncePerLight(BOOL value, TimeValue t = 0);
        void SetSupportedLightType(int value, TimeValue t = 0);
        void SetHardwareCull(int value, TimeValue t = 0);
        void SetSoftwareCull(int value, TimeValue t = 0);
        void SetFogFunction(int value, TimeValue t = 0);
        void SetFogColor(Color value, TimeValue t = 0);
        void SetFogStart(float value, TimeValue t = 0);
        void SetFogEnd(float value, TimeValue t = 0);
        void SetFogDensity(float value, TimeValue t = 0);

        BOOL IsVertexShaderEnabled();
        BOOL IsPixelShaderEnabled();

        void EnableVertexShader(BOOL enable);
        void EnablePixelShader(BOOL enable);

        AColor EvalColor(ShadeContext& sc);
        Point3 EvalNormalPerturb(ShadeContext& sc);

        RefResult NotifyRefChanged
            (
            Interval changeInt, 
            RefTargetHandle hTarget, 
	        PartID& partID, 
            RefMessage message
            );

        void Update(TimeValue t, Interval& valid);

        RefTargetHandle Clone(RemapDir& remap);
        void Reset();

        Interval Validity(TimeValue t);

        ParamDlg* CreateParamDlg(HWND hWndMaterialEdit, IMtlParams* materialParams);

        void SetParamDlg(OgreMaxPassDlg* paramDlg);

        int NumSubTexmaps();
	    Texmap* GetSubTexmap(int i);
	    int MapSlotType(int i);
	    void SetSubTexmap(int i, Texmap* m);
	    TSTR GetSubTexmapSlotName(int i);
	    int SubTexmapOn(int i);

        static ClassDesc* GetClassDesc();
        static Class_ID GetClassClassID();
        Class_ID ClassID();
	    SClass_ID SuperClassID();
	    void GetClassName(TSTR& s);
        void DeleteThis();

        TSTR SubAnimName(int i);
        int NumSubs();
        Animatable* SubAnim(int i);
        int NumRefs();
        RefTargetHandle GetReference(int i);
        void SetReference(int i, RefTargetHandle rtarg);
        
        ULONG GetFlags();
        BOOL TestFlag(ULONG f);
        void SetFlag(ULONG f, ULONG value);

        BOOL IsTextureUnitEnabled(int index);
        void EnableTextureUnit(int index, ULONG enable);

        TSTR GetTextureUnitName(int index);
        TextureUnitSlot* GetTextureUnit(int index);
        
        IOResult Save(ISave* isave);
        IOResult Load(ILoad* iload);

    public:
        OgreMaxPassDlg* paramDlg;

        ULONG flags;
	    int rollScroll;

        /** Reference indices. */
        enum 
        {
            TEXTURE_UNITS_REF = 0,
            PARAM_BLOCK_REF = 1,
            NUM_REFS = 2
        };

        /** ParamBlock indices. */
        enum
        {
            PB_AMBIENT = 0,
            PB_DIFFUSE = 1,
            PB_SPECULAR = 2,
            PB_GLOSSINESS = 3,
            PB_OPACITY = 4,
            PB_EMISSIVE = 5,
            PB_SOURCE_BLEND_FACTOR = 6,
            PB_DESTINATION_BLEND_FACTOR = 7,
            PB_SHADING = 8,
            PB_ENABLE_DEPTH_CHECK = 9,
            PB_ENABLE_DEPTH_WRITE = 10,
            PB_DEPTH_FUNCTION = 11,
            PB_DEPTH_BIAS = 12,
            PB_ALPHA_FUNCTION = 13,
            PB_ALPHA_VALUE = 14,
            PB_ENABLE_LIGHTING = 15,
            PB_ENABLE_COLOR_WRITE = 16,
            PB_MAX_LIGHTS = 17,
            PB_ONCE_PER_LIGHT = 18,
            PB_SUPPORTED_LIGHT_TYPE = 19,
            PB_HARDWARE_CULL = 20,
            PB_SOFTWARE_CULL = 21,
            PB_FOG_FUNCTION = 22,
            PB_FOG_COLOR = 23,
            PB_FOG_START = 24,
            PB_FOG_END = 25,
            PB_FOG_DENSITY = 26,
            NUM_PB_ENTRIES = 27
        };

        /** Reference 0. */
	    OgreMaxTextureUnits* textureUnits;

        /** Reference 1. */
        IParamBlock* paramBlock;

        /** Validity interval. */
        Interval ivalid;        
    };
}


#endif