
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "OgreMaxPassDlg.h"
#include "OgreMaxPass.h"
#include "ControlUtilities.h"
#include "PluginGlobals.h"
#include "resource.h"

using namespace OgreMax;


//Local functions--------------------------------------------------------------
static int TextureUnitIndexToCheckBoxID(int i)
{
    switch (i)
    {
        case 0: return IDC_TEXTURE_UNIT_ENABLE_CHECK1;
        case 1: return IDC_TEXTURE_UNIT_ENABLE_CHECK2;
        case 2: return IDC_TEXTURE_UNIT_ENABLE_CHECK3;
        case 3: return IDC_TEXTURE_UNIT_ENABLE_CHECK4;
    }

    return 0;
}

static int TextureUnitIndexToButtonID(int i)
{
    switch (i)
    {
        case 0: return IDC_TEXTURE_UNIT_BUTTON1;
        case 1: return IDC_TEXTURE_UNIT_BUTTON2;
        case 2: return IDC_TEXTURE_UNIT_BUTTON3;
        case 3: return IDC_TEXTURE_UNIT_BUTTON4;
    }

    return 0;
}

//Implementation---------------------------------------------------------------
OgreMaxPassDlg::OgreMaxPassDlg
    (
    HWND hWndMaterialEdit, 
    IMtlParams* materialParams, 
    OgreMaxPass* pass
    )
{
    this->hWndMaterialEdit = hWndMaterialEdit;
    this->passParams = materialParams;
    this->pass = pass;

    this->mainPanel = NULL;
    this->shaderPanel = NULL;

    this->dragAndDropManager.Init(this);

    for (int i = 0; i < OgreMaxTextureUnits::MAX_TEXTURE_UNITS; i++) 
        this->textureUnitButton[i] = NULL;

    this->ambientColorSwatch = NULL;
    this->diffuseColorSwatch = NULL;
    this->specularColorSwatch = NULL;
    this->emissiveColorSwatch = NULL;
    this->fogColorSwatch = NULL;

    this->glossinessSpin = NULL;
    this->opacitySpin = NULL;
    this->depthBiasSpin = NULL;
    this->alphaValueSpin = NULL;
    this->maxLightsSpin = NULL;
    this->fogStartSpin = NULL;
    this->fogEndSpin = NULL;
    this->fogDensitySpin = NULL;
    
    this->currentTime = 0;

    this->initializedCount = 0;
}

OgreMaxPassDlg::~OgreMaxPassDlg()
{
    //Capture some final rollup values
    this->pass->SetFlag
        (
        OgreMaxPass::MAIN_ROLLUP_OPEN, 
        this->passParams->IsRollupPanelOpen(this->mainPanel)
        );
    this->pass->SetFlag
        (
        OgreMaxPass::SHADER_ROLLUP_OPEN, 
        this->passParams->IsRollupPanelOpen(this->shaderPanel)
        );
	this->pass->rollScroll = this->passParams->GetRollupScrollPos();

    //Remove the parameter dialog
	this->pass->SetParamDlg(NULL);

    //Release the custom buttons used for the texture units
	for (int i = 0; i < OgreMaxTextureUnits::MAX_TEXTURE_UNITS; i++) 
    	ReleaseICustButton(this->textureUnitButton[i]);

    //Release color swatch controls
    ReleaseIColorSwatch(this->ambientColorSwatch);
    ReleaseIColorSwatch(this->diffuseColorSwatch);
    ReleaseIColorSwatch(this->specularColorSwatch);
    ReleaseIColorSwatch(this->emissiveColorSwatch);
    ReleaseIColorSwatch(this->fogColorSwatch);

    //Release spinner controls
    ReleaseISpinner(this->glossinessSpin);
    ReleaseISpinner(this->opacitySpin);
    ReleaseISpinner(this->depthBiasSpin);
    ReleaseISpinner(this->alphaValueSpin);
    ReleaseISpinner(this->maxLightsSpin);
    ReleaseISpinner(this->fogStartSpin);
    ReleaseISpinner(this->fogEndSpin);
    ReleaseISpinner(this->fogDensitySpin);
        
    //Remove the user data that we previously set
	SetWindowLongPtr(this->mainPanel, GWLP_USERDATA, NULL);	
    SetWindowLongPtr(this->shaderPanel, GWLP_USERDATA, NULL);	
}

Class_ID OgreMaxPassDlg::ClassID() 
{
    return OgreMaxPass::GetClassClassID();  
}

void OgreMaxPassDlg::SetThing(ReferenceTarget* refTarget)
{
    assert(refTarget->SuperClassID() == TEXMAP_CLASS_ID);
    assert(refTarget->ClassID() == OgreMaxPass::GetClassClassID());

	if (this->pass) 
		this->pass->SetParamDlg(NULL);
	
    this->pass = static_cast<OgreMaxPass*>(refTarget);
    if (this->pass)
	    this->pass->SetParamDlg(this);
	
    LoadDialog();	
}

ReferenceTarget* OgreMaxPassDlg::GetThing() 
{ 
    return this->pass; 
}

void OgreMaxPassDlg::DeleteThis() 
{ 
    delete this;  
}	

void OgreMaxPassDlg::SetTime(TimeValue t)
{
    if (t != this->currentTime) 
    {
		this->currentTime = t;
		Interval v;
        this->pass->Update(this->passParams->GetTime(), v);
		LoadDialog();
	}
}

int OgreMaxPassDlg::FindSubTexFromHWND(HWND hWnd)
{
    for (int i = 0; i < OgreMaxTextureUnits::MAX_TEXTURE_UNITS; i++) 
    {
		if (this->textureUnitButton[i]->GetHwnd() == hWnd) 
			return i;
	}	
	
	return -1;
}

void OgreMaxPassDlg::ReloadDialog()
{
    Interval v;
	this->pass->Update(this->passParams->GetTime(), v);
	LoadDialog();
}

void OgreMaxPassDlg::ActivateDlg(BOOL activate)
{
}

void OgreMaxPassDlg::LoadDialog()
{
    if (this->pass != NULL)
    {
        if (this->mainPanel != NULL)
        {
            this->ambientColorSwatch->SetColor(this->pass->GetAmbientColor());
            this->diffuseColorSwatch->SetColor(this->pass->GetDiffuseColor());
            this->specularColorSwatch->SetColor(this->pass->GetSpecularColor());
            this->glossinessSpin->SetValue(this->pass->GetGlossiness(), FALSE);
            this->opacitySpin->SetValue(this->pass->GetOpacity(), FALSE);
            this->emissiveColorSwatch->SetColor(this->pass->GetEmissiveColor());
            SendMessage(GetDlgItem(this->mainPanel, IDC_SOURCE_BLEND_COMBO), CB_SETCURSEL, this->pass->GetSourceBlend(), 0);
            SendMessage(GetDlgItem(this->mainPanel, IDC_DESTINATION_BLEND_COMBO), CB_SETCURSEL, this->pass->GetDestinationBlend(), 0);
            SendMessage(GetDlgItem(this->mainPanel, IDC_SHADING_COMBO), CB_SETCURSEL, this->pass->GetShading(), 0);
            SetCheckBox(this->mainPanel, IDC_ENABLE_DEPTH_CHECK, this->pass->GetEnableDepthCheck());
            SetCheckBox(this->mainPanel, IDC_ENABLE_DEPTH_WRITE_CHECK, this->pass->GetEnableDepthWrite());
            SendMessage(GetDlgItem(this->mainPanel, IDC_DEPTH_FUNCTION_COMBO), CB_SETCURSEL, this->pass->GetDepthFunction(), 0);
            this->depthBiasSpin->SetValue(this->pass->GetDepthBias(), FALSE);
            SendMessage(GetDlgItem(this->mainPanel, IDC_ALPHA_FUNCTION_COMBO), CB_SETCURSEL, this->pass->GetAlphaFunction(), 0);
            this->alphaValueSpin->SetValue(this->pass->GetAlphaValue(), FALSE);
            SetCheckBox(this->mainPanel, IDC_ENABLE_LIGHTING_CHECK, this->pass->GetEnableLighting());
            SetCheckBox(this->mainPanel, IDC_ENABLE_COLOR_WRITE_CHECK, this->pass->GetEnableColorWrite());
            this->maxLightsSpin->SetValue(this->pass->GetMaxLights(), FALSE);
            SetCheckBox(this->mainPanel, IDC_ONCE_PER_LIGHT_CHECK, this->pass->GetOncePerLight());
            SendMessage(GetDlgItem(this->mainPanel, IDC_LIGHT_TYPE_COMBO), CB_SETCURSEL, this->pass->GetSupportedLightType(), 0);
            SendMessage(GetDlgItem(this->mainPanel, IDC_HARDWARE_CULL_COMBO), CB_SETCURSEL, this->pass->GetHardwareCull(), 0);
            SendMessage(GetDlgItem(this->mainPanel, IDC_SOFTWARE_CULL_COMBO), CB_SETCURSEL, this->pass->GetSoftwareCull(), 0);
            SendMessage(GetDlgItem(this->mainPanel, IDC_FOG_FUNCTION_COMBO), CB_SETCURSEL, this->pass->GetFogFunction(), 0);
            this->fogColorSwatch->SetColor(this->pass->GetFogColor());            
            this->fogStartSpin->SetValue(this->pass->GetFogStart(), FALSE);
            this->fogEndSpin->SetValue(this->pass->GetFogEnd(), FALSE);
            this->fogDensitySpin->SetValue(this->pass->GetFogDensity(), FALSE);            

            for (int i = 0; i < OgreMaxTextureUnits::MAX_TEXTURE_UNITS; i++)
                UpdateTextureUnitDisplay(i);
        }

        if (this->shaderPanel != NULL)
        {
            SetCheckBox(this->shaderPanel, IDC_ENABLE_VERTEX_SHADER_CHECK, this->pass->IsVertexShaderEnabled());
            SetCheckBox(this->shaderPanel, IDC_ENABLE_PIXEL_SHADER_CHECK, this->pass->IsPixelShaderEnabled());
        }
    }
}

void OgreMaxPassDlg::BuildDialog() 
{
    this->mainPanel = 
	    this->passParams->AddRollupPage
            ( 
            PluginGlobals::GetInstanceHandle(),
		    MAKEINTRESOURCE(IDD_PASS_MAIN_ROLLUP),
		    MainPanelDlgProc, 
		    PluginGlobals::GetString(IDS_PASS_MAIN_PARAMETERS), 
		    reinterpret_cast<LPARAM>(this),
            this->pass->flags & OgreMaxPass::MAIN_ROLLUP_OPEN ? 0 : APPENDROLL_CLOSED
		    );

    this->shaderPanel =
        this->passParams->AddRollupPage
            ( 
            PluginGlobals::GetInstanceHandle(),
		    MAKEINTRESOURCE(IDD_PASS_SHADERS_ROLLUP),
		    ShaderPanelDlgProc, 
		    PluginGlobals::GetString(IDS_PASS_SHADER_PARAMETERS), 
		    reinterpret_cast<LPARAM>(this),
            this->pass->flags & OgreMaxPass::SHADER_ROLLUP_OPEN ? 0 : APPENDROLL_CLOSED
		    );

    this->passParams->SetRollupScrollPos(this->pass->rollScroll);    
}

void OgreMaxPassDlg::UpdateTextureUnitDisplay(int i) 
{
    if (this->pass != NULL)
    {
        if (this->pass->textureUnits != NULL && i != -1)
        {
            //Check or uncheck checkbox
	        SetCheckBox
                (
                this->mainPanel, 
                TextureUnitIndexToCheckBoxID(i), 
                this->pass->IsTextureUnitEnabled(i)
                );

            //Set button text
	        TSTR name = this->pass->GetTextureUnitName(i);
	        this->textureUnitButton[i]->SetText(name.data());	
        }
    }
}

void OgreMaxPassDlg::Invalidate()
{
    InvalidateRect(this->mainPanel, NULL, 0);
    InvalidateRect(this->shaderPanel, NULL, 0);    
}

BOOL OgreMaxPassDlg::MainPanelProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    int id = LOWORD(wParam);
	int code = HIWORD(wParam);
    switch (message)
    {
		case WM_INITDIALOG:
		{
			this->textureUnitButton[0] = GetICustButton(GetDlgItem(hDlg, IDC_TEXTURE_UNIT_BUTTON1));
            this->textureUnitButton[0]->SetDADMgr(&this->dragAndDropManager);
            
            this->textureUnitButton[1] = GetICustButton(GetDlgItem(hDlg, IDC_TEXTURE_UNIT_BUTTON2));
            this->textureUnitButton[1]->SetDADMgr(&this->dragAndDropManager);
            
            this->textureUnitButton[2] = GetICustButton(GetDlgItem(hDlg, IDC_TEXTURE_UNIT_BUTTON3));
            this->textureUnitButton[2]->SetDADMgr(&this->dragAndDropManager);
            
            this->textureUnitButton[3] = GetICustButton(GetDlgItem(hDlg, IDC_TEXTURE_UNIT_BUTTON4));
            this->textureUnitButton[3]->SetDADMgr(&this->dragAndDropManager);

            this->ambientColorSwatch = GetIColorSwatch(GetDlgItem(hDlg, IDC_AMBIENT_COLOR));
            this->diffuseColorSwatch = GetIColorSwatch(GetDlgItem(hDlg, IDC_DIFFUSE_COLOR));
            this->specularColorSwatch = GetIColorSwatch(GetDlgItem(hDlg, IDC_SPECULAR_COLOR));
            this->emissiveColorSwatch = GetIColorSwatch(GetDlgItem(hDlg, IDC_EMISSIVE_COLOR));
            this->fogColorSwatch = GetIColorSwatch(GetDlgItem(hDlg, IDC_FOG_COLOR));

            this->glossinessSpin = SetupFloatSpinner(hDlg, IDC_GLOSSINESS_SPIN, IDC_GLOSSINESS_EDIT, 0.0f, 100.0f, 50.0f, .1f);
            this->opacitySpin = SetupFloatSpinner(hDlg, IDC_OPACITY_SPIN, IDC_OPACITY_EDIT, 0.0f, 100.0f, 100.0f, .1f);
            this->depthBiasSpin = SetupIntSpinner(hDlg, IDC_DEPTH_BIAS_SPIN, IDC_DEPTH_BIAS_EDIT, 0, 15, 0);
            this->alphaValueSpin = SetupIntSpinner(hDlg, IDC_ALPHA_VALUE_SPIN, IDC_ALPHA_VALUE_EDIT, 0, 255, 0);
            this->maxLightsSpin = SetupIntSpinner(hDlg, IDC_MAX_LIGHTS_SPIN, IDC_MAX_LIGHTS_EDIT, 0, 8, 8);
            this->fogStartSpin = SetupFloatSpinner(hDlg, IDC_FOG_START_SPIN, IDC_FOG_START_EDIT, 0.0f, 100000.0f, 100.0f, 100.0f);
            this->fogEndSpin = SetupFloatSpinner(hDlg, IDC_FOG_END_SPIN, IDC_FOG_END_EDIT, 0.0f, 100000.0f, 100000.0f, 100.0f);
            this->fogDensitySpin = SetupFloatSpinner(hDlg, IDC_FOG_DENSITY_SPIN, IDC_FOG_DENSITY_EDIT, 0.0f, 1.0f, 0.002f, 0.01f);
                        
            ControlUtilities::FillBlendFactorComboBox(GetDlgItem(hDlg, IDC_SOURCE_BLEND_COMBO));
            ControlUtilities::FillBlendFactorComboBox(GetDlgItem(hDlg, IDC_DESTINATION_BLEND_COMBO));
            ControlUtilities::FillShadingComboBox(GetDlgItem(hDlg, IDC_SHADING_COMBO));
            ControlUtilities::FillDepthFunctionComboBox(GetDlgItem(hDlg, IDC_DEPTH_FUNCTION_COMBO));
            ControlUtilities::FillAlphaFunctionComboBox(GetDlgItem(hDlg, IDC_ALPHA_FUNCTION_COMBO));
            ControlUtilities::FillHardwareCullComboBox(GetDlgItem(hDlg, IDC_HARDWARE_CULL_COMBO));
            ControlUtilities::FillSoftwareCullComboBox(GetDlgItem(hDlg, IDC_SOFTWARE_CULL_COMBO));
            ControlUtilities::FillFogFunctionComboBox(GetDlgItem(hDlg, IDC_FOG_FUNCTION_COMBO));
            ControlUtilities::FillLightTypeComboBox(GetDlgItem(hDlg, IDC_LIGHT_TYPE_COMBO));
                        
            return TRUE;
		}			
		case WM_COMMAND: 
		{ 
            switch (id) 
            {
                case IDC_TEXTURE_UNIT_BUTTON1:
                {
                    PostMessage(this->hWndMaterialEdit, WM_TEXMAP_BUTTON, 0, (LPARAM)this->pass);
				    break;
                }
                case IDC_TEXTURE_UNIT_BUTTON2:
                {
                    PostMessage(this->hWndMaterialEdit, WM_TEXMAP_BUTTON, 1, (LPARAM)this->pass);
				    break;
                }
                case IDC_TEXTURE_UNIT_BUTTON3:
                {
                    PostMessage(this->hWndMaterialEdit, WM_TEXMAP_BUTTON, 2, (LPARAM)this->pass);
				    break;
                }
                case IDC_TEXTURE_UNIT_BUTTON4:
                {
                    PostMessage(this->hWndMaterialEdit, WM_TEXMAP_BUTTON, 3, (LPARAM)this->pass);
				    break;
                }
                case IDC_TEXTURE_UNIT_ENABLE_CHECK1:
                {
                    this->pass->EnableTextureUnit(0, GetCheckBox(hDlg, IDC_TEXTURE_UNIT_ENABLE_CHECK1));
                    break;
                }
                case IDC_TEXTURE_UNIT_ENABLE_CHECK2:
                {
                    this->pass->EnableTextureUnit(1, GetCheckBox(hDlg, IDC_TEXTURE_UNIT_ENABLE_CHECK2));
                    break;
                }
                case IDC_TEXTURE_UNIT_ENABLE_CHECK3:
                {
                    this->pass->EnableTextureUnit(2, GetCheckBox(hDlg, IDC_TEXTURE_UNIT_ENABLE_CHECK3));
                    break;
                }
                case IDC_TEXTURE_UNIT_ENABLE_CHECK4:
                {
                    this->pass->EnableTextureUnit(3, GetCheckBox(hDlg, IDC_TEXTURE_UNIT_ENABLE_CHECK4));
                    break;
                }
                case IDC_SOURCE_BLEND_COMBO:
                {
                    this->pass->SetSourceBlend(SendMessage(GetDlgItem(hDlg, IDC_SOURCE_BLEND_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_DESTINATION_BLEND_COMBO:
                {
                    this->pass->SetDestinationBlend(SendMessage(GetDlgItem(hDlg, IDC_DESTINATION_BLEND_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_SHADING_COMBO:
                {
                    this->pass->SetShading(SendMessage(GetDlgItem(hDlg, IDC_SHADING_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_DEPTH_FUNCTION_COMBO:
                {
                    this->pass->SetDepthFunction(SendMessage(GetDlgItem(hDlg, IDC_DEPTH_FUNCTION_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_ALPHA_FUNCTION_COMBO:
                {
                    this->pass->SetAlphaFunction(SendMessage(GetDlgItem(hDlg, IDC_ALPHA_FUNCTION_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_HARDWARE_CULL_COMBO:
                {
                    this->pass->SetHardwareCull(SendMessage(GetDlgItem(hDlg, IDC_HARDWARE_CULL_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_SOFTWARE_CULL_COMBO:
                {
                    this->pass->SetSoftwareCull(SendMessage(GetDlgItem(hDlg, IDC_SOFTWARE_CULL_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_FOG_FUNCTION_COMBO:
                {
                    this->pass->SetFogFunction(SendMessage(GetDlgItem(hDlg, IDC_FOG_FUNCTION_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_LIGHT_TYPE_COMBO:
                {
                    this->pass->SetSupportedLightType(SendMessage(GetDlgItem(hDlg, IDC_LIGHT_TYPE_COMBO), CB_GETCURSEL, 0, 0));                    
                    break;
                }
                case IDC_ENABLE_DEPTH_CHECK:
                {
                    this->pass->SetEnableDepthCheck(GetCheckBox(hDlg, IDC_ENABLE_DEPTH_CHECK));
                    break;
                }
                case IDC_ENABLE_DEPTH_WRITE_CHECK:
                {
                    this->pass->SetEnableDepthWrite(GetCheckBox(hDlg, IDC_ENABLE_DEPTH_WRITE_CHECK));
                    break;
                }
                case IDC_ENABLE_LIGHTING_CHECK:
                {
                    this->pass->SetEnableLighting(GetCheckBox(hDlg, IDC_ENABLE_LIGHTING_CHECK));
                    break;
                }
                case IDC_ENABLE_COLOR_WRITE_CHECK:
                {
                    this->pass->SetEnableColorWrite(GetCheckBox(hDlg, IDC_ENABLE_COLOR_WRITE_CHECK));
                    break;
                }
                case IDC_ONCE_PER_LIGHT_CHECK:
                {
                    this->pass->SetOncePerLight(GetCheckBox(hDlg, IDC_ONCE_PER_LIGHT_CHECK));
                    break;
                }
			}

			return TRUE;
        }
        case CC_COLOR_SEL:
		{
			id = LOWORD(wParam);
			SaveEditedColor(id);
            break;
		}	
		case CC_COLOR_DROP:
		{
			id = LOWORD(wParam);
			SaveEditedColor(id);
            break;
		}			
		case CC_COLOR_BUTTONDOWN:            
        {
			theHold.Begin();
			break;
        }
		case CC_COLOR_BUTTONUP:
        {
			if (HIWORD(wParam)) 
                theHold.Accept(PluginGlobals::GetString(IDS_PARAMETER_CHANGE));
			else 
                theHold.Cancel();
			
            break;
        }
		case CC_COLOR_CHANGE:
		{			
			id = LOWORD(wParam);			
            int buttonUp = HIWORD(wParam); 			

			if (buttonUp) 
                theHold.Begin();
            
            SaveEditedColor(id);
			
            if (buttonUp) 
                theHold.Accept(PluginGlobals::GetString(IDS_PARAMETER_CHANGE));

			break;
        }
        case CC_SPINNER_CHANGE: 
        {
			if (!theHold.Holding()) 
                theHold.Begin();

            SaveEditedSpinner(id);
            
			break;
        }
		case CC_SPINNER_BUTTONDOWN:            
        {
			theHold.Begin();
			break;
        }
		case WM_CUSTEDIT_ENTER:
		case CC_SPINNER_BUTTONUP: 
        {
			if (HIWORD(wParam) || message == WM_CUSTEDIT_ENTER) 
                theHold.Accept(PluginGlobals::GetString(IDS_PARAMETER_CHANGE));
			else 
				theHold.Cancel();
			
            break;
    	}
		case WM_PAINT:
        {
            if (this->initializedCount++ == PANEL_COUNT)
            	ReloadDialog();
			
            return FALSE;
        }
    }

	return FALSE;
}

BOOL OgreMaxPassDlg::ShaderPanelProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    int id = LOWORD(wParam);
	int code = HIWORD(wParam);
    switch (message)
    {
		case WM_INITDIALOG:
		{
            return TRUE;
		}			
		case WM_COMMAND: 
		{ 
            switch (id) 
            {
                case IDC_ENABLE_VERTEX_SHADER_CHECK:
                {
                    this->pass->EnableVertexShader(GetCheckBox(hDlg, IDC_ENABLE_VERTEX_SHADER_CHECK));
				    break;
                }
                case IDC_ENABLE_PIXEL_SHADER_CHECK:
                {
                    this->pass->EnablePixelShader(GetCheckBox(hDlg, IDC_ENABLE_PIXEL_SHADER_CHECK));
				    break;
                }
			}

			return TRUE;
        }
		case WM_PAINT:
        {
            if (this->initializedCount++ == PANEL_COUNT)
            	ReloadDialog();
			return FALSE;
        }
    }

	return FALSE;
}

INT_PTR CALLBACK OgreMaxPassDlg::MainPanelDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) 
{
    OgreMaxPassDlg* dialog = NULL;
	if (message == WM_INITDIALOG) 
    {
		dialog = reinterpret_cast<OgreMaxPassDlg*>(lParam);
		dialog->mainPanel = hWnd;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, lParam);
	}
	else 
    {
        dialog = reinterpret_cast<OgreMaxPassDlg*>(GetWindowLongPtr(hWnd, GWLP_USERDATA));
	    if (dialog == NULL)
			return FALSE; 
	}
	
    return dialog->MainPanelProc(hWnd, message, wParam, lParam);
}

INT_PTR CALLBACK OgreMaxPassDlg::ShaderPanelDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam)
{
    OgreMaxPassDlg* dialog = NULL;
	if (message == WM_INITDIALOG) 
    {
		dialog = reinterpret_cast<OgreMaxPassDlg*>(lParam);
		dialog->shaderPanel = hWnd;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, lParam);
	}
	else 
    {
        dialog = reinterpret_cast<OgreMaxPassDlg*>(GetWindowLongPtr(hWnd, GWLP_USERDATA));
	    if (dialog == NULL)
			return FALSE; 
	}
	
    return dialog->ShaderPanelProc(hWnd, message, wParam, lParam);
}

void OgreMaxPassDlg::SaveEditedColor(int controlID) 
{
    switch (controlID)
    {
        case IDC_AMBIENT_COLOR:
        {
            this->ambientColorSwatch->EditThis(FALSE);
            this->pass->SetAmbientColor(Color(this->ambientColorSwatch->GetColor()));
            break;
        }
        case IDC_DIFFUSE_COLOR:
        {
            this->diffuseColorSwatch->EditThis(FALSE);
            this->pass->SetDiffuseColor(Color(this->diffuseColorSwatch->GetColor()));
            break;
        }
        case IDC_SPECULAR_COLOR:
        {
            this->specularColorSwatch->EditThis(FALSE);
            this->pass->SetSpecularColor(Color(this->specularColorSwatch->GetColor()));
            break;
        }
        case IDC_EMISSIVE_COLOR:
        {
            this->emissiveColorSwatch->EditThis(FALSE);
            this->pass->SetEmissiveColor(Color(this->emissiveColorSwatch->GetColor()));
            break;
        }
        case IDC_FOG_COLOR:
        {
            this->fogColorSwatch->EditThis(FALSE);
            this->pass->SetFogColor(Color(this->fogColorSwatch->GetColor()));
            break;
        }
    }	
}

void OgreMaxPassDlg::SaveEditedSpinner(int controlID)
{
    switch (controlID) 
    {
		case IDC_GLOSSINESS_SPIN: 
        {
            this->pass->SetGlossiness(this->glossinessSpin->GetFVal());
            break;
        }
		case IDC_OPACITY_SPIN: 
        {
            this->pass->SetOpacity(this->opacitySpin->GetFVal());
            break;
        }
		case IDC_DEPTH_BIAS_SPIN: 
        {
            this->pass->SetDepthBias(this->depthBiasSpin->GetIVal());            
            break;
        }
		case IDC_ALPHA_VALUE_SPIN: 
        {
            this->pass->SetAlphaValue(this->alphaValueSpin->GetIVal());
            break;
        }
		case IDC_MAX_LIGHTS_SPIN: 
        {
            this->pass->SetMaxLights(this->maxLightsSpin->GetIVal());
            break;
        }
		case IDC_FOG_START_SPIN: 
        {
            this->pass->SetFogStart(this->fogStartSpin->GetFVal());
            break;
        }
		case IDC_FOG_END_SPIN: 
        {
			this->pass->SetFogEnd(this->fogEndSpin->GetFVal());
			break;
        }
        case IDC_FOG_DENSITY_SPIN:
        {
            this->pass->SetFogDensity(this->fogDensitySpin->GetFVal());
            break;
        }				
	}
}