#ifndef OgreMax_OgreMaxPassDlg_INCLUDED
#define OgreMax_OgreMaxPassDlg_INCLUDED


//Includes---------------------------------------------------------------------
#include "OgreMaxTextureUnits.h"


//Classes----------------------------------------------------------------------
namespace OgreMax
{
    class OgreMaxPass;

    /**
     * Manages the rollup(s) for an OgreMaxTextureUnit in the 3DS Max material editor.
     */
    class OgreMaxPassDlg : public ParamDlg 
    {
    public:
        OgreMaxPassDlg(HWND hWndMaterialEdit, IMtlParams* passParams, OgreMaxPass* pass); 
	    ~OgreMaxPassDlg();

	    Class_ID ClassID();
        void SetThing(ReferenceTarget* refTarget);
	    ReferenceTarget* GetThing();
	    void DeleteThis();
	    void SetTime(TimeValue t);
	    int FindSubTexFromHWND(HWND hWnd);
        
        /** Updates material and puts parameters into dialog. */
        void ReloadDialog();

        /** Called to notify the dialog it's being activated/deactiviated. */
        void ActivateDlg(BOOL activate);

        /**
         * Puts parameters into dialog.
         */
        void LoadDialog();

        /**
         * Loads the dialog resources and puts them into the dialog.
         */
        void BuildDialog();

        /**
         * Updates the information for the specified texture unit in the UI.
         * @param textureUnitIndex [in] - The index of the texture unit to update in the UI.
         */
        void UpdateTextureUnitDisplay(int textureUnitIndex);

        /**
         * Invalidates all the rollup panels.
         */
        void Invalidate();

    private:
        BOOL MainPanelProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
        BOOL ShaderPanelProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

        static INT_PTR CALLBACK MainPanelDlgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);
        static INT_PTR CALLBACK ShaderPanelDlgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam);

        void SaveEditedColor(int controlID);
        void SaveEditedSpinner(int controlID);

    private:
        HWND hWndMaterialEdit;
        IMtlParams* passParams;
        OgreMaxPass* pass;

        enum {PANEL_COUNT = 2};
        HWND mainPanel;
        HWND shaderPanel;

        TexDADMgr dragAndDropManager;	
        ICustButton* textureUnitButton[OgreMaxTextureUnits::MAX_TEXTURE_UNITS];	    

        IColorSwatch* ambientColorSwatch;
        IColorSwatch* diffuseColorSwatch;
        IColorSwatch* specularColorSwatch;
        IColorSwatch* emissiveColorSwatch;
        IColorSwatch* fogColorSwatch;

        ISpinnerControl* glossinessSpin;
        ISpinnerControl* opacitySpin;
        ISpinnerControl* depthBiasSpin;
        ISpinnerControl* alphaValueSpin;
        ISpinnerControl* maxLightsSpin;
        ISpinnerControl* fogStartSpin;
        ISpinnerControl* fogEndSpin;
        ISpinnerControl* fogDensitySpin;

        TimeValue currentTime;
        
        size_t initializedCount;
    };
}


#endif
