
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "OgreMaxPasses.h"
#include "OgreMaxGlobalUtilities.h"
#include "PluginGlobals.h"
#include "resource.h"

using namespace OgreMax;


//Local classes----------------------------------------------------------------
class OgreMaxPassesClassDesc : public ClassDesc 
{
public:
	int IsPublic()              {return 0;}
	void* Create(BOOL loading)  {return new OgreMaxPasses(NULL);}
    const TCHAR* ClassName()    {return PluginGlobals::GetString(IDS_PASSES_CLASS_NAME);}
	SClass_ID SuperClassID()    {return TEXMAP_CONTAINER_CLASS_ID;}
    Class_ID ClassID()          {return OgreMaxPasses::GetClassClassID();}
    const TCHAR* Category()     {return PluginGlobals::GetString(IDS_PLUGINS_CATEGORY);}

public:
    static OgreMaxPassesClassDesc instance;
};
OgreMaxPassesClassDesc OgreMaxPassesClassDesc::instance;


//Implementation---------------------------------------------------------------

//PassSlot implementation-----------------------------
PassSlot::PassSlot() 
{ 
    this->map = NULL;
    this->on = TRUE;
}

void PassSlot::Update(TimeValue t, Interval& ivalid) 
{
	if (IsOn() && HasMap()) 
		this->map->Update(t, ivalid);
}

BOOL PassSlot::IsOn() 
{ 
    return this->on;
}

void PassSlot::SetOn(BOOL on)
{
    this->on = on;
}

BOOL PassSlot::HasMap() 
{
    return this->map != 0 ? TRUE : FALSE; 
}

Texmap* PassSlot::GetMap()
{
    return this->map;
}

//OgreMaxPasses implementation-------------------
OgreMaxPasses::OgreMaxPasses() 
{
    this->client = NULL;
}
					
OgreMaxPasses::OgreMaxPasses(MtlBase* mtlBase) 
{
	this->client = mtlBase;
}

void OgreMaxPasses::SetName(long i, TSTR& name)
{
    this->passes[i].name = name;
}

TSTR& OgreMaxPasses::GetName(long i)
{
    return this->passes[i].name; 
} 

void OgreMaxPasses::SetClientPtr(MtlBase* mtlBase)
{
    this->client = mtlBase;
}

PassSlot& OgreMaxPasses::operator[](int i)
{
    return this->passes[i];
}

PassSlot* OgreMaxPasses::GetPass(int i)
{
    return &this->passes[i];
}

ClassDesc* OgreMaxPasses::GetClassDesc() 
{ 
    return &OgreMaxPassesClassDesc::instance;  
}

Class_ID OgreMaxPasses::GetClassClassID() 
{
    return Class_ID(0x197bc63d, 0x99731d81);
}

Class_ID OgreMaxPasses::ClassID() 
{ 
    return GetClassClassID(); 
}

void OgreMaxPasses::DeleteThis()
{
    delete this;
}

RefTargetHandle OgreMaxPasses::Clone(RemapDir& remap) 
{
	OgreMaxPasses* newTexmaps = new OgreMaxPasses(NULL);
	
    for (int i = 0; i< MAX_PASSES; i++) 
    {
		newTexmaps->passes[i].on = this->passes[i].on;
		newTexmaps->passes[i].map = NULL;
		if (this->passes[i].map) 
			newTexmaps->ReplaceReference(i, remap.CloneRef(this->passes[i].map));
	}
	
    BaseClone(this, newTexmaps, remap);
	
    return newTexmaps;
}

RefResult OgreMaxPasses::NotifyRefChanged
    (
    Interval changeInt, 
    RefTargetHandle hTarget, 
    PartID& partID, 
    RefMessage message 
    ) 
{
	switch (message) 
    {
		case REFMSG_GET_PARAM_DIM: 
        {
			GetParamDim* gpd = (GetParamDim*)partID;
			gpd->dim = defaultDim; 
			break;
		}
		case REFMSG_GET_PARAM_NAME: 
        {
			GetParamName* gpn = (GetParamName*)partID;
			return REF_STOP; 
		}
	}
	
    return REF_SUCCEED;
}

void OgreMaxPasses::RescaleWorldUnits(float f) 
{
	if (TestAFlag(A_WORK1))
		return;
	SetAFlag(A_WORK1);

	for (int i = 0; i < NumRefs(); i++) 
    {
        ReferenceMaker* ref = GetReference(i);
		if (ref != NULL) 
            ref->RescaleWorldUnits(f);
	}		
}

BOOL OgreMaxPasses::AssignController(Animatable* control, int subAnim) 
{
	ReplaceReference(SubNumToRefNum(subAnim), static_cast<ReferenceTarget*>(control));
	return TRUE;
}

int OgreMaxPasses::NumSubs()
{ 
    //Each technique has two subanims.
    //One is for the LOD distance, the other for the technique map itself
    return MAX_PASSES; 
}  

Animatable* OgreMaxPasses::SubAnim(int i) 
{
	return this->passes[i].map;    
}

TSTR OgreMaxPasses::SubAnimName(int i) 
{
	return client->GetSubTexmapTVName(i);    
}

int OgreMaxPasses::SubNumToRefNum(int subNum)
{
    //There is no mapping required
    return subNum;
}

int OgreMaxPasses::NumRefs() 
{ 
    return NumSubs();
}

RefTargetHandle OgreMaxPasses::GetReference(int i) 
{
	return this->passes[i].map;    
}

void OgreMaxPasses::SetReference(int i, RefTargetHandle refTarget) 
{
	this->passes[i].map = static_cast<Texmap*>(refTarget);    
}

SvGraphNodeReference OgreMaxPasses::SvTraverseAnimGraph
    (
    IGraphObjectManager* gom, 
    Animatable* owner, 
    int id, 
    DWORD flags
    )
{
	//If not testing for maps, we're done already
	if (!gom->TestFilter(SV_FILTER_MAPS))
		return SvGraphNodeReference();

    //Figure out how many slots are needed for the schematic view
    int usedSlotCount = 0;
    if (this->passes != NULL)
    {
	    for (int i = 0; i < MAX_PASSES; i++)
        {
		    if (this->passes[i].GetMap() != NULL)
			    usedSlotCount++;
        }
    }
    
    //Return a "used" result or an empty one, depending on the count
	if (usedSlotCount)
		return SvStdTraverseAnimGraph(gom, owner, id, flags);
	else
		return SvGraphNodeReference();
}

#define PASSES_ON_OFF_CHUNK 0x5003
#define PASSES_VERSION_CHUNK 0x5300

#define PASSES_VERSION 1

IOResult OgreMaxPasses::Save(ISave* isave) 
{ 
    int i;
    ULONG byteCount;

    //Save the version
    isave->BeginChunk(PASSES_VERSION_CHUNK);
	int version = PASSES_VERSION;
	isave->Write(&version, sizeof(version), &byteCount);			
	isave->EndChunk();

    //Save the on/off flags
    ULONG onFlags = 0;    
	for (i = 0; i < MAX_PASSES; i++)
    {
		if (this->passes[i].on) 
            onFlags |= (1<<i);
    }
    isave->BeginChunk(PASSES_ON_OFF_CHUNK);
	isave->Write(&onFlags, sizeof(onFlags), &byteCount);			
	isave->EndChunk();

    return IO_OK;
}

IOResult OgreMaxPasses::Load(ILoad* iload) 
{ 
	ULONG byteCount;
	int id;
    int version = 0;
	IOResult result;

	while ((result = iload->OpenChunk()) == IO_OK) 
    {
		switch (id = iload->CurChunkID())  
        {
            case PASSES_VERSION_CHUNK:
            {
				result = iload->Read(&version, sizeof(version), &byteCount);
				break;
            }
			case PASSES_ON_OFF_CHUNK:
			{
			    ULONG onFlags;
			    result = iload->Read(&onFlags, sizeof(onFlags), &byteCount);
			    for (int i = 0; i < MAX_PASSES; i++) 
			        this->passes[i].on = (onFlags & (1<<i)) ? TRUE : FALSE;

                break;
			}			
		}

		iload->CloseChunk();
		if (result != IO_OK) 
			return result;
	}

	return IO_OK;                                                      
}

