//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "PluginGlobals.h"
#include "OgreMaxRenderTexture.h"
#include "OgreMaxRenderTextureDlg.h"
#include "resource.h"

using namespace OgreMax;


//Local classes-----------------------------------------------------------------
class OgreMaxRenderTextureClassDesc : public ClassDesc
{
public:
	int IsPublic()              {return 1;}
	void* Create(BOOL loading)  {return new OgreMaxRenderTexture(loading);}
	const TCHAR* ClassName()    {return PluginGlobals::GetString(IDS_RENDER_TEXTURE_CLASS_NAME);}
	SClass_ID SuperClassID()    {return TEXMAP_CLASS_ID;}
    Class_ID ClassID()          {return OgreMaxRenderTexture::GetClassClassID();}
	const TCHAR* Category()     {return PluginGlobals::GetString(IDS_PLUGINS_CATEGORY);}

public:
    static OgreMaxRenderTextureClassDesc instance;
};
OgreMaxRenderTextureClassDesc OgreMaxRenderTextureClassDesc::instance;


//Implementation---------------------------------------------------------------
OgreMaxRenderTexture::OgreMaxRenderTexture(BOOL loading)
{
    this->paramDlg = NULL;

    this->flags = ROLLUP_OPEN;
	this->rollScroll = 0;

    //this->renderTexture = NULL;
    
    if (!loading)
		Reset();
}

AColor OgreMaxRenderTexture::EvalColor(ShadeContext& sc)
{
    return AColor(0, 0, 0);
}

Point3 OgreMaxRenderTexture::EvalNormalPerturb(ShadeContext& sc)
{
    return Point3(0, 0, 0);
}

RefResult OgreMaxRenderTexture::NotifyRefChanged
    (
    Interval changeInt, 
    RefTargetHandle hTarget, 
    PartID& partID, 
    RefMessage message
    )
{
    return REF_SUCCEED;
}

void OgreMaxRenderTexture::Update(TimeValue t, Interval& valid)
{
    Point3 p;
	if (!this->ivalid.InInterval(t)) 
    {
		this->ivalid.SetInfinite();

        //if (this->renderTexture != NULL)
            //this->renderTexture->Update(t, this->ivalid);
	}
	valid &= ivalid;
}

RefTargetHandle OgreMaxRenderTexture::Clone(RemapDir& remap)
{
    OgreMaxRenderTexture* newTechnique = new OgreMaxRenderTexture();

    //Copy superclass stuff
    *static_cast<MtlBase*>(newTechnique) = *this;
	
    //Copy everything else
	//newTechnique->ReplaceReference(RENDER_TEXTURE_REF, remap.CloneRef(this->renderTexture));	
	newTechnique->flags = flags;
	
    BaseClone(this, newTechnique, remap);
	
    return newTechnique;
}

void OgreMaxRenderTexture::Reset()
{
    SuspendSetKeyMode();

    this->ivalid.SetEmpty();

    this->flags = ROLLUP_OPEN;
	
    //ReplaceReference(RENDER_TEXTURE_REF, NULL);
		
	ResumeSetKeyMode();
}

Interval OgreMaxRenderTexture::Validity(TimeValue t)
{
    Interval v;
	Update(t, v);
	return this->ivalid;
}

ParamDlg* OgreMaxRenderTexture::CreateParamDlg(HWND hWndMaterialEdit, IMtlParams* materialParams)
{
    Interval v;
	Update(materialParams->GetTime(), v);
	OgreMaxRenderTextureDlg* dlg = new OgreMaxRenderTextureDlg(hWndMaterialEdit, materialParams, this);    
	dlg->BuildDialog();
	dlg->LoadDialog();	 
	SetParamDlg(dlg);
	return dlg;
}

void OgreMaxRenderTexture::SetParamDlg(OgreMaxRenderTextureDlg* paramDlg)
{
    this->paramDlg = paramDlg;
}

ClassDesc* OgreMaxRenderTexture::GetClassDesc()
{
    return &OgreMaxRenderTextureClassDesc::instance;
}

Class_ID OgreMaxRenderTexture::GetClassClassID()
{
    return Class_ID(0xd0627ab5, 0xc6964942);
}

Class_ID OgreMaxRenderTexture::ClassID()
{
    return GetClassClassID();
}

SClass_ID OgreMaxRenderTexture::SuperClassID()
{ 
    return TEXMAP_CLASS_ID; 
}

void OgreMaxRenderTexture::GetClassName(TSTR& s)
{
    s = PluginGlobals::GetString(IDS_RENDER_TEXTURE_CLASS_NAME); 
}  

void OgreMaxRenderTexture::DeleteThis() 
{
	delete this;
}

TSTR OgreMaxRenderTexture::SubAnimName(int i) 
{ 
    TSTR name;

    /*switch (i)
    {
        case RENDER_TEXTURE_REF:
        {
            name = PluginGlobals::GetString(IDS_RENDER_TO_TEXTURE);
            break;
        }        
    }*/

    return name;
}		

int OgreMaxRenderTexture::NumSubs()
{
    return NUM_REFS; 
}  

Animatable* OgreMaxRenderTexture::SubAnim(int i) 
{
    /*
	switch (i)
    {
		case RENDER_TEXTURE_REF: return this->renderTexture;
		default: 
        {
            assert(0); 
            return NULL;
        }
	}
    */

    return NULL;
}

int OgreMaxRenderTexture::NumRefs()
{
    return NUM_REFS; 
}

RefTargetHandle OgreMaxRenderTexture::GetReference(int i) 
{
	/*switch (i) 
    {
		case RENDER_TEXTURE_REF: return this->renderTexture;
		default: 
        {
            assert(0); 
            return NULL;
        }
	}
    */
    return NULL;
}

void OgreMaxRenderTexture::SetReference(int i, RefTargetHandle refTarget) 
{
    /*
	switch (i) 
    {
		case RENDER_TEXTURE_REF:	
        {
            this->renderTexture = reinterpret_cast<Texmap*>(refTarget);
            //if (this->renderTexture != NULL)
              //  this->renderTexture->SetClientPtr(this);
            
            break;
        }
		default: assert(0);
	}
    */
}

ULONG OgreMaxRenderTexture::GetFlags()
{
    return this->flags;
}

void OgreMaxRenderTexture::SetFlag(ULONG f, ULONG value) 
{
	if (value) 
    {
        //Add to the flags
        this->flags |= f; 
    }
	else 
    {
        //Remove the flags
        this->flags &= ~f;
    }
}

#define RENDER_TEXTURE_HEADER_CHUNK 0x4000
#define RENDER_TEXTURE_FLAGS_CHUNK 0x5000
#define RENDER_TEXTURE_VERSION_CHUNK 0x5300

#define RENDER_TEXTURE_VERSION 1

IOResult OgreMaxRenderTexture::Save(ISave* isave)
{
    ULONG byteCount;
    IOResult result;

	isave->BeginChunk(RENDER_TEXTURE_HEADER_CHUNK);
	result = MtlBase::Save(isave);
	if (result != IO_OK) 
        return result;
	isave->EndChunk();

    isave->BeginChunk(RENDER_TEXTURE_VERSION_CHUNK);
	int version = RENDER_TEXTURE_VERSION;
	isave->Write(&version, sizeof(version), &byteCount);			
	isave->EndChunk();

	isave->BeginChunk(RENDER_TEXTURE_FLAGS_CHUNK);
	isave->Write(&this->flags, sizeof(this->flags), &byteCount);			
	isave->EndChunk();

	return IO_OK;
}

IOResult OgreMaxRenderTexture::Load(ILoad* iload)
{
    ULONG byteCount;
	int id;
	int version = 0;
	IOResult result;

	while ((result = iload->OpenChunk()) == IO_OK) 
    {
		switch (id = iload->CurChunkID())  
        {
			case RENDER_TEXTURE_HEADER_CHUNK:
            {
				result = MtlBase::Load(iload);
				this->ivalid.SetEmpty();
				break;
            }
            case RENDER_TEXTURE_VERSION_CHUNK:
            {
				result = iload->Read(&version, sizeof(version), &byteCount);
				break;
            }
			case RENDER_TEXTURE_FLAGS_CHUNK:
            {
				result = iload->Read(&this->flags, sizeof(this->flags), &byteCount);
				break;
            }
		}
		iload->CloseChunk();
		if (result != IO_OK) 
			return result;
	}
	
	return IO_OK;
}
