
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "OgreMaxRenderTextureDlg.h"
#include "OgreMaxRenderTexture.h"
#include "PluginGlobals.h"
#include "resource.h"

using namespace OgreMax;


//Local functions--------------------------------------------------------------


//Implementation---------------------------------------------------------------
OgreMaxRenderTextureDlg::OgreMaxRenderTextureDlg
    (
    HWND hWndMaterialEdit, 
    IMtlParams* materialParams, 
    OgreMaxRenderTexture* renderTexture
    )
{
    this->hWndMaterialEdit = hWndMaterialEdit;
    this->renderTextureParams = materialParams;
    this->renderTexture = renderTexture;

    this->mainPanel = NULL;
    
    this->dragAndDropManager.Init(this);

    this->currentTime = 0;

    this->initialized = false;
}

OgreMaxRenderTextureDlg::~OgreMaxRenderTextureDlg()
{
    //Capture some final rollup values
    this->renderTexture->SetFlag
        (
        OgreMaxRenderTexture::ROLLUP_OPEN, 
        this->renderTextureParams->IsRollupPanelOpen(this->mainPanel)
        );
	this->renderTexture->rollScroll = this->renderTextureParams->GetRollupScrollPos();

    //Remove the parameter dialog
	this->renderTexture->SetParamDlg(NULL);

    //Remove the user data that we previously set
	SetWindowLongPtr(this->mainPanel, GWLP_USERDATA, NULL);	
}

Class_ID OgreMaxRenderTextureDlg::ClassID() 
{
    return OgreMaxRenderTexture::GetClassClassID();  
}

void OgreMaxRenderTextureDlg::SetThing(ReferenceTarget* refTarget)
{
    assert(refTarget->SuperClassID() == TEXMAP_CLASS_ID);
    assert(refTarget->ClassID() == OgreMaxRenderTexture::GetClassClassID());

	if (this->renderTexture) 
		this->renderTexture->SetParamDlg(NULL);
	
    this->renderTexture = static_cast<OgreMaxRenderTexture*>(refTarget);
    if (this->renderTexture)
	    this->renderTexture->SetParamDlg(this);
	
    LoadDialog();	
}

ReferenceTarget* OgreMaxRenderTextureDlg::GetThing() 
{ 
    return this->renderTexture; 
}

void OgreMaxRenderTextureDlg::DeleteThis() 
{ 
    delete this;  
}	

void OgreMaxRenderTextureDlg::SetTime(TimeValue t)
{
    if (t != this->currentTime) 
    {
		this->currentTime = t;
		Interval v;
        this->renderTexture->Update(this->renderTextureParams->GetTime(), v);
		LoadDialog();
	}
}

int OgreMaxRenderTextureDlg::FindSubTexFromHWND(HWND hWnd)
{
    /*for (int i = 0; i < OgreMaxRenderTextures::MAX_TEXTURE_UNITS; i++) 
    {
		if (this->renderTextureButton[i]->GetHwnd() == hWnd) 
			return i;
	}	*/
	
	return -1;
}

void OgreMaxRenderTextureDlg::ReloadDialog()
{
    Interval v;
	this->renderTexture->Update(this->renderTextureParams->GetTime(), v);
	LoadDialog();
}

void OgreMaxRenderTextureDlg::ActivateDlg(BOOL activate)
{
}

void OgreMaxRenderTextureDlg::LoadDialog()
{
    if (this->renderTexture != NULL && this->mainPanel != NULL)
    {
		Interval v;

        ULONG flags = this->renderTexture->GetFlags();
	}
}

void OgreMaxRenderTextureDlg::BuildDialog() 
{
    this->mainPanel = 
	    this->renderTextureParams->AddRollupPage
            ( 
            PluginGlobals::GetInstanceHandle(),
		    MAKEINTRESOURCE(IDD_RENDER_TEXTURE_MAIN_ROLLUP),
		    MainPanelDlgProc, 
		    PluginGlobals::GetString(IDS_RENDER_TEXTURE_PARAMETERS), 
		    reinterpret_cast<LPARAM>(this),
            this->renderTexture->flags & OgreMaxRenderTexture::ROLLUP_OPEN ? 0 : APPENDROLL_CLOSED
		    );

    this->renderTextureParams->SetRollupScrollPos(this->renderTexture->rollScroll);    
}

BOOL OgreMaxRenderTextureDlg::MainPanelProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    int id = LOWORD(wParam);
	int code = HIWORD(wParam);
    switch (message)
    {
		case WM_INITDIALOG:
		{
			return TRUE;
		}			
		case WM_COMMAND: 
		{ 
            /*
            switch (id) 
            {
                case IDC_TEXTURE_UNIT_ENABLE_CHECK4:
                {
                    this->renderTexture->EnableTextureUnit(3, GetCheckBox(hDlg, IDC_TEXTURE_UNIT_ENABLE_CHECK4));
                    break;
                }                
			}
            */

			return TRUE;
        }
		case WM_PAINT:
        {
            if (!this->initialized)
            {
				this->initialized = true;
				ReloadDialog();
			}

			return FALSE;
        }
    }

	return FALSE;
}

INT_PTR CALLBACK OgreMaxRenderTextureDlg::MainPanelDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) 
{
    OgreMaxRenderTextureDlg* dialog = NULL;
	if (message == WM_INITDIALOG) 
    {
		dialog = reinterpret_cast<OgreMaxRenderTextureDlg*>(lParam);
		dialog->mainPanel = hWnd;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, lParam);
	}
	else 
    {
        dialog = reinterpret_cast<OgreMaxRenderTextureDlg*>(GetWindowLongPtr(hWnd, GWLP_USERDATA));
	    if (dialog == NULL)
			return FALSE; 
	}
	
    return dialog->MainPanelProc(hWnd, message, wParam, lParam);
}
