//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "PluginGlobals.h"
#include "OgreMaxTechnique.h"
#include "OgreMaxTechniqueDlg.h"
#include "resource.h"

using namespace OgreMax;


//Local classes-----------------------------------------------------------------
class OgreMaxTechniqueClassDesc : public ClassDesc 
{
public:
	int IsPublic()              {return 1;}
	void* Create(BOOL loading)  {return new OgreMaxTechnique(loading);}
	const TCHAR* ClassName()    {return PluginGlobals::GetString(IDS_TECHNIQUE_CLASS_NAME);}
	SClass_ID SuperClassID()    {return TEXMAP_CLASS_ID;}
    Class_ID ClassID()          {return OgreMaxTechnique::GetClassClassID();}
	const TCHAR* Category()     {return PluginGlobals::GetString(IDS_PLUGINS_CATEGORY);}

public:
    static OgreMaxTechniqueClassDesc instance;
};
OgreMaxTechniqueClassDesc OgreMaxTechniqueClassDesc::instance;


//Implementation---------------------------------------------------------------
OgreMaxTechnique::OgreMaxTechnique(BOOL loading)
{
    this->paramDlg = NULL;

    this->flags = ROLLUP_OPEN;
	this->rollScroll = 0;

    this->passes = NULL;
    
    if (!loading)
		Reset();
}

AColor OgreMaxTechnique::EvalColor(ShadeContext& sc)
{
    return AColor(0, 0, 0);
}

Point3 OgreMaxTechnique::EvalNormalPerturb(ShadeContext& sc)
{
    return Point3(0, 0, 0);
}

RefResult OgreMaxTechnique::NotifyRefChanged
    (
    Interval changeInt, 
    RefTargetHandle hTarget, 
    PartID& partID, 
    RefMessage message
    )
{
    return REF_SUCCEED;
}

void OgreMaxTechnique::Update(TimeValue t, Interval& valid)
{
    Point3 p;
	if (!this->ivalid.InInterval(t)) 
    {
		this->ivalid.SetInfinite();

        if (this->passes != NULL)
        {
    	    for (int i = 0; i < OgreMaxPasses::MAX_PASSES; i++) 	
            {
                if (GetPass(i)->IsOn())
				    GetPass(i)->Update(t, this->ivalid);
		    }
        }
	}
	valid &= ivalid;
}

RefTargetHandle OgreMaxTechnique::Clone(RemapDir& remap)
{
    OgreMaxTechnique* newTechnique = new OgreMaxTechnique();

    //Copy superclass stuff
    *static_cast<MtlBase*>(newTechnique) = *this;
	
    //Copy everything else
	newTechnique->ReplaceReference(PASSES_REF, remap.CloneRef(this->passes));	
	newTechnique->flags = flags;
	
    BaseClone(this, newTechnique, remap);
	
    return newTechnique;
}

void OgreMaxTechnique::Reset()
{
    SuspendSetKeyMode();

    this->ivalid.SetEmpty();

    this->flags = ROLLUP_OPEN;
	
    ReplaceReference(PASSES_REF, new OgreMaxPasses(this));	
		
	ResumeSetKeyMode();
}

Interval OgreMaxTechnique::Validity(TimeValue t)
{
    Interval v;
	Update(t, v);
	return this->ivalid;
}

ParamDlg* OgreMaxTechnique::CreateParamDlg(HWND hWndMaterialEdit, IMtlParams* materialParams)
{
    Interval v;
	Update(materialParams->GetTime(), v);
	OgreMaxTechniqueDlg* dlg = new OgreMaxTechniqueDlg(hWndMaterialEdit, materialParams, this);    
	dlg->BuildDialog();
	dlg->LoadDialog();	 
	SetParamDlg(dlg);
	return dlg;
}

void OgreMaxTechnique::SetParamDlg(OgreMaxTechniqueDlg* paramDlg)
{
    this->paramDlg = paramDlg;
}

int OgreMaxTechnique::NumSubTexmaps() 
{ 
    return OgreMaxPasses::MAX_PASSES; 
}

Texmap* OgreMaxTechnique::GetSubTexmap(int i)
{
    if (this->passes != NULL)
        return GetPass(i)->GetMap();
    else
        return NULL;
}

int OgreMaxTechnique::MapSlotType(int i) 
{ 
    return MAPSLOT_TEXTURE; 
}

void OgreMaxTechnique::SetSubTexmap(int i, Texmap* m)
{
    assert(i < OgreMaxPasses::MAX_PASSES);
	
    if (this->passes != NULL)
        this->passes->ReplaceReference(i, m);	    
    
	if (this->paramDlg)
		this->paramDlg->UpdatePassDisplay(i);
}

TSTR OgreMaxTechnique::GetSubTexmapSlotName(int i)
{
    return PluginGlobals::GetString(IDS_TECHNIQUE_SLOT_NAME);
}

int OgreMaxTechnique::SubTexmapOn(int i) 
{ 
    return this->passes->GetPass(i)->IsOn();
} 

ClassDesc* OgreMaxTechnique::GetClassDesc()
{
    return &OgreMaxTechniqueClassDesc::instance;
}

Class_ID OgreMaxTechnique::GetClassClassID()
{
    return Class_ID(0x6ab4ebd5, 0xc6afce9b);
}

Class_ID OgreMaxTechnique::ClassID()
{
    return GetClassClassID();
}

SClass_ID OgreMaxTechnique::SuperClassID()
{ 
    return TEXMAP_CLASS_ID; 
}

void OgreMaxTechnique::GetClassName(TSTR& s)
{
    s = PluginGlobals::GetString(IDS_TECHNIQUE_CLASS_NAME); 
}  

void OgreMaxTechnique::DeleteThis() 
{
	delete this;
}

TSTR OgreMaxTechnique::SubAnimName(int i) 
{ 
    TSTR name;

    switch (i)
    {
        case PASSES_REF:
        {
            name = PluginGlobals::GetString(IDS_PASSES); 
            break;
        }        
    }

    return name;
}		

int OgreMaxTechnique::NumSubs()
{
    return NUM_REFS; 
}  

Animatable* OgreMaxTechnique::SubAnim(int i) 
{
	switch (i) 
    {
		case PASSES_REF: return this->passes;
		default: 
        {
            assert(0); 
            return NULL;
        }
	}
}

int OgreMaxTechnique::NumRefs()
{
    return NUM_REFS; 
}

RefTargetHandle OgreMaxTechnique::GetReference(int i) 
{
	switch (i) 
    {
		case PASSES_REF: return this->passes;
		default: 
        {
            assert(0); 
            return NULL;
        }
	}
}

void OgreMaxTechnique::SetReference(int i, RefTargetHandle refTarget) 
{
	switch (i) 
    {
		case PASSES_REF:	
        {
            this->passes = reinterpret_cast<OgreMaxPasses*>(refTarget);
            if (this->passes != NULL)
                this->passes->SetClientPtr(this);
            
            break;
        }
		default: assert(0);
	}
}

ULONG OgreMaxTechnique::GetFlags()
{
    return this->flags;
}

void OgreMaxTechnique::SetFlag(ULONG f, ULONG value) 
{
	if (value) 
    {
        //Add to the flags
        this->flags |= f; 
    }
	else 
    {
        //Remove the flags
        this->flags &= ~f;
    }
}

BOOL OgreMaxTechnique::IsPassEnabled(int index)
{
    if (this->passes != NULL)
        return GetPass(index)->IsOn();
    else
        return FALSE;
}

void OgreMaxTechnique::EnablePass(int index, ULONG enable)
{
    if (this->passes != NULL)
        GetPass(index)->SetOn(enable);
}

TSTR OgreMaxTechnique::GetPassName(int index)
{
    TSTR name;
    if (this->passes != NULL && GetPass(index)->GetMap() != NULL)
    {
        Texmap* texmap = this->passes->GetPass(index)->GetMap();
        name = texmap->GetName();
    }
    else
        name = PluginGlobals::GetString(IDS_NONE);
    return name;
}

PassSlot* OgreMaxTechnique::GetPass(int index)
{
    assert(this->passes);
    return this->passes->GetPass(index);
}

#define TECHNIQUE_HEADER_CHUNK 0x4000
#define TECHNIQUE_FLAGS_CHUNK 0x5000
#define TECHNIQUE_VERSION_CHUNK 0x5300

#define TECHNIQUE_VERSION 1

IOResult OgreMaxTechnique::Save(ISave* isave)
{
    ULONG byteCount;
    IOResult result;

	isave->BeginChunk(TECHNIQUE_HEADER_CHUNK);
	result = MtlBase::Save(isave);
	if (result != IO_OK) 
        return result;
	isave->EndChunk();

    isave->BeginChunk(TECHNIQUE_VERSION_CHUNK);
	int version = TECHNIQUE_VERSION;
	isave->Write(&version, sizeof(version), &byteCount);			
	isave->EndChunk();

	isave->BeginChunk(TECHNIQUE_FLAGS_CHUNK);
	isave->Write(&this->flags, sizeof(this->flags), &byteCount);			
	isave->EndChunk();

	return IO_OK;
}

IOResult OgreMaxTechnique::Load(ILoad* iload)
{
    ULONG byteCount;
	int id;
	int version = 0;
	IOResult result;

	while ((result = iload->OpenChunk()) == IO_OK) 
    {
		switch (id = iload->CurChunkID())  
        {
			case TECHNIQUE_HEADER_CHUNK:
            {
				result = MtlBase::Load(iload);
				this->ivalid.SetEmpty();
				break;
            }
            case TECHNIQUE_VERSION_CHUNK:
            {
				result = iload->Read(&version, sizeof(version), &byteCount);
				break;
            }
			case TECHNIQUE_FLAGS_CHUNK:
            {
				result = iload->Read(&this->flags, sizeof(this->flags), &byteCount);
				break;
            }
		}
		iload->CloseChunk();
		if (result != IO_OK) 
			return result;
	}
	
	return IO_OK;
}
