
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "OgreMaxTechniqueDlg.h"
#include "OgreMaxTechnique.h"
#include "PluginGlobals.h"
#include "resource.h"

using namespace OgreMax;


//Local functions--------------------------------------------------------------
static int PassIndexToCheckBoxID(int i)
{
    switch (i)
    {
        case 0: return IDC_PASS_ENABLE_CHECK1;
        case 1: return IDC_PASS_ENABLE_CHECK2;
        case 2: return IDC_PASS_ENABLE_CHECK3;
        case 3: return IDC_PASS_ENABLE_CHECK4;
    }

    return 0;
}

static int PassIndexToButtonID(int i)
{
    switch (i)
    {
        case 0: return IDC_PASS_BUTTON1;
        case 1: return IDC_PASS_BUTTON2;
        case 2: return IDC_PASS_BUTTON3;
        case 3: return IDC_PASS_BUTTON4;
    }

    return 0;
}


//Implementation---------------------------------------------------------------
OgreMaxTechniqueDlg::OgreMaxTechniqueDlg
    (
    HWND hWndMaterialEdit, 
    IMtlParams* materialParams, 
    OgreMaxTechnique* technique
    )
{
    this->hWndMaterialEdit = hWndMaterialEdit;
    this->techniqueParams = materialParams;
    this->technique = technique;

    this->mainPanel = NULL;

    this->dragAndDropManager.Init(this);

    for (int i = 0; i < OgreMaxPasses::MAX_PASSES; i++) 
        this->passButton[i] = NULL;
    
    this->currentTime = 0;

    this->initialized = false;
}

OgreMaxTechniqueDlg::~OgreMaxTechniqueDlg()
{
    //Capture some final rollup values
    this->technique->SetFlag
        (
        OgreMaxTechnique::ROLLUP_OPEN, 
        this->techniqueParams->IsRollupPanelOpen(this->mainPanel)
        );
	this->technique->rollScroll = this->techniqueParams->GetRollupScrollPos();

    //Remove the parameter dialog
	this->technique->SetParamDlg(NULL);

    //Release the custom buttons used for the passes
	for (int i = 0; i < OgreMaxPasses::MAX_PASSES; i++) 
    	ReleaseICustButton(this->passButton[i]);
        
    //Remove the user data that we previously set
	SetWindowLongPtr(this->mainPanel, GWLP_USERDATA, NULL);	
}

Class_ID OgreMaxTechniqueDlg::ClassID() 
{
    return OgreMaxTechnique::GetClassClassID();  
}

void OgreMaxTechniqueDlg::SetThing(ReferenceTarget* refTarget)
{
    assert(refTarget->SuperClassID() == TEXMAP_CLASS_ID);
    assert(refTarget->ClassID() == OgreMaxTechnique::GetClassClassID());

	if (this->technique) 
		this->technique->SetParamDlg(NULL);
	
    this->technique = static_cast<OgreMaxTechnique*>(refTarget);
    if (this->technique)
	    this->technique->SetParamDlg(this);
	
    LoadDialog();	
}

ReferenceTarget* OgreMaxTechniqueDlg::GetThing() 
{ 
    return this->technique; 
}

void OgreMaxTechniqueDlg::DeleteThis() 
{ 
    delete this;  
}	

void OgreMaxTechniqueDlg::SetTime(TimeValue t)
{
    if (t != this->currentTime) 
    {
		this->currentTime = t;
		Interval v;
        this->technique->Update(this->techniqueParams->GetTime(), v);
		LoadDialog();
	}
}

int OgreMaxTechniqueDlg::FindSubTexFromHWND(HWND hWnd)
{
    for (int i = 0; i < OgreMaxPasses::MAX_PASSES; i++) 
    {
		if (this->passButton[i]->GetHwnd() == hWnd) 
			return i;
	}	
	
	return -1;
}

void OgreMaxTechniqueDlg::ReloadDialog()
{
    Interval v;
	this->technique->Update(this->techniqueParams->GetTime(), v);
	LoadDialog();
}

void OgreMaxTechniqueDlg::ActivateDlg(BOOL activate)
{
}

void OgreMaxTechniqueDlg::LoadDialog()
{
    if (this->technique != NULL && this->mainPanel != NULL)
    {		
		for (int i = 0; i < OgreMaxPasses::MAX_PASSES; i++)
            UpdatePassDisplay(i);
	}
}

void OgreMaxTechniqueDlg::BuildDialog() 
{
    this->mainPanel = 
	    this->techniqueParams->AddRollupPage
            ( 
            PluginGlobals::GetInstanceHandle(),
		    MAKEINTRESOURCE(IDD_TECHNIQUE_MAIN_ROLLUP),
		    MainPanelDlgProc, 
		    PluginGlobals::GetString(IDS_TECHNIQUE_PARAMETERS), 
		    reinterpret_cast<LPARAM>(this),
            this->technique->flags & OgreMaxTechnique::ROLLUP_OPEN ? 0 : APPENDROLL_CLOSED
		    );

    this->techniqueParams->SetRollupScrollPos(this->technique->rollScroll);
}

void OgreMaxTechniqueDlg::UpdatePassDisplay(int i) 
{
    if (this->technique != NULL)
    {
        if (this->technique->passes != NULL && i != -1)
        {
            //Set checkbox
	        SetCheckBox
                (
                this->mainPanel, 
                PassIndexToCheckBoxID(i), 
                this->technique->IsPassEnabled(i)
                );

            //Set button text
	        TSTR name = this->technique->GetPassName(i);
	        this->passButton[i]->SetText(name.data());	
        }
    }
}

BOOL OgreMaxTechniqueDlg::MainPanelProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    int id = LOWORD(wParam);
	int code = HIWORD(wParam);
    switch (message)
    {
		case WM_INITDIALOG:
		{
			this->passButton[0] = GetICustButton(GetDlgItem(hDlg, IDC_PASS_BUTTON1));
            this->passButton[0]->SetDADMgr(&this->dragAndDropManager);
            
            this->passButton[1] = GetICustButton(GetDlgItem(hDlg, IDC_PASS_BUTTON2));
            this->passButton[1]->SetDADMgr(&this->dragAndDropManager);
            
            this->passButton[2] = GetICustButton(GetDlgItem(hDlg, IDC_PASS_BUTTON3));
            this->passButton[2]->SetDADMgr(&this->dragAndDropManager);
            
            this->passButton[3] = GetICustButton(GetDlgItem(hDlg, IDC_PASS_BUTTON4));
            this->passButton[3]->SetDADMgr(&this->dragAndDropManager);
            
            return TRUE;
		}			
		case WM_COMMAND: 
		{ 
            switch (id) 
            {
                case IDC_PASS_BUTTON1:
                {
                    PostMessage(this->hWndMaterialEdit, WM_TEXMAP_BUTTON, 0, (LPARAM)this->technique);
				    break;
                }
                case IDC_PASS_BUTTON2:
                {
                    PostMessage(this->hWndMaterialEdit, WM_TEXMAP_BUTTON, 1, (LPARAM)this->technique);
				    break;
                }
                case IDC_PASS_BUTTON3:
                {
                    PostMessage(this->hWndMaterialEdit, WM_TEXMAP_BUTTON, 2, (LPARAM)this->technique);
				    break;
                }
                case IDC_PASS_BUTTON4:
                {
                    PostMessage(this->hWndMaterialEdit, WM_TEXMAP_BUTTON, 3, (LPARAM)this->technique);
				    break;
                }
                case IDC_PASS_ENABLE_CHECK1:
                {
                    this->technique->EnablePass(0, GetCheckBox(hDlg, IDC_PASS_ENABLE_CHECK1));
                    break;
                }
                case IDC_PASS_ENABLE_CHECK2:
                {
                    this->technique->EnablePass(1, GetCheckBox(hDlg, IDC_PASS_ENABLE_CHECK2));
                    break;
                }
                case IDC_PASS_ENABLE_CHECK3:
                {
                    this->technique->EnablePass(2, GetCheckBox(hDlg, IDC_PASS_ENABLE_CHECK3));
                    break;
                }
                case IDC_PASS_ENABLE_CHECK4:
                {
                    this->technique->EnablePass(3, GetCheckBox(hDlg, IDC_PASS_ENABLE_CHECK4));
                    break;
                }                
			}
			
            return TRUE;
        }
		case WM_PAINT:
        {
            if (!this->initialized)
            {
				this->initialized = true;
				ReloadDialog();
			}

			return FALSE;
        }
    }

	return FALSE;
}

INT_PTR CALLBACK OgreMaxTechniqueDlg::MainPanelDlgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam) 
{
    OgreMaxTechniqueDlg* dialog = NULL;
	if (msg == WM_INITDIALOG) 
    {
		dialog = reinterpret_cast<OgreMaxTechniqueDlg*>(lParam);
		dialog->mainPanel = hWnd;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, lParam);
	}
	else 
    {
        dialog = reinterpret_cast<OgreMaxTechniqueDlg*>(GetWindowLongPtr(hWnd, GWLP_USERDATA));
	    if (dialog == NULL)
			return FALSE; 
	}
	
    return dialog->MainPanelProc(hWnd, msg, wParam, lParam);
}
