
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "OgreMaxTechniques.h"
#include "PluginGlobals.h"
#include "resource.h"

using namespace OgreMax;


//Local classes----------------------------------------------------------------
class OgreMaxTechniqueTexmapsClassDesc : public ClassDesc 
{
public:
	int IsPublic()              {return 0;}
	void* Create(BOOL loading)  {return new OgreMaxTechniques(NULL);}
    const TCHAR* ClassName()    {return PluginGlobals::GetString(IDS_TECHNIQUES_CLASS_NAME);}
	SClass_ID SuperClassID()    {return TEXMAP_CONTAINER_CLASS_ID;}
    Class_ID ClassID()          {return OgreMaxTechniques::GetClassClassID();}
    const TCHAR* Category()     {return PluginGlobals::GetString(IDS_PLUGINS_CATEGORY);}

public:
    static OgreMaxTechniqueTexmapsClassDesc instance;
};
OgreMaxTechniqueTexmapsClassDesc OgreMaxTechniqueTexmapsClassDesc::instance;


//Implementation---------------------------------------------------------------

//TechniqueSlot implementation-----------------------------
TechniqueSlot::TechniqueSlot() 
{ 
    this->distance = 0.0f;
    this->distanceControl = NULL;
    this->map = NULL;
    this->on = TRUE;
}

void TechniqueSlot::Update(TimeValue t, Interval& ivalid) 
{
	if (IsOn() && HasMap()) 
		this->map->Update(t, ivalid);

	if (this->distanceControl != NULL) 
    	distanceControl->GetValue(t, &this->distance, ivalid);
}

float TechniqueSlot::GetDistance() 
{
    TimeValue t = GetCOREInterface()->GetTime();
    float f;
	Interval v;	

	if (this->distanceControl) 
    	this->distanceControl->GetValue(t, &f, v);			
	else 
        f = this->distance;

    return f;
} 

void TechniqueSlot::SetDistance(float distance)
{
    if (this->distanceControl != NULL)
        this->distanceControl->SetValue(0, &distance);
    
    this->distance = distance;
}

BOOL TechniqueSlot::IsOn() 
{ 
    return this->on;
}

void TechniqueSlot::SetOn(BOOL on)
{
    this->on = on;
}

BOOL TechniqueSlot::HasMap() 
{
    return this->map != 0 ? TRUE : FALSE; 
}

Texmap* TechniqueSlot::GetMap()
{
    return this->map;
}

Control* TechniqueSlot::GetDistanceControl()
{
    return this->distanceControl;
}

//OgreMaxTechniques implementation-------------------
OgreMaxTechniques::OgreMaxTechniques() 
{
    this->client = NULL;
}
					
OgreMaxTechniques::OgreMaxTechniques(MtlBase* mtlBase) 
{
	this->client = mtlBase;
}

void OgreMaxTechniques::SetName(long i, TSTR& name)
{
    this->techniques[i].name = name;
}

TSTR& OgreMaxTechniques::GetName(long i)
{
    return this->techniques[i].name; 
} 

void OgreMaxTechniques::SetClientPtr(MtlBase* mtlBase)
{
    this->client = mtlBase;
}

TechniqueSlot& OgreMaxTechniques::operator[](int i)
{
    return this->techniques[i];
}

TechniqueSlot* OgreMaxTechniques::GetTechnique(int i)
{
    return &this->techniques[i];
}

ClassDesc* OgreMaxTechniques::GetClassDesc() 
{ 
    return &OgreMaxTechniqueTexmapsClassDesc::instance;  
}

Class_ID OgreMaxTechniques::GetClassClassID() 
{
    return Class_ID(0x391aeda0, 0x5de85d74);
}

Class_ID OgreMaxTechniques::ClassID() 
{ 
    return GetClassClassID(); 
}

void OgreMaxTechniques::DeleteThis()
{
    delete this;
}

RefTargetHandle OgreMaxTechniques::Clone(RemapDir& remap) 
{
	OgreMaxTechniques* newTexmaps = new OgreMaxTechniques(NULL);
	
    for (int i = 0; i< MAX_TECHNIQUES; i++) 
    {
		newTexmaps->techniques[i].distance = this->techniques[i].distance;
		newTexmaps->techniques[i].on = this->techniques[i].on;
		newTexmaps->techniques[i].map = NULL;
		if (this->techniques[i].distanceControl) 
			newTexmaps->ReplaceReference(2*i, remap.CloneRef(this->techniques[i].distanceControl));
		if (this->techniques[i].map) 
			newTexmaps->ReplaceReference(2*i+1, remap.CloneRef(this->techniques[i].map));
	}
	
    BaseClone(this, newTexmaps, remap);
	
    return newTexmaps;
}

RefResult OgreMaxTechniques::NotifyRefChanged
    (
    Interval changeInt, 
    RefTargetHandle hTarget, 
    PartID& partID, 
    RefMessage message 
    ) 
{
	switch (message) 
    {
		case REFMSG_GET_PARAM_DIM: 
        {
			GetParamDim* gpd = (GetParamDim*)partID;
			gpd->dim = defaultDim; 
			break;
		}
		case REFMSG_GET_PARAM_NAME: 
        {
			GetParamName* gpn = (GetParamName*)partID;
			return REF_STOP; 
		}
	}
	
    return REF_SUCCEED;
}

void OgreMaxTechniques::RescaleWorldUnits(float f) 
{
	if (TestAFlag(A_WORK1))
		return;
	SetAFlag(A_WORK1);

	for (int i = 0; i < NumRefs(); i++) 
    {
        //Only rescale units for maps
		if (i & 1)
        {            
		    ReferenceMaker* ref = GetReference(i);
		    if (ref != NULL) 
        	    ref->RescaleWorldUnits(f);
        }
	}		
}

BOOL OgreMaxTechniques::AssignController(Animatable* control, int subAnim) 
{
	ReplaceReference(SubNumToRefNum(subAnim), static_cast<ReferenceTarget*>(control));
	return TRUE;
}

int OgreMaxTechniques::NumSubs()
{ 
    //Each technique has two subanims.
    //One is for the LOD distance, the other for the technique map itself
    return MAX_TECHNIQUES * 2; 
}  

Animatable* OgreMaxTechniques::SubAnim(int i) 
{
	if (i & 1)
    {
        //Odd numbered index. Return the map
		return this->techniques[i / 2].map;
    }
	else 
    {
        //Even numbered index. Return the distance controller
		return this->techniques[i / 2].distanceControl;
    }
}

TSTR OgreMaxTechniques::SubAnimName(int i) 
{
	if (i & 1)
    {
        //Odd numbered index. Return the map name
		return client->GetSubTexmapTVName(i / 2);
    }
	else  
    {
        //Even numbered index. Return the distance controller name
		TSTR name;
		name = this->techniques[i / 2].name;
		name += _T(" ");
        name += PluginGlobals::GetString(IDS_LOD_DISTANCE);
		return name;
	}
}

int OgreMaxTechniques::SubNumToRefNum(int subNum)
{
    //There is no mapping required
    return subNum;
}

int OgreMaxTechniques::NumRefs() 
{ 
    return NumSubs();
}

RefTargetHandle OgreMaxTechniques::GetReference(int i) 
{
	if (i & 1)
    {
        //Odd numbered index. Return the map
		return this->techniques[i / 2].map;
    }
	else 
    {
        //Even numbered index. Return the distance controller
		return this->techniques[i / 2].distanceControl;
    }
}

void OgreMaxTechniques::SetReference(int i, RefTargetHandle refTarget) 
{
	if (i & 1)
    {
        //Odd numbered index. Set the map
		this->techniques[i / 2].map = static_cast<Texmap*>(refTarget);
    }
	else 
    {
        //Even numbered index. Set the distance controller
		this->techniques[i / 2].distanceControl = static_cast<Control*>(refTarget);	
    }
}

SvGraphNodeReference OgreMaxTechniques::SvTraverseAnimGraph
    (
    IGraphObjectManager* gom, 
    Animatable* owner, 
    int id, 
    DWORD flags
    )
{
	//If not testing for maps, we're done already
	if (!gom->TestFilter(SV_FILTER_MAPS))
		return SvGraphNodeReference();

    //Figure out how many slots are needed for the schematic view
    int usedSlotCount = 0;
    if (this->techniques != NULL)
    {
	    for (int i = 0; i < MAX_TECHNIQUES; i++)
        {
		    if (this->techniques[i].GetMap() != NULL)
			    usedSlotCount++;
        }
    }
    
    //Return a "used" result or an empty one, depending on the count
	if (usedSlotCount)
		return SvStdTraverseAnimGraph(gom, owner, id, flags);
	else
		return SvGraphNodeReference();
}

#define TECHNIQUES_ON_OFF_CHUNK 0x5003
#define TECHNIQUES_VERSION_CHUNK 0x5300

#define TECHNIQUES_VERSION 1
#define TECHNIQUES_LOD_DISTANCE0 0x5100
#define TECHNIQUES_LOD_DISTANCE1 0x5101
#define TECHNIQUES_LOD_DISTANCE2 0x5102
#define TECHNIQUES_LOD_DISTANCE3 0x5103

IOResult OgreMaxTechniques::Save(ISave* isave) 
{ 
    int i;
    ULONG byteCount;

    //Save the version
    isave->BeginChunk(TECHNIQUES_VERSION_CHUNK);
	int version = TECHNIQUES_VERSION;
	isave->Write(&version, sizeof(version), &byteCount);			
	isave->EndChunk();

    //Save the on/off flags
    ULONG onFlags = 0;    
	for (i = 0; i < MAX_TECHNIQUES; i++)
    {
		if (this->techniques[i].on) 
            onFlags |= (1<<i);
    }
    isave->BeginChunk(TECHNIQUES_ON_OFF_CHUNK);
	isave->Write(&onFlags, sizeof(onFlags), &byteCount);			
	isave->EndChunk();

    //Save the LOD distances
	for (i = 0; i < MAX_TECHNIQUES; i++) 
    {
		if (this->techniques[i].distance != 0.0f)
        {
			isave->BeginChunk(TECHNIQUES_LOD_DISTANCE0 + i);
			isave->Write(&this->techniques[i].distance, sizeof(float), &byteCount);			
			isave->EndChunk();
		}
	}

	return IO_OK;
}

IOResult OgreMaxTechniques::Load(ILoad *iload) 
{ 
	ULONG byteCount;
	int id;
    int version = 0;
	IOResult result;

	while ((result = iload->OpenChunk()) == IO_OK) 
    {
		switch (id = iload->CurChunkID())  
        {
            case TECHNIQUES_VERSION_CHUNK:
            {
				result = iload->Read(&version, sizeof(version), &byteCount);
				break;
            }
			case TECHNIQUES_ON_OFF_CHUNK:
			{
			    ULONG onFlags;
			    result = iload->Read(&onFlags, sizeof(onFlags), &byteCount);
			    for (int i = 0; i < MAX_TECHNIQUES; i++) 
			        this->techniques[i].on = (onFlags & (1<<i)) ? TRUE : FALSE;

                break;
			}				
			case TECHNIQUES_LOD_DISTANCE0: 
            case TECHNIQUES_LOD_DISTANCE1:
			case TECHNIQUES_LOD_DISTANCE2:	
            case TECHNIQUES_LOD_DISTANCE3:
			{
				result = 
                    iload->Read
                        (
                        &this->techniques[id - TECHNIQUES_LOD_DISTANCE0].distance, 
                        sizeof(float), 
                        &byteCount
                        );
				break;
            }
		}

		iload->CloseChunk();
		if (result != IO_OK) 
			return result;
	}

	return IO_OK;                                                      
}

