//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "PluginGlobals.h"
#include "OgreMaxTextureUnit.h"
#include "OgreMaxTextureUnitDlg.h"
#include "OgreMaxPass.h"
#include "resource.h"

using namespace OgreMax;


//Local classes-----------------------------------------------------------------
class OgreMaxTextureUnitClassDesc : public ClassDesc
{
public:
	int IsPublic()              {return 1;}
	void* Create(BOOL loading)  {return new OgreMaxTextureUnit(loading);}
	const TCHAR* ClassName()    {return PluginGlobals::GetString(IDS_TEXTURE_UNIT_CLASS_NAME);}
	SClass_ID SuperClassID()    {return TEXMAP_CLASS_ID;}
    Class_ID ClassID()          {return OgreMaxTextureUnit::GetClassClassID();}
	const TCHAR* Category()     {return PluginGlobals::GetString(IDS_PLUGINS_CATEGORY);}

public:
    static OgreMaxTextureUnitClassDesc instance;
};
OgreMaxTextureUnitClassDesc OgreMaxTextureUnitClassDesc::instance;


//Globals----------------------------------------------------------------------

//Parameter block descriptor
static ParamBlockDescID ogreMaxTextureUnitParamBlock[] = 
{
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_BITMAP_DIMENSION},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_MAP_CHANNEL},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_COLOR_BLEND_FUNCTION},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_COLOR_BLEND_SOURCE},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_COLOR_BLEND_DESTINATION},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_COLOR_BLEND_ALPHA},
    {TYPE_RGBA, NULL, FALSE, OgreMaxTextureUnit::PB_COLOR_BLEND_MANUAL_COLOR1},
    {TYPE_RGBA, NULL, FALSE, OgreMaxTextureUnit::PB_COLOR_BLEND_MANUAL_COLOR2},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_ALPHA_BLEND_FUNCTION},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_ALPHA_BLEND_SOURCE},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_ALPHA_BLEND_DESTINATION},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_ALPHA_BLEND_ALPHA},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_ALPHA_BLEND_MANUAL_ALPHA1},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_ALPHA_BLEND_MANUAL_ALPHA2},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_LAYER_BLEND_FALLBACK_SOURCE},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_LAYER_BLEND_FALLBACK_DESTINATION},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_FRAMES_DURATION},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_ENVIRONMENT_MAPPING},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_ADDRESSING_MODE},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_FILTERING_MODE},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_MAX_ANISOTROPY},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_TRANSFORM_SCROLL_U},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_TRANSFORM_SCROLL_V},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_TRANSFORM_SCALE_U},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_TRANSFORM_SCALE_V},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_TRANSFORM_ROTATE},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_TRANSFORM_ANIMATION_SCROLL_U},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_TRANSFORM_ANIMATION_SCROLL_V},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_TRANSFORM_ANIMATION_ROTATE},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_TRANSFORM_ANIMATION_TRANSFORM_TYPE},
    {TYPE_INT, NULL, FALSE, OgreMaxTextureUnit::PB_TRANSFORM_ANIMATION_WAVE_TYPE},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_TRANSFORM_ANIMATION_BASE},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_TRANSFORM_ANIMATION_FREQUENCY},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_TRANSFORM_ANIMATION_PHASE},
    {TYPE_FLOAT, NULL, FALSE, OgreMaxTextureUnit::PB_TRANSFORM_ANIMATION_AMPLITUDE}
};

#define PARAM_BLOCK_VERSION 1


//Implementation---------------------------------------------------------------
OgreMaxTextureUnit::OgreMaxTextureUnit(BOOL loading)
{
    this->paramDlg = NULL;

    this->flags = ROLLUP_OPEN;
	this->rollScroll = 0;

    this->renderTexture = NULL;
    this->paramBlock = NULL;
    
    if (!loading)
		Reset();
}           

void OgreMaxTextureUnit::SetBitmapDimension(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_BITMAP_DIMENSION, t, value);
}

void OgreMaxTextureUnit::SetMapChannel(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_MAP_CHANNEL, t, value);
}

void OgreMaxTextureUnit::SetColorBlendFunction(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_COLOR_BLEND_FUNCTION, t, value);
}

void OgreMaxTextureUnit::SetColorBlendSource(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_COLOR_BLEND_SOURCE, t, value);
}

void OgreMaxTextureUnit::SetColorBlendDestination(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_COLOR_BLEND_DESTINATION, t, value);
}

void OgreMaxTextureUnit::SetColorBlendAlpha(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_COLOR_BLEND_ALPHA, t, value);
}

void OgreMaxTextureUnit::SetColorBlendManualColor1(Color value, TimeValue t)
{
    this->paramBlock->SetValue(PB_COLOR_BLEND_MANUAL_COLOR1, t, value);
}

void OgreMaxTextureUnit::SetColorBlendManualColor2(Color value, TimeValue t)
{
    this->paramBlock->SetValue(PB_COLOR_BLEND_MANUAL_COLOR2, t, value);
}

void OgreMaxTextureUnit::SetAlphaBlendFunction(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_ALPHA_BLEND_FUNCTION, t, value);
}

void OgreMaxTextureUnit::SetAlphaBlendSource(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_ALPHA_BLEND_SOURCE, t, value);
}

void OgreMaxTextureUnit::SetAlphaBlendDestination(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_ALPHA_BLEND_DESTINATION, t, value);
}

void OgreMaxTextureUnit::SetAlphaBlendAlpha(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_ALPHA_BLEND_ALPHA, t, value);
}

void OgreMaxTextureUnit::SetAlphaBlendManualAlpha1(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_ALPHA_BLEND_MANUAL_ALPHA1, t, value);
}

void OgreMaxTextureUnit::SetAlphaBlendManualAlpha2(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_ALPHA_BLEND_MANUAL_ALPHA2, t, value);
}

void OgreMaxTextureUnit::SetLayerBlendFallbackSource(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_LAYER_BLEND_FALLBACK_SOURCE, t, value);    
}

void OgreMaxTextureUnit::SetLayerBlendFallbackDestination(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_LAYER_BLEND_FALLBACK_DESTINATION, t, value);    
}

void OgreMaxTextureUnit::SetFramesDuration(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_FRAMES_DURATION, t, value);    
}

void OgreMaxTextureUnit::SetEnvironmentMapping(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_ENVIRONMENT_MAPPING, t, value);
}

void OgreMaxTextureUnit::SetAddressingMode(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_ADDRESSING_MODE, t, value);
}

void OgreMaxTextureUnit::SetFilteringMode(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_FILTERING_MODE, t, value);
}

void OgreMaxTextureUnit::SetMaxAnisotropy(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_MAX_ANISOTROPY, t, value);
}

void OgreMaxTextureUnit::SetTransformScrollU(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_TRANSFORM_SCROLL_U, t, value);
}

void OgreMaxTextureUnit::SetTransformScrollV(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_TRANSFORM_SCROLL_V, t, value);
}

void OgreMaxTextureUnit::SetTransformScaleU(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_TRANSFORM_SCALE_U, t, value);
}

void OgreMaxTextureUnit::SetTransformScaleV(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_TRANSFORM_SCALE_V, t, value);
}

void OgreMaxTextureUnit::SetTransformRotate(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_TRANSFORM_ROTATE, t, value);
}

void OgreMaxTextureUnit::SetTransformAnimationScrollU(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_TRANSFORM_ANIMATION_SCROLL_U, t, value);
}

void OgreMaxTextureUnit::SetTransformAnimationScrollV(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_TRANSFORM_ANIMATION_SCROLL_V, t, value);
}

void OgreMaxTextureUnit::SetTransformAnimationRotate(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_TRANSFORM_ANIMATION_ROTATE, t, value);
}

void OgreMaxTextureUnit::SetTransformAnimationType(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_TRANSFORM_ANIMATION_TRANSFORM_TYPE, t, value);
}

void OgreMaxTextureUnit::SetTransformAnimationWaveType(int value, TimeValue t)
{
    this->paramBlock->SetValue(PB_TRANSFORM_ANIMATION_WAVE_TYPE, t, value);
}

void OgreMaxTextureUnit::SetTransformAnimationBase(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_TRANSFORM_ANIMATION_BASE, t, value);
}

void OgreMaxTextureUnit::SetTransformAnimationFrequency(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_TRANSFORM_ANIMATION_FREQUENCY, t, value);
}

void OgreMaxTextureUnit::SetTransformAnimationPhase(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_TRANSFORM_ANIMATION_PHASE, t, value);
}

void OgreMaxTextureUnit::SetTransformAnimationAmplitude(float value, TimeValue t)
{
    this->paramBlock->SetValue(PB_TRANSFORM_ANIMATION_AMPLITUDE, t, value);
}

int OgreMaxTextureUnit::GetBitmapDimension(TimeValue t)
{
    return this->paramBlock->GetInt(PB_BITMAP_DIMENSION, t);
}

int OgreMaxTextureUnit::GetMapChannel(TimeValue t)
{
    return this->paramBlock->GetInt(PB_MAP_CHANNEL, t);
}

int OgreMaxTextureUnit::GetColorBlendFunction(TimeValue t)
{
    return this->paramBlock->GetInt(PB_COLOR_BLEND_FUNCTION, t);
}

int OgreMaxTextureUnit::GetColorBlendSource(TimeValue t)
{
    return this->paramBlock->GetInt(PB_COLOR_BLEND_SOURCE, t);
}

int OgreMaxTextureUnit::GetColorBlendDestination(TimeValue t)
{
    return this->paramBlock->GetInt(PB_COLOR_BLEND_DESTINATION, t);
}

float OgreMaxTextureUnit::GetColorBlendAlpha(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_COLOR_BLEND_ALPHA, t);
}

Color OgreMaxTextureUnit::GetColorBlendManualColor1(TimeValue t)
{
    return this->paramBlock->GetColor(PB_COLOR_BLEND_MANUAL_COLOR1, t);
}

Color OgreMaxTextureUnit::GetColorBlendManualColor2(TimeValue t)
{
    return this->paramBlock->GetColor(PB_COLOR_BLEND_MANUAL_COLOR2, t);
}

int OgreMaxTextureUnit::GetAlphaBlendFunction(TimeValue t)
{
    return this->paramBlock->GetInt(PB_ALPHA_BLEND_FUNCTION, t);
}

int OgreMaxTextureUnit::GetAlphaBlendSource(TimeValue t)
{
    return this->paramBlock->GetInt(PB_ALPHA_BLEND_SOURCE, t);
}

int OgreMaxTextureUnit::GetAlphaBlendDestination(TimeValue t)
{
    return this->paramBlock->GetInt(PB_ALPHA_BLEND_DESTINATION, t);
}

float OgreMaxTextureUnit::GetAlphaBlendAlpha(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_ALPHA_BLEND_ALPHA, t);
}

float OgreMaxTextureUnit::GetAlphaBlendManualAlpha1(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_ALPHA_BLEND_MANUAL_ALPHA1, t);
}

float OgreMaxTextureUnit::GetAlphaBlendManualAlpha2(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_ALPHA_BLEND_MANUAL_ALPHA2, t);
}

int OgreMaxTextureUnit::GetLayerBlendFallbackSource(TimeValue t)
{
    return this->paramBlock->GetInt(PB_LAYER_BLEND_FALLBACK_SOURCE, t);    
}

int OgreMaxTextureUnit::GetLayerBlendFallbackDestination(TimeValue t)
{
    return this->paramBlock->GetInt(PB_LAYER_BLEND_FALLBACK_DESTINATION, t);    
}

float OgreMaxTextureUnit::GetFramesDuration(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_FRAMES_DURATION, t);    
}

int OgreMaxTextureUnit::GetEnvironmentMapping(TimeValue t)
{
    return this->paramBlock->GetInt(PB_ENVIRONMENT_MAPPING, t);
}

int OgreMaxTextureUnit::GetAddressingMode(TimeValue t)
{
    return this->paramBlock->GetInt(PB_ADDRESSING_MODE, t);
}

int OgreMaxTextureUnit::GetFilteringMode(TimeValue t)
{
    return this->paramBlock->GetInt(PB_FILTERING_MODE, t);
}

int OgreMaxTextureUnit::GetMaxAnisotropy(TimeValue t)
{
    return this->paramBlock->GetInt(PB_MAX_ANISOTROPY, t);
}

float OgreMaxTextureUnit::GetTransformScrollU(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_TRANSFORM_SCROLL_U, t);
}

float OgreMaxTextureUnit::GetTransformScrollV(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_TRANSFORM_SCROLL_V, t);
}

float OgreMaxTextureUnit::GetTransformScaleU(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_TRANSFORM_SCALE_U, t);
}

float OgreMaxTextureUnit::GetTransformScaleV(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_TRANSFORM_SCALE_V, t);
}

float OgreMaxTextureUnit::GetTransformRotate(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_TRANSFORM_ROTATE, t);
}

float OgreMaxTextureUnit::GetTransformAnimationScrollU(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_TRANSFORM_ANIMATION_SCROLL_U, t);
}

float OgreMaxTextureUnit::GetTransformAnimationScrollV(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_TRANSFORM_ANIMATION_SCROLL_V, t);
}

float OgreMaxTextureUnit::GetTransformAnimationRotate(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_TRANSFORM_ANIMATION_ROTATE, t);
}

int OgreMaxTextureUnit::GetTransformAnimationType(TimeValue t)
{
    return this->paramBlock->GetInt(PB_TRANSFORM_ANIMATION_TRANSFORM_TYPE, t);
}

int OgreMaxTextureUnit::GetTransformAnimationWaveType(TimeValue t)
{
    return this->paramBlock->GetInt(PB_TRANSFORM_ANIMATION_WAVE_TYPE, t);
}

float OgreMaxTextureUnit::GetTransformAnimationBase(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_TRANSFORM_ANIMATION_BASE, t);
}

float OgreMaxTextureUnit::GetTransformAnimationFrequency(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_TRANSFORM_ANIMATION_FREQUENCY, t);
}

float OgreMaxTextureUnit::GetTransformAnimationPhase(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_TRANSFORM_ANIMATION_PHASE, t);
}

float OgreMaxTextureUnit::GetTransformAnimationAmplitude(TimeValue t)
{
    return this->paramBlock->GetFloat(PB_TRANSFORM_ANIMATION_AMPLITUDE, t);
}

AColor OgreMaxTextureUnit::EvalColor(ShadeContext& sc)
{
    return AColor(0, 0, 0);
}

Point3 OgreMaxTextureUnit::EvalNormalPerturb(ShadeContext& sc)
{
    return Point3(0, 0, 0);
}

RefResult OgreMaxTextureUnit::NotifyRefChanged
    (
    Interval changeInt, 
    RefTargetHandle hTarget, 
    PartID& partID, 
    RefMessage message
    )
{
    switch (message) 
    {
		case REFMSG_WANT_SHOWPARAMLEVEL:
		{
			BOOL* result = reinterpret_cast<BOOL*>(partID);
			*result = TRUE;
			return REF_STOP;
		}
		case REFMSG_CHANGE:
        {
			this->ivalid.SetEmpty();
			if (this->paramDlg != NULL)
            	this->paramDlg->Invalidate();
            break;
        }
		case REFMSG_GET_PARAM_DIM: 
        {
			GetParamDim* gpd = reinterpret_cast<GetParamDim*>(partID);
            gpd->dim = defaultDim;
			return REF_STOP; 
		}
		case REFMSG_GET_PARAM_NAME: 
        {
			GetParamName* gpn = reinterpret_cast<GetParamName*>(partID);			
            switch (gpn->index) 
            {
                case PB_BITMAP_DIMENSION: gpn->name = PluginGlobals::GetString(IDS_BITMAP_DIMENSION); break;
                case PB_MAP_CHANNEL: gpn->name = PluginGlobals::GetString(IDS_MAP_CHANNEL); break;
                case PB_COLOR_BLEND_FUNCTION: gpn->name = PluginGlobals::GetString(IDS_COLOR_BLEND_FUNCTION); break;
                case PB_COLOR_BLEND_SOURCE: gpn->name = PluginGlobals::GetString(IDS_COLOR_BLEND_SOURCE); break;
                case PB_COLOR_BLEND_DESTINATION: gpn->name = PluginGlobals::GetString(IDS_COLOR_BLEND_DESTINATION); break;
                case PB_COLOR_BLEND_ALPHA: gpn->name = PluginGlobals::GetString(IDS_COLOR_BLEND_ALPHA); break;
                case PB_COLOR_BLEND_MANUAL_COLOR1: gpn->name = PluginGlobals::GetString(IDS_COLOR_BLEND_MANUAL_COLOR1); break;
                case PB_COLOR_BLEND_MANUAL_COLOR2: gpn->name = PluginGlobals::GetString(IDS_COLOR_BLEND_MANUAL_COLOR2); break;
                case PB_ALPHA_BLEND_FUNCTION: gpn->name = PluginGlobals::GetString(IDS_ALPHA_BLEND_FUNCTION); break;
                case PB_ALPHA_BLEND_SOURCE: gpn->name = PluginGlobals::GetString(IDS_ALPHA_BLEND_SOURCE); break;
                case PB_ALPHA_BLEND_DESTINATION: gpn->name = PluginGlobals::GetString(IDS_ALPHA_BLEND_DESTINATION); break;
                case PB_ALPHA_BLEND_ALPHA: gpn->name = PluginGlobals::GetString(IDS_ALPHA_BLEND_ALPHA); break;
                case PB_ALPHA_BLEND_MANUAL_ALPHA1: gpn->name = PluginGlobals::GetString(IDS_ALPHA_BLEND_MANUAL_ALPHA1); break;
                case PB_ALPHA_BLEND_MANUAL_ALPHA2: gpn->name = PluginGlobals::GetString(IDS_ALPHA_BLEND_MANUAL_ALPHA2); break;
                case PB_LAYER_BLEND_FALLBACK_SOURCE: gpn->name = PluginGlobals::GetString(IDS_LAYER_BLEND_FALLBACK_SOURCE); break;
                case PB_LAYER_BLEND_FALLBACK_DESTINATION: gpn->name = PluginGlobals::GetString(IDS_LAYER_BLEND_FALLBACK_DESTINATION); break;
                case PB_FRAMES_DURATION: gpn->name = PluginGlobals::GetString(IDS_FRAMES_DURATION); break;
                case PB_ENVIRONMENT_MAPPING: gpn->name = PluginGlobals::GetString(IDS_ENVIRONMENT_MAPPING); break;
                case PB_ADDRESSING_MODE: gpn->name = PluginGlobals::GetString(IDS_ADDRESSING_MODE); break;
                case PB_FILTERING_MODE: gpn->name = PluginGlobals::GetString(IDS_FILTERING_MODE); break;
                case PB_MAX_ANISOTROPY: gpn->name = PluginGlobals::GetString(IDS_MAX_ANISOTROPY); break;
                case PB_TRANSFORM_SCROLL_U: gpn->name = PluginGlobals::GetString(IDS_TRANSFORM_SCROLL_U); break;
                case PB_TRANSFORM_SCROLL_V: gpn->name = PluginGlobals::GetString(IDS_TRANSFORM_SCROLL_V); break;
                case PB_TRANSFORM_SCALE_U: gpn->name = PluginGlobals::GetString(IDS_TRANSFORM_SCALE_U); break;
                case PB_TRANSFORM_SCALE_V: gpn->name = PluginGlobals::GetString(IDS_TRANSFORM_SCALE_V); break;
                case PB_TRANSFORM_ROTATE: gpn->name = PluginGlobals::GetString(IDS_TRANSFORM_ROTATE); break;
                case PB_TRANSFORM_ANIMATION_SCROLL_U: gpn->name = PluginGlobals::GetString(IDS_TRANSFORM_ANIMATION_SCROLL_U); break;
                case PB_TRANSFORM_ANIMATION_SCROLL_V: gpn->name = PluginGlobals::GetString(IDS_TRANSFORM_ANIMATION_SCROLL_V); break;
                case PB_TRANSFORM_ANIMATION_ROTATE: gpn->name = PluginGlobals::GetString(IDS_TRANSFORM_ANIMATION_ROTATE); break;
                case PB_TRANSFORM_ANIMATION_TRANSFORM_TYPE: gpn->name = PluginGlobals::GetString(IDS_TRANSFORM_ANIMATION_TRANSFORM_TYPE); break;
                case PB_TRANSFORM_ANIMATION_WAVE_TYPE: gpn->name = PluginGlobals::GetString(IDS_TRANSFORM_ANIMATION_WAVE_TYPE); break;
                case PB_TRANSFORM_ANIMATION_BASE: gpn->name = PluginGlobals::GetString(IDS_TRANSFORM_ANIMATION_BASE); break;
                case PB_TRANSFORM_ANIMATION_FREQUENCY: gpn->name = PluginGlobals::GetString(IDS_TRANSFORM_ANIMATION_FREQUENCY); break;
                case PB_TRANSFORM_ANIMATION_PHASE: gpn->name = PluginGlobals::GetString(IDS_TRANSFORM_ANIMATION_PHASE); break;
                case PB_TRANSFORM_ANIMATION_AMPLITUDE: gpn->name = PluginGlobals::GetString(IDS_TRANSFORM_ANIMATION_AMPLITUDE); break;
			}
			return REF_STOP; 
		}
	}

    return REF_SUCCEED;
}

void OgreMaxTextureUnit::Update(TimeValue t, Interval& valid)
{
    Point3 p;
	if (!this->ivalid.InInterval(t)) 
    {
		this->ivalid.SetInfinite();

        if (this->renderTexture != NULL)
            this->renderTexture->Update(t, this->ivalid);
	}
	valid &= ivalid;
}

RefTargetHandle OgreMaxTextureUnit::Clone(RemapDir& remap)
{
    OgreMaxTextureUnit* newTechnique = new OgreMaxTextureUnit();

    //Copy superclass stuff
    *static_cast<MtlBase*>(newTechnique) = *this;
	
    //Copy everything else
	newTechnique->ReplaceReference(RENDER_TEXTURE_REF, remap.CloneRef(this->renderTexture));	
    newTechnique->ReplaceReference(PARAM_BLOCK_REF, remap.CloneRef(this->paramBlock));	

    newTechnique->flags = flags;
	
    BaseClone(this, newTechnique, remap);
	
    return newTechnique;
}

void OgreMaxTextureUnit::Reset()
{
    SuspendSetKeyMode();

    this->ivalid.SetEmpty();

    this->flags = ROLLUP_OPEN;
	
    ReplaceReference(RENDER_TEXTURE_REF, NULL);
    ReplaceReference
        (
        PARAM_BLOCK_REF, 
        CreateParameterBlock(ogreMaxTextureUnitParamBlock, NUM_PB_ENTRIES, PARAM_BLOCK_VERSION)
        );	

    //Set the default values    
    SetBitmapDimension(BITMAP_2D);
    SetMapChannel(0);
    SetColorBlendFunction(BLEND_FUNCTION_MODULATE);    
    SetColorBlendSource(BLEND_LOCATION_TEXTURE);
    SetColorBlendDestination(BLEND_LOCATION_CURRENT);
    SetColorBlendAlpha(0.0f);
    SetColorBlendManualColor1(Color(1.0f, 1.0f, 1.0f));
    SetColorBlendManualColor2(Color(1.0f, 1.0f, 1.0f));
    SetAlphaBlendFunction(BLEND_FUNCTION_MODULATE);
    SetAlphaBlendSource(BLEND_LOCATION_TEXTURE);
    SetAlphaBlendDestination(BLEND_LOCATION_CURRENT);
    SetAlphaBlendAlpha(0.0f);
    SetAlphaBlendManualAlpha1(1.0f);
    SetAlphaBlendManualAlpha2(1.0f);
    SetLayerBlendFallbackSource(OgreMaxPass::BLEND_FACTOR_DESTINATION_COLOR);
    SetLayerBlendFallbackDestination(OgreMaxPass::BLEND_FACTOR_ZERO);
    SetFramesDuration(1.0f);    
    SetEnvironmentMapping(ENVIRONMENT_MAPPING_OFF);
    SetAddressingMode(ADDRESSING_MODE_WRAP);
    SetFilteringMode(FILTERING_MODE_TRILINEAR);
    SetMaxAnisotropy(3);
    SetTransformScrollU(0.0f);
    SetTransformScrollV(0.0f);
    SetTransformScaleU(1.0f);
    SetTransformScaleV(1.0f);
    SetTransformRotate(0.0f);
    SetTransformAnimationScrollU(0.0f);
    SetTransformAnimationScrollV(0.0f);
    SetTransformAnimationRotate(0.0f);
    SetTransformAnimationType(TRANSFORM_ANIMATION_SCROLL_U);
    SetTransformAnimationWaveType(TRANSFORM_ANIMATION_WAVE_SINE);    
    SetTransformAnimationBase(1.0f);
    SetTransformAnimationFrequency(0.2f);
    SetTransformAnimationPhase(0.0f);
    SetTransformAnimationAmplitude(0.0f);
		
	ResumeSetKeyMode();
}

Interval OgreMaxTextureUnit::Validity(TimeValue t)
{
    Interval v;
	Update(t, v);
	return this->ivalid;
}

ParamDlg* OgreMaxTextureUnit::CreateParamDlg(HWND hWndMaterialEdit, IMtlParams* materialParams)
{
    Interval v;
	Update(materialParams->GetTime(), v);
	OgreMaxTextureUnitDlg* dlg = new OgreMaxTextureUnitDlg(hWndMaterialEdit, materialParams, this);    
	dlg->BuildDialog();
	dlg->LoadDialog();	 
	SetParamDlg(dlg);
	return dlg;
}

void OgreMaxTextureUnit::SetParamDlg(OgreMaxTextureUnitDlg* paramDlg)
{
    this->paramDlg = paramDlg;
}

int OgreMaxTextureUnit::NumSubTexmaps() 
{ 
    return NUM_SUBTEXMAPS;
}

Texmap* OgreMaxTextureUnit::GetSubTexmap(int i)
{
    assert(i == RENDER_TEXTURE_SUBTEXMAP);

    return this->renderTexture;
}

int OgreMaxTextureUnit::MapSlotType(int i) 
{ 
    return MAPSLOT_TEXTURE; 
}

void OgreMaxTextureUnit::SetSubTexmap(int i, Texmap* m)
{
    assert(i == RENDER_TEXTURE_SUBTEXMAP);
	
    ReplaceReference(i, m);
    
	if (this->paramDlg)
		this->paramDlg->UpdateRenderTextureDisplay();
}

TSTR OgreMaxTextureUnit::GetSubTexmapSlotName(int i)
{
    return PluginGlobals::GetString(IDS_RENDER_TO_TEXTURE_SLOT_NAME);
}

int OgreMaxTextureUnit::SubTexmapOn(int i) 
{ 
    return TRUE;
} 

ClassDesc* OgreMaxTextureUnit::GetClassDesc()
{
    return &OgreMaxTextureUnitClassDesc::instance;
}

Class_ID OgreMaxTextureUnit::GetClassClassID()
{
    return Class_ID(0xac89cdc, 0x9d8c4538);
}

Class_ID OgreMaxTextureUnit::ClassID()
{
    return GetClassClassID();
}

SClass_ID OgreMaxTextureUnit::SuperClassID()
{ 
    return TEXMAP_CLASS_ID; 
}

void OgreMaxTextureUnit::GetClassName(TSTR& s)
{
    s = PluginGlobals::GetString(IDS_TEXTURE_UNIT_CLASS_NAME); 
}  

void OgreMaxTextureUnit::DeleteThis() 
{
	delete this;
}

TSTR OgreMaxTextureUnit::SubAnimName(int i) 
{ 
    TSTR name;

    switch (i)
    {
        case RENDER_TEXTURE_REF:
        {
            name = PluginGlobals::GetString(IDS_RENDER_TO_TEXTURE);
            break;
        }
        case PARAM_BLOCK_REF:
        {
            name = PluginGlobals::GetString(IDS_PARAMETERS);
            break;
        }
    }

    return name;
}		

int OgreMaxTextureUnit::NumSubs()
{
    return NUM_REFS; 
}  

Animatable* OgreMaxTextureUnit::SubAnim(int i) 
{
	switch (i)
    {
		case RENDER_TEXTURE_REF: return this->renderTexture;
        case PARAM_BLOCK_REF: return this->paramBlock;
		default: 
        {
            assert(0); 
            return NULL;
        }
	}
}

int OgreMaxTextureUnit::NumRefs()
{
    return NUM_REFS; 
}

RefTargetHandle OgreMaxTextureUnit::GetReference(int i) 
{
	switch (i) 
    {
		case RENDER_TEXTURE_REF: return this->renderTexture;
        case PARAM_BLOCK_REF: return this->paramBlock;
		default: 
        {
            assert(0); 
            return NULL;
        }
	}
}

void OgreMaxTextureUnit::SetReference(int i, RefTargetHandle refTarget) 
{
	switch (i) 
    {
		case RENDER_TEXTURE_REF:	
        {
            this->renderTexture = reinterpret_cast<Texmap*>(refTarget);
            break;
        }
        case PARAM_BLOCK_REF:
        {
            this->paramBlock = reinterpret_cast<IParamBlock*>(refTarget);
            break;
        }
		default: assert(0);
	}
}

ULONG OgreMaxTextureUnit::GetFlags()
{
    return this->flags;
}

void OgreMaxTextureUnit::SetFlag(ULONG f, ULONG value) 
{
	if (value) 
    {
        //Add to the flags
        this->flags |= f; 
    }
	else 
    {
        //Remove the flags
        this->flags &= ~f;
    }
}

TSTR OgreMaxTextureUnit::GetRenderTextureName()
{
    TSTR name;
    if (this->renderTexture != NULL)
        name = this->renderTexture->GetName();
    else
        name = PluginGlobals::GetString(IDS_NONE);
    return name;
}

TSTR OgreMaxTextureUnit::GetBitmapFileName()
{
    TSTR name;
    if (this->bitmapFileName.Length() > 0)
        name = this->bitmapFileName;
    else
        name = PluginGlobals::GetString(IDS_NONE);
    return name;
}

void OgreMaxTextureUnit::AddFrame(TSTR fileName)
{
    this->frameFileNames.push_back(fileName);
}

void OgreMaxTextureUnit::RemoveFrame(int index)
{
    std::vector<TSTR>::iterator it;
    for (it = this->frameFileNames.begin(); it != this->frameFileNames.end(); ++it)
    {
        if (index == 0)
            break;
        index--;
    }
    if (it != this->frameFileNames.end())
        this->frameFileNames.erase(it);
}

const TSTR* OgreMaxTextureUnit::GetFrames()
{
    if (!this->frameFileNames.empty())
        return &this->frameFileNames[0];
    else
        return NULL;
}

int OgreMaxTextureUnit::GetFrameCount()
{
    return (int)this->frameFileNames.size();
}

const TSTR& OgreMaxTextureUnit::GetFrame(int index)
{
    return this->frameFileNames[index];
}

#define TEXTURE_UNIT_HEADER_CHUNK 0x4000
#define TEXTURE_UNIT_FLAGS_CHUNK 0x5000
#define TEXTURE_UNIT_VERSION_CHUNK 0x5300
#define TEXTURE_UNIT_BITMAP_FILENAME_CHUNK 0x5400
#define TEXTURE_UNIT_BITMAP_FILENAME_FRAME_CHUNK 0x5500

#define TEXTURE_UNIT_VERSION 1

IOResult OgreMaxTextureUnit::Save(ISave* isave)
{
    ULONG byteCount;
    IOResult result;

	isave->BeginChunk(TEXTURE_UNIT_HEADER_CHUNK);
	result = MtlBase::Save(isave);
	if (result != IO_OK) 
        return result;
	isave->EndChunk();

    isave->BeginChunk(TEXTURE_UNIT_VERSION_CHUNK);
	int version = TEXTURE_UNIT_VERSION;
	isave->Write(&version, sizeof(version), &byteCount);			
	isave->EndChunk();

	isave->BeginChunk(TEXTURE_UNIT_FLAGS_CHUNK);
	isave->Write(&this->flags, sizeof(this->flags), &byteCount);			
	isave->EndChunk();

    if (this->bitmapFileName.Length() > 0)
    {
        isave->BeginChunk(TEXTURE_UNIT_BITMAP_FILENAME_CHUNK);
	    isave->WriteWString(this->bitmapFileName.data());
	    isave->EndChunk();
    }

    for (size_t i = 0; i < this->frameFileNames.size(); i++)
    {
        isave->BeginChunk(TEXTURE_UNIT_BITMAP_FILENAME_FRAME_CHUNK);
	    isave->WriteWString(this->frameFileNames[i].data());
	    isave->EndChunk();
    }

	return IO_OK;
}

IOResult OgreMaxTextureUnit::Load(ILoad* iload)
{
    ULONG byteCount;
	int id;
	int version = 0;
	IOResult result;

	while ((result = iload->OpenChunk()) == IO_OK) 
    {
		switch (id = iload->CurChunkID())  
        {
			case TEXTURE_UNIT_HEADER_CHUNK:
            {
				result = MtlBase::Load(iload);
				this->ivalid.SetEmpty();
				break;
            }
            case TEXTURE_UNIT_VERSION_CHUNK:
            {
				result = iload->Read(&version, sizeof(version), &byteCount);
				break;
            }
			case TEXTURE_UNIT_FLAGS_CHUNK:
            {
				result = iload->Read(&this->flags, sizeof(this->flags), &byteCount);
				break;
            }
            case TEXTURE_UNIT_BITMAP_FILENAME_CHUNK:
            {
                TCHAR* name;
                result = iload->ReadWStringChunk(&name);
                if (result == IO_OK)
                    this->bitmapFileName = name;
                break;
            }
            case TEXTURE_UNIT_BITMAP_FILENAME_FRAME_CHUNK:
            {
                TCHAR* name;
                result = iload->ReadWStringChunk(&name);
                if (result == IO_OK)
                    this->frameFileNames.push_back(name);
                break;
            }
		}
		iload->CloseChunk();
		if (result != IO_OK) 
			return result;
	}
	
	return IO_OK;
}
