
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "OgreMaxTextureUnitDlg.h"
#include "OgreMaxTextureUnit.h"
#include "ControlUtilities.h"
#include "PluginGlobals.h"
#include "resource.h"

using namespace OgreMax;


//Local functions--------------------------------------------------------------


//Implementation---------------------------------------------------------------
OgreMaxTextureUnitDlg::OgreMaxTextureUnitDlg
    (
    HWND hWndMaterialEdit, 
    IMtlParams* materialParams, 
    OgreMaxTextureUnit* pass
    )
{
    this->hWndMaterialEdit = hWndMaterialEdit;
    this->textureUnitParams = materialParams;
    this->textureUnit = pass;

    this->mainPanel = NULL;
    
    this->displayBitmap = NULL;

    this->dragAndDropManager.Init(this);

    this->colorBlendManualColor1Swatch = NULL;
    this->colorBlendManualColor2Swatch = NULL;

    this->mapChannelSpin = NULL;
    this->colorBlendAlphaSpin = NULL;
    this->alphaBlendAlphaSpin = NULL;
    this->alphaBlendManualAlpha1Spin = NULL;
    this->alphaBlendManualAlpha2Spin = NULL;
    this->framesDurationSpin = NULL;
    this->maxAnisotropySpin = NULL;
    this->transformScrollUSpin = NULL;
    this->transformScrollVSpin = NULL;
    this->transformScaleUSpin = NULL;
    this->transformScaleVSpin = NULL;
    this->transformRotateSpin = NULL;
    this->transformAnimationScrollUSpin = NULL;
    this->transformAnimationScrollVSpin = NULL;
    this->transformAnimationRotateSpin = NULL;
    this->transformAnimationBaseSpin = NULL;
    this->transformAnimationFrequencySpin = NULL;
    this->transformAnimationPhaseSpin = NULL;
    this->transformAnimationAmplitudeSpin = NULL;

    this->currentTime = 0;

    this->initialized = false;
}

OgreMaxTextureUnitDlg::~OgreMaxTextureUnitDlg()
{
    //Capture some final rollup values
    this->textureUnit->SetFlag
        (
        OgreMaxTextureUnit::ROLLUP_OPEN, 
        this->textureUnitParams->IsRollupPanelOpen(this->mainPanel)
        );
	this->textureUnit->rollScroll = this->textureUnitParams->GetRollupScrollPos();

    //Remove the parameter dialog
	this->textureUnit->SetParamDlg(NULL);

    //Delete the display bitmap
    if (this->displayBitmap != NULL)
        this->displayBitmap->DeleteThis();                            

    //Release color swatch controls
    ReleaseIColorSwatch(this->colorBlendManualColor1Swatch);
    ReleaseIColorSwatch(this->colorBlendManualColor2Swatch);

    //Release spinner controls
    ReleaseISpinner(this->mapChannelSpin);
    ReleaseISpinner(this->colorBlendAlphaSpin);
    ReleaseISpinner(this->alphaBlendAlphaSpin);
    ReleaseISpinner(this->alphaBlendManualAlpha1Spin);
    ReleaseISpinner(this->alphaBlendManualAlpha2Spin);
    ReleaseISpinner(this->framesDurationSpin);
    ReleaseISpinner(this->maxAnisotropySpin);
    ReleaseISpinner(this->transformScrollUSpin);
    ReleaseISpinner(this->transformScrollVSpin);
    ReleaseISpinner(this->transformScaleUSpin);
    ReleaseISpinner(this->transformScaleVSpin);
    ReleaseISpinner(this->transformRotateSpin);
    ReleaseISpinner(this->transformAnimationScrollUSpin);
    ReleaseISpinner(this->transformAnimationScrollVSpin);
    ReleaseISpinner(this->transformAnimationRotateSpin);
    ReleaseISpinner(this->transformAnimationBaseSpin);
    ReleaseISpinner(this->transformAnimationFrequencySpin);
    ReleaseISpinner(this->transformAnimationPhaseSpin);
    ReleaseISpinner(this->transformAnimationAmplitudeSpin);

    //Remove the user data that we previously set
	SetWindowLongPtr(this->mainPanel, GWLP_USERDATA, NULL);	
}

Class_ID OgreMaxTextureUnitDlg::ClassID() 
{
    return OgreMaxTextureUnit::GetClassClassID();  
}

void OgreMaxTextureUnitDlg::SetThing(ReferenceTarget* refTarget)
{
    assert(refTarget->SuperClassID() == TEXMAP_CLASS_ID);
    assert(refTarget->ClassID() == OgreMaxTextureUnit::GetClassClassID());

	if (this->textureUnit) 
		this->textureUnit->SetParamDlg(NULL);
	
    this->textureUnit = static_cast<OgreMaxTextureUnit*>(refTarget);
    if (this->textureUnit)
	    this->textureUnit->SetParamDlg(this);
	
    LoadDialog();	
}

ReferenceTarget* OgreMaxTextureUnitDlg::GetThing() 
{ 
    return this->textureUnit; 
}

void OgreMaxTextureUnitDlg::DeleteThis() 
{ 
    delete this;  
}	

void OgreMaxTextureUnitDlg::SetTime(TimeValue t)
{
    if (t != this->currentTime) 
    {
		this->currentTime = t;
		Interval v;
        this->textureUnit->Update(this->textureUnitParams->GetTime(), v);
		LoadDialog();
	}
}

int OgreMaxTextureUnitDlg::FindSubTexFromHWND(HWND hWnd)
{
    if (GetDlgItem(this->mainPanel, IDC_RENDER_TEXTURE_BUTTON) == hWnd)
        return OgreMaxTextureUnit::RENDER_TEXTURE_SUBTEXMAP;
    
	return -1;
}

void OgreMaxTextureUnitDlg::ReloadDialog()
{
    Interval v;
	this->textureUnit->Update(this->textureUnitParams->GetTime(), v);
	LoadDialog();
}

void OgreMaxTextureUnitDlg::ActivateDlg(BOOL activate)
{
}

void OgreMaxTextureUnitDlg::LoadDialog()
{
    if (this->textureUnit != NULL && this->mainPanel != NULL)
    {        
        UpdateRenderTextureDisplay();
        UpdateBitmapName();
        ControlUtilities::FillListBox
            (
            GetDlgItem(this->mainPanel, IDC_FRAMES_LIST), 
            this->textureUnit->GetFrames(), 
            this->textureUnit->GetFrameCount()
            );
        
        SendMessage(GetDlgItem(this->mainPanel, IDC_BITMAP_DIMENSION_COMBO), CB_SETCURSEL, this->textureUnit->GetBitmapDimension(), 0);
		this->mapChannelSpin->SetValue(this->textureUnit->GetMapChannel(), FALSE);
        SendMessage(GetDlgItem(this->mainPanel, IDC_COLOR_BLEND_FUNCTION_COMBO), CB_SETCURSEL, this->textureUnit->GetColorBlendFunction(), 0);
        SendMessage(GetDlgItem(this->mainPanel, IDC_COLOR_BLEND_SOURCE_COMBO), CB_SETCURSEL, this->textureUnit->GetColorBlendSource(), 0);
        SendMessage(GetDlgItem(this->mainPanel, IDC_COLOR_BLEND_DESTINATION_COMBO), CB_SETCURSEL, this->textureUnit->GetColorBlendDestination(), 0);
        this->colorBlendAlphaSpin->SetValue(this->textureUnit->GetColorBlendAlpha(), FALSE);
        this->colorBlendManualColor1Swatch->SetColor(this->textureUnit->GetColorBlendManualColor1(), FALSE);
        this->colorBlendManualColor2Swatch->SetColor(this->textureUnit->GetColorBlendManualColor2(), FALSE);
        SendMessage(GetDlgItem(this->mainPanel, IDC_ALPHA_BLEND_FUNCTION_COMBO), CB_SETCURSEL, this->textureUnit->GetAlphaBlendFunction(), 0);
        SendMessage(GetDlgItem(this->mainPanel, IDC_ALPHA_BLEND_SOURCE_COMBO), CB_SETCURSEL, this->textureUnit->GetAlphaBlendSource(), 0);
        SendMessage(GetDlgItem(this->mainPanel, IDC_ALPHA_BLEND_DESTINATION_COMBO), CB_SETCURSEL, this->textureUnit->GetAlphaBlendDestination(), 0);
        this->alphaBlendAlphaSpin->SetValue(this->textureUnit->GetAlphaBlendAlpha(), FALSE);
        this->alphaBlendManualAlpha1Spin->SetValue(this->textureUnit->GetAlphaBlendManualAlpha1(), FALSE);
        this->alphaBlendManualAlpha2Spin->SetValue(this->textureUnit->GetAlphaBlendManualAlpha2(), FALSE);
        SendMessage(GetDlgItem(this->mainPanel, IDC_LAYER_BLEND_FALLBACK_SOURCE_COMBO), CB_SETCURSEL, this->textureUnit->GetLayerBlendFallbackSource(), 0);
        SendMessage(GetDlgItem(this->mainPanel, IDC_LAYER_BLEND_FALLBACK_DESTINATION_COMBO), CB_SETCURSEL, this->textureUnit->GetLayerBlendFallbackDestination(), 0);        
        this->framesDurationSpin->SetValue(this->textureUnit->GetFramesDuration(), FALSE);
        SendMessage(GetDlgItem(this->mainPanel, IDC_ENVIRONMENT_MAPPING_COMBO), CB_SETCURSEL, this->textureUnit->GetEnvironmentMapping(), 0);
        SendMessage(GetDlgItem(this->mainPanel, IDC_ADDRESSING_MODE_COMBO), CB_SETCURSEL, this->textureUnit->GetAddressingMode(), 0);
        SendMessage(GetDlgItem(this->mainPanel, IDC_FILTERING_MODE_COMBO), CB_SETCURSEL, this->textureUnit->GetFilteringMode(), 0);
        this->maxAnisotropySpin->SetValue(this->textureUnit->GetMaxAnisotropy(), FALSE);
        this->transformScrollUSpin->SetValue(this->textureUnit->GetTransformScrollU(), FALSE);
        this->transformScrollVSpin->SetValue(this->textureUnit->GetTransformScrollV(), FALSE);
        this->transformScaleUSpin->SetValue(this->textureUnit->GetTransformScaleU(), FALSE);
        this->transformScaleVSpin->SetValue(this->textureUnit->GetTransformScaleV(), FALSE);
        this->transformRotateSpin->SetValue(this->textureUnit->GetTransformRotate(), FALSE); 
        this->transformAnimationScrollUSpin->SetValue(this->textureUnit->GetTransformAnimationScrollU(), FALSE);
        this->transformAnimationScrollVSpin->SetValue(this->textureUnit->GetTransformAnimationScrollV(), FALSE);
        this->transformAnimationRotateSpin->SetValue(this->textureUnit->GetTransformAnimationRotate(), FALSE);
        SendMessage(GetDlgItem(this->mainPanel, IDC_TRANSFORM_ANIMATION_TRANSFORM_TYPE_COMBO), CB_SETCURSEL, this->textureUnit->GetTransformAnimationType(), 0);
        SendMessage(GetDlgItem(this->mainPanel, IDC_TRANSFORM_ANIMATION_WAVE_TYPE_COMBO), CB_SETCURSEL, this->textureUnit->GetTransformAnimationWaveType(), 0);
        this->transformAnimationBaseSpin->SetValue(this->textureUnit->GetTransformAnimationBase(), FALSE);
        this->transformAnimationFrequencySpin->SetValue(this->textureUnit->GetTransformAnimationFrequency(), FALSE);
        this->transformAnimationPhaseSpin->SetValue(this->textureUnit->GetTransformAnimationPhase(), FALSE);
        this->transformAnimationAmplitudeSpin->SetValue(this->textureUnit->GetTransformAnimationAmplitude(), FALSE);        
	}
}

void OgreMaxTextureUnitDlg::BuildDialog() 
{
    this->mainPanel = 
	    this->textureUnitParams->AddRollupPage
            ( 
            PluginGlobals::GetInstanceHandle(),
		    MAKEINTRESOURCE(IDD_TEXTURE_UNIT_MAIN_ROLLUP),
		    MainPanelDlgProc, 
		    PluginGlobals::GetString(IDS_TEXTURE_UNIT_PARAMETERS), 
		    reinterpret_cast<LPARAM>(this),
            this->textureUnit->flags & OgreMaxTextureUnit::ROLLUP_OPEN ? 0 : APPENDROLL_CLOSED
		    );

    this->textureUnitParams->SetRollupScrollPos(this->textureUnit->rollScroll);    
}

void OgreMaxTextureUnitDlg::UpdateRenderTextureDisplay() 
{
    if (this->textureUnit != NULL)
    {
        //Set button text
        TSTR name = this->textureUnit->GetRenderTextureName();
        SetDlgItemText(this->mainPanel, IDC_RENDER_TEXTURE_BUTTON, name.data());        
    }
}

void OgreMaxTextureUnitDlg::UpdateBitmapName()
{
    TSTR name = this->textureUnit->GetBitmapFileName();
    ControlUtilities::SetText(GetDlgItem(this->mainPanel, IDC_BITMAP_BUTTON), name.data());
}

void OgreMaxTextureUnitDlg::Invalidate()
{
    InvalidateRect(this->mainPanel, NULL, 0);    
}

BOOL OgreMaxTextureUnitDlg::MainPanelProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
    int id = LOWORD(wParam);
	int code = HIWORD(wParam);
    switch (message)
    {
		case WM_INITDIALOG:
		{
            this->colorBlendManualColor1Swatch = GetIColorSwatch(GetDlgItem(hDlg, IDC_COLOR_BLEND_MANUAL_COLOR1));
            this->colorBlendManualColor2Swatch = GetIColorSwatch(GetDlgItem(hDlg, IDC_COLOR_BLEND_MANUAL_COLOR2));

            this->mapChannelSpin = SetupIntSpinner(hDlg, IDC_MAP_CHANNEL_SPIN, IDC_MAP_CHANNEL_EDIT, 0, 20, 0);
            this->colorBlendAlphaSpin = SetupFloatSpinner(hDlg, IDC_COLOR_BLEND_ALPHA_SPIN, IDC_COLOR_BLEND_ALPHA_EDIT, 0.0f, 1.0f, 0.0f, 0.01f);
            this->alphaBlendAlphaSpin = SetupFloatSpinner(hDlg, IDC_ALPHA_BLEND_ALPHA_SPIN, IDC_ALPHA_BLEND_ALPHA_EDIT, 0.0f, 1.0f, 0.0f, 0.01f);
            this->alphaBlendManualAlpha1Spin = SetupFloatSpinner(hDlg, IDC_ALPHA_BLEND_MANUAL_ALPHA1_SPIN, IDC_ALPHA_BLEND_MANUAL_ALPHA1_EDIT, 0.0f, 1.0f, 1.0f, 0.01f);
            this->alphaBlendManualAlpha2Spin = SetupFloatSpinner(hDlg, IDC_ALPHA_BLEND_MANUAL_ALPHA2_SPIN, IDC_ALPHA_BLEND_MANUAL_ALPHA2_EDIT, 0.0f, 1.0f, 1.0f, 0.01f);
            this->framesDurationSpin = SetupFloatSpinner(hDlg, IDC_FRAMES_DURATION_SPIN, IDC_FRAMES_DURATION_EDIT, 0.0f, 100.0f, 1.0f, 1.0f);
            this->maxAnisotropySpin = SetupIntSpinner(hDlg, IDC_MAX_ANISOTROPY_SPIN, IDC_MAX_ANISOTROPY_EDIT, 0, 20, 1);
            this->transformScrollUSpin = SetupFloatSpinner(hDlg, IDC_TRANSFORM_SCROLL_U_SPIN, IDC_TRANSFORM_SCROLL_U_EDIT, 0.0f, 100.0f, 0.0f, 0.01f);
            this->transformScrollVSpin = SetupFloatSpinner(hDlg, IDC_TRANSFORM_SCROLL_V_SPIN, IDC_TRANSFORM_SCROLL_V_EDIT, 0.0f, 100.0f, 0.0f, 0.01f);
            this->transformScaleUSpin = SetupFloatSpinner(hDlg, IDC_TRANSFORM_SCALE_U_SPIN, IDC_TRANSFORM_SCALE_U_EDIT, 0.0f, 100.0f, 1.0f, 0.1f);
            this->transformScaleVSpin = SetupFloatSpinner(hDlg, IDC_TRANSFORM_SCALE_V_SPIN, IDC_TRANSFORM_SCALE_V_EDIT, 0.0f, 100.0f, 1.0f, 0.1f);
            this->transformRotateSpin = SetupFloatSpinner(hDlg, IDC_TRANSFORM_ROTATE_SPIN, IDC_TRANSFORM_ROTATE_EDIT, 0.0f, 100.0f, 0.0f, 0.01f);
            this->transformAnimationScrollUSpin = SetupFloatSpinner(hDlg, IDC_TRANSFORM_ANIMATION_SCROLL_U_SPIN, IDC_TRANSFORM_ANIMATION_SCROLL_U_EDIT, 0.0f, 100.0f, 0.0f, 0.01f);
            this->transformAnimationScrollVSpin = SetupFloatSpinner(hDlg, IDC_TRANSFORM_ANIMATION_SCROLL_V_SPIN, IDC_TRANSFORM_ANIMATION_SCROLL_V_EDIT, 0.0f, 100.0f, 0.0f, 0.01f);
            this->transformAnimationRotateSpin = SetupFloatSpinner(hDlg, IDC_TRANSFORM_ANIMATION_ROTATE_SPIN, IDC_TRANSFORM_ANIMATION_ROTATE_EDIT, 0.0f, 100.0f, 0.0f, 0.01f);
            this->transformAnimationBaseSpin = SetupFloatSpinner(hDlg, IDC_TRANSFORM_ANIMATION_BASE_SPIN, IDC_TRANSFORM_ANIMATION_BASE_EDIT, 0.0f, 100.0f, 1.0f, 1.0f);
            this->transformAnimationFrequencySpin = SetupFloatSpinner(hDlg, IDC_TRANSFORM_ANIMATION_FREQUENCY_SPIN, IDC_TRANSFORM_ANIMATION_FREQUENCY_EDIT, 0.0f, 100.0f, 0.2f, 0.1f);
            this->transformAnimationPhaseSpin = SetupFloatSpinner(hDlg, IDC_TRANSFORM_ANIMATION_PHASE_SPIN, IDC_TRANSFORM_ANIMATION_PHASE_EDIT, 0.0f, 100.0f, 0.0f, 0.1f);
            this->transformAnimationAmplitudeSpin = SetupFloatSpinner(hDlg, IDC_TRANSFORM_ANIMATION_AMPLITUDE_SPIN, IDC_TRANSFORM_ANIMATION_AMPLITUDE_EDIT, 0.0f, 100.0f, 0.0f, 0.1f);
            
            ControlUtilities::FillBitmapDimensionComboBox(GetDlgItem(hDlg, IDC_BITMAP_DIMENSION_COMBO));
            ControlUtilities::FillColorAlphaBlendFunctionComboBox(GetDlgItem(hDlg, IDC_COLOR_BLEND_FUNCTION_COMBO));
            ControlUtilities::FillColorAlphaBlendSourceComboBox(GetDlgItem(hDlg, IDC_COLOR_BLEND_SOURCE_COMBO));
            ControlUtilities::FillColorAlphaBlendDestinationComboBox(GetDlgItem(hDlg, IDC_COLOR_BLEND_DESTINATION_COMBO));
            ControlUtilities::FillColorAlphaBlendFunctionComboBox(GetDlgItem(hDlg, IDC_ALPHA_BLEND_FUNCTION_COMBO));
            ControlUtilities::FillColorAlphaBlendSourceComboBox(GetDlgItem(hDlg, IDC_ALPHA_BLEND_SOURCE_COMBO));
            ControlUtilities::FillColorAlphaBlendDestinationComboBox(GetDlgItem(hDlg, IDC_ALPHA_BLEND_DESTINATION_COMBO));
            ControlUtilities::FillBlendFactorComboBox(GetDlgItem(hDlg, IDC_LAYER_BLEND_FALLBACK_SOURCE_COMBO));
            ControlUtilities::FillBlendFactorComboBox(GetDlgItem(hDlg, IDC_LAYER_BLEND_FALLBACK_DESTINATION_COMBO));            
            ControlUtilities::FillEnvironmentMappingComboBox(GetDlgItem(hDlg, IDC_ENVIRONMENT_MAPPING_COMBO));
            ControlUtilities::FillAddressingModeComboBox(GetDlgItem(hDlg, IDC_ADDRESSING_MODE_COMBO));
            ControlUtilities::FillFilteringModeComboBox(GetDlgItem(hDlg, IDC_FILTERING_MODE_COMBO));            
            ControlUtilities::FillTextureTransformAnimationTransformTypeComboBox(GetDlgItem(hDlg, IDC_TRANSFORM_ANIMATION_TRANSFORM_TYPE_COMBO));
            ControlUtilities::FillTextureTransformAnimationWaveTypeComboBox(GetDlgItem(hDlg, IDC_TRANSFORM_ANIMATION_WAVE_TYPE_COMBO));
            
			return TRUE;
		}			
		case WM_COMMAND: 
		{ 
            switch (id) 
            {
                case IDC_BITMAP_BUTTON:
                {                    
                    //Show the dialog box to choose an image
                    BitmapInfo bitmapInfo;
                    BOOL result = 
                        TheManager->SelectFileInputEx
                            (
                            &bitmapInfo, 
                            GetCOREInterface()->GetMAXHWnd(), 
                            PluginGlobals::GetString(IDS_SELECT_BITMAP_FILE)
                            );

                    if (result)
                    {
                        //Update the OgreMaxTextureUnit file name
                        this->textureUnit->bitmapFileName = bitmapInfo.Name();

                        //Update the bitmap's display in the UI
                        UpdateBitmapName();
                    }
                    
                    break;
                }
                case IDC_BITMAP_RELOAD_BUTTON:
                {
                    //TODO: Reload the bitmap and notify the rest of the system that it was 
                    //reloaded.
                    break;
                }
                case IDC_BITMAP_VIEW_BUTTON:
                {
                    if (this->textureUnit->bitmapFileName.Length() > 0)
                    {
                        //There is a file name specified                        
                        
                        BitmapInfo bitmapInfo(this->textureUnit->bitmapFileName.data());
                        if (bitmapInfo.Width() > 0)
                        {
                            //The file exists

                            //Delete the existing bitmap
                            if (this->displayBitmap != NULL)
                                this->displayBitmap->DeleteThis();
                            
                            //Load the new one
                            this->displayBitmap = TheManager->Load(&bitmapInfo);
                            
                            //Display it if all is well
                            if (this->displayBitmap != NULL)
                                this->displayBitmap->Display();
                        }
                        else
                        {
                            //Show an error message box
                            MessageBox
                                (
                                GetActiveWindow(), 
                                PluginGlobals::GetString(IDS_BITMAP_FILE_NOT_FOUND), 
                                PluginGlobals::GetString2(IDS_ERROR_VIEWING_BITMAP), 
                                MB_OK | MB_ICONEXCLAMATION
                                );
                        }
                    }
                    break;
                }
                case IDC_FRAMES_ADD_BUTTON:
                {
                    BitmapInfo bitmapInfo;
                    BOOL result = 
                        TheManager->SelectFileInputEx
                            (
                            &bitmapInfo, 
                            GetCOREInterface()->GetMAXHWnd(), 
                            PluginGlobals::GetString(IDS_SELECT_BITMAP_FILE)
                            );
                    if (result)
                    {
                        this->textureUnit->AddFrame(bitmapInfo.Name());
                        SendMessage(GetDlgItem(this->mainPanel, IDC_FRAMES_LIST), LB_ADDSTRING, 0, reinterpret_cast<LPARAM>(bitmapInfo.Name())); 
                    }
                    break;
                }
                case IDC_FRAMES_REMOVE_BUTTON:
                {
                    int curSel = SendMessage(GetDlgItem(this->mainPanel, IDC_FRAMES_LIST), LB_GETCURSEL, 0, 0);
                    if (curSel >= 0)
                    {
                        this->textureUnit->RemoveFrame(curSel);
                        SendMessage(GetDlgItem(this->mainPanel, IDC_FRAMES_LIST), LB_DELETESTRING, curSel, 0);
                    }

                    break;
                }
                case IDC_RENDER_TEXTURE_BUTTON:
                {
                    PostMessage(this->hWndMaterialEdit, WM_TEXMAP_BUTTON, 0, (LPARAM)this->textureUnit);
                    break;
                }
                case IDC_BITMAP_DIMENSION_COMBO:
                {
                    this->textureUnit->SetBitmapDimension(SendMessage(GetDlgItem(hDlg, IDC_BITMAP_DIMENSION_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_COLOR_BLEND_FUNCTION_COMBO:
                {
                    this->textureUnit->SetColorBlendFunction(SendMessage(GetDlgItem(hDlg, IDC_COLOR_BLEND_FUNCTION_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_COLOR_BLEND_SOURCE_COMBO:
                {
                    this->textureUnit->SetColorBlendSource(SendMessage(GetDlgItem(hDlg, IDC_COLOR_BLEND_SOURCE_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_COLOR_BLEND_DESTINATION_COMBO:
                {                    
                    this->textureUnit->SetColorBlendDestination(SendMessage(GetDlgItem(hDlg, IDC_COLOR_BLEND_DESTINATION_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_ALPHA_BLEND_FUNCTION_COMBO:
                {                    
                    this->textureUnit->SetAlphaBlendFunction(SendMessage(GetDlgItem(hDlg, IDC_ALPHA_BLEND_FUNCTION_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_ALPHA_BLEND_SOURCE_COMBO:
                {
                    this->textureUnit->SetAlphaBlendSource(SendMessage(GetDlgItem(hDlg, IDC_ALPHA_BLEND_SOURCE_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_ALPHA_BLEND_DESTINATION_COMBO:
                {
                    this->textureUnit->SetAlphaBlendDestination(SendMessage(GetDlgItem(hDlg, IDC_ALPHA_BLEND_DESTINATION_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_LAYER_BLEND_FALLBACK_SOURCE_COMBO:
                {
                    this->textureUnit->SetLayerBlendFallbackSource(SendMessage(GetDlgItem(hDlg, IDC_LAYER_BLEND_FALLBACK_SOURCE_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_LAYER_BLEND_FALLBACK_DESTINATION_COMBO:
                {
                    this->textureUnit->SetLayerBlendFallbackDestination(SendMessage(GetDlgItem(hDlg, IDC_LAYER_BLEND_FALLBACK_DESTINATION_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_ENVIRONMENT_MAPPING_COMBO:
                {
                    this->textureUnit->SetEnvironmentMapping(SendMessage(GetDlgItem(hDlg, IDC_ENVIRONMENT_MAPPING_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_ADDRESSING_MODE_COMBO:
                {
                    this->textureUnit->SetAddressingMode(SendMessage(GetDlgItem(hDlg, IDC_ADDRESSING_MODE_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_FILTERING_MODE_COMBO:
                {
                    this->textureUnit->SetFilteringMode(SendMessage(GetDlgItem(hDlg, IDC_FILTERING_MODE_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_TRANSFORM_ANIMATION_TRANSFORM_TYPE_COMBO:
                {
                    this->textureUnit->SetTransformAnimationType(SendMessage(GetDlgItem(hDlg, IDC_TRANSFORM_ANIMATION_TRANSFORM_TYPE_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }
                case IDC_TRANSFORM_ANIMATION_WAVE_TYPE_COMBO:
                {
                    this->textureUnit->SetTransformAnimationWaveType(SendMessage(GetDlgItem(hDlg, IDC_TRANSFORM_ANIMATION_WAVE_TYPE_COMBO), CB_GETCURSEL, 0, 0));
                    break;
                }                
			}
            
            return TRUE;
        }
        case CC_COLOR_SEL:
		{
			id = LOWORD(wParam);
			SaveEditedColor(id);
            break;
		}	
		case CC_COLOR_DROP:
		{
			id = LOWORD(wParam);
			SaveEditedColor(id);
            break;
		}			
		case CC_COLOR_BUTTONDOWN:            
        {
			theHold.Begin();
			break;
        }
		case CC_COLOR_BUTTONUP:
        {
			if (HIWORD(wParam)) 
                theHold.Accept(PluginGlobals::GetString(IDS_PARAMETER_CHANGE));
			else 
                theHold.Cancel();
			
            break;
        }
		case CC_COLOR_CHANGE:
		{			
			id = LOWORD(wParam);			
            int buttonUp = HIWORD(wParam); 			

			if (buttonUp) 
                theHold.Begin();
            
            SaveEditedColor(id);
			
            if (buttonUp) 
                theHold.Accept(PluginGlobals::GetString(IDS_PARAMETER_CHANGE));

			break;
        }
        case CC_SPINNER_CHANGE: 
        {
			if (!theHold.Holding()) 
                theHold.Begin();

            SaveEditedSpinner(id);
            
			break;
        }
		case CC_SPINNER_BUTTONDOWN:            
        {
			theHold.Begin();
			break;
        }
		case WM_CUSTEDIT_ENTER:
		case CC_SPINNER_BUTTONUP: 
        {
			if (HIWORD(wParam) || message == WM_CUSTEDIT_ENTER) 
                theHold.Accept(PluginGlobals::GetString(IDS_PARAMETER_CHANGE));
			else 
				theHold.Cancel();
			
            break;
    	}
		case WM_PAINT:
        {
            if (!this->initialized)
            {
				this->initialized = true;
				ReloadDialog();
			}

			return FALSE;
        }
    }

	return FALSE;
}

INT_PTR CALLBACK OgreMaxTextureUnitDlg::MainPanelDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam) 
{
    OgreMaxTextureUnitDlg* dialog = NULL;
	if (message == WM_INITDIALOG) 
    {
		dialog = reinterpret_cast<OgreMaxTextureUnitDlg*>(lParam);
		dialog->mainPanel = hWnd;
		SetWindowLongPtr(hWnd, GWLP_USERDATA, lParam);
	}
	else 
    {
        dialog = reinterpret_cast<OgreMaxTextureUnitDlg*>(GetWindowLongPtr(hWnd, GWLP_USERDATA));
	    if (dialog == NULL)
			return FALSE; 
	}
	
    return dialog->MainPanelProc(hWnd, message, wParam, lParam);
}

void OgreMaxTextureUnitDlg::SaveEditedColor(int controlID) 
{
    switch (controlID)
    {
        case IDC_COLOR_BLEND_MANUAL_COLOR1:
        {
            this->colorBlendManualColor1Swatch->EditThis(FALSE);
            this->textureUnit->SetColorBlendManualColor1(Color(this->colorBlendManualColor1Swatch->GetColor()));            
            break;
        }
        case IDC_COLOR_BLEND_MANUAL_COLOR2:
        {
            this->colorBlendManualColor2Swatch->EditThis(FALSE);
            this->textureUnit->SetColorBlendManualColor2(Color(this->colorBlendManualColor2Swatch->GetColor()));
            break;
        }
    }        
}

void OgreMaxTextureUnitDlg::SaveEditedSpinner(int controlID)
{
    switch (controlID) 
    {
		case IDC_MAP_CHANNEL_SPIN: 
        {
            this->textureUnit->SetMapChannel(this->mapChannelSpin->GetIVal());
            break;
        }
		case IDC_COLOR_BLEND_ALPHA_SPIN: 
        {
            this->textureUnit->SetColorBlendAlpha(this->colorBlendAlphaSpin->GetFVal());
            break;
        }
		case IDC_ALPHA_BLEND_ALPHA_SPIN: 
        {
            this->textureUnit->SetAlphaBlendAlpha(this->alphaBlendAlphaSpin->GetFVal());
            break;
        }
		case IDC_ALPHA_BLEND_MANUAL_ALPHA1_SPIN: 
        {
            this->textureUnit->SetAlphaBlendManualAlpha1(this->alphaBlendManualAlpha1Spin->GetFVal());
            break;
        }
		case IDC_ALPHA_BLEND_MANUAL_ALPHA2_SPIN: 
        {
            this->textureUnit->SetAlphaBlendManualAlpha2(this->alphaBlendManualAlpha2Spin->GetFVal());
            break;
        }
		case IDC_FRAMES_DURATION_SPIN: 
        {
            this->textureUnit->SetFramesDuration(this->framesDurationSpin->GetFVal());
            break;
        }
		case IDC_MAX_ANISOTROPY_SPIN: 
        {
			this->textureUnit->SetMaxAnisotropy(this->maxAnisotropySpin->GetIVal());
			break;
        }
        case IDC_TRANSFORM_SCROLL_U_SPIN:
        {
            this->textureUnit->SetTransformScrollU(this->transformScrollUSpin->GetFVal());
            break;
        }				
        case IDC_TRANSFORM_SCROLL_V_SPIN:
        {
            this->textureUnit->SetTransformScrollV(this->transformScrollVSpin->GetFVal());
            break;
        }
        case IDC_TRANSFORM_SCALE_U_SPIN:
        {
            this->textureUnit->SetTransformScaleU(this->transformScaleUSpin->GetFVal());
            break;
        }
        case IDC_TRANSFORM_SCALE_V_SPIN:
        {
            this->textureUnit->SetTransformScaleV(this->transformScaleVSpin->GetFVal());
            break;
        }
        case IDC_TRANSFORM_ROTATE_SPIN:
        {
            this->textureUnit->SetTransformRotate(this->transformRotateSpin->GetFVal());
            break;
        }
        case IDC_TRANSFORM_ANIMATION_SCROLL_U_SPIN:
        {
            this->textureUnit->SetTransformAnimationScrollU(this->transformAnimationScrollUSpin->GetFVal());
            break;
        }
        case IDC_TRANSFORM_ANIMATION_SCROLL_V_SPIN:
        {
            this->textureUnit->SetTransformAnimationScrollV(this->transformAnimationScrollVSpin->GetFVal());
            break;
        }
        case IDC_TRANSFORM_ANIMATION_ROTATE_SPIN:
        {
            this->textureUnit->SetTransformAnimationRotate(this->transformAnimationRotateSpin->GetFVal());
            break;
        }
        case IDC_TRANSFORM_ANIMATION_BASE_SPIN:
        {
            this->textureUnit->SetTransformAnimationBase(this->transformAnimationBaseSpin->GetFVal());
            break;
        }
        case IDC_TRANSFORM_ANIMATION_FREQUENCY_SPIN:
        {
            this->textureUnit->SetTransformAnimationFrequency(this->transformAnimationFrequencySpin->GetFVal());
            break;
        }
        case IDC_TRANSFORM_ANIMATION_PHASE_SPIN:
        {
            this->textureUnit->SetTransformAnimationPhase(this->transformAnimationPhaseSpin->GetFVal());
            break;
        }
        case IDC_TRANSFORM_ANIMATION_AMPLITUDE_SPIN:
        {
            this->textureUnit->SetTransformAnimationAmplitude(this->transformAnimationAmplitudeSpin->GetFVal());
            break;
        }
	}
}