#ifndef OgreMax_OgreMaxTextureUnitDlg_INCLUDED
#define OgreMax_OgreMaxTextureUnitDlg_INCLUDED


//Includes---------------------------------------------------------------------
#include "OgreMaxTextureUnits.h"


//Classes----------------------------------------------------------------------
namespace OgreMax
{
    class OgreMaxTextureUnit;

    /**
     * Manages the rollup(s) for an OgreMaxTextureUnit in the 3DS Max material editor.
     */
    class OgreMaxTextureUnitDlg : public ParamDlg 
    {
    public:
        OgreMaxTextureUnitDlg(HWND hWndMaterialEdit, IMtlParams* passParams, OgreMaxTextureUnit* pass); 
	    ~OgreMaxTextureUnitDlg();

	    Class_ID ClassID();
        void SetThing(ReferenceTarget* refTarget);
	    ReferenceTarget* GetThing();
	    void DeleteThis();
	    void SetTime(TimeValue t);
	    int FindSubTexFromHWND(HWND hWnd);
        
        /** Updates material and puts parameters into dialog. */
        void ReloadDialog();

        /** Called to notify the dialog it's being activated/deactiviated. */
        void ActivateDlg(BOOL activate);

        /**
         * Puts parameters into dialog.
         */
        void LoadDialog();

        /**
         * Loads the dialog resources and puts them into the dialog.
         */
        void BuildDialog();

        /**
         * Updates the information for the render texture in the UI.
         */
        void UpdateRenderTextureDisplay();

        /**
         * Updates the information for the bitmap name in the UI.
         */
        void UpdateBitmapName();

        /**
         * Invalidates all the rollup panels.
         */
        void Invalidate();

    private:
        BOOL MainPanelProc(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);

        static INT_PTR CALLBACK MainPanelDlgProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);

        void SaveEditedColor(int controlID);
        void SaveEditedSpinner(int controlID);

    private:
        HWND hWndMaterialEdit;
        IMtlParams* textureUnitParams;
        OgreMaxTextureUnit* textureUnit;

        HWND mainPanel;

        Bitmap* displayBitmap;
        
        TexDADMgr dragAndDropManager;	

        IColorSwatch* colorBlendManualColor1Swatch;
        IColorSwatch* colorBlendManualColor2Swatch;

        ISpinnerControl* mapChannelSpin;
        ISpinnerControl* colorBlendAlphaSpin;
        ISpinnerControl* alphaBlendAlphaSpin;
        ISpinnerControl* alphaBlendManualAlpha1Spin;
        ISpinnerControl* alphaBlendManualAlpha2Spin;
        ISpinnerControl* framesDurationSpin;
        ISpinnerControl* maxAnisotropySpin;
        ISpinnerControl* transformScrollUSpin;
        ISpinnerControl* transformScrollVSpin;
        ISpinnerControl* transformScaleUSpin;
        ISpinnerControl* transformScaleVSpin;
        ISpinnerControl* transformRotateSpin;
        ISpinnerControl* transformAnimationScrollUSpin;
        ISpinnerControl* transformAnimationScrollVSpin;
        ISpinnerControl* transformAnimationRotateSpin;
        ISpinnerControl* transformAnimationBaseSpin;
        ISpinnerControl* transformAnimationFrequencySpin;
        ISpinnerControl* transformAnimationPhaseSpin;
        ISpinnerControl* transformAnimationAmplitudeSpin;

        TimeValue currentTime;

        bool initialized;
    };
}


#endif
