
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "OgreMaxTextureUnits.h"
#include "OgreMaxGlobalUtilities.h"
#include "PluginGlobals.h"
#include "resource.h"

using namespace OgreMax;


//Local classes----------------------------------------------------------------
class OgreMaxTextureUnitsClassDesc : public ClassDesc 
{
public:
	int IsPublic()              {return 0;}
	void* Create(BOOL loading)  {return new OgreMaxTextureUnits(NULL);}
    const TCHAR* ClassName()    {return PluginGlobals::GetString(IDS_TEXTURE_UNITS_CLASS_NAME);}
	SClass_ID SuperClassID()    {return TEXMAP_CONTAINER_CLASS_ID;}
    Class_ID ClassID()          {return OgreMaxTextureUnits::GetClassClassID();}
    const TCHAR* Category()     {return PluginGlobals::GetString(IDS_PLUGINS_CATEGORY);}

public:
    static OgreMaxTextureUnitsClassDesc instance;
};
OgreMaxTextureUnitsClassDesc OgreMaxTextureUnitsClassDesc::instance;


//Implementation---------------------------------------------------------------

//TextureUnitSlot implementation-----------------------------
TextureUnitSlot::TextureUnitSlot() 
{ 
    this->map = NULL;
    this->on = TRUE;
}

void TextureUnitSlot::Update(TimeValue t, Interval& ivalid) 
{
	if (IsOn() && HasMap()) 
		this->map->Update(t, ivalid);
}

BOOL TextureUnitSlot::IsOn() 
{ 
    return this->on;
}

void TextureUnitSlot::SetOn(BOOL on)
{
    this->on = on;
}

BOOL TextureUnitSlot::HasMap() 
{
    return this->map != 0 ? TRUE : FALSE; 
}

Texmap* TextureUnitSlot::GetMap()
{
    return this->map;
}

//OgreMaxTextureUnits implementation-------------------
OgreMaxTextureUnits::OgreMaxTextureUnits() 
{
    this->client = NULL;
}
					
OgreMaxTextureUnits::OgreMaxTextureUnits(MtlBase* mtlBase) 
{
	this->client = mtlBase;
}

void OgreMaxTextureUnits::SetName(long i, TSTR& name)
{
    this->textureUnits[i].name = name;
}

TSTR& OgreMaxTextureUnits::GetName(long i)
{
    return this->textureUnits[i].name; 
} 

void OgreMaxTextureUnits::SetClientPtr(MtlBase* mtlBase)
{
    this->client = mtlBase;
}

TextureUnitSlot& OgreMaxTextureUnits::operator[](int i)
{
    return this->textureUnits[i];
}

TextureUnitSlot* OgreMaxTextureUnits::GetTextureUnit(int i)
{
    return &this->textureUnits[i];
}

ClassDesc* OgreMaxTextureUnits::GetClassDesc() 
{ 
    return &OgreMaxTextureUnitsClassDesc::instance;  
}

Class_ID OgreMaxTextureUnits::GetClassClassID() 
{
    return Class_ID(0x199bc63c, 0x99734d80);
}

Class_ID OgreMaxTextureUnits::ClassID() 
{ 
    return GetClassClassID(); 
}

void OgreMaxTextureUnits::DeleteThis()
{
    delete this;
}

RefTargetHandle OgreMaxTextureUnits::Clone(RemapDir& remap) 
{
	OgreMaxTextureUnits* newTexmaps = new OgreMaxTextureUnits(NULL);
	
    for (int i = 0; i< MAX_TEXTURE_UNITS; i++) 
    {
		newTexmaps->textureUnits[i].on = this->textureUnits[i].on;
		newTexmaps->textureUnits[i].map = NULL;
		if (this->textureUnits[i].map) 
			newTexmaps->ReplaceReference(i, remap.CloneRef(this->textureUnits[i].map));
	}
	
    BaseClone(this, newTexmaps, remap);
	
    return newTexmaps;
}

RefResult OgreMaxTextureUnits::NotifyRefChanged
    (
    Interval changeInt, 
    RefTargetHandle hTarget, 
    PartID& partID, 
    RefMessage message 
    ) 
{
	switch (message) 
    {
		case REFMSG_GET_PARAM_DIM: 
        {
			GetParamDim* gpd = (GetParamDim*)partID;
			gpd->dim = defaultDim; 
			break;
		}
		case REFMSG_GET_PARAM_NAME: 
        {
			GetParamName* gpn = (GetParamName*)partID;
			return REF_STOP; 
		}
	}
	
    return REF_SUCCEED;
}

void OgreMaxTextureUnits::RescaleWorldUnits(float f) 
{
	if (TestAFlag(A_WORK1))
		return;
	SetAFlag(A_WORK1);

	for (int i = 0; i < NumRefs(); i++) 
    {
        ReferenceMaker* ref = GetReference(i);
		if (ref != NULL) 
            ref->RescaleWorldUnits(f);
	}		
}

BOOL OgreMaxTextureUnits::AssignController(Animatable* control, int subAnim) 
{
	ReplaceReference(SubNumToRefNum(subAnim), static_cast<ReferenceTarget*>(control));
	return TRUE;
}

int OgreMaxTextureUnits::NumSubs()
{ 
    //Each technique has two subanims.
    //One is for the LOD distance, the other for the technique map itself
    return MAX_TEXTURE_UNITS; 
}  

Animatable* OgreMaxTextureUnits::SubAnim(int i) 
{
	return this->textureUnits[i].map;    
}

TSTR OgreMaxTextureUnits::SubAnimName(int i) 
{
	return client->GetSubTexmapTVName(i);    
}

int OgreMaxTextureUnits::SubNumToRefNum(int subNum)
{
    //There is no mapping required
    return subNum;
}

int OgreMaxTextureUnits::NumRefs() 
{ 
    return NumSubs();
}

RefTargetHandle OgreMaxTextureUnits::GetReference(int i) 
{
	return this->textureUnits[i].map;    
}

void OgreMaxTextureUnits::SetReference(int i, RefTargetHandle refTarget) 
{
	this->textureUnits[i].map = static_cast<Texmap*>(refTarget);    
}

SvGraphNodeReference OgreMaxTextureUnits::SvTraverseAnimGraph
    (
    IGraphObjectManager* gom, 
    Animatable* owner, 
    int id, 
    DWORD flags
    )
{
	//If not testing for maps, we're done already
	if (!gom->TestFilter(SV_FILTER_MAPS))
		return SvGraphNodeReference();

    //Figure out how many slots are needed for the schematic view
    int usedSlotCount = 0;
    if (this->textureUnits != NULL)
    {
	    for (int i = 0; i < MAX_TEXTURE_UNITS; i++)
        {
		    if (this->textureUnits[i].GetMap() != NULL)
			    usedSlotCount++;
        }
    }
    
    //Return a "used" result or an empty one, depending on the count
	if (usedSlotCount)
		return SvStdTraverseAnimGraph(gom, owner, id, flags);
	else
		return SvGraphNodeReference();
}

#define TEXTURE_UNITS_ON_OFF_CHUNK 0x5003
#define TEXTURE_UNITS_VERSION_CHUNK 0x5300

#define TEXTURE_UNITS_VERSION 1

IOResult OgreMaxTextureUnits::Save(ISave* isave) 
{ 
    int i;
    ULONG byteCount;

    //Save the version
    isave->BeginChunk(TEXTURE_UNITS_VERSION_CHUNK);
	int version = TEXTURE_UNITS_VERSION;
	isave->Write(&version, sizeof(version), &byteCount);			
	isave->EndChunk();

    //Save the on/off flags
    ULONG onFlags = 0;    
	for (i = 0; i < MAX_TEXTURE_UNITS; i++)
    {
		if (this->textureUnits[i].on) 
            onFlags |= (1<<i);
    }
    isave->BeginChunk(TEXTURE_UNITS_ON_OFF_CHUNK);
	isave->Write(&onFlags, sizeof(onFlags), &byteCount);			
	isave->EndChunk();

    return IO_OK;
}

IOResult OgreMaxTextureUnits::Load(ILoad* iload) 
{ 
	ULONG byteCount;
	int id;
    int version = 0;
	IOResult result;

	while ((result = iload->OpenChunk()) == IO_OK) 
    {
		switch (id = iload->CurChunkID())  
        {
            case TEXTURE_UNITS_VERSION_CHUNK:
            {
				result = iload->Read(&version, sizeof(version), &byteCount);
				break;
            }
			case TEXTURE_UNITS_ON_OFF_CHUNK:
			{
			    ULONG onFlags;
			    result = iload->Read(&onFlags, sizeof(onFlags), &byteCount);
			    for (int i = 0; i < MAX_TEXTURE_UNITS; i++) 
			        this->textureUnits[i].on = (onFlags & (1<<i)) ? TRUE : FALSE;

                break;
			}			
		}

		iload->CloseChunk();
		if (result != IO_OK) 
			return result;
	}

	return IO_OK;                                                      
}

