#ifndef OgreMax_PluginGlobals_INCLUDED
#define OgreMax_PluginGlobals_INCLUDED


//Includes---------------------------------------------------------------------
#include "FileVersionInfo.h"


//Classes----------------------------------------------------------------------
namespace OgreMax
{
    /**
     * Globals used by most plugins.
     */
    class PluginGlobals
    {
    public:
        /**
         * Called by the DLL entry point to initialize the globals.
         * @param hInstance [in] - The global DLL HINSTANCE.
         */
        static void Initialize(HINSTANCE hInstance);

        /** The maximum length of a string that can be loaded by GetString(). */
        enum {MAX_GET_STRING_LENGTH = 512};

        /**
         * Loads a string with the specified resource ID.
         * @param id [in] - The resource identifier.
         * @return A pointer to the loaded string. The string should not be
         * any longer than MAX_GET_STRING_LENGTH and should not be freed.
         * Note that the returned string points to a global buffer and should
         * copied to a new buffer if the string needs to be kept between
         * success successive calls to GetString(). If no string could 
         * be located for the specified ID, null is returned.
         */
        static TCHAR* GetString(int id);

        /**
         * Same functionality as GetString(), but uses a different buffer.
         * This is handy for situations where two resource strings need to be
         * passed into another function, such as when calling MessageBox().
         */
        static TCHAR* GetString2(int id);

        /** Gets the global DLL instance handle set with Initialize(). */
        static HINSTANCE GetInstanceHandle();

        /** 
         * Gets the number of class descriptions provided by the plugin.
         */
        static int GetClassDescriptionCount();

        /**
         * Gets the class description for the specified index.
         */
        static ClassDesc* GetClassDescription(size_t index);

    private:
        static HINSTANCE hInstance;

    public:
        static FileVersionInfo versionInfo;      

        static const TSTR CONTROL_NEWLINE;

        static const TSTR OgreMaxUserPropertyMarker;
        static const TSTR OgreMaxUserDataPropertyMarker;

    private:
        enum {MAX_CLASS_DESCRIPTIONS = 20};
        static ClassDesc* classDescriptions[MAX_CLASS_DESCRIPTIONS];
        static int classDescriptionCount;
    };
}


#endif