
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "SceneSettings.h"
#include "PropertyValues.h"
#include "Utilities.h"

using namespace OgreMax;


//Implementation---------------------------------------------------------------
void SceneSettings::ClearDefaults()
{
    UserDataSettings::ClearDefaults();
    LodSettings::ClearDefaults();
    if (Utilities::IsSameNoCase(this->sceneManager, PropertyValues::StandardSceneManager::GENERIC))
        this->sceneManager = _T("");
    if (Utilities::IsSameNoCase(this->environmentNear, _T("0")))
        this->environmentNear = _T("");
    if (Utilities::IsSameNoCase(this->environmentFar, _T("0")))
        this->environmentFar = _T("");
    if (Utilities::ToPoint3(this->shadowColor) == Point3(0,0,0))
        this->shadowColor = _T("");
    if (Utilities::IsSameNoCase(this->shadowTechnique, PropertyValues::ShadowTechnique::NONE))
        this->shadowTechnique = _T("");
    if (!Values::IsTextureShadowTechnique(this->shadowTechnique))
        this->shadowCameraType = _T("");
    if (Utilities::IsSameNoCase(this->selfShadow, PropertyValues::TRUE_TEXT))
        this->selfShadow = _T("");
    if (Utilities::IsSameNoCase(this->upAxis, PropertyValues::UpAxis::Y_UP))
        this->upAxis = _T("");
    if (Utilities::IsSameNoCase(this->exportMeshes, PropertyValues::TRUE_TEXT))
        this->exportMeshes = _T("");
    if (Utilities::IsSameNoCase(this->exportVertexColors, PropertyValues::FALSE_TEXT))
        this->exportVertexColors = _T("");
    if (Utilities::IsSameNoCase(this->exportTextMeshes, PropertyValues::TRUE_TEXT))
        this->exportTextMeshes = _T("");
    if (Utilities::IsSameNoCase(this->exportBinaryMeshes, PropertyValues::TRUE_TEXT))
        this->exportBinaryMeshes = _T("");
    if (Utilities::IsSameNoCase(this->generateTangents, PropertyValues::TRUE_TEXT))
        this->generateTangents = _T("");
    if (Utilities::IsSameNoCase(this->generateEdgeLists, PropertyValues::FALSE_TEXT))
        this->generateEdgeLists = _T("");
    if (Utilities::IsSameNoCase(this->vertexColorFormat, PropertyValues::VertexColorFormat::DIRECT3D))
        this->vertexColorFormat = _T("");
    if (Utilities::IsSameNoCase(this->endian, PropertyValues::ByteOrder::NATIVE))
        this->endian = _T("");
    if (Utilities::IsSameNoCase(this->sortBoneAssignments, PropertyValues::TRUE_TEXT))
        this->sortBoneAssignments = _T("");
}

bool SceneSettings::Values::IsYUpAxis(const TSTR& text)
{
    return 
        Utilities::IsNull(text) || 
        Utilities::IsSameNoCase(text, _T("y"));
}

TSTR SceneSettings::Values::GetSceneScale(const TSTR& sceneScaleText)
{
    TSTR scale = sceneScaleText;

    int divideIndex = Utilities::FindDivide(sceneScaleText);
    if (divideIndex != -1)
    {
        scale = sceneScaleText.Substr(0, divideIndex);
        Utilities::TrimWhitespace(scale);
    }

    return scale;
}

bool SceneSettings::Values::IsSceneScaleDivisionActive(const TSTR& scaneScaleText)
{
    return Utilities::FindDivide(scaneScaleText) > 0;
}

TSTR SceneSettings::Values::GetSceneScaleDivideBy(const TSTR& sceneScaleText)
{
    TSTR divideBy;

    int divideIndex = Utilities::FindDivide(sceneScaleText);
    if (divideIndex != -1)
    {
        divideBy = sceneScaleText.Substr(divideIndex + 1, sceneScaleText.length() - divideIndex - 1);
        Utilities::TrimWhitespace(divideBy);
    }

    return divideBy;
}

bool SceneSettings::Values::IsValidEnvironmentFarBox(const TSTR& text)
{
    return GetCOREInterface()->GetINodeByName(text.data()) != NULL;
}

bool SceneSettings::Values::IsVertexColorsEnabled(const TSTR& text)
{
    return Utilities::ToBool(text);
}

bool SceneSettings::Values::IsTextMeshesEnabled(const TSTR& text)
{
    return 
        Utilities::IsNull(text) || 
        Utilities::ToBool(text);
}

bool SceneSettings::Values::IsBinaryMeshesEnabled(const TSTR& text)
{
    return 
        Utilities::IsNull(text) || 
        Utilities::ToBool(text);
}

bool SceneSettings::Values::IsGenerateTangentsEnabled(const TSTR& text)
{
    return 
        Utilities::IsNull(text) || 
        Utilities::ToBool(text);
}

bool SceneSettings::Values::IsGenerateEdgeListsEnabled(const TSTR& text)
{
    return Utilities::ToBool(text);
}

bool SceneSettings::Values::IsSortBoneAssignmentsEnabled(const TSTR& text)
{
    return 
        Utilities::IsNull(text) || 
        Utilities::ToBool(text);
}

bool SceneSettings::Values::ExportMeshes(const TSTR& text)
{
    return 
        Utilities::IsNull(text) || 
        Utilities::ToBool(text);
}

bool SceneSettings::Values::IsSelfShadow(const TSTR& text)
{
    return 
        Utilities::IsNull(text) || 
        Utilities::ToBool(text);
}

bool SceneSettings::Values::IsTextureShadowTechnique(const TSTR& text)
{
    return 
        Utilities::IsSameNoCase(text, PropertyValues::ShadowTechnique::TEXTURE_ADDITIVE) ||
        Utilities::IsSameNoCase(text, PropertyValues::ShadowTechnique::TEXTURE_MODULATIVE) ||
        Utilities::IsSameNoCase(text, PropertyValues::ShadowTechnique::TEXTURE_ADDITIVE_INTEGRATED) ||
        Utilities::IsSameNoCase(text, PropertyValues::ShadowTechnique::TEXTURE_MODULATIVE_INTEGRATED);
}