
#ifndef OgreMax_SceneSettings_INCLUDED
#define OgreMax_SceneSettings_INCLUDED


//Includes---------------------------------------------------------------------
#include "CommonSettings.h"


//Classes----------------------------------------------------------------------
namespace OgreMax 
{
    struct SceneSettings : UserDataSettings, LodSettings
    {
        TSTR sceneManager;
        TSTR environmentNear;
        TSTR environmentFar;
        TSTR environmentBoxObjectName;
        TSTR fogMode;
        TSTR fogDensity;

        TSTR shadowTechnique;
        TSTR selfShadow;
        TSTR shadowColor;
        TSTR shadowTextureSize;
        TSTR shadowTextureCount;
        TSTR shadowCameraType;
        TSTR shadowPlaneObjectName;

        TSTR upAxis;
        TSTR sceneScale;

        TSTR exportPath;                        //Path to directory that contains the exported files
        TSTR meshPath;                          //Path to directory that contains the exported meshes
        TSTR materialPath;                      //Path to directory that contains the exported materials
        TSTR modelPath;                         //Path to directory that contains the exported models
        TSTR logPath;                           //Path to directory that will contain log files                    

        TSTR exportMeshes;
        std::vector<ItemAllow> allowExportMeshes;
        TSTR exportVertexColors;
        TSTR exportTextMeshes;
        TSTR exportBinaryMeshes;
        TSTR generateTangents;                  //Indicates whether tangents should be generated
        TSTR generateEdgeLists;                 //Indicates whether edge lists should be generated (used in shadow volumes)
        TSTR vertexColorFormat;    //Direct3D or OpenGL
        TSTR endian;        //Byte ordering

        //This can be modified to turn on/off sorting for the output bone assignments
        //Sorting makes it easier to visually inspect the output, no sorting is faster
        TSTR sortBoneAssignments;

        TSTR exportMaterials;                   //Indicates whether materials should be exported
        std::vector<ItemAllow> allowExportMaterials;
        TSTR noMaterialName;                    //Name of material to use when no material is applied to a mesh

        std::vector<Rename> renameBitmaps;
        std::vector<Rename> renameBitmapExtensions;

        void ClearDefaults();

        struct Values
        {
            static bool IsYUpAxis(const TSTR& text);

            static TSTR GetSceneScale(const TSTR& sceneScaleText);
            static bool IsSceneScaleDivisionActive(const TSTR& text);
            static TSTR GetSceneScaleDivideBy(const TSTR& sceneScaleText);

            static bool IsStandardSceneManager(const TSTR& text);

            static bool IsValidEnvironmentFarBox(const TSTR& text);

            static bool IsVertexColorsEnabled(const TSTR& text);

            static bool IsTextMeshesEnabled(const TSTR& text);
            static bool IsBinaryMeshesEnabled(const TSTR& text);            

            static bool IsGenerateTangentsEnabled(const TSTR& text);

            static bool IsGenerateEdgeListsEnabled(const TSTR& text);

            static bool IsSortBoneAssignmentsEnabled(const TSTR& text);

            static bool ExportMeshes(const TSTR& text);

            static bool IsSelfShadow(const TSTR& text);

            static bool IsTextureShadowTechnique(const TSTR& text);
        };
    };
}


#endif