
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "SceneSettingsDialog.h"
#include "Utilities.h"
#include "resource.h"

using namespace OgreMax;


//Globals----------------------------------------------------------------------
static const int tabNameIDs[] = 
    {
    IDS_SCENE_SETTINGS_GENERAL_TAB, 
    IDS_SETTINGS_USER_DATA_TAB, 
    IDS_SCENE_SETTINGS_MESHES_TAB,
    IDS_SCENE_SETTINGS_MATERIALS_TAB,                
    IDS_SCENE_SETTINGS_BITMAPS_TAB,
    IDS_SCENE_SETTINGS_SHADOWS_TAB,
    IDS_SCENE_SETTINGS_DIRECTORIES_TAB
    };
static const int tabPageIDs[] = 
    {
    IDD_SCENE_SETTINGS_GENERAL_PAGE, 
    IDD_SETTINGS_USER_DATA_PAGE, 
    IDD_SCENE_SETTINGS_MESHES_PAGE,
    IDD_SCENE_SETTINGS_MATERIALS_PAGE,                
    IDD_SCENE_SETTINGS_BITMAPS_PAGE,
    IDD_SCENE_SETTINGS_SHADOWS_PAGE,
    IDD_SCENE_SETTINGS_DIRECTORIES_PAGE
    };

//Implementation---------------------------------------------------------------
SceneSettingsDialog::SceneSettingsDialog(SceneSettings& sceneSettings) : 
    settings(sceneSettings),
    generalTabPage(sceneSettings),
    userDataTabPage(sceneSettings),
    meshesTabPage(sceneSettings),
    materialsTabPage(sceneSettings),
    bitmapsTabPage(sceneSettings),
    shadowsTabPage(sceneSettings),
    directoriesTabPage(sceneSettings)
{
    this->tabPages.reserve(COUNT_OF(tabNameIDs));
    this->tabPages.push_back(&this->generalTabPage);
    this->tabPages.push_back(&this->userDataTabPage);
    this->tabPages.push_back(&this->meshesTabPage);
    this->tabPages.push_back(&this->materialsTabPage);
    this->tabPages.push_back(&this->bitmapsTabPage);
    this->tabPages.push_back(&this->shadowsTabPage);
    this->tabPages.push_back(&this->directoriesTabPage);
}

SceneSettingsDialog::~SceneSettingsDialog()
{
}

int SceneSettingsDialog::DoModal()
{
    return DialogBase::DoModal(IDD_SCENE_SETTINGS_DIALOG);
}

INT_PTR SceneSettingsDialog::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    switch (message)
    {
        case WM_INITDIALOG:
        {
            CenterWindow();

            this->tabControl.Create(GetDlgItem(IDC_SCENE_SETTINGS_TAB), true);

            for (size_t i = 0; i < this->tabPages.size(); i++)
                this->tabPages[i]->Attach(this->tabControl.AddPage(tabNameIDs[i], tabPageIDs[i]));

            handled = true;

            break;
        }
        case WM_DESTROY:
        {
            for (size_t i = 0; i < this->tabPages.size(); i++)
                this->tabPages[i]->Detach();

            handled = true;

            break;
        }
        case WM_COMMAND:
        {
            int id = LOWORD(wParam);
            int code = HIWORD(wParam);

            switch (id)
            {
                case IDOK:
                {
                    //Get the GUI data
                    for (size_t i = 0; i < this->tabPages.size(); i++)
                        this->tabPages[i]->GetGUIData();

                    this->settings.ClearDefaults();

                    handled = true;
                    EndDialog(this->hDlg, TRUE);
                    return TRUE;
                }
                case IDCANCEL:
                {
                    handled = true;
                    EndDialog(this->hDlg, FALSE);
                    return FALSE;
                }
            }

            break;
        }
        case WM_NOTIFY:
        {
            NMHDR* pnm = (NMHDR*)lParam;
            UINT controlID = (UINT)wParam;
            switch (controlID)
            {
                case IDC_SCENE_SETTINGS_TAB: 
                {
                    this->tabControl.Notify(pnm);
                    handled = true;
                    return FALSE;
                }
            }
        }
    }

    return 0;
}