
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "SceneSettingsSerializer.h"
#include "Utilities.h"
#include "StringSplitter.h"
#include "PropertyKeys.h"
#include "PluginGlobals.h"

using namespace OgreMax;


//Implementation---------------------------------------------------------------
SceneSettingsSerializer::SceneSettingsSerializer()
{
}

void SceneSettingsSerializer::ParseProperty
    (
    const TSTR& key, 
    const TSTR& value, 
    int line, 
    std::vector<int>& handledLines
    )
{
    if (Utilities::IsSameNoCase(key, PropertyKeys::ID))
    {
        handledLines.push_back(line);
        this->settings.id = value;
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::USER_DATA_REFERENCE))
    {
        handledLines.push_back(line);
        this->settings.userDataReference = value;
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::USER_DATA_MARKER))
    {
        this->settings.userDataMarker = value;
        handledLines.push_back(line);

        //Get the user data
        int firstLine = this->textLines.FindLine(this->settings.userDataMarker);
        if (firstLine != -1)
        {
            int lastLine = this->textLines.FindLine(this->settings.userDataMarker, firstLine + 1);
            if (lastLine != -1)
            {
                //Mark all of the user data lines for removal
                for (int i = firstLine; i <= lastLine; i++)
                    handledLines.push_back(i);

                //Extract user data
                this->settings.userData = this->textLines.Join(PluginGlobals::CONTROL_NEWLINE, firstLine + 1, lastLine - firstLine - 1);
                Utilities::TrimWhitespace(this->settings.userData);
            }
        }
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::IS_USER_DATA_CDATA))
    {
        this->settings.isUserDataCDATA = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::GENERATE_LOD))
    {
        this->settings.generateLod = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::NUM_LOD_LEVELS))
    {
        this->settings.numLodLevels = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::LOD_DISTANCE))
    {
        this->settings.lodDistance = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::LOD_REDUCTION_TYPE))
    {
        this->settings.lodReductionType = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::LOD_PERCENT_REDUCTION))
    {
        this->settings.lodReductionAmount = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::LOD_VERTEX_REDUCTION))
    {
        this->settings.lodReductionAmount = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::LOD_REDUCTION_AMOUNT))
    {
        this->settings.lodReductionAmount = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::SCENE_MANAGER))
    {
        this->settings.sceneManager = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::ENVIRONMENT_NEAR))
    {
        this->settings.environmentNear = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::ENVIRONMENT_FAR))
    {
        this->settings.environmentFar = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::ENVIRONMENT_BOX))
    {
        this->settings.environmentBoxObjectName = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::FOG_MODE))
    {
        this->settings.fogMode = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::FOG_EXP_DENSITY))
    {
        this->settings.fogDensity = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::SHADOW_TECHNIQUE))
    {
        this->settings.shadowTechnique = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::SELF_SHADOW))
    {
        this->settings.selfShadow = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::SHADOW_COLOR))
    {
        this->settings.shadowColor = value;
        handledLines.push_back(line);
    }        
    else if (Utilities::IsSameNoCase(key, PropertyKeys::SHADOW_TEXTURE_SIZE))
    {
        this->settings.shadowTextureSize = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::SHADOW_TEXTURE_COUNT))
    {
        this->settings.shadowTextureCount = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::SHADOW_CAMERA_TYPE))
    {
        this->settings.shadowCameraType = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::SHADOW_PLANE))
    {
        this->settings.shadowPlaneObjectName = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::UP_AXIS))
    {
        this->settings.upAxis = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::SCALE))
    {
        this->settings.sceneScale = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::EXPORT_PATH))
    {
        this->settings.exportPath = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::MESH_PATH))
    {
        this->settings.meshPath = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::MATERIAL_PATH))
    {
        this->settings.materialPath = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::MODEL_PATH))
    {
        this->settings.modelPath = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::LOG_PATH))
    {
        this->settings.logPath = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::EXPORT_MESHES))
    {
        this->settings.exportMeshes = value;
        handledLines.push_back(line);
    }
    else if (Utilities::StartsWithNoCase(key, PropertyKeys::EXPORT_MESH))
    {
        TSTR keyValueLine = key + _T("=") + value;
        
        ItemAllow exportMesh;
        if (Utilities::ParsePrefixedKeyValueLine(keyValueLine, exportMesh.name, exportMesh.allow) &&
            !Utilities::ToBool(exportMesh.allow))
        {
            this->settings.allowExportMeshes.push_back(exportMesh);
            handledLines.push_back(line);
        }        
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::EXPORT_VERTEX_COLORS))
    {
        this->settings.exportVertexColors = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::EXPORT_TEXT_MESH))
    {
        this->settings.exportTextMeshes = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::EXPORT_BINARY_MESH))
    {
        this->settings.exportBinaryMeshes = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::GENERATE_TANGENTS))
    {
        this->settings.generateTangents = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::GENERATE_EDGE_LISTS))
    {
        this->settings.generateEdgeLists = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::VERTEX_COLOR_FORMAT))
    {
        this->settings.vertexColorFormat = value;
        handledLines.push_back(line);
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::ENDIAN))
    {
        this->settings.endian = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::SORT_BONE_ASSIGNMENTS))
    {
        this->settings.sortBoneAssignments = value;
        handledLines.push_back(line);
    }

    else if (Utilities::IsSameNoCase(key, PropertyKeys::EXPORT_MATERIALS))
    {
        this->settings.exportMaterials = value;
        handledLines.push_back(line);
    }
    else if (Utilities::StartsWithNoCase(key, PropertyKeys::EXPORT_MATERIAL))
    {
        TSTR keyValueLine = key + _T("=") + value;

        ItemAllow exportMaterial;
        if (Utilities::ParsePrefixedKeyValueLine(keyValueLine, exportMaterial.name, exportMaterial.allow) &&
            !Utilities::ToBool(exportMaterial.allow))
        {
            this->settings.allowExportMaterials.push_back(exportMaterial);
            handledLines.push_back(line);
        }
    }
    else if (Utilities::IsSameNoCase(key, PropertyKeys::NO_MATERIAL_NAME))
    {
        this->settings.noMaterialName = value;
        handledLines.push_back(line);
    }

    else if (Utilities::StartsWithNoCase(key, PropertyKeys::RENAME_BITMAP_EXTENSION))
    {
        TSTR keyValueLine = key + _T("=") + value;

        Rename rename;
        if (Utilities::ParsePrefixedKeyValueLine(keyValueLine, rename.from, rename.to))
        {
            this->settings.renameBitmapExtensions.push_back(rename);
            handledLines.push_back(line);
        }
    }
    else if (Utilities::StartsWithNoCase(key, PropertyKeys::RENAME_BITMAP))
    {
        TSTR keyValueLine = key + _T("=") + value;

        Rename rename;
        if (Utilities::ParsePrefixedKeyValueLine(keyValueLine, rename.from, rename.to))
        {
            this->settings.renameBitmaps.push_back(rename);
            handledLines.push_back(line);
        }
    }    
}

void SceneSettingsSerializer::GetPropertiesText(TSTR& text, const TSTR& newline)
{
    text = _T("");

    if (!Utilities::IsNull(this->settings.id))
        Utilities::AppendKeyAndValue(text, PropertyKeys::ID, this->settings.id, newline);
    if (!Utilities::IsNull(this->settings.userDataReference))
        Utilities::AppendKeyAndValue(text, PropertyKeys::USER_DATA_REFERENCE, this->settings.userDataReference, newline);
    if (!Utilities::IsNull(this->settings.userDataMarker) && !Utilities::IsNull(this->settings.userData))
    {
        Utilities::AppendKeyAndValue(text, PropertyKeys::USER_DATA_MARKER, this->settings.userDataMarker, newline);
        Utilities::AppendMarkedText(text, this->settings.userDataMarker, this->settings.userData, newline);
    }
    if (!Utilities::IsNull(this->settings.isUserDataCDATA))
        Utilities::AppendKeyAndValue(text, PropertyKeys::IS_USER_DATA_CDATA, this->settings.isUserDataCDATA, newline);    

    if (!Utilities::IsNull(this->settings.generateLod))
        Utilities::AppendKeyAndValue(text, PropertyKeys::GENERATE_LOD, this->settings.generateLod, newline);    
    if (!Utilities::IsNull(this->settings.numLodLevels))
        Utilities::AppendKeyAndValue(text, PropertyKeys::NUM_LOD_LEVELS, this->settings.numLodLevels, newline);    
    if (!Utilities::IsNull(this->settings.lodDistance))
        Utilities::AppendKeyAndValue(text, PropertyKeys::LOD_DISTANCE, this->settings.lodDistance, newline);    
    if (!Utilities::IsNull(this->settings.lodReductionType))
        Utilities::AppendKeyAndValue(text, PropertyKeys::LOD_REDUCTION_TYPE, this->settings.lodReductionType, newline);    
    if (!Utilities::IsNull(this->settings.lodReductionAmount))
        Utilities::AppendKeyAndValue(text, PropertyKeys::LOD_REDUCTION_AMOUNT, this->settings.lodReductionAmount, newline);    

    if (!Utilities::IsNull(this->settings.sceneManager))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SCENE_MANAGER, this->settings.sceneManager, newline);    
    if (!Utilities::IsNull(this->settings.environmentNear))
        Utilities::AppendKeyAndValue(text, PropertyKeys::ENVIRONMENT_NEAR, this->settings.environmentNear, newline);    
    if (!Utilities::IsNull(this->settings.environmentFar))
        Utilities::AppendKeyAndValue(text, PropertyKeys::ENVIRONMENT_FAR, this->settings.environmentFar, newline);    
    if (!Utilities::IsNull(this->settings.environmentBoxObjectName))
        Utilities::AppendKeyAndValue(text, PropertyKeys::ENVIRONMENT_BOX, this->settings.environmentBoxObjectName, newline);    
    if (!Utilities::IsNull(this->settings.fogMode))
        Utilities::AppendKeyAndValue(text, PropertyKeys::FOG_MODE, this->settings.fogMode, newline);    
    if (!Utilities::IsNull(this->settings.fogDensity))
        Utilities::AppendKeyAndValue(text, PropertyKeys::FOG_EXP_DENSITY, this->settings.fogDensity, newline);    
    if (!Utilities::IsNull(this->settings.shadowTechnique))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SHADOW_TECHNIQUE, this->settings.shadowTechnique, newline);    

    if (!Utilities::IsNull(this->settings.selfShadow))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SELF_SHADOW, this->settings.selfShadow, newline);    
    if (!Utilities::IsNull(this->settings.shadowColor))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SHADOW_COLOR, this->settings.shadowColor, newline);    
    if (!Utilities::IsNull(this->settings.shadowTextureSize))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SHADOW_TEXTURE_SIZE, this->settings.shadowTextureSize, newline);    
    if (!Utilities::IsNull(this->settings.shadowTextureCount))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SHADOW_TEXTURE_COUNT, this->settings.shadowTextureCount, newline);    
    if (!Utilities::IsNull(this->settings.shadowCameraType))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SHADOW_CAMERA_TYPE, this->settings.shadowCameraType, newline);    
    if (!Utilities::IsNull(this->settings.shadowPlaneObjectName))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SHADOW_PLANE, this->settings.shadowPlaneObjectName, newline);    

    if (!Utilities::IsNull(this->settings.upAxis))
        Utilities::AppendKeyAndValue(text, PropertyKeys::UP_AXIS, this->settings.upAxis, newline);    
    if (!Utilities::IsNull(this->settings.sceneScale))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SCALE, this->settings.sceneScale, newline);    

    if (!Utilities::IsNull(this->settings.exportPath))
        Utilities::AppendKeyAndValue(text, PropertyKeys::EXPORT_PATH, this->settings.exportPath, newline);    
    if (!Utilities::IsNull(this->settings.meshPath))
        Utilities::AppendKeyAndValue(text, PropertyKeys::MESH_PATH, this->settings.meshPath, newline);    
    if (!Utilities::IsNull(this->settings.materialPath))
        Utilities::AppendKeyAndValue(text, PropertyKeys::MATERIAL_PATH, this->settings.materialPath, newline);    
    if (!Utilities::IsNull(this->settings.modelPath))
        Utilities::AppendKeyAndValue(text, PropertyKeys::MODEL_PATH, this->settings.modelPath, newline);    
    if (!Utilities::IsNull(this->settings.logPath))
        Utilities::AppendKeyAndValue(text, PropertyKeys::LOG_PATH, this->settings.logPath, newline);    

    if (!Utilities::IsNull(this->settings.exportMeshes))
        Utilities::AppendKeyAndValue(text, PropertyKeys::EXPORT_MESHES, this->settings.exportMeshes, newline);    
    for (size_t i = 0; i < this->settings.allowExportMeshes.size(); i++)
        Utilities::AppendPrefixedKeyAndValue(text, PropertyKeys::EXPORT_MESH, this->settings.allowExportMeshes[i].name, this->settings.allowExportMeshes[i].allow, newline);
    if (!Utilities::IsNull(this->settings.exportVertexColors))
        Utilities::AppendKeyAndValue(text, PropertyKeys::EXPORT_VERTEX_COLORS, this->settings.exportVertexColors, newline);    
    if (!Utilities::IsNull(this->settings.exportTextMeshes))
        Utilities::AppendKeyAndValue(text, PropertyKeys::EXPORT_TEXT_MESH, this->settings.exportTextMeshes, newline);    
    if (!Utilities::IsNull(this->settings.exportBinaryMeshes))
        Utilities::AppendKeyAndValue(text, PropertyKeys::EXPORT_BINARY_MESH, this->settings.exportBinaryMeshes, newline);    
    if (!Utilities::IsNull(this->settings.generateTangents))
        Utilities::AppendKeyAndValue(text, PropertyKeys::GENERATE_TANGENTS, this->settings.generateTangents, newline);    
    if (!Utilities::IsNull(this->settings.generateEdgeLists))
        Utilities::AppendKeyAndValue(text, PropertyKeys::GENERATE_EDGE_LISTS, this->settings.generateEdgeLists, newline);    
    if (!Utilities::IsNull(this->settings.vertexColorFormat))
        Utilities::AppendKeyAndValue(text, PropertyKeys::VERTEX_COLOR_FORMAT, this->settings.vertexColorFormat, newline);    
    if (!Utilities::IsNull(this->settings.endian))
        Utilities::AppendKeyAndValue(text, PropertyKeys::ENDIAN, this->settings.endian, newline);    

    if (!Utilities::IsNull(this->settings.sortBoneAssignments))
        Utilities::AppendKeyAndValue(text, PropertyKeys::SORT_BONE_ASSIGNMENTS, this->settings.sortBoneAssignments, newline);    

    if (!Utilities::IsNull(this->settings.exportMaterials))
        Utilities::AppendKeyAndValue(text, PropertyKeys::EXPORT_MATERIALS, this->settings.exportMaterials, newline);    
    for (size_t i = 0; i < this->settings.allowExportMaterials.size(); i++)
        Utilities::AppendPrefixedKeyAndValue(text, PropertyKeys::EXPORT_MATERIAL, this->settings.allowExportMaterials[i].name, this->settings.allowExportMaterials[i].allow, newline);
    if (!Utilities::IsNull(this->settings.noMaterialName))
        Utilities::AppendKeyAndValue(text, PropertyKeys::NO_MATERIAL_NAME, this->settings.noMaterialName, newline);    
    
    for (size_t i = 0; i < this->settings.renameBitmaps.size(); i++)
        Utilities::AppendPrefixedKeyAndValue(text, PropertyKeys::RENAME_BITMAP, this->settings.renameBitmaps[i].from, this->settings.renameBitmaps[i].to, newline);
    for (size_t i = 0; i < this->settings.renameBitmapExtensions.size(); i++)
        Utilities::AppendPrefixedKeyAndValue(text, PropertyKeys::RENAME_BITMAP_EXTENSION, this->settings.renameBitmapExtensions[i].from, this->settings.renameBitmapExtensions[i].to, newline);
}