
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "SceneSettingsTabPages.h"
#include "PluginGlobals.h"
#include "Utilities.h"
#include "PropertyValues.h"
#include "StringSplitter.h"
#include "RenameDialog.h"
#include "resource.h"

using namespace OgreMax;


//Globals-----------------------------------------------------------------------
static const int scaleComboStringIDs[] = { IDS_METERS, IDS_STANDARD, IDS_ACTIVE, IDS_KILOMETERS, IDS_FEET, IDS_MILES};
static const TCHAR* STANDARD_SCENE_SCALES[] = {_T(""),  _T("standard"), _T("active"), _T("kilometers"), _T("feet"), _T("miles")};
static const int STANDARD_SCENE_SCALE_COUNT = COUNT_OF(STANDARD_SCENE_SCALES);

static const int standardSceneManagerComboStringIDs[] = {IDS_GENERIC, IDS_EXTERIOR_CLOSE, IDS_EXTERIOR_FAR, IDS_EXTERIOR_REAL_FAR};
static const TCHAR* STANDARD_SCENE_MANAGERS[] = {_T(""), _T("exteriorClose"), _T("exteriorFar"), _T("exteriorRealFar")};
static const int STANDARD_SCENE_MANAGER_COUNT = COUNT_OF(STANDARD_SCENE_MANAGERS);

static const int vertexColorFormatComboStringIDs[] = {IDS_DIRECT3D, IDS_OPENGL};
static const TCHAR* VERTEX_COLOR_FORMATS[] = {_T("direct3d"), _T("opengl")};
static const int VERTEX_COLOR_FORMAT_COUNT = COUNT_OF(VERTEX_COLOR_FORMATS);

static const int byteOrderComboStringIDs[] = {IDS_NATIVE, IDS_LITTLE_ENDIAN, IDS_BIG_ENDIAN};
static const TCHAR* BYTE_ORDERS[] = {_T(""), _T("little"), _T("big")};
static const int BYTE_ORDER_COUNT = COUNT_OF(BYTE_ORDERS);

static const int lodReductionTypeComboStringIDs[] = {IDS_PERCENT, IDS_VERTEX};

static const int shadowTechniqueComboStringIDs[] = {IDS_NONE, IDS_STENCIL_MODULATIVE, IDS_STENCIL_ADDITIVE, IDS_TEXTURE_MODULATIVE, IDS_TEXTURE_ADDITIVE, IDS_TEXTURE_MODULATIVE_INTEGRATED, IDS_TEXTURE_ADDITIVE_INTEGRATED};
static const TCHAR* SHADOW_TECHNIQUES[] = {_T(""), _T("stencilModulative"), _T("stencilAdditive"), _T("textureModulative"), _T("textureAdditive"), _T("textureModulativeIntegrated"), _T("textureAdditiveIntegrated")};
static const int SHADOW_TECHNIQUE_COUNT = COUNT_OF(SHADOW_TECHNIQUES);

static const int shadowCameraTypeComboStringIDs[] = {IDS_UNIFORM, IDS_UNIFORM_FOCUSED, IDS_LISPSM, IDS_PLANE_OPTIMAL};
static const TCHAR* SHADOW_CAMERA_TYPES[] = {_T("uniform"), _T("uniformFocused"), _T("lispsm"), _T("planeOptimal")};
static const int SHADOW_CAMERA_TYPE_COUNT = COUNT_OF(SHADOW_CAMERA_TYPES);

static const int fogModeComboStringIDs[] = {IDS_NONE, IDS_LINEAR, IDS_EXP, IDS_EXP2};
static const TCHAR* FOG_MODES[] = {_T(""), _T("linear"), _T("exp"), _T("exp2")};
static const int FOG_MODE_COUNT = COUNT_OF(fogModeComboStringIDs);


//Local functions--------------------------------------------------------------
static int GetScaleValueComboStringIndex(const TSTR& value)
{
    TSTR scale = SceneSettings::Values::GetSceneScale(value);
    return Utilities::GetValueIndex(scale, STANDARD_SCENE_SCALE_COUNT, STANDARD_SCENE_SCALES);    
}

static int GetStandardSceneManagerValueComboStringIndex(const TSTR& value)
{
    return Utilities::GetValueIndex(value, STANDARD_SCENE_MANAGER_COUNT, STANDARD_SCENE_MANAGERS);
}

static int GetFogModeValueComboStringIndex(const TSTR& value)
{
    return Utilities::GetValueIndex(value, FOG_MODE_COUNT, FOG_MODES);
}

static int GetVertexColorFormatValueComboStringIndex(const TSTR& value)
{
    return Utilities::GetValueIndex(value, VERTEX_COLOR_FORMAT_COUNT, VERTEX_COLOR_FORMATS);
}

static int GetByteOrderValueComboStringIndex(const TSTR& value)
{
    return Utilities::GetValueIndex(value, BYTE_ORDER_COUNT, BYTE_ORDERS);
}

static int GetLODReductionTypeValueComboStringIndex(const TSTR& value)
{
    return Utilities::GetValueIndex(value, CommonSettings::Values::LOD_REDUCTION_TYPE_COUNT, CommonSettings::Values::LOD_REDUCTION_TYPES);
}

static int GetShadowTechniqueValueComboStringIndex(const TSTR& value)
{
    return Utilities::GetValueIndex(value, SHADOW_TECHNIQUE_COUNT, SHADOW_TECHNIQUES);
}

static int GetShadowCameraTypeValueComboStringIndex(const TSTR& value)
{
    return Utilities::GetValueIndex(value, SHADOW_CAMERA_TYPE_COUNT, SHADOW_CAMERA_TYPES);
}

static const TSTR& GetSceneScaleComboValue(int i)
{
    static const TSTR values[] =
        {
        _T(""),
        PropertyValues::SceneScale::ACTIVE,
        PropertyValues::SceneScale::METERS,
        PropertyValues::SceneScale::KILOMETERS,
        PropertyValues::SceneScale::FEET,
        PropertyValues::SceneScale::MILES
        };
    return values[i];
}

static const TSTR& GetStandardSceneManagerComboValue(int i)
{
    static const TSTR values[] = 
        {
        _T(""),
        PropertyValues::StandardSceneManager::EXTERIOR_CLOSE,
        PropertyValues::StandardSceneManager::EXTERIOR_FAR,
        PropertyValues::StandardSceneManager::EXTERIOR_REAL_FAR
        };
    return values[i];
}

static const TSTR& GetFogModeComboValue(int i)
{
    static const TSTR values[] = 
        {
        _T(""),
        PropertyValues::FogMode::LINEAR,
        PropertyValues::FogMode::EXP,
        PropertyValues::FogMode::EXP2
        };
    return values[i];
}

static const TSTR& GetVertexColorFormatComboValue(int i)
{
    static const TSTR values[] = 
        {
        _T(""),
        PropertyValues::VertexColorFormat::DIRECT3D,
        PropertyValues::VertexColorFormat::OPENGL
        };
    return values[i];
}

static const TSTR& GetShadowTechniqueComboValue(int i)
{
    static const TSTR values[] = 
        {
        _T(""),
        PropertyValues::ShadowTechnique::STENCIL_MODULATIVE,
        PropertyValues::ShadowTechnique::STENCIL_ADDITIVE,
        PropertyValues::ShadowTechnique::TEXTURE_MODULATIVE,
        PropertyValues::ShadowTechnique::TEXTURE_ADDITIVE,
        PropertyValues::ShadowTechnique::TEXTURE_MODULATIVE_INTEGRATED,
        PropertyValues::ShadowTechnique::TEXTURE_ADDITIVE_INTEGRATED
        };
    return values[i];
}

static const TSTR& GetShadowCameraTypeComboValue(int i)
{
    static const TSTR values[] = 
        {
        PropertyValues::ShadowCameraType::UNIFORM,
        PropertyValues::ShadowCameraType::UNIFORM_FOCUSED,
        PropertyValues::ShadowCameraType::LISPSM,
        PropertyValues::ShadowCameraType::PLANE_OPTIMAL
        };
    return values[i];
}

static const TSTR& GetByteOrderComboValue(int i)
{
    static const TSTR values[] = 
        {
        _T(""),
        PropertyValues::ByteOrder::LITTLE_ENDIAN,
        PropertyValues::ByteOrder::BIG_ENDIAN
        };
    return values[i];    
}

static const TSTR& GetLODReductionTypeComboValue(int i)
{
    static const TSTR values[] =
        {
        PropertyValues::LODReductionType::PERCENT,
        PropertyValues::LODReductionType::VERTEX
        };
    return values[i];
}

static bool IsStandardSceneManager(const TSTR& text)
{
    if (Utilities::IsNull(text))
        return true;

    for (int i = 0; i < STANDARD_SCENE_MANAGER_COUNT; i++)
    {
        if (Utilities::IsSameNoCase(text, STANDARD_SCENE_MANAGERS[i]))
            return true;
    }

    return false;
}


//Implementation----------------------------------------------------------------

//SceneSettingsGeneralTabPage-------------------------
SceneSettingsGeneralTabPage::SceneSettingsGeneralTabPage(SceneSettings& sceneSettings) : settings(sceneSettings)
{
    this->nearDistanceSpin = NULL;
    this->farDistanceSpin = NULL;
}

void SceneSettingsGeneralTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void SceneSettingsGeneralTabPage::Detach()
{
    ReleaseISpinner(this->nearDistanceSpin);
    ReleaseISpinner(this->farDistanceSpin);
}

void SceneSettingsGeneralTabPage::GetGUIData()
{
    //Up axis radio buttons
    this->settings.upAxis = IsDlgButtonChecked(IDC_UP_AXIS_Z_RADIO) ? PropertyValues::UpAxis::Z_UP : PropertyValues::UpAxis::Y_UP;

    //Scene scale units combo
    HWND scaleComboHwnd = GetDlgItem(IDC_SCENE_SCALE_COMBO);
    int selectedScaleIndex = ComboBox_GetCurSel(scaleComboHwnd);
    this->settings.sceneScale = GetSceneScaleComboValue(selectedScaleIndex);
    
    //Scale "divide by" edit
    if (!this->settings.sceneScale.isNull())
    {
        TSTR scaleDivideByText;
        GetDlgItemText(IDC_SCENE_SCALE_DIVIDE_BY_EDIT, scaleDivideByText);
        Utilities::TrimWhitespace(scaleDivideByText);
        if (!scaleDivideByText.isNull())
        {
            this->settings.sceneScale += _T("/");
            this->settings.sceneScale += scaleDivideByText;            
        }
    }

    //Scene manager radio buttons
    this->settings.sceneManager = _T("");
    if (IsDlgButtonChecked(IDC_SCENE_MANAGER_STANDARD_RADIO))
    {
        HWND sceneManagerComboHwnd = GetDlgItem(IDC_SCENE_MANAGER_STANDARD_COMBO);
        int selectedSceneManagerIndex = ComboBox_GetCurSel(sceneManagerComboHwnd);
        this->settings.sceneManager = GetStandardSceneManagerComboValue(selectedSceneManagerIndex);
    }
    else
        GetDlgItemText(IDC_SCENE_MANAGER_CUSTOM_EDIT, this->settings.sceneManager);
    
    //Environment range near edit
    this->settings.environmentNear = Utilities::ToString(this->nearDistanceSpin->GetFVal());

    //Environment range far type radio
    this->settings.environmentFar = _T("");
    this->settings.environmentBoxObjectName = _T("");
    if (IsDlgButtonChecked(IDC_ENVIRONMENT_RANGE_FAR_DISTANCE_RADIO))
        this->settings.environmentFar = Utilities::ToString(this->farDistanceSpin->GetFVal());
    else if (IsDlgButtonChecked(IDC_ENVIRONMENT_RANGE_FAR_BOX_RADIO))
        GetDlgItemText(IDC_ENVIRONMENT_RANGE_FAR_BOX_EDIT, this->settings.environmentBoxObjectName);    

    //Fog mode combo
    HWND fogModeComboHwnd = GetDlgItem(IDC_FOG_MODE_COMBO);
    int selectedFogModeIndex = ComboBox_GetCurSel(fogModeComboHwnd);
    this->settings.fogMode = GetFogModeComboValue(selectedFogModeIndex);

    //Fog exp density edit
    GetDlgItemText(IDC_FOG_EXP_DENSITY_EDIT, this->settings.fogDensity);
}

void SceneSettingsGeneralTabPage::SetGUIData()
{
    //Up axis radio buttons
    int upAxisControlID = SceneSettings::Values::IsYUpAxis(this->settings.upAxis) ? IDC_UP_AXIS_Y_RADIO : IDC_UP_AXIS_Z_RADIO;
    CheckRadioButton(IDC_UP_AXIS_Y_RADIO, IDC_UP_AXIS_Z_RADIO, upAxisControlID);

    //Scene scale units combo
    HWND scaleComboHwnd = GetDlgItem(IDC_SCENE_SCALE_COMBO);
    for (int i = 0; i < STANDARD_SCENE_SCALE_COUNT; i++)
        ComboBox_AddString(scaleComboHwnd, PluginGlobals::GetString(scaleComboStringIDs[i]));
    ComboBox_SetCurSel(scaleComboHwnd, GetScaleValueComboStringIndex(this->settings.sceneScale));

    //Scale "divide by" value    
    TSTR scaleDivideByText = SceneSettings::Values::GetSceneScaleDivideBy(this->settings.sceneScale);
    SetDlgItemText(IDC_SCENE_SCALE_DIVIDE_BY_EDIT, scaleDivideByText);

    //Scene manager radio buttons
    bool isStandardSceneManager = IsStandardSceneManager(this->settings.sceneManager);
    int sceneManagerControlID = isStandardSceneManager ? IDC_SCENE_MANAGER_STANDARD_RADIO : IDC_SCENE_MANAGER_CUSTOM_RADIO;
    CheckRadioButton(IDC_SCENE_MANAGER_STANDARD_RADIO, IDC_SCENE_MANAGER_CUSTOM_RADIO, sceneManagerControlID);

    //Standard scene manager combo
    HWND sceneManagerComboHwnd = GetDlgItem(IDC_SCENE_MANAGER_STANDARD_COMBO);
    for (int i = 0; i < STANDARD_SCENE_MANAGER_COUNT; i++)
        ComboBox_AddString(sceneManagerComboHwnd, PluginGlobals::GetString(standardSceneManagerComboStringIDs[i]));
    ComboBox_SetCurSel(sceneManagerComboHwnd, GetStandardSceneManagerValueComboStringIndex(this->settings.sceneManager));

    //Custom scene manager edit
    if (!isStandardSceneManager)
        SetDlgItemText(IDC_SCENE_MANAGER_CUSTOM_EDIT, this->settings.sceneManager);

    //Environment range near edit/spin
    this->nearDistanceSpin = SetupUniverseSpinner(this->hDlg, IDC_ENVIRONMENT_RANGE_NEAR_DISTANCE_SPIN, IDC_ENVIRONMENT_RANGE_NEAR_DISTANCE_EDIT, 0, 100000, Utilities::ToFloat(this->settings.environmentNear));
    
    //Environment range far type radio
    bool isValidEnvironmentFarBox = SceneSettings::Values::IsValidEnvironmentFarBox(this->settings.environmentBoxObjectName);
    int environmentFarControlID = isValidEnvironmentFarBox ? IDC_ENVIRONMENT_RANGE_FAR_BOX_RADIO : IDC_ENVIRONMENT_RANGE_FAR_DISTANCE_RADIO;
    CheckRadioButton(IDC_ENVIRONMENT_RANGE_FAR_DISTANCE_RADIO, IDC_ENVIRONMENT_RANGE_FAR_BOX_RADIO, environmentFarControlID);

    //Environment range far edit/spin
    this->farDistanceSpin = SetupUniverseSpinner(this->hDlg, IDC_ENVIRONMENT_RANGE_FAR_DISTANCE_SPIN, IDC_ENVIRONMENT_RANGE_FAR_DISTANCE_EDIT, 0, 100000, Utilities::ToFloat(this->settings.environmentFar));
    
    //Environment range far box object edit
    SetDlgItemText(IDC_ENVIRONMENT_RANGE_FAR_BOX_EDIT, this->settings.environmentBoxObjectName);

    //Fog mode combo
    HWND fogModeComboHwnd = GetDlgItem(IDC_FOG_MODE_COMBO);
    for (int i = 0; i < FOG_MODE_COUNT; i++)
        ComboBox_AddString(fogModeComboHwnd, PluginGlobals::GetString(fogModeComboStringIDs[i]));
    ComboBox_SetCurSel(fogModeComboHwnd, GetFogModeValueComboStringIndex(this->settings.fogMode));

    //Fog exp density edit
    SetDlgItemText(IDC_FOG_EXP_DENSITY_EDIT, this->settings.fogDensity);
}

INT_PTR SceneSettingsGeneralTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    return 0;
}

//SceneSettingsMeshesTabPage-------------------------
SceneSettingsMeshesTabPage::SceneSettingsMeshesTabPage(SceneSettings& sceneSettings) : settings(sceneSettings)
{
    this->lodDistanceSpin = NULL;
}

void SceneSettingsMeshesTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void SceneSettingsMeshesTabPage::Detach()
{
    ReleaseISpinner(this->lodDistanceSpin);    
}

void SceneSettingsMeshesTabPage::GetGUIData()
{
    //Vertex colors enabled checkbox
    this->settings.exportVertexColors = IsDlgButtonCheckedText(IDC_EXPORT_VERTEX_COLORS_CHECK);

    //Vertex color format combo
    HWND vertexColorComboHwnd = GetDlgItem(IDC_EXPORT_VERTEX_COLORS_COMBO);
    int selectedVertexColorIndex = ComboBox_GetCurSel(vertexColorComboHwnd);
    this->settings.vertexColorFormat = GetVertexColorFormatComboValue(selectedVertexColorIndex);
    
    //Text files enabled checkbox
    this->settings.exportTextMeshes = IsDlgButtonCheckedText(IDC_EXPORT_TEXT_MESHES_CHECK);

    //Binary files enabled checkbox
    this->settings.exportBinaryMeshes = IsDlgButtonCheckedText(IDC_EXPORT_BINARY_MESHES_CHECK);

    //Byte ordering
    HWND byteOrderComboHwnd = GetDlgItem(IDC_BINARY_MESH_BYTE_ORDER);
    int selectedByteOrderIndex = ComboBox_GetCurSel(byteOrderComboHwnd);
    this->settings.endian = GetByteOrderComboValue(selectedByteOrderIndex);

    //Generate tangents checkbox
    this->settings.generateTangents = IsDlgButtonCheckedText(IDC_GENERATE_TANGENTS_CHECK);

    //Generate edge lists checkbox
    this->settings.generateEdgeLists = IsDlgButtonCheckedText(IDC_GENERATE_EDGE_LISTS_CHECK);

    //Sort bone assignments checkbox
    this->settings.sortBoneAssignments = IsDlgButtonCheckedText(IDC_SORT_BONE_ASSIGNMENTS_CHECK);

    //Generate LOD checkbox
    this->settings.generateLod = IsDlgButtonCheckedText(IDC_GENERATE_LOD_CHECK);

    //Number of LOD levels edit
    GetDlgItemText(IDC_NUM_LOD_LEVELS_EDIT, this->settings.numLodLevels);

    //LOD distance edit/spin
    this->settings.lodDistance = Utilities::ToString(this->lodDistanceSpin->GetFVal());

    //LOD reduction amount edit
    GetDlgItemText(IDC_LOD_REDUCE_AMOUNT_EDIT, this->settings.lodReductionAmount);

    //LOD reduction type combo
    HWND lodReductionTypeComboHwnd = GetDlgItem(IDC_LOD_REDUCE_TYPE_COMBO);
    int selectedLodReductionTypeIndex = ComboBox_GetCurSel(lodReductionTypeComboHwnd);
    this->settings.lodReductionType = GetLODReductionTypeComboValue(selectedLodReductionTypeIndex);
    
    //Ignore meshes radio button
    this->settings.exportMeshes = _T("");
    this->settings.allowExportMeshes.clear();
    if (IsDlgButtonChecked(IDC_IGNORE_ALL_MESHES_RADIO))
        this->settings.exportMeshes = PropertyValues::FALSE_TEXT;
    else if (IsDlgButtonChecked(IDC_IGNORE_SOME_MESHES_RADIO))
    {
        TSTR meshesText;
        GetDlgItemText(IDC_IGNORE_SOME_MESHES_EDIT, meshesText);    
        StringSplitter meshNames(meshesText, _T('\n'));

        this->settings.allowExportMaterials.reserve(meshNames.size());

        ItemAllow allowMesh;
        allowMesh.allow = PropertyValues::FALSE_TEXT;
        for (size_t i = 0; i < meshNames.size(); i++)
        {
            Utilities::TrimWhitespace(meshNames[i]);
            if (!meshNames[i].isNull())
            {         
                allowMesh.name = meshNames[i];
                this->settings.allowExportMeshes.push_back(allowMesh);
            }
        }
    }
}

void SceneSettingsMeshesTabPage::SetGUIData()
{
    //Vertex colors enabled checkbox
    CheckDlgButton(IDC_EXPORT_VERTEX_COLORS_CHECK, SceneSettings::Values::IsVertexColorsEnabled(this->settings.exportVertexColors));

    //Vertex color format combo
    HWND vertexColorComboHwnd = GetDlgItem(IDC_EXPORT_VERTEX_COLORS_COMBO);
    for (int i = 0; i < VERTEX_COLOR_FORMAT_COUNT; i++)
        ComboBox_AddString(vertexColorComboHwnd, PluginGlobals::GetString(vertexColorFormatComboStringIDs[i]));
    ComboBox_SetCurSel(vertexColorComboHwnd, GetVertexColorFormatValueComboStringIndex(this->settings.vertexColorFormat));    

    //Text files enabled checkbox
    CheckDlgButton(IDC_EXPORT_TEXT_MESHES_CHECK, SceneSettings::Values::IsTextMeshesEnabled(this->settings.exportTextMeshes));

    //Binary files enabled checkbox
    CheckDlgButton(IDC_EXPORT_BINARY_MESHES_CHECK, SceneSettings::Values::IsBinaryMeshesEnabled(this->settings.exportBinaryMeshes));

    //Byte ordering
    HWND byteOrderComboHwnd = GetDlgItem(IDC_BINARY_MESH_BYTE_ORDER);
    for (int i = 0; i < BYTE_ORDER_COUNT; i++)
        ComboBox_AddString(byteOrderComboHwnd, PluginGlobals::GetString(byteOrderComboStringIDs[i]));
    ComboBox_SetCurSel(byteOrderComboHwnd, GetByteOrderValueComboStringIndex(this->settings.endian));    

    //Generate tangents checkbox
    CheckDlgButton(IDC_GENERATE_TANGENTS_CHECK, SceneSettings::Values::IsGenerateTangentsEnabled(this->settings.generateTangents));

    //Generate edge lists checkbox
    CheckDlgButton(IDC_GENERATE_EDGE_LISTS_CHECK, SceneSettings::Values::IsGenerateEdgeListsEnabled(this->settings.generateEdgeLists));

    //Sort bone assignments checkbox
    CheckDlgButton(IDC_SORT_BONE_ASSIGNMENTS_CHECK, SceneSettings::Values::IsSortBoneAssignmentsEnabled(this->settings.sortBoneAssignments));

    //Generate LOD checkbox
    CheckDlgButton(IDC_GENERATE_LOD_CHECK, CommonSettings::Values::IsGenerateLodEnabled(this->settings.generateLod));

    //Number of LOD levels edit
    SetDlgItemText(IDC_NUM_LOD_LEVELS_EDIT, this->settings.numLodLevels);

    //LOD distance edit/spin
    this->lodDistanceSpin = SetupUniverseSpinner(this->hDlg, IDC_LOD_DISTANCE_SPIN, IDC_LOD_DISTANCE_EDIT, 0, 100000, Utilities::ToFloat(this->settings.lodDistance));
    
    //LOD reduction amount edit
    SetDlgItemText(IDC_LOD_REDUCE_AMOUNT_EDIT, this->settings.lodReductionAmount);

    //LOD reduction type combo
    HWND lodReductionTypeComboHwnd = GetDlgItem(IDC_LOD_REDUCE_TYPE_COMBO);
    for (int i = 0; i < CommonSettings::Values::LOD_REDUCTION_TYPE_COUNT; i++)
        ComboBox_AddString(lodReductionTypeComboHwnd, PluginGlobals::GetString(lodReductionTypeComboStringIDs[i]));
    ComboBox_SetCurSel(lodReductionTypeComboHwnd, GetLODReductionTypeValueComboStringIndex(this->settings.lodReductionType));    

    //Ignore meshes radio button
    bool exportMeshes = SceneSettings::Values::ExportMeshes(this->settings.exportMeshes);
    int exportMeshesControlID = exportMeshes ? IDC_IGNORE_SOME_MESHES_RADIO : IDC_IGNORE_ALL_MESHES_RADIO;
    CheckRadioButton(IDC_IGNORE_ALL_MESHES_RADIO, IDC_IGNORE_SOME_MESHES_RADIO, exportMeshesControlID);

    //Ignore meshes edit
    TSTR meshesText;
    CommonSettings::WriteNames(meshesText, this->settings.allowExportMeshes, PluginGlobals::CONTROL_NEWLINE);
    SetDlgItemText(IDC_IGNORE_SOME_MESHES_EDIT, meshesText);    
}

INT_PTR SceneSettingsMeshesTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    return 0;
}

//SceneSettingsMaterialsTabPage-------------------------
SceneSettingsMaterialsTabPage::SceneSettingsMaterialsTabPage(SceneSettings& sceneSettings) : settings(sceneSettings)
{    
}

void SceneSettingsMaterialsTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void SceneSettingsMaterialsTabPage::GetGUIData()
{
    //"No material" name edit
    GetDlgItemText(IDC_NO_MATERIAL_NAME_EDIT, this->settings.noMaterialName);

    //Ignore materials radio button
    this->settings.exportMaterials = _T("");
    this->settings.allowExportMaterials.clear();
    if (IsDlgButtonChecked(IDC_IGNORE_ALL_MATERIALS_RADIO))
        this->settings.exportMaterials = PropertyValues::FALSE_TEXT;
    else if (IsDlgButtonChecked(IDC_IGNORE_SOME_MATERIALS_RADIO))
    {
        TSTR materialsText;
        GetDlgItemText(IDC_IGNORE_SOME_MATERIALS_EDIT, materialsText);        
        StringSplitter materialNames(materialsText, _T('\n'));
        
        this->settings.allowExportMaterials.reserve(materialNames.size());
        
        ItemAllow allowMaterial;
        allowMaterial.allow = PropertyValues::FALSE_TEXT;
        for (size_t i = 0; i < materialNames.size(); i++)
        {
            Utilities::TrimWhitespace(materialNames[i]);
            if (!materialNames[i].isNull())
            {         
                allowMaterial.name = materialNames[i];
                this->settings.allowExportMaterials.push_back(allowMaterial);                
            }
        }
    }
}

void SceneSettingsMaterialsTabPage::SetGUIData()
{    
    //"No material" name edit
    SetDlgItemText(IDC_NO_MATERIAL_NAME_EDIT, this->settings.noMaterialName);

    //Ignore materials radio button
    bool exportMaterials = SceneSettings::Values::ExportMeshes(this->settings.exportMaterials);
    int exportMaterialsControlID = exportMaterials ? IDC_IGNORE_SOME_MATERIALS_RADIO : IDC_IGNORE_ALL_MATERIALS_RADIO;
    CheckRadioButton(IDC_IGNORE_ALL_MATERIALS_RADIO, IDC_IGNORE_SOME_MATERIALS_RADIO, exportMaterialsControlID);

    //Ignore materials edit
    TSTR materialsText;
    CommonSettings::WriteNames(materialsText, this->settings.allowExportMaterials, PluginGlobals::CONTROL_NEWLINE);
    SetDlgItemText(IDC_IGNORE_SOME_MATERIALS_EDIT, materialsText);    
}

INT_PTR SceneSettingsMaterialsTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    return 0;
}

//SceneSettingsBitmapsTabPage-------------------------
SceneSettingsBitmapsTabPage::SceneSettingsBitmapsTabPage(SceneSettings& sceneSettings) : settings(sceneSettings)
{
}

void SceneSettingsBitmapsTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void SceneSettingsBitmapsTabPage::GetGUIData()
{
    TSTR renameText;
    Rename rename;
    StringSplitter bitmapNames;
    
    //Rename bitmaps text
    GetDlgItemText(IDC_RENAME_BITMAPS_EDIT, renameText);    
    bitmapNames.Split(renameText, _T('\n'));
    this->settings.renameBitmaps.clear();
    this->settings.renameBitmaps.reserve(bitmapNames.size());    
    for (size_t i = 0; i < bitmapNames.size(); i++)
    {
        if (Utilities::ParseKeyValueLine(bitmapNames[i], rename.from, rename.to))
            this->settings.renameBitmaps.push_back(rename);
    }

    //Rename bitmap extensions
    GetDlgItemText(IDC_RENAME_BITMAP_EXTENSIONS_EDIT, renameText);    
    bitmapNames.Split(renameText, _T('\n'));
    this->settings.renameBitmapExtensions.clear();
    this->settings.renameBitmapExtensions.reserve(bitmapNames.size());    
    for (size_t i = 0; i < bitmapNames.size(); i++)
    {
        if (Utilities::ParseKeyValueLine(bitmapNames[i], rename.from, rename.to))
            this->settings.renameBitmapExtensions.push_back(rename);
    }
}

void SceneSettingsBitmapsTabPage::SetGUIData()
{ 
    //Rename bitmaps text
    TSTR renameText;
    CommonSettings::Write(renameText, this->settings.renameBitmaps, PluginGlobals::CONTROL_NEWLINE);
    SetDlgItemText(IDC_RENAME_BITMAPS_EDIT, renameText);    

    //Rename bitmap extensions
    renameText = _T("");
    CommonSettings::Write(renameText, this->settings.renameBitmapExtensions, PluginGlobals::CONTROL_NEWLINE);
    SetDlgItemText(IDC_RENAME_BITMAP_EXTENSIONS_EDIT, renameText);    
}

INT_PTR SceneSettingsBitmapsTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    switch (message)
    {
        case WM_COMMAND:
        {
            int id = LOWORD(wParam);
            int code = HIWORD(wParam);

            switch (id)
            {
                case IDC_RENAME_BITMAPS_ADD_BUTTON:
                {
                    switch (code)
                    {
                        case BN_CLICKED:
                        {
                            RenameDialog dialog;
                            dialog.title = PluginGlobals::GetString(IDS_RENAME_BITMAP);
                            if (dialog.DoModal(this->hDlg) == IDOK)
                            {
                                TSTR text;
                                GetDlgItemText(IDC_RENAME_BITMAPS_EDIT, text);
                                Utilities::AppendKeyAndValue(text, dialog.from, dialog.to, PluginGlobals::CONTROL_NEWLINE);
                                SetDlgItemText(IDC_RENAME_BITMAPS_EDIT, text);
                            }

                            handled = true;
                            
                            break;
                        }
                    }
                    break;
                }
                case IDC_RENAME_BITMAP_EXTENSIONS_ADD_BUTTON:
                {
                    switch (code)
                    {
                        case BN_CLICKED:
                        {
                            RenameDialog dialog;
                            dialog.title = PluginGlobals::GetString(IDS_RENAME_BITMAP_EXTENSION);
                            if (dialog.DoModal(this->hDlg) == IDOK)
                            {
                                TSTR text;
                                GetDlgItemText(IDC_RENAME_BITMAP_EXTENSIONS_EDIT, text);
                                Utilities::AppendKeyAndValue(text, dialog.from, dialog.to, PluginGlobals::CONTROL_NEWLINE);
                                SetDlgItemText(IDC_RENAME_BITMAP_EXTENSIONS_EDIT, text);
                            }

                            handled = true;
                            
                            break;
                        }
                    }
                    break;
                }
            }

            break;
        }        
    }

    return 0;
}

//SceneSettingsShadowsTabPage-------------------------
SceneSettingsShadowsTabPage::SceneSettingsShadowsTabPage(SceneSettings& sceneSettings) : settings(sceneSettings)
{
    this->colorSwatch = NULL;
}

void SceneSettingsShadowsTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void SceneSettingsShadowsTabPage::Detach()
{
    ReleaseIColorSwatch(this->colorSwatch);
}

void SceneSettingsShadowsTabPage::GetGUIData()
{
    //Technique units combo
    HWND techniqueComboHwnd = GetDlgItem(IDC_SHADOW_TECHNIQUE_COMBO);
    int selectedTechniqueIndex = ComboBox_GetCurSel(techniqueComboHwnd);
    this->settings.shadowTechnique = GetShadowTechniqueComboValue(selectedTechniqueIndex);

    //Self shadow checkbox
    this->settings.selfShadow = IsDlgButtonCheckedText(IDC_SHADOW_TECHNIQUE_SELF_SHADOW_CHECK);

    //Color
    Color color(this->colorSwatch->GetColor());
    this->settings.shadowColor = Utilities::ToString(color);    
    
    //Texture size value    
    GetDlgItemText(IDC_SHADOW_TEXTURE_SIZE_EDIT, this->settings.shadowTextureSize);

    //Texture count value    
    GetDlgItemText(IDC_SHADOW_TEXTURE_COUNT_EDIT, this->settings.shadowTextureCount);

    //Standard scene manager combo
    HWND shadowCameraTypeHwnd = GetDlgItem(IDC_SHADOW_CAMERA_TYPE_COMBO);
    int selectedShadowCameraIndex = ComboBox_GetCurSel(shadowCameraTypeHwnd);
    this->settings.shadowCameraType = GetShadowCameraTypeComboValue(selectedShadowCameraIndex);

    //Shadow texture plane object edit
    GetDlgItemText(IDC_SHADOW_PLANE_OBJECT_EDIT, this->settings.shadowPlaneObjectName);
}

void SceneSettingsShadowsTabPage::SetGUIData()
{   
    //Technique units combo
    HWND techniqueComboHwnd = GetDlgItem(IDC_SHADOW_TECHNIQUE_COMBO);
    for (int i = 0; i < SHADOW_TECHNIQUE_COUNT; i++)
        ComboBox_AddString(techniqueComboHwnd, PluginGlobals::GetString(shadowTechniqueComboStringIDs[i]));
    ComboBox_SetCurSel(techniqueComboHwnd, GetShadowTechniqueValueComboStringIndex(this->settings.shadowTechnique));

    //Self shadow checkbox
    CheckDlgButton(IDC_SHADOW_TECHNIQUE_SELF_SHADOW_CHECK, SceneSettings::Values::IsSelfShadow(this->settings.selfShadow));

    //Color
    this->colorSwatch = GetIColorSwatch(GetDlgItem(IDC_SHADOW_COLOR_SWATCH));
    Point3 color3 = Utilities::ToPoint3(this->settings.shadowColor);
    Color color(color3.x, color3.y, color3.z);
    this->colorSwatch->SetColor(color, FALSE);
    
    //Texture size value    
    SetDlgItemText(IDC_SHADOW_TEXTURE_SIZE_EDIT, this->settings.shadowTextureSize);

    //Texture count value    
    SetDlgItemText(IDC_SHADOW_TEXTURE_COUNT_EDIT, this->settings.shadowTextureCount);

    //Standard scene manager combo
    HWND shadowCameraTypeHwnd = GetDlgItem(IDC_SHADOW_CAMERA_TYPE_COMBO);
    for (int i = 0; i < SHADOW_CAMERA_TYPE_COUNT; i++)
        ComboBox_AddString(shadowCameraTypeHwnd, PluginGlobals::GetString(shadowCameraTypeComboStringIDs[i]));
    ComboBox_SetCurSel(shadowCameraTypeHwnd, GetShadowCameraTypeValueComboStringIndex(this->settings.shadowCameraType));

    //Shadow texture plane object edit
    SetDlgItemText(IDC_SHADOW_PLANE_OBJECT_EDIT, this->settings.shadowPlaneObjectName);
}

INT_PTR SceneSettingsShadowsTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    return 0;
}

//SceneSettingsDirectoriesTabPage-------------------------
SceneSettingsDirectoriesTabPage::SceneSettingsDirectoriesTabPage(SceneSettings& sceneSettings) : settings(sceneSettings)
{
}

void SceneSettingsDirectoriesTabPage::Attach(HWND hDlg)
{
    DialogBase::Attach(hDlg);

    SetGUIData();
}

void SceneSettingsDirectoriesTabPage::GetGUIData()
{
    GetDlgItemText(IDC_EXPORT_DIRECTORY_EDIT, this->settings.exportPath);
    GetDlgItemText(IDC_MATERIAL_DIRECTORY_EDIT, this->settings.materialPath);
    GetDlgItemText(IDC_MESH_DIRECTORY_EDIT, this->settings.meshPath);
    GetDlgItemText(IDC_MODEL_DIRECTORY_EDIT, this->settings.modelPath);
    GetDlgItemText(IDC_LOG_DIRECTORY_EDIT, this->settings.logPath);    
}

void SceneSettingsDirectoriesTabPage::SetGUIData()
{    
    SetDlgItemText(IDC_EXPORT_DIRECTORY_EDIT, this->settings.exportPath);
    SetDlgItemText(IDC_MATERIAL_DIRECTORY_EDIT, this->settings.materialPath);
    SetDlgItemText(IDC_MESH_DIRECTORY_EDIT, this->settings.meshPath);
    SetDlgItemText(IDC_MODEL_DIRECTORY_EDIT, this->settings.modelPath);
    SetDlgItemText(IDC_LOG_DIRECTORY_EDIT, this->settings.logPath);    
}

INT_PTR SceneSettingsDirectoriesTabPage::DialogProc(UINT message, WPARAM wParam, LPARAM lParam, bool& handled)
{
    return 0;
}

