
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "SettingsSerializer.h"
#include "Utilities.h"
#include "StringSplitter.h"
#include "PluginGlobals.h"

using namespace OgreMax;


//Local functions--------------------------------------------------------------
static void RemoveTrailingCarriage(TSTR& line)
{
    int length = line.Length();
    if (length > 0 && line[length - 1] == _T('\r'))
        line[length - 1] = 0;
}


//Implementation---------------------------------------------------------------
SettingsSerializer::SettingsSerializer()
{
    this->propertiesInMarker = false;
    this->includeMarker = true;
}

void SettingsSerializer::Read(const TSTR& text)
{
    //Extract properties text
    TSTR propertiesText;
    this->propertiesInMarker = this->textInMarker.Parse(text, PluginGlobals::OgreMaxUserPropertyMarker);
    if (this->propertiesInMarker)
        propertiesText = this->textInMarker.GetMarkedText();
    else
        propertiesText = text;

    //Parse lines and properties
    this->textLines.Split(text, _T('\n'));
    for (size_t i = 0; i < this->textLines.size(); i++)
        RemoveTrailingCarriage(this->textLines[i]);
    
    KeyedValues properties;
    properties.Parse(propertiesText);    

    //Iterate over the properties, keeping a note of which lines to remove
    std::vector<int> handledLines;
    handledLines.reserve(this->textLines.size());
    for (KeyedValues::Iterator mapItem = properties.GetIterator();
        mapItem != properties.GetIteratorEnd();
        ++mapItem)
    {
        ParseProperty(mapItem->second.originalKey, mapItem->second.value, mapItem->second.line, handledLines);
    }

    //Remove the parsed lines if the properties are not in an marked OgreMax area
    if (!this->propertiesInMarker)
    {        
        //Sort the line numbers into ascending order
        std::sort(handledLines.begin(), handledLines.end());

        //It's more efficient to go backwards
        for (int i = (int)handledLines.size() - 1; i >= 0; i--)
            this->textLines.Remove(handledLines[i]);
        this->textLines.RemoveTrailingEmptyLines();
    }
}

void SettingsSerializer::Write(TSTR& text)
{    
    TSTR propertiesText;
    GetPropertiesText(propertiesText, PluginGlobals::CONTROL_NEWLINE);

    if (this->propertiesInMarker)
    {
        //This is easy. Since the text originally had an OgreMax marker, just
        //replace the text within the marker
        TSTR replaceWith = PluginGlobals::CONTROL_NEWLINE;
        replaceWith += propertiesText;
        this->textInMarker.ReplaceMarkedText(text, replaceWith, true);
    }
    else
    {
        //Join the remaining text lines. These are lines containing non-OgreMax text
        TSTR nonOgreMaxText = this->textLines.Join(PluginGlobals::CONTROL_NEWLINE.data());
        
        //Remove any instances of the marker as a cleanup step, just in case
        //there happened to be a stray marker somewhere, which would screw up
        //later parses
        Utilities::Replace(text, nonOgreMaxText, PluginGlobals::OgreMaxUserPropertyMarker, _T(""));        

        //Add the OgreMax properties in a marker
        if (!nonOgreMaxText.isNull())
        {
            text += PluginGlobals::CONTROL_NEWLINE;
            text += PluginGlobals::CONTROL_NEWLINE;
        }

        if (this->includeMarker)
        {
            text += PluginGlobals::OgreMaxUserPropertyMarker;
            text += PluginGlobals::CONTROL_NEWLINE;
        }
        text += propertiesText;
        if (this->includeMarker)
            text += PluginGlobals::OgreMaxUserPropertyMarker;
    }
}
