
#ifndef OgreMax_SettingsSerializer_INCLUDED
#define OgreMax_SettingsSerializer_INCLUDED


//Includes---------------------------------------------------------------------
#include "TextInMarkerParser.h"
#include "KeyedValues.h"
#include "StringSplitter.h"


//Classes----------------------------------------------------------------------
namespace OgreMax 
{
    /** Parses and writes text-based settings */
    class SettingsSerializer
    {
    public:
        SettingsSerializer();
        virtual ~SettingsSerializer() {}

        /** 
         * Reads/parses the settings from the specified text 
         * @param text [in] - The user text, which can contain a mix of OgreMax 
         * settings and non-OgreMax text. The non-OgreMax text will not
         * be affected, and the OgreMax settings will be collected from the OgreMax-marked
         * section if present, or the general text area if not present
         */
        void Read(const TSTR& text);

        /** 
         * Writes the OgreMax settings to the specified text, along with the 
         * non-OgreMax text. The OgreMax settings will be placed into an OgreMax-marked section
         */
        void Write(TSTR& text);

    protected:
        /** 
         * Parses a key/value property
         * @param key [in] - The property's key 
         * @param value [in] - The property's value
         * @param line [in] - The line number on which the key/value pair was parsed
         * @param handledLines [out] - The lines that should be removed as a result of the parsing
         */
        virtual void ParseProperty
            (
            const TSTR& key, 
            const TSTR& value, 
            int line, 
            std::vector<int>& handledLines
            ) = 0;

        /** 
         * Gets all the properties as text
         * @param text [out] - The properties, formatted as text
         */
        virtual void GetPropertiesText(TSTR& text, const TSTR& newline) = 0;

    protected:
        /** The non-OgreMax text lines */
        StringSplitter textLines;

        /** Indicates if the OgreMax settings were in an OgreMax-marked area */
        bool propertiesInMarker;

        /** The parser used to read/write OgreMax-marked settings */
        TextInMarkerParser textInMarker;        

        bool includeMarker;
    };
}


#endif