
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "StringSplitter.h"

using namespace OgreMax;


//Implementation---------------------------------------------------------------
StringSplitter::StringSplitter()
{	
}

StringSplitter::StringSplitter(const TSTR& strToSplit, TCHAR splitChar)
{
	Split(strToSplit, splitChar);
}

StringSplitter::StringSplitter(const TCHAR* strToSplit, TCHAR splitChar)
{
	Split(strToSplit, splitChar);
}

int StringSplitter::FindLine(const TSTR& text, int startLine) const
{
    for (int index = startLine; index < (int)size(); index++)
    {
        if ((*this)[index] == text)
            return index;
    }

    return -1;
}

size_t StringSplitter::TotalLength() const
{
    size_t length = 0;

    for (size_t index = 0; index < size(); index++)
        length += (*this)[index].Length();
    
    return length;
}

size_t StringSplitter::TotalLength(int startLine, int lineCount) const
{
    size_t length = 0;

    for (int index = startLine; index < startLine + lineCount; index++)
        length += (*this)[index].Length();
    
    return length;
}

static int Find(const TCHAR* text, TCHAR find, int start)
{
    for (; text[start]; start++)
    {
        if (text[start] == find)
            return start;
    }

    return -1;
}

static TCHAR* Substring(TCHAR* result, const TCHAR* text, int start, int count)
{
    _tcsncpy(result, &text[start], count);
    result[count] = 0;
            
    return result;
}

size_t StringSplitter::Split(const TSTR& strToSplit, TCHAR splitChar)
{
    return Split(strToSplit.data(), splitChar);
}

size_t StringSplitter::Split(const TCHAR* strToSplit, TCHAR splitChar)
{
	clear();
	
	size_t splitCount = CountInstancesOf(strToSplit, splitChar);
	
    resize(splitCount + 1);
	
    std::vector<TCHAR> substring;
    size_t currentString = 0;
    int currentIndex = -1;
    int nextIndex;
    int length;
    while (currentString < size())
	{
        nextIndex = Find(strToSplit, splitChar, currentIndex + 1);
		if (nextIndex != -1)
		{
            length = nextIndex - currentIndex - 1;
			if (length == 0)
			{
                //There are two split chars next to each other
                (*this)[currentString] = _T("");
			}
			else
			{
                //Normal case of a non-empty string
                substring.resize(length + 1);
                (*this)[currentString] = Substring(&substring[0], strToSplit, currentIndex + 1, length);
			}

            //The current index is the now the next
			currentIndex = nextIndex;			
		}			
		else
        {
            //The last string
			(*this)[currentString] = &strToSplit[currentIndex + 1];
        }
		
        currentString++;
	}
	
	return splitCount;
}

TSTR StringSplitter::Join(TCHAR joinChar)
{
	TCHAR joinText[2] = {joinChar, 0};
	
	return Join(joinText);
}

TSTR StringSplitter::Join(const TCHAR* joinText)
{
    TSTR joined;    

    if (size() > 0)
	{
        joined += (*this)[0];
		for (size_t index = 1; index < size(); index++)
		{
			joined += joinText;
			joined += (*this)[index];
		}
	}
	
    return joined;
}

TSTR StringSplitter::Join(const TSTR& joinText)
{
    return Join(joinText.data());
}

size_t StringSplitter::CountInstancesOf(const TCHAR* str, TCHAR c)
{
	size_t count = 0;
	for (size_t index = 0; str[index] != 0; index++)
	{
		if (str[index] == c)
			count++;
	}
	return count;
}

TSTR StringSplitter::Join(TCHAR joinChar, int startLine, int lineCount)
{
    TCHAR joinText[2] = {joinChar, 0};

    return Join(joinText, startLine, lineCount);
}

TSTR StringSplitter::Join(const TCHAR* joinText, int startLine, int lineCount)
{
    TSTR joined;

    if (lineCount > 0)
    {
        joined += (*this)[startLine];
        for (int index = startLine + 1; index < startLine + lineCount; index++)
        {
            joined += joinText;
			joined += (*this)[index];
        }
    }

    return joined;
}

TSTR StringSplitter::Join(const TSTR& joinText, int startLine, int lineCount)
{
    return Join(joinText.data(), startLine, lineCount);
}

void StringSplitter::Remove(int line)
{
    for (int index = line; index < (int)this->size() - 1; index++)
        (*this)[index] = (*this)[index + 1];

    pop_back();
}

void StringSplitter::RemoveTrailingEmptyLines()
{
    while (!empty() && back().isNull())
        pop_back();
}