
//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "TextInMarkerParser.h"

using namespace OgreMax;


//Implementation---------------------------------------------------------------
TextInMarkerParser::TextInMarkerParser()
{
    this->offsetToMarkerStart = -1;
    this->offsetToMarkerStartEnd = -1;
    this->offsetToMarkerEnd = -1;
    this->offsetToMarkerEndEnd = -1;
}

bool TextInMarkerParser::Parse(const TSTR& text, const TSTR& marker)
{
    return Parse(text, marker, marker);
}

bool TextInMarkerParser::Parse(const TSTR& text, const TSTR& markerStart, const TSTR& markerEnd)
{
    this->offsetToMarkerStart = -1;
    this->offsetToMarkerStartEnd = -1;
    this->offsetToMarkerEnd = -1;
    this->offsetToMarkerEndEnd = -1;
    this->markedText = _T("");

    //Find marker start
    const TCHAR* markerStartFoundAt = _tcsstr(text.data(), markerStart.data());
    if (!markerStartFoundAt)
        return false;

    //Find marker end
    const TCHAR* markerEndFoundAt = _tcsstr(markerStartFoundAt + markerStart.Length(), markerEnd.data());
    if (!markerEndFoundAt)
        return false;

    this->text = text;
    this->offsetToMarkerStart = markerStartFoundAt - text.data();
    this->offsetToMarkerStartEnd = this->offsetToMarkerStart + markerStart.Length();
    this->offsetToMarkerEnd = markerEndFoundAt - text.data();
    this->offsetToMarkerEndEnd = this->offsetToMarkerEnd + markerEnd.Length();
    this->markedText = text.Substr(this->offsetToMarkerStartEnd, this->offsetToMarkerEnd - this->offsetToMarkerStartEnd);

    return true;
}

const TSTR& TextInMarkerParser::GetMarkedText() const
{
    return this->markedText;
}

void TextInMarkerParser::ReplaceMarkedText(TSTR& result, const TSTR& replaceWith, bool includeMarkers) const
{
    if (this->offsetToMarkerStart != -1 && this->offsetToMarkerEnd != -1)
    {
        if (includeMarkers)
            result = this->text.Substr(0, this->offsetToMarkerStartEnd);
        else
            result = this->text.Substr(0, this->offsetToMarkerStart);

        result += replaceWith;

        if (includeMarkers)
            result += this->text.Substr(this->offsetToMarkerEnd, this->text.Length() - this->offsetToMarkerEnd);
        else
            result += this->text.Substr(this->offsetToMarkerEndEnd, this->text.Length() - this->offsetToMarkerEndEnd);
    }
    else
        result = _T("");
}