#ifndef OgreMax_UniqueCoordinateTab_INCLUDED
#define OgreMax_UniqueCoordinateTab_INCLUDED


//Classes----------------------------------------------------------------------
namespace OgreMax
{
    /**
     * A lookup table that only allows unique coordinates to be added.
     * Uniqueness is determined by making sure newly added coordinates
     * aren't within a specified minimum distance from other coordinates
     */
    class UniqueCoordinateTab : Tab<float>
    {
    public:
        UniqueCoordinateTab(float minDistance = .001f)
        {
            this->minDistance = minDistance;
        }

        bool AddCoordinate(float coordinate)
        {
            float distance;

            //If the specified coordinate is within the prescribed minimum 
            //distance, exit early to prevent it from being added
            for (int index = 0; index < Count(); index++)
            {
                distance = abs((*this)[index] - coordinate);
                if (distance < this->minDistance)
                    return false;
            }

            //The coordinate is unique. Add it
            Tab::Append(1, &coordinate);

            return true;
        }

        int Count()
        {
            return Tab<float>::Count();
        }

    protected:
        float minDistance;
    };
}

#endif