//Includes---------------------------------------------------------------------
#include "stdafx.h"
#include "Vertex.h"

using namespace OgreMax;


//Implementation---------------------------------------------------------------
Vertex::Vertex(float x, float y, float z) 
{
	this->position.x = x;
	this->position.y = y;
	this->position.z = z;
}

void Vertex::SetNormal(float x, float y, float z) 
{
	this->normal.x = x;
	this->normal.y = y;
	this->normal.z = z;
}

void Vertex::SetColor(float r, float g, float b, float a) 
{
	this->color.r = r;
	this->color.g = g;
	this->color.b = b;
	this->color.a = a;
}

void Vertex::AddTexCoord(int mapIndex, float u, float v, float w) 
{
	Ogre::Vector3 uvw(u, v, w);
	this->uvwMap[mapIndex] = uvw;
}

bool Vertex::HasSameTexCoords(const TexCoordMap& uvwMap) const
{
	for (TexCoordMap::const_iterator mappedVertex = this->uvwMap.begin(); 
        mappedVertex != this->uvwMap.end(); 
        ++mappedVertex)
    {
        TexCoordMap::const_iterator foundMappedVertex = uvwMap.find(mappedVertex->first);
        if (foundMappedVertex != uvwMap.end() && foundMappedVertex->second != mappedVertex->second)
			return false;
	}

	return true;
}

bool Vertex::operator == (const Vertex& other) const
{
	return 
        this->position == other.position &&
		this->normal == other.normal &&
		this->color == other.color &&
		other.HasSameTexCoords(this->uvwMap);
}

VertexList::VertexList()
{
    this->vertexCount = 0;
}

unsigned int VertexList::Add(Vertex& v, bool* isNew, bool fastAdd) 
{
    unsigned int index = 0;

    if (fastAdd)
    {
        //Add the vertex without checking for duplicates
        index = this->vertexCount;
        this->vertexList.push_back(v);
        this->vertexCount++;
        
        if (isNew != NULL)
            *isNew = true;
    }
    else
    {
        //Check for duplicates
        std::list<Vertex>::iterator vertexIterator = this->vertexList.begin();
	    for (; vertexIterator != this->vertexList.end(); ++vertexIterator, ++index) 
        {
		    if (*vertexIterator == v)
			    break;
	    }

        //Only add the vertex if we reached the end
        if (vertexIterator == this->vertexList.end())
        {
            this->vertexList.push_back(v);
            this->vertexCount++;
        }

        if (isNew != NULL)
            *isNew = vertexIterator == this->vertexList.end();
    }

    return index;
}