
#ifndef OgreMax_Vertex_INCLUDED
#define OgreMax_Vertex_INCLUDED


//Classes----------------------------------------------------------------------
namespace OgreMax 
{
	class Vertex 
    {
	public:
		Vertex(float x, float y, float z);

		void SetNormal(float x, float y, float z);
		void SetColor(float r, float g, float b, float a = 1.0);
		void AddTexCoord(int mapIndex, float u, float v, float w = 0.0);

		bool operator == (const Vertex& other) const;

        typedef std::map<int, Ogre::Vector3> TexCoordMap;

		const Ogre::Vector3& GetUVW(int mapIndex) const 
        { 
            return this->uvwMap.find(mapIndex)->second; 
        }

    private:
		bool HasSameTexCoords(const TexCoordMap& uvwMap) const;
	
    public:
		Ogre::Vector3 position;
		Ogre::Vector3 normal;
		Ogre::ColourValue color;
		TexCoordMap	uvwMap;
	};

	class VertexList 
    {
	public:
        VertexList();

		// returns the index into the list for the inserted element
		unsigned int Add(Vertex& v, bool* isNew = NULL, bool fastAdd = false);
		size_t GetSize() { return this->vertexList.size(); }
		const Vertex& GetFront() { return this->vertexList.front(); }
		void Pop() { this->vertexList.pop_front(); }

	private:
		std::list<Vertex> vertexList;
        unsigned int vertexCount;
	};
}

#endif