#include <assert.h>
#include <string>
#include<map>
#include <set>
#include <stdarg.h>
#ifdef _WIN32
#include <direct.h>
#include <windows.h>
#else
#include <sys/stat.h>
#endif
#include "generate_shaders.hpp"
static bool beginsWithConst(const std::string&shader_name,const std::string&prop) {
    return shader_name.find(prop)==0;
}
static bool starts(std::string &shader_name,const std::string&prop) {
    std::string::size_type where=shader_name.find(prop);
    if (where==0) {
        shader_name=shader_name.substr(prop.length());
        return true;
    }
    return false;
}

static bool appendDef(std::string&macro,const std::string&sp,std::string&shader_name,const std::string&prop,const std::string &defn, bool uv_needed) {
    macro+=sp;
    macro+=defn;
    macro+='=';    
    bool retval=false;
    if (starts(shader_name,prop)) {
        macro+="1";
        retval=true;
    }else {
        macro+="0";
    }
    if (uv_needed) {
        macro+=sp;
        macro+=defn.substr(4)+"_UV=";
        if (shader_name[0]>='0'&&shader_name[0]<='2') {
            macro+=shader_name[0];
            shader_name=shader_name.substr(1);
        }else {
            macro+='0';
        }
    }
    
    return retval;
}
static std::string compilerType(std::string name) {
    size_t len=name.length();
    if (name[len-4]=='c'&&name[len-3]=='g')
        return "cg";
    if (name[len-4]=='h'&&name[len-3]=='l')
        return "hlsl";
    if (name[len-4]=='g'&&name[len-3]=='l')
      return "glsl";
    if (name[len-3]=='c'&&name[len-2]=='g')
        return "cg";
    if (name[len-3]=='h'&&name[len-2]=='l')
        return "hlsl";
    if (name[len-3]=='g'&&name[len-2]=='l')
        return "glsl";

    if (name[len-2]=='c'&&name[len-1]=='g')
        return "cg";
    if (name[len-2]=='h'&&name[len-1]=='l')
        return "hlsl";
    if (name[len-2]=='g'&&name[len-1]=='l')
        return "glsl";
    return "cg";
}
static bool isVertexProgram(std::string name) {
    size_t len=name.length();
    if (name[len-5]=='P'&&name[len-6]=='V')
        return true;
    if (name[len-5]=='P'&&name[len-6]=='F')
        return false;
    if (name[len-3]=='P'&&name[len-4]=='V')
        return true;
    if (name[len-3]=='P'&&name[len-4]=='F')
        return false;
    if (name[len-1]=='P'&&name[len-2]=='V')
        return true;
    if (name[len-1]=='P'&&name[len-2]=='F')
        return false;
    if (name[len-2]=='P'&&name[len-3]=='V')
        return true;
    if (name[len-2]=='P'&&name[len-3]=='F')
        return false;
    if (name[len-4]=='P'&&name[len-5]=='V')
        return true;
    if (name[len-4]=='P'&&name[len-5]=='F')
        return false;
    return false;
}
namespace ProgramDefines {
std::string nickname[NUM_NICK]={"Skin",  "TwoSided"  ,"Amb","Env",      "Occl",                "Sub",          "Diffuse",    "Dirt",          "Grunge","Cruftmal",           "Point",          "Spec",       "Dirt",            "Grunge",           "Gloss",        "Glow",   "Dirt",        "Grunge",       "Norm",     "Bump",          "Reflect",         "Shad",   "Deriv",    "Lev",   "Adv",        "H",             "VP",   "FP",   "glsl",   "hlsl",   "cg",   "GL"};
static bool check (ProgramDefines::NICKNAMES lastOption, int opt[ProgramDefines::NUM_NICK], NICKNAMES a, NICKNAMES b) {
    return (lastOption==a&&opt[b])||(lastOption==b&&opt[a]);
}
bool capCompat(ProgramDefines::NICKNAMES lastOption, int opt[ProgramDefines::NUM_NICK]) {
    int oldopt=lastOption<NUM_NICK?opt[lastOption]:0;
    if (lastOption<NUM_NICK)
        opt[lastOption]=1;
    bool retval=true;
    bool second_cruft_slot=opt[SPECULAR_CRUFT_NICK]||((opt[AMBIENT_ENVIRONMENT_NICK]||opt[DIFFUSE_CRUFT_NICK])&&opt[DIRT_NORMAL_NICK]);
    if (opt[DIFFUSE_CRUFT_NICK]&&opt[AMBIENT_ENVIRONMENT_NICK])
        retval=false;
    bool third_cruft_slot=(opt[SPECULAR_CRUFT_NICK]&&(opt[AMBIENT_ENVIRONMENT_NICK]||opt[DIFFUSE_CRUFT_NICK])&&opt[DIRT_NORMAL_NICK]);
		
	if (third_cruft_slot&&(opt[REFLECTION_NICK]!=0||opt[GLOW_NICK]!=0))
        retval=false;//ambient env means that dirt normal needs to live in specular cruft and if it cant fit there then both reflection and glow had better be "no"
    if (second_cruft_slot&&opt[GLOW_NICK]&&opt[REFLECTION_NICK])
        retval=false;//second slot and both glow and reflection present, so glow can't live in last slot
    
    if (lastOption<NUM_NICK)
        opt[lastOption]=oldopt;
    return retval;
    int lastNorm=opt[DIFFUSE_CRUFT_NICK];
    int lastSpec=opt[SPECULAR_CRUFT_NICK];
    if (check(lastOption,opt,DIFFUSE_CRUFT_NICK,DIRT_NORMAL_NICK))
        opt[SPECULAR_CRUFT_NICK]=opt[DIRT_NORMAL_NICK];//normal fallback to specular slot
    if (opt[DIRT_NORMAL_NICK])
        opt[DIFFUSE_CRUFT_NICK]=opt[DIRT_NORMAL_NICK];
    if (lastOption==DIRT_NORMAL_NICK)
        lastOption=DIFFUSE_CRUFT_NICK;
    if (check(lastOption,opt,AMBIENT_ENVIRONMENT_NICK,DIFFUSE_CRUFT_NICK))
        if (lastNorm==0)
            opt[SPECULAR_CRUFT_NICK]=opt[DIRT_NORMAL_NICK];//normal fallback to specular slot
        else
            retval=false;
    if (check(lastOption,opt,GLOW_NICK,SPECULAR_CRUFT_NICK)&&opt[REFLECTION_NICK]!=0)
        retval=false;
    if (check(lastOption,opt,REFLECTION_NICK,SPECULAR_CRUFT_NICK)&&opt[GLOW_NICK]!=0)
        retval=false;
    opt[DIFFUSE_CRUFT_NICK]=lastNorm;
    opt[SPECULAR_CRUFT_NICK]=lastSpec;
    return true;
}
}

static std::string itostr(unsigned int i) {
    char test[16];
    sprintf(test,"%d",i);
    return test;
}

static std::string generatePreprocessorString(std::string name, const std::string &sp,bool &skinning,bool&diffuse_environment, bool &diffuse_cruft, bool &specular_cruft,bool &dirt_normal, bool&reflection,bool&glow, bool &spotlight, unsigned int &num_lights, unsigned int &num_shadow_lights, bool&use_bump) {
    using namespace ProgramDefines;
    bool vertexShader=isVertexProgram(name);
    bool fragShader=!vertexShader;
    
    std::string retval(sp.find(';')==std::string::npos?sp:std::string());
    retval+="DEBUG=0";
    skinning=false;
    if (appendDef(retval,sp,name,nickname[SKINNING_NICK],"USE_SKINNING",false)){
        retval+=sp+"NUM_BONES=3";
        skinning=true;
    }
	appendDef(retval,sp,name,nickname[TWO_SIDED_NICK],"USE_TWO_SIDED",false);
    appendDef(retval,sp,name,nickname[AMBIENT_NICK],"USE_AMBIENT",false);
    diffuse_environment=beginsWithConst(name,nickname[AMBIENT_ENVIRONMENT_NICK]);
    appendDef(retval,sp,name,nickname[AMBIENT_ENVIRONMENT_NICK],"USE_AMBIENT_ENVIRONMENT",false);
    appendDef(retval,sp,name,nickname[AMBIENT_OCCLUSION_NICK],"USE_AMBIENT_OCCLUSION",true);
    appendDef(retval,sp,name,nickname[SUBSURFACE_NICK],"USE_SUBSURFACE",false);
    appendDef(retval,sp,name,nickname[DIFFUSE_NICK],"USE_DIFFUSE",true);
    if (appendDef(retval,sp,name,nickname[DIFFUSE_DIRT_NICK],"USE_DIFFUSE_DIRT",true)) {
        if (starts(name,"M")) {
            retval+=sp+"COMBINE_DIFFUSE_DIRT=combineMultiply";
        }else if (starts(name,"O")) {
            retval+=sp+"COMBINE_DIFFUSE_DIRT=combineOverlay";
        }else {
            retval+=sp+"COMBINE_DIFFUSE_DIRT=combineOver";
        }
    }
    diffuse_cruft=false;
    if (appendDef(retval,sp,name,nickname[DIFFUSE_CRUFT_NICK],"USE_DIFFUSE_CRUFT",true)) {
        diffuse_cruft=true;
        if (starts(name,"M")) {
            retval+=sp+"COMBINE_DIFFUSE_CRUFT=combineMultiply";
        }else if (starts(name,"O")) {
            retval+=sp+"COMBINE_DIFFUSE_CRUFT=combineOverlay";
        }else {
            retval+=sp+"COMBINE_DIFFUSE_CRUFT=combineOver";
        }
    }
    dirt_normal=false;
    if (appendDef(retval,sp,name,nickname[DIRT_NORMAL_NICK],"USE_DIRT_NORMAL",true)) {
        dirt_normal=true;
        if (starts(name,"M")) {
            retval+=sp+"COMBINE_DIRT_NORMAL=combineAddAlpha";
        }else if (starts(name,"O")) {
            retval+=sp+"COMBINE_DIRT_NORMAL=combineAdd";
        }else {
            retval+=sp+"COMBINE_DIRT_NORMAL=combineAlpha";
        }
    }
    if (starts(name,nickname[NO_SPOTLIGHT_NICK])) {
        retval+=sp+"USE_SPOTLIGHT=0";//see it's inverted, so our scheme wn
        spotlight=false;
    }else {
        retval+=sp+"USE_SPOTLIGHT=1";
        spotlight=true;
    }
    appendDef(retval,sp,name,nickname[SPECULAR_NICK],"USE_SPECULAR",true);
    if (appendDef(retval,sp,name,nickname[SPECULAR_DIRT_NICK],"USE_SPECULAR_DIRT",true)) {
        if (starts(name,"M")) {
            retval+=sp+"COMBINE_SPECULAR_DIRT=combineMultiply";
        }else if (starts(name,"O")) {
            retval+=sp+"COMBINE_SPECULAR_DIRT=combineOverlay";
        }else {
            retval+=sp+"COMBINE_SPECULAR_DIRT=combineOver";
        }
    }
    specular_cruft=false;
    if (appendDef(retval,sp,name,nickname[SPECULAR_CRUFT_NICK],"USE_SPECULAR_CRUFT",true)) {
        specular_cruft=true;
        if (starts(name,"M")) {
            retval+=sp+"COMBINE_SPECULAR_CRUFT=combineMultiply";
        }else if (starts(name,"O")) {
            retval+=sp+"COMBINE_SPECULAR_CRUFT=combineOverlay";
        }else {
            retval+=sp+"COMBINE_SPECULAR_CRUFT=combineOver";
        }
    }
    appendDef(retval,sp,name,nickname[GLOSSINESS_NICK],"USE_GLOSSINESS",false);    
    glow=false;
    if (appendDef(retval,sp,name,nickname[GLOW_NICK],"USE_GLOW",true)) {
        glow=true;
    }
    if (appendDef(retval,sp,name,nickname[GLOW_DIRT_NICK],"USE_GLOW_DIRT",true)) {
        if (starts(name,"M")) {
            retval+=sp+"COMBINE_GLOW_DIRT=combineMultiply";
        }else if (starts(name,"O")) {
            retval+=sp+"COMBINE_GLOW_DIRT=combineOverlay";
        }else {
            retval+=sp+"COMBINE_GLOW_DIRT=combineOver";
        }
    }
    if (appendDef(retval,sp,name,nickname[GLOW_CRUFT_NICK],"USE_GLOW_CRUFT",true)) {
        if (starts(name,"M")) {
            retval+=sp+"COMBINE_GLOW_CRUFT=combineMultiply";
        }else if (starts(name,"O")) {
            retval+=sp+"COMBINE_GLOW_CRUFT=combineOverlay";
        }else {
            retval+=sp+"COMBINE_GLOW_CRUFT=combineOver";
        }
    }
    appendDef(retval,sp,name,nickname[NORMAL_NICK],"USE_NORMAL",true);
    if (appendDef(retval,sp,name,nickname[BUMP_NICK],"USE_BUMP",true)) {
        use_bump=true;
    }else {
        use_bump=false;
    }
    reflection=appendDef(retval,sp,name,nickname[REFLECTION_NICK],"USE_REFLEECTION",false);
    bool shadows=appendDef(retval,sp,name,nickname[SHADOW_NICK],"USE_SHADOWS",false);
    retval+=sp+"START_LIGHT=0";
    appendDef(retval,sp,name,nickname[DERIVATIVE_NICK],"DERIVATIVE_SUPPORT",false);
    appendDef(retval,sp,name,nickname[LOD_NICK],"HAVE_LOD",false);
    bool adv=appendDef(retval,sp,name,nickname[ADVANCED_NICK],"HAVE_TEN_TEXCOORD",false);
    retval+=sp+"NUM_LIGHTS=";
    if (starts(name,nickname[NUL_LIGHTS_NICK])==false) {
        fprintf(stderr,"SHADER %s does not start with %s",name.c_str(),nickname[NUL_LIGHTS_NICK].c_str());
    }
    std::string lights=name.substr(0,1);
    num_lights=atoi(lights.c_str());
    retval+=lights;
    name=name.substr(1);
    if (shadows){
        retval+=sp+"NUM_SHADOW_LIGHTS=";
        num_shadow_lights=adv?num_lights:(num_lights>4?4:num_lights);
		
		retval+=itostr(num_shadow_lights);
    }else {
        retval+=sp+"NUM_SHADOW_LIGHTS=0";
        num_shadow_lights=0;
    }
    retval+=sp+"MAX_LIGHTS=";
    retval+=lights;
    bool cg=false,hlsl=false,glsl=false;
    bool DX=true;
    if (starts(name,nickname[VP_NICK])) {
        retval+=sp+"USE_VERTEX_SHADER=1";
        retval+=sp+"main_vp=main";
        if (starts(name,nickname[CG_NICK])) {
            if (starts(name,nickname[GL_NICK])) {
                DX=false;
                if (skinning) {                
                    retval+=sp+"BLENDINDICES=ATTR7";
                    retval+=sp+"BLENDWEIGHT=ATTR1 -profileopts MaxLocalParams=144";
                }
            }
        }
    }else {
        retval+=sp+"USE_VERTEX_SHADER=0";
        retval+=sp+"main_fp=main";
    }
    appendDef(retval,sp,name,nickname[FP_NICK],"USE_FRAGMENT_SHADER",false);
    appendDef(retval,sp,name,nickname[GLSL_NICK],"USE_GLSL",false);
    return retval;
}


static std::string programDefinition(std::string name, std::string profile,bool &ambient_environment, bool&reflection,bool&glow, unsigned int &num_shadow_lights) {
    unsigned int num_lights=0;
    bool vp=isVertexProgram(name);
    std::string retval;
    if (vp) {
        retval+="vertex_program ";
    }else retval+="fragment_program ";
    retval+=name;
    retval +=' ';
    std::string compiler_type=compilerType(name);
    retval+=compiler_type;
    retval+="\n{\n\tsource UeberShader.hlsl\n";
    std::string preprocessor_separator=";";
    if (compiler_type=="hlsl") {
        retval+="\tentry_point main\n";
        retval+="\ttarget "+profile+"\n\tpreprocessor_defines ";

    }else if (compiler_type=="cg") {
        retval+="\tentry_point main\n";
        retval+="\tprofiles "+profile+"\n\tcompile_arguments ";
        preprocessor_separator=" -D";
    }else {
        retval+="\n\tpreprocessor_defines ";
    }
    bool skinning=false;
    bool spotlight=true;
    bool dirt_normal=false;
    bool diffuse_cruft=false;
    bool specular_cruft=false;
    bool use_bump=false;
    retval+=generatePreprocessorString(name,
        preprocessor_separator,
        skinning,
        ambient_environment,
        diffuse_cruft,
        specular_cruft,
        dirt_normal,
        reflection,
        glow,
        spotlight,
        num_lights,
        num_shadow_lights,
        use_bump);
    if (skinning)
        retval+="\n\tincludes_skeletal_animation true";
    retval+="\n\tdefault_params\n\t{\n\t\t";

    if (compiler_type=="glsl"&&vp==false) {

        retval+="param_named ambientOcclusionMap int 0\n\t\t"
            "param_named diffuseMap int 1\n\t\t"
            "param_named diffuseDirtMap int 2\n\t\t";
        if (ambient_environment) {
            retval+="param_named diffuseEnvironmentMap int 3\n\t\t";
        }else {
            if (diffuse_cruft||dirt_normal==false)
                retval+="param_named diffuseCruftMap int 3\n\t\t";
            else
                retval+="param_named dirtNormalMap int 3\n\t\t";
        }
        retval+="param_named bumpMap int 4\n\t\t"
            "param_named normalMap int 5\n\t\t"
            "param_named specularMap int 6\n\t\t"
            "param_named specularDirtMap int 7\n\t\t";
        if (glow==false||reflection==false) {
            if (specular_cruft) {
                retval+="param_named specularCruftMap int 8\n\t\t";
            }else {
                if (dirt_normal&&(ambient_environment!=false||diffuse_cruft!=false))  {
                    retval+="param_named dirtNormalMap int 8\n\t\t";
                }else {
                    
                }
            }
        } else {
            retval+="param_named glowMap int 8\n\t\t";
        }
        if (reflection)
            retval+="param_named reflectionMap int 9\n\t\t";
        else {
            if (glow) {
                retval+="param_named glowMap int 9\n\t\t";
            }else if (specular_cruft&&dirt_normal&&glow==false&&reflection==false&&(ambient_environment!=false||diffuse_cruft!=false)) {
                retval+="param_named dirtNormalMap int 9\n\t\t";
            }
        }
        if (num_shadow_lights>0) {
            if (num_shadow_lights>1){
                retval+="param_named shadowMap[0] int 10\n\t\t";
                retval+="param_named shadowMap[1] int 11\n\t\t";
            }else {
                retval+="param_named shadowMap int 10\n\t\t";                
            }
            if (num_shadow_lights>2)
                retval+="param_named shadowMap[2] int 12\n\t\t";
            if (num_shadow_lights>3)
                retval+="param_named shadowMap[3] int 13\n\t\t";
            if (num_shadow_lights>4)
                retval+="param_named shadowMap[4] int 14\n\t\t";
            if (num_shadow_lights>5)
                retval+="param_named shadowMap[5] int 15\n\t\t";            
        }
    }
    if (vp) {
        if (skinning) {
            retval+="param_named_auto worldMatrix3x4Array world_matrix_array_3x4\n\t\t"
                "param_named_auto worldViewProj viewproj_matrix\n\t\t";
        }else {
            retval+="param_named_auto worldMatrix world_matrix\n\t\t"
                "param_named_auto worldViewProj worldviewproj_matrix\n\t\t";
        }
    }else {
        if (!ambient_environment)
            retval+="param_named_auto derived_ambient_color ambient_light_colour\n\t\t";
        std::string nlights=itostr(num_lights);
        if (num_lights>1) {
            retval+="param_named_auto light_position light_position_array "+nlights+"\n\t\t";
            if (spotlight) {
                retval+="param_named_auto spotlight_direction light_direction_array "+nlights+"\n\t\t";
                retval+="param_named_auto light_spotlight_params spotlight_params_array "+nlights+"\n\t\t";
            }
            retval+="param_named_auto light_attenuation light_attenuation_array "+nlights+"\n\t\t";
            retval+="param_named_auto derived_light_specular_color light_specular_colour_power_scaled_array "+nlights+"\n\t\t";
            retval+="param_named_auto derived_light_diffuse_color light_diffuse_colour_power_scaled_array "+nlights+"\n\t\t";
        }else if (num_lights==1) {
            retval+="param_named_auto light_position light_position 0\n\t\t";
            if (spotlight) {
                retval+="param_named_auto spotlight_direction light_direction 0\n\t\t";
                retval+="param_named_auto light_spotlight_params spotlight_params 0\n\t\t";
            }
            retval+="param_named_auto light_attenuation light_attenuation 0\n\t\t";
            retval+="param_named_auto derived_light_specular_color light_specular_colour_power_scaled 0\n\t\t";
            retval+="param_named_auto derived_light_diffuse_color light_diffuse_colour_power_scaled 0\n\t\t";
        }
        retval+="param_named_auto eyePosition camera_position\n\t\tparam_named_auto colorVariance custom 0\n";
        if (use_bump) {
            retval+="\t\tparam_named_auto numParallaxSteps custom 1\n";
        }
    }            
    retval+="\t}\n}\n";
    return retval;    
}
namespace DXCaps {

struct ShaderCapabilities {
    int values[ProgramDefines::NUM_NICK];
};
enum DELEGATE_TYPE {
    ps_3_0,  ps_2_b,  ps_2_0,  cg_ps_3_0,  cg_ps_2_xfp40,  cg_ps_2_0,  arbfp1,  glsl,
    vs_3_0,  vs_2_b,  vs_2_0,  cg_vs_3_0,  cg_vs_2_xvp40,  cg_vs_2_0,  arbvp1,  glslv,
    NUM_DELEGATES
};
static const char* profileData[NUM_DELEGATES]={0};
int delegateCaps[NUM_DELEGATES][ProgramDefines::NUM_NICK]={0};
static int setDelegateCaps() {
    using namespace ProgramDefines;
    using namespace DXCaps;
    profileData[ps_3_0]="ps_3_0";
    profileData[ps_2_b]="ps_2_b";
    profileData[ps_2_0]="ps_2_0";
    profileData[cg_ps_3_0]="ps_3_0 fp40unlimited";
    profileData[cg_ps_2_xfp40]="ps_2_x fp40";
    profileData[cg_ps_2_0]="ps_2_0";
    profileData[arbfp1]="arbfp1";
    profileData[glsl]="glsl";

    profileData[vs_3_0]="vs_3_0";
    profileData[vs_2_b]="vs_2_0";
    profileData[vs_2_0]="vs_2_0";
    profileData[cg_vs_3_0]="vs_3_0 vp40unlimited";
    profileData[cg_vs_2_xvp40]="vs_2_0 vp40";
    profileData[cg_vs_2_0]="vs_2_0";
    profileData[arbvp1]="arbvp1";
    profileData[glslv]="glsl";

    for(int i=0;i<=FP_NICK||i<=VP_NICK;++i) {
        if (i!=SKINNING_NICK&&i!=VP_NICK) {
            delegateCaps[ps_3_0][i]=1;
            delegateCaps[cg_ps_3_0][i]=1;
            delegateCaps[glsl][i]=1;
            if (i!=ADVANCED_NICK&&i!=LOD_NICK) {
				if (i!=BUMP_NICK) {
	                delegateCaps[cg_ps_2_xfp40][i]=1;
	                if (i!=DERIVATIVE_NICK) {
		                delegateCaps[ps_2_b][i]=1;
			            delegateCaps[arbfp1][i]=1;
				        if (i!=SPECULAR_CRUFT_NICK&&i!=DIFFUSE_CRUFT_NICK&&i!=DIRT_NORMAL_NICK) {
					        delegateCaps[cg_ps_2_0][i]=1;
						    delegateCaps[ps_2_0][i]=1;
						}
                    }
                }
            }                
        }
    }

    for (int i=0;i<vs_3_0;++i) {
        if (delegateCaps[i][AMBIENT_OCCLUSION_NICK]) delegateCaps[i][AMBIENT_OCCLUSION_NICK]=4;
        if (delegateCaps[i][DIFFUSE_NICK]) delegateCaps[i][DIFFUSE_NICK]=4;
        if (delegateCaps[i][SPECULAR_NICK]) delegateCaps[i][SPECULAR_NICK]=4;
        if (delegateCaps[i][GLOW_NICK])delegateCaps[i][GLOW_NICK]=4;
        if(delegateCaps[i][BUMP_NICK])delegateCaps[i][BUMP_NICK]=4;
        if (delegateCaps[i][NORMAL_NICK])delegateCaps[i][NORMAL_NICK]=4;
        if(delegateCaps[i][DIRT_NORMAL_NICK])delegateCaps[i][DIRT_NORMAL_NICK]=11;
        if(delegateCaps[i][DIFFUSE_DIRT_NICK])delegateCaps[i][DIFFUSE_DIRT_NICK]=11;
        if(delegateCaps[i][DIFFUSE_CRUFT_NICK])delegateCaps[i][DIFFUSE_CRUFT_NICK]=11;
        if(delegateCaps[i][SPECULAR_DIRT_NICK])delegateCaps[i][SPECULAR_DIRT_NICK]=11;
        if(delegateCaps[i][SPECULAR_CRUFT_NICK])delegateCaps[i][SPECULAR_CRUFT_NICK]=11;
    }    
    delegateCaps[ps_2_0][DIFFUSE_DIRT_NICK]=7;
    delegateCaps[ps_2_0][SPECULAR_DIRT_NICK]=7; 
    delegateCaps[cg_ps_2_0][DIFFUSE_DIRT_NICK]=7;
    delegateCaps[cg_ps_2_0][SPECULAR_DIRT_NICK]=7;    

    delegateCaps[cg_ps_3_0][NUL_LIGHTS_NICK]=6;
    delegateCaps[ps_3_0][NUL_LIGHTS_NICK]=6;
    delegateCaps[glsl][NUL_LIGHTS_NICK]=6;
    delegateCaps[arbfp1][NUL_LIGHTS_NICK]=4;
    delegateCaps[cg_ps_2_xfp40][NUL_LIGHTS_NICK]=4;
    delegateCaps[ps_2_b][NUL_LIGHTS_NICK]=4;
    delegateCaps[ps_2_0][NUL_LIGHTS_NICK]=1;
    delegateCaps[cg_ps_2_0][NUL_LIGHTS_NICK]=1;
 
    delegateCaps[ps_3_0][HLSL_NICK]=1;
    delegateCaps[ps_2_b][HLSL_NICK]=1;
    delegateCaps[ps_2_0][HLSL_NICK]=1;
    delegateCaps[cg_ps_3_0][CG_NICK]=1;
    delegateCaps[cg_ps_2_xfp40][CG_NICK]=1;
    delegateCaps[cg_ps_2_0][CG_NICK]=1;
    delegateCaps[arbfp1][CG_NICK]=1;
    delegateCaps[arbfp1][GL_NICK]=1;
    delegateCaps[glsl][GLSL_NICK]=1;

    for(int i=0;i<=FP_NICK||i<=VP_NICK;++i) {
        if (i!=SUBSURFACE_NICK&&
            i!=AMBIENT_NICK&&
            i!=AMBIENT_OCCLUSION_NICK&&//doesn't matter to vp
            i!=NO_SPOTLIGHT_NICK&&//doesn't matter to vp...just xtra arg to fp
            i!=DIFFUSE_NICK&&i!=SPECULAR_NICK&&//not important to vp
            i!=GLOSSINESS_NICK&&//doesn't matter to vp, just alpha chan
            i!=GLOW_NICK&&//doesn't matter to vp(dirt does)
            i!=REFLECTION_NICK&&//just a texture later on
            //NEED DERIVATIVES TO KEEP BUMP ALIVE IN FACE OF DIRT in Vertex shader i!=DERIVATIVE_NICK&&//no derivatives in vp
            i!=LOD_NICK&&//no tex lookups in vp, and even so they would not use lod
            i!=FP_NICK&&
            i!=TWO_SIDED_NICK&&
            i!=NORMAL_NICK&&
            i!=DIFFUSE_DIRT_NICK&&
            i!=DIFFUSE_CRUFT_NICK&&
            i!=SPECULAR_DIRT_NICK&&
            i!=SPECULAR_CRUFT_NICK&&
            i!=DIRT_NORMAL_NICK) {//not a fp
            delegateCaps[vs_3_0][i]=1;
            delegateCaps[glslv][i]=1;
            if (i!=ADVANCED_NICK) {
                delegateCaps[cg_vs_3_0][i]=1;//they SHOULD support 10 texcoords
                delegateCaps[cg_vs_2_xvp40][i]=1;
                if (i!=DERIVATIVE_NICK) {
	                delegateCaps[vs_2_b][i]=1;
	                delegateCaps[cg_vs_2_0][i]=1;
		            delegateCaps[vs_2_0][i]=1;
			        delegateCaps[arbvp1][i]=1;                
				}
            }                
        }
    }
    delegateCaps[cg_vs_3_0][NUL_LIGHTS_NICK]=6;
    delegateCaps[vs_3_0][NUL_LIGHTS_NICK]=6;
    delegateCaps[glslv][NUL_LIGHTS_NICK]=6;
    delegateCaps[arbvp1][NUL_LIGHTS_NICK]=4;
    delegateCaps[cg_vs_2_xvp40][NUL_LIGHTS_NICK]=4;
    delegateCaps[vs_2_b][NUL_LIGHTS_NICK]=4;
    delegateCaps[vs_2_0][NUL_LIGHTS_NICK]=1;
    delegateCaps[cg_vs_2_0][NUL_LIGHTS_NICK]=1;

    delegateCaps[cg_vs_2_xvp40][CG_NICK]=1;
    delegateCaps[vs_2_b][HLSL_NICK]=1;
    delegateCaps[vs_3_0][HLSL_NICK]=1;    
    delegateCaps[vs_2_0][HLSL_NICK]=1;
    delegateCaps[cg_vs_2_0][CG_NICK]=1;
    delegateCaps[cg_vs_3_0][CG_NICK]=1;
    delegateCaps[arbvp1][CG_NICK]=1;
    delegateCaps[arbvp1][GL_NICK]=1;
    delegateCaps[glslv][GLSL_NICK]=1;
   return 0;


}
static int nothing=setDelegateCaps();
//enum OffOrOn{PROP_OFF=0,PROP_ON_TC0=1,PROP_ON_TC1=2,PROP_ON_TC2=3,PROP_MUL=4,PROP_MUL_TC0=5,PROP_MUL_TC1=6,PROP_MUL_TC2=7,PROP_OVERLAY=8,PROP_OVERLAY_TC0=9,PROP_OVERLAY_TC1=10,PROP_OVERLAY_TC2=11};
static int mymin (int a, int b){
  return a<b?a:b;
}
}
namespace ProgramDefines {
std::string capName(int program_properties[ProgramDefines::NUM_NICK], bool include_lighting) {
    using namespace ProgramDefines;
    std::string programName;
    for (int i=0;i<NUM_NICK;++i) {
        if (i==NUL_LIGHTS_NICK) {
            if (include_lighting)
                programName+=nickname[i]+itostr(program_properties[i]);
        }else if(program_properties[i]){
            programName+=nickname[i];
            if (program_properties[i]%4>1) programName+=itostr(program_properties[i]%4-1);
            if (program_properties[i]/4==1) programName+='M';
            if (program_properties[i]/4==2) programName+='O';
        }                
    }
    return programName;
}

#define MATERIAL_PATH "materials/scripts/"
static FILE * openShaderFile(std::string programName, char mode) {
    char cmode[3]={mode,'b','\0'};
    programName=MATERIAL_PATH+programName;
    return fopen(programName.c_str(),cmode);
}
static void addProfile(const std::string &programName, std::string&program,DXCaps::DELEGATE_TYPE profile) {
    //FIXME need to add profile if it is not there
    using namespace DXCaps;
    std::string prof=profileData[profile];
    if (program.find(prof)==std::string::npos) {
        std::string filename=programName+".program";
        FILE * fp=openShaderFile(filename,'r');
        fseek(fp,0,SEEK_END);
        size_t length=ftell(fp);
        fseek(fp,0,SEEK_SET);
        char * data=(char*)malloc(length);
        fread(data,1,length,fp);
        fclose(fp);
        std::string program(data,length);
        free(data);
        
        
        std::string::size_type where=program.find("target ");
        std::string::size_type where2=program.find("profiles ");
        if (where2<where&&where2!=std::string::npos) {
            where=where2+strlen("profiles ");
        }else {
            where+=strlen("target ");
        }
        if (where!=std::string::npos) {
            program=program.substr(0,where)+prof+" "+program.substr(where);
            fprintf (stderr,"Located profile %s in %s\n",prof.c_str(),program.c_str());
        }else {
            fprintf (stderr,"Error cannot locate profile in %s\n",program.c_str());
        }
        fp=openShaderFile(filename,'w');
        fwrite(program.data(),program.length(),1,fp);
        fclose(fp);
    }
}
static std::string makeDelegate(int program_properties[ProgramDefines::NUM_NICK], DXCaps::DELEGATE_TYPE profile,std::map<std::string,std::string> &needed_programs, bool doWrite) {
  using namespace DXCaps;
 using namespace ProgramDefines;
  std::string programName;
  for (int i=0;i<NUM_NICK;++i) {
    int property=delegateCaps[profile][i];
    if (i<=SHADOW_NICK) {
        property=mymin(program_properties[i],delegateCaps[profile][i]);
    }
    if (i==LOD_NICK&&program_properties[REFLECTION_NICK]==0&&program_properties[AMBIENT_ENVIRONMENT_NICK]==0) {
        //LOD is only needed in the presence of environment maps
        property=0;
    }
    if (i==LOD_NICK&&program_properties[REFLECTION_NICK]==0&&program_properties[AMBIENT_ENVIRONMENT_NICK]==0) {
        //LOD is only needed in the presence of environment maps
        property=0;
    }
    if (i==ADVANCED_NICK&&(program_properties[SHADOW_NICK]==0||program_properties[NUL_LIGHTS_NICK]<=3)) {
        //10 texture coordinates only needed if there are more than 3 shadow casters
        property=0;
    }
    if (i==DERIVATIVE_NICK&&(program_properties[BUMP_NICK]==0||(program_properties[DIFFUSE_DIRT_NICK]==0&&
                                                                program_properties[SPECULAR_DIRT_NICK]==0&&
                                                                program_properties[GLOW_DIRT_NICK]==0&&
                                                                program_properties[DIFFUSE_CRUFT_NICK]==0&&
                                                                program_properties[SPECULAR_CRUFT_NICK]==0&&
                                                                program_properties[GLOW_CRUFT_NICK]==0&&
                                                                program_properties[DIRT_NORMAL_NICK]==0))) {
        property=0;//derivatives are only needed for dirt or cruft maps *if* bump mapping is enabled
    }
    if (i==BUMP_NICK&&(!(program_properties[DIFFUSE_DIRT_NICK]==0&&
                         program_properties[SPECULAR_DIRT_NICK]==0&&
                         program_properties[GLOW_DIRT_NICK]==0&&
                         program_properties[DIFFUSE_CRUFT_NICK]==0&&
                         program_properties[SPECULAR_CRUFT_NICK]==0&&
                         program_properties[GLOW_CRUFT_NICK]==0&&
                         program_properties[DIRT_NORMAL_NICK]==0))&&delegateCaps[profile][DERIVATIVE_NICK]==0) {
        property=0;//don't have bump mapping with more than 1 property unless derivatives supported
        
    }
    if (i==NUL_LIGHTS_NICK) {
      programName+=nickname[i]+itostr(property);
    }else if (property) {
        programName+=nickname[i];
        if (property%4>1)
            programName+=itostr(property%4-1);
        if (property/4==1) {
            programName+="M";
        }else if (property/4==2)
            programName+="O";    
    }
  }
  //ambient_environment=program_properties[AMBIENT_ENVIRONMENT_NICK]!=0;
  //reflection=program_properites[REFLECTION_NICK]!=0;
  //glow=program_properties[GLOW_NICK]!=0;
  //num_shadow_lights=program_properties[SHADOW_NICK]!=0?program_propreties[NUL_LIGHTS_NICK]:0;
  if (needed_programs.find(programName)==needed_programs.end()) {
    bool ambient_environment,reflection,glow;
    unsigned int num_shadow_lights;
	//char num[128];
	//sprintf(num,"writing %d",doWrite);
	//std::string data="Writing "+programName;
	//MessageBox(NULL,num,data.c_str(),MB_OK|MB_ICONEXCLAMATION);

    if (doWrite) {
        FILE *fp=openShaderFile(programName+".program",'w');
        std::string dat=programDefinition(programName,profileData[profile],ambient_environment,reflection,glow,num_shadow_lights);
        fwrite(dat.data(),dat.length(),1,fp);
        fclose(fp);
    }
    needed_programs[programName]=profileData[profile];    
  }else {
      if (needed_programs[programName].find(profileData[profile])==std::string::npos) {
          if (doWrite) {
              addProfile(programName,needed_programs[programName],profile);
          }
          needed_programs[programName]+=std::string(" ")+profileData[profile];
      }
  }
  return programName;
}
#define TEX_WHITECLEAR "\t\t\t\ttexture whiteclear.png\n"
#define TEX_WHITE "\t\t\t\ttexture white.png\n"
#define TEX_BLACK "\t\t\t\ttexture black.png\n"
#define TEX_BLACKCLEAR "\t\t\t\ttexture blackclear.png\n"
#define TEX_BLACKCUBE "\t\t\t\tcubic_texture blackcube.png combinedUVW\n"
#define TEX_WHITECUBE "\t\t\t\tcubic_texture whitecube.png combinedUVW\n"
#define TEX_GRAYTRANSPARENT "\t\t\t\ttexture graytrans.png\n"
static std::string makeDelegates(std::string material_name, int program_properties[ProgramDefines::NUM_NICK], std::map<std::string,std::string> &needed_delegates,std::map<std::string,std::string> &needed_programs, bool doWrite, const std::set<DXCaps::DELEGATE_TYPE>&disallowed_profiles) {
  using namespace ProgramDefines;
  using namespace DXCaps;
    bool VertexProgram = program_properties[VP_NICK]!=0;
    std::string firstName;
    for (int i=0;i<NUM_NICK;++i) {
        if (program_properties[i]&&i!=NUL_LIGHTS_NICK) {
            firstName+=nickname[i];
            if (program_properties[i]%4>1) firstName+=itostr(program_properties[i]%4-1);
            if (program_properties[i]/4==1) firstName+="M";
            if (program_properties[i]/4==2) firstName+="O";
        }
        if (i==NUL_LIGHTS_NICK) {
            firstName+=nickname[i]+itostr(program_properties[i]);
        }
    }
    std::string space(" ");
    std::string mangled_name=material_name+firstName;
    if (needed_delegates.find(mangled_name)==needed_delegates.end()) {
        std::string retval;
        std::string supported;
        if (VertexProgram) {
            retval+="vertex_program "+mangled_name+" unified\n{";
            if (disallowed_profiles.find(vs_3_0)==disallowed_profiles.end()) {
                retval+="\n\tdelegate "+makeDelegate(program_properties,vs_3_0,needed_programs,doWrite);
                supported+=profileData[vs_3_0]+space;
            }
            if (disallowed_profiles.find(cg_vs_3_0)==disallowed_profiles.end()) {
                retval+="\n\tdelegate "+makeDelegate(program_properties,cg_vs_3_0,needed_programs,doWrite);
                supported+=profileData[cg_vs_3_0]+space;
            }
            if (disallowed_profiles.find(vs_2_b)==disallowed_profiles.end()) {
                retval+="\n\tdelegate "+makeDelegate(program_properties,vs_2_b,needed_programs,doWrite);
                supported+=profileData[vs_2_b]+space;
            }
            if (disallowed_profiles.find(cg_vs_2_xvp40)==disallowed_profiles.end()) {
                retval+="\n\tdelegate "+makeDelegate(program_properties,cg_vs_2_xvp40,needed_programs,doWrite);
                supported+=profileData[cg_vs_2_xvp40]+space;
            }
            if (disallowed_profiles.find(glslv)==disallowed_profiles.end()) {
                retval+="\n\tdelegate "+makeDelegate(program_properties,glslv,needed_programs,doWrite);
                supported+=profileData[glslv]+space;
            }
            if (disallowed_profiles.find(arbvp1)==disallowed_profiles.end()) {
                retval+="\n\tdelegate "+makeDelegate(program_properties,arbvp1,needed_programs,doWrite);
                supported+=profileData[arbvp1]+space;
            }
            if (program_properties[NUL_LIGHTS_NICK]<=1) {
                if (disallowed_profiles.find(vs_2_0)==disallowed_profiles.end()) {
                    retval+="\n\tdelegate "+makeDelegate(program_properties,vs_2_0,needed_programs,doWrite);
                    supported+=profileData[vs_2_0]+space;
                }
                if (disallowed_profiles.find(cg_vs_2_0)==disallowed_profiles.end()) {
                    retval+="\n\tdelegate "+makeDelegate(program_properties,cg_vs_2_0,needed_programs,doWrite);
                    supported+=profileData[cg_vs_2_0]+space;
                }
            }
        }else {
            retval+="fragment_program "+mangled_name+" unified\n{";
            if (disallowed_profiles.find(ps_3_0)==disallowed_profiles.end()) {
                retval+="\n\tdelegate "+makeDelegate(program_properties,ps_3_0,needed_programs,doWrite);
                supported+=profileData[ps_3_0]+space;
            }
            if (disallowed_profiles.find(cg_ps_3_0)==disallowed_profiles.end()) {
                retval+="\n\tdelegate "+makeDelegate(program_properties,cg_ps_3_0,needed_programs,doWrite);
                supported+=profileData[cg_ps_3_0]+space;
            }
            if (disallowed_profiles.find(ps_2_b)==disallowed_profiles.end()) {
                retval+="\n\tdelegate "+makeDelegate(program_properties,ps_2_b,needed_programs,doWrite);
                supported+=profileData[ps_2_b]+space;
            }
            if (disallowed_profiles.find(cg_ps_2_xfp40)==disallowed_profiles.end()) {
                retval+="\n\tdelegate "+makeDelegate(program_properties,cg_ps_2_xfp40,needed_programs,doWrite);
                supported+=profileData[cg_ps_2_xfp40]+space;
            }
            if (disallowed_profiles.find(glsl)==disallowed_profiles.end()) {
                retval+="\n\tdelegate "+makeDelegate(program_properties,glsl,needed_programs,doWrite);
                supported+=profileData[glsl]+space;
            }
            if (disallowed_profiles.find(arbfp1)==disallowed_profiles.end()) {
                retval+="\n\tdelegate "+makeDelegate(program_properties,arbfp1,needed_programs,doWrite);
                supported+=profileData[arbfp1]+space;
            }
            if (program_properties[NUL_LIGHTS_NICK]<=1) {
                if (disallowed_profiles.find(ps_2_0)==disallowed_profiles.end()) {
                    retval+="\n\tdelegate "+makeDelegate(program_properties,ps_2_0,needed_programs,doWrite);
                    supported+=profileData[ps_2_0]+space;
                }
                //if (disallowed_profiles.find(cg_ps_2_0)==disallowed_profiles.end())//cg doens't build efficient ps_2_0 progs
                //retval+="\n\tdelegate "+makeDelegate(program_properties,cg_ps_2_0,needed_programs,doWrite);
                //                supported+=profileData[cg_ps_2_0]+" ";
            }
        }
        
        retval+="\n}\n";
        needed_delegates[mangled_name]=retval;        
    }
    return mangled_name;
}


std::string setTextureUnits(int technique_caps[NUM_NICK], bool fixed_function, std::string*texunits) {

    std::string retval;
    if (technique_caps[AMBIENT_OCCLUSION_NICK]||fixed_function==false) {
        retval+="\t\t\ttexture_unit AmbientOcclusion\n\t\t\t{\n";
        if (texunits)
            retval+=texunits[AMBIENT_OCCLUSION_NICK];
        else
            retval+=TEX_WHITE;
        retval+="\t\t\t}\n";
    }
    retval+="\t\t\ttexture_unit Diffuse\n\t\t\t{\n";
    if (texunits)
        retval+=texunits[DIFFUSE_NICK];
    else
        retval+=TEX_WHITE;
    retval+="\t\t\t}\n";
    if (technique_caps[DIFFUSE_DIRT_NICK]/4==2) {//PROP_OVERLAY
        if (!fixed_function ) {
            retval+="\t\t\ttexture_unit DiffuseDirt\n\t\t\t{\n";
            if (texunits)
                retval+=texunits[DIFFUSE_DIRT_NICK];
            else
                retval+=TEX_GRAYTRANSPARENT;
            retval+="\t\t\t}\n";
        }
    }else if (technique_caps[DIFFUSE_DIRT_NICK]!=0||fixed_function==false){
        retval+="\t\t\ttexture_unit DiffuseDirt\n\t\t\t{\n";
        if (texunits)
            retval+=texunits[DIFFUSE_DIRT_NICK];
        else
            retval+=TEX_WHITECLEAR;
        if (fixed_function&&technique_caps[DIFFUSE_DIRT_NICK]/4==0) {
            retval+="\t\t\t\tcolour_op alpha_blend\n";
        }      
        if (fixed_function)
            retval+="\t\t\t\ttex_coord_set 1\n";
        retval+="\t\t\t}\n";
    }
    bool wrotenormaldirt=false;
    if (technique_caps[AMBIENT_ENVIRONMENT_NICK]==0) {
        if (fixed_function==false&&technique_caps[DIRT_NORMAL_NICK]!=0&&technique_caps[DIFFUSE_CRUFT_NICK]==0) {
            retval+="\t\t\ttexture_unit NormalDirt\n\t\t\t{\n";
            if (texunits)
                retval+=texunits[DIRT_NORMAL_NICK];
            else
                retval+=TEX_GRAYTRANSPARENT;
            retval+="\t\t\t}\n";
            wrotenormaldirt=true;
        }else if (technique_caps[DIFFUSE_CRUFT_NICK]!=0||fixed_function==false) {
            if (technique_caps[DIFFUSE_CRUFT_NICK]/4==2) {//PROP_OVERLAY
                if (!fixed_function) {
                    retval+="\t\t\ttexture_unit DiffuseGrunge\n\t\t\t{\n";
                    if (texunits)
                        retval+=texunits[DIFFUSE_CRUFT_NICK];
                    else
                        retval+=TEX_GRAYTRANSPARENT; 
                    retval+="\t\t\t}\n";
                }
            }else {
                retval+="\t\t\ttexture_unit DiffuseGrunge\n\t\t\t{\n";
                if (texunits)
                    retval+=texunits[DIFFUSE_CRUFT_NICK];
                else
                    retval+=TEX_WHITECLEAR;
                if (fixed_function&&technique_caps[DIFFUSE_CRUFT_NICK]/4==0) {
                    retval+="\t\t\t\tcolour_op alpha_blend\n";
                }
                if (fixed_function)
                    retval+="\t\t\t\ttex_coord_set 2\n";
                retval+="\t\t\t}\n";
            }
        }
    }else if (!fixed_function) {
        retval+="\t\t\ttexture_unit AmbientEnvironment\n\t\t\t{\n";
        if (texunits)
            retval+=texunits[AMBIENT_ENVIRONMENT_NICK];
        else
            retval+=TEX_WHITECUBE;
        retval+="\t\t\t}\n";      
    }
    if (!fixed_function) {
        retval+="\t\t\ttexture_unit Bump\n\t\t\t{\n";
        if (texunits)
            retval+=texunits[BUMP_NICK];
        else
            retval+=TEX_BLACK;
        retval+="\t\t\t}\n";
        retval+="\t\t\ttexture_unit Normal\n\t\t\t{\n";
        if (texunits)
            retval+=texunits[NORMAL_NICK];
        else
            retval+=TEX_GRAYTRANSPARENT;
        retval+="\t\t\t}\n";
        retval+="\t\t\ttexture_unit Specular\n\t\t\t{\n";
        if (texunits)
            retval+=texunits[SPECULAR_NICK];
        else
            retval+=TEX_WHITECLEAR;
        retval+="\t\t\t}\n";
        if (technique_caps[SPECULAR_DIRT_NICK]/4==2) {//PROP_OVERLAY
            retval+="\t\t\ttexture_unit SpecularDirt\n\t\t\t{\n";
            if (texunits)
                retval+=texunits[SPECULAR_DIRT_NICK];
            else
                retval+=TEX_GRAYTRANSPARENT;
            retval+="\t\t\t}\n";
        }else {
            retval+="\t\t\ttexture_unit SpecularDirt\n\t\t\t{\n";
            if (texunits)
                retval+=texunits[SPECULAR_DIRT_NICK];
            else
                retval+=TEX_WHITECLEAR;            
            retval+="\t\t\t}\n";
        }
        if (technique_caps[REFLECTION_NICK]==0||technique_caps[GLOW_NICK]==0) {
            if (technique_caps[SPECULAR_CRUFT_NICK]!=0||(technique_caps[DIRT_NORMAL_NICK]==0||(technique_caps[AMBIENT_ENVIRONMENT_NICK]==0&&technique_caps[DIFFUSE_CRUFT_NICK]==0))) {
                //if dirt normal went elsewhere or is off, print spec grunge even if this is off
                if (technique_caps[SPECULAR_CRUFT_NICK]/4==2){//==PROP_OVERLAY){
                    retval+="\t\t\ttexture_unit SpecularGrunge\n\t\t\t{\n";
                    if (texunits)
                        retval+=texunits[SPECULAR_CRUFT_NICK];
                    else
                        retval+=TEX_GRAYTRANSPARENT;
                    retval+="\t\t\t}\n";
                }else {
                    retval+="\t\t\ttexture_unit SpecularGrunge\n\t\t\t{\n";
                    if (texunits)
                        retval+=texunits[SPECULAR_CRUFT_NICK];
                    else
                        retval+=TEX_WHITECLEAR;
                    retval+="\t\t\t}\n";
                }
            }
            if (technique_caps[DIRT_NORMAL_NICK]&&(technique_caps[AMBIENT_ENVIRONMENT_NICK]||technique_caps[DIFFUSE_CRUFT_NICK])) {
                assert(wrotenormaldirt==0);
                retval+="\t\t\ttexture_unit NormalDirt\n\t\t\t{\n";
                if (texunits)
                    retval+=texunits[DIRT_NORMAL_NICK];
                else
                    retval+=TEX_GRAYTRANSPARENT;
                retval+="\t\t\t}\n";                
            }else {
            }
        }
    }
    if (technique_caps[GLOW_NICK]!=0||technique_caps[REFLECTION_NICK]==0) { 
        retval+="\t\t\ttexture_unit Glow\n\t\t\t{\n";
        if (texunits)
            retval+=texunits[GLOW_NICK];
        else
            retval+=TEX_BLACK;
        if (fixed_function) {
            retval+="\t\t\t\tcolour_op add\n";
        }
        retval+="\t\t\t}\n";     
    }
    if (!fixed_function) {
        if (technique_caps[REFLECTION_NICK]!=0) {
            retval+="\t\t\ttexture_unit Reflection\n\t\t\t{\n";
            if (texunits)
                retval+=texunits[REFLECTION_NICK];
            else
                retval+=TEX_BLACKCUBE;
            retval+="\t\t\t}\n";     
        }
        if(!texunits) {//do not want to reiterate the shadow nicks
            for (int i=0;i<technique_caps[NUL_LIGHTS_NICK]&&technique_caps[SHADOW_NICK]&&i<6;++i) {
                retval+="\t\t\ttexture_unit\n\t\t\t{\n\t\t\t\tcontent_type shadow\n\t\t\t}\n";     
            }
        }
    }
    return retval;
}
}
static std::string makePass(const std::string&material_name,const std::string &PassName, int technique_caps[::ProgramDefines::NUM_NICK], std::map<std::string,std::string>&needed_delegates, std::map<std::string,std::string>&needed_programs,bool doWrite, const std::set<DXCaps::DELEGATE_TYPE>&disallowed_profiles,const std::string &per_something="once") {
    using namespace ProgramDefines;
    std::string retval;
    if (doWrite) {
        retval+="\t\tpass ";
        retval+=PassName;
        retval+="\n\t\t{\n";
        if (per_something!="once") {// this is the default, in case someone wants to make a fur shader that runs a bunch of times afterwards
            retval+="\t\t\titeration "+per_something+"\n";
        }
    }
    technique_caps[FP_NICK]=0;
    technique_caps[VP_NICK]=1;

    std::string vpname=makeDelegates(material_name,technique_caps,needed_delegates,needed_programs,doWrite,disallowed_profiles);
    technique_caps[VP_NICK]=0;
    technique_caps[FP_NICK]=1;
    std::string fpname=makeDelegates(material_name,technique_caps,needed_delegates,needed_programs,doWrite,disallowed_profiles);
    technique_caps[FP_NICK]=0;
    if (doWrite) {
        retval+="\t\t\tvertex_program_ref ";
        retval+=vpname;
        retval+="\n\t\t\t{\n\t\t\t}\n";
        retval+="\t\t\tfragment_program_ref ";
        retval+=fpname;
        retval+="\n\t\t\t{\n\t\t\t}\n";
        retval+=setTextureUnits(technique_caps,false,NULL);
        retval+="\t\t}\n";
    }
    return retval;
}
static std::string makeTechnique(const std::string&material_name,const std::string &TechniqueName, int technique_caps[::ProgramDefines::NUM_NICK], std::map<std::string,std::string>&needed_delegates, std::map<std::string,std::string>&needed_programs, bool doWrite, std::string scheme="", const std::set<DXCaps::DELEGATE_TYPE> &disallowed_profiles=std::set<DXCaps::DELEGATE_TYPE>() ) {
    std::string retval;
    using namespace ProgramDefines;
    int pass_caps[NUM_NICK];
    if (doWrite) {
        retval+="\ttechnique "+TechniqueName+"\n";
        retval+="\t{\n";
        if (scheme.length()) {
            retval+="\t\tscheme "+scheme+"\n";
        }
    }
    std::string firstPassName="OnlyPass";
    memcpy(pass_caps,technique_caps,NUM_NICK*sizeof(int));    
    if (technique_caps[NUL_LIGHTS_NICK]==1){
        firstPassName="AmbientPass";
        //shave off a few of the technique caps for this ambient-only pass
        pass_caps[NO_SPOTLIGHT_NICK]=1;
        if (0==pass_caps[REFLECTION_NICK]) {
            pass_caps[SPECULAR_NICK]=
                pass_caps[SPECULAR_DIRT_NICK]=
                pass_caps[SPECULAR_CRUFT_NICK]=
                pass_caps[GLOSSINESS_NICK]=0;
            pass_caps[NUL_LIGHTS_NICK]=0;
            pass_caps[SHADOW_NICK]=0;
            pass_caps[NORMAL_NICK]=0;            
        }
    }
    retval+=makePass(material_name,firstPassName,technique_caps,needed_delegates,needed_programs,doWrite,disallowed_profiles);
    if (technique_caps[NUL_LIGHTS_NICK]==1){        
        memcpy(pass_caps,technique_caps,NUM_NICK*sizeof(int));        
        pass_caps[AMBIENT_ENVIRONMENT_NICK]=0;
        pass_caps[AMBIENT_NICK]=0;
        pass_caps[GLOW_NICK]=
            pass_caps[GLOW_DIRT_NICK]=
            pass_caps[GLOW_CRUFT_NICK]=0;
        pass_caps[REFLECTION_NICK]=0;        
        //shave off a few of the technique caps for this per-light pass
        retval+=makePass(material_name,"PerSpotLightPass",technique_caps,needed_delegates,needed_programs,doWrite, disallowed_profiles,"once_per_light spot");
        pass_caps[NO_SPOTLIGHT_NICK]=1;
        retval+=makePass(material_name,"PerPointLightPass",technique_caps,needed_delegates,needed_programs, doWrite, disallowed_profiles,"once_per_light point");
        retval+=makePass(material_name,"PerDirectionalLightPass",technique_caps,needed_delegates,needed_programs,doWrite,disallowed_profiles,"once_per_light directional");
        
    }
    if (doWrite) {
        retval+="\t}\n";
    }
    return retval;
}
static std::string makeFixedFunctionTechnique(std::string TechniqueName, int technique_caps[::ProgramDefines::NUM_NICK], bool doWrite,const std::string& scheme="") {
    std::string retval;
    if (doWrite) {
        retval+="\ttechnique "+TechniqueName+"\n";
        retval+="\t{\n";
        if (scheme.length()) {
            retval+="\t\tscheme "+scheme+"\n";
        }
        retval+="\t\tpass OnlyPass\n\t\t{\n";
		retval+=ProgramDefines::setTextureUnits(technique_caps,true,NULL);
        retval+="\t\t}\n\t}\n";
    }
    return retval;
}
static void makeMaterial(std::string name, const int wanted_caps[::ProgramDefines::NUM_NICK],std::map<std::string,std::string> &needed_programs, bool doWrite) {
    using namespace ProgramDefines;
	int caps[NUM_NICK];
	memcpy(&caps[0],&wanted_caps[0],sizeof(int)*NUM_NICK);
    std::map<std::string,std::string> needed_delegates;
    std::string tech;
    if (doWrite) {
        tech="material ";
        tech+=name;
        tech+="\n{\n";
    }
    unsigned int oldlight=caps[NUL_LIGHTS_NICK];
    caps[NUL_LIGHTS_NICK]=1;
    tech+=makeTechnique(name,"Stencil",caps,needed_delegates,needed_programs,doWrite,"stencil");
    caps[NUL_LIGHTS_NICK]=oldlight;
    caps[SHADOW_NICK]=1;
    tech+=makeTechnique(name,"ShadowMap",caps,needed_delegates,needed_programs,doWrite,"shadowmap");
    caps[SHADOW_NICK]=0;
    tech+=makeTechnique(name,"Base",caps,needed_delegates,needed_programs,doWrite);
    caps[NUL_LIGHTS_NICK]=1;
    caps[BUMP_NICK]=0;
    //FIXME: not sure about this one: can we afford to be rid of it caps[DIRT_NORMAL_NICK]=0;
    tech+=makeTechnique(name,"Fallback",caps,needed_delegates,needed_programs,doWrite);
    caps[NUL_LIGHTS_NICK]=oldlight;
    tech+=makeFixedFunctionTechnique("FixedFunction",caps,doWrite);
    if (oldlight>3)
        caps[NUL_LIGHTS_NICK]=3;
    caps[GLOSSINESS_NICK]=0;
    caps[SUBSURFACE_NICK]=0;
    caps[REFLECTION_NICK]=0;
    caps[AMBIENT_ENVIRONMENT_NICK]=0;
    tech+=makeTechnique(name,"Lite",caps,needed_delegates,needed_programs,doWrite,"lite");
    caps[DIRT_NORMAL_NICK]=0;
    tech+=makeFixedFunctionTechnique("LiteFixedFunction",caps,doWrite,"lite");
    caps[NUL_LIGHTS_NICK]=oldlight;
    if (doWrite) {
        tech+="}\n";
        std::string filename=name+".material";
        FILE *fp=openShaderFile(filename,'w');
        for (std::map<std::string,std::string>::iterator i=needed_delegates.begin(),ie=needed_delegates.end();i!=ie;++i) {
            fwrite(i->second.data(),i->second.length(),1,fp);
            fwrite("\n",1,1,fp);
        }
        fwrite(tech.data(),tech.length(),1,fp);
        fclose(fp);
    }
/*
    for (std::map<std::string,std::string>::iterator i=needed_delegates.begin(),ie=needed_delegates.end();i!=ie;++i) {
        global_needed_delegates[i->first]=i->second;
    }        
*/  
}
void makeMaterial(const std::string&name, int caps[::ProgramDefines::NUM_NICK]) {
    std::map<std::string,std::string>need;
    makeMaterial(name,caps,need,true);
}
static int myabs(int a) {
    if (a>=0) return a;
    return -a;//this way we know that be doesn't support the technique
}
static int compare(int a, int b) {
	if (a!=0&&b==0) return ::ProgramDefines::NUM_NICK*16;
	if (a>1&&a!=b) return ::ProgramDefines::NUM_NICK*4;
    return myabs(a-b);
}
/// Compare two Capability strings. a is the desired technique...b better have all that and more
static int stringdiff(const DXCaps::ShaderCapabilities& a, const DXCaps::ShaderCapabilities& b) {
    int diff=0;
    int num_nick=::ProgramDefines::NUM_NICK;
	for (int i=0;i < num_nick; ++i) {
        diff+=compare(a.values[i],b.values[i]);
    }
    return diff;
}
static DXCaps::ShaderCapabilities OptionsToCode(int values[::ProgramDefines::NUM_NICK]) {
    DXCaps::ShaderCapabilities retval;
	memcpy(&retval.values[0],&values[0],sizeof(int)*::ProgramDefines::NUM_NICK);
    return retval;
}
static void genProgSpecialization(::ProgramDefines::NICKNAMES BO [::ProgramDefines::NUM_NICK],int i,int CurOptions[::ProgramDefines::NUM_NICK],std::map<std::string,std::string> &needed_programs,std::map <std::string,DXCaps::ShaderCapabilities> &needed_materials, bool write, bool calc_dependencies) {
    using namespace ProgramDefines;
    //USE OPTIONS
    if (BO[0]==NUM_NICK) {
        if (capCompat(NUM_NICK,CurOptions)) {
            std::string nam=capName(CurOptions,false);
            if (needed_materials.find(nam)==needed_materials.end()) {
                if (calc_dependencies) {
                    makeMaterial(nam,CurOptions,needed_programs,write);
                }
                needed_materials[nam]=OptionsToCode(CurOptions);
            }
        }
    }else {
        std::string retval;
        NICKNAMES lastOption=BO[i-1];
        NICKNAMES backup[NUM_NICK];
        BO[i-1]=NUM_NICK;//turn off option
        memcpy(backup,BO,sizeof(NICKNAMES)*i);
        int prevOption=CurOptions[lastOption];
        if (capCompat(lastOption,CurOptions)&&prevOption==0) 
        {
            CurOptions[lastOption]=1;
            genProgSpecialization(BO,i-1,CurOptions,needed_programs,needed_materials,write,calc_dependencies);
            CurOptions[lastOption]=prevOption;        

        }
        if (capCompat(NUM_NICK,CurOptions)) {
            genProgSpecialization(backup,i-1,CurOptions,needed_programs,needed_materials,write,calc_dependencies);
        }
    }
}
static void genProgramSpecializations(std::map<std::string,std::string> &needed_programs,std::map<std::string,DXCaps::ShaderCapabilities>&needed_materials, int *onlyOption,bool doWrite, bool calc_dependencies, ::ProgramDefines::NICKNAMES *bothOptions=NULL) {
    using namespace ProgramDefines;
    int opts[NUM_NICK]={0};    
    opts[NUL_LIGHTS_NICK]=8;
    NICKNAMES BO[NUM_NICK+1];
    unsigned int i;
    for (i=0;i<NUM_NICK;++i) {
        opts[i]=onlyOption[i];
    }
    for (i=0;i<NUM_NICK&&bothOptions&&bothOptions[i]!=NUM_NICK;++i) {        
        BO[i]=bothOptions[i];
    }
    BO[i]=NUM_NICK;
    genProgSpecialization(BO,i,opts,needed_programs,needed_materials,doWrite,calc_dependencies);
}


static void generateAllShaders (std::map<std::string, std::string> &needed_programs,
                         std::map<std::string,DXCaps::ShaderCapabilities> &needed_materials,
                         bool dependencies_and_write) {
    using namespace ProgramDefines;
    int caps[NUM_NICK]={0};

    caps[AMBIENT_NICK]=1;
    caps[AMBIENT_OCCLUSION_NICK]=1;

    caps[NUL_LIGHTS_NICK]=8;
    
    //string tech=makeMaterial("Awesome",caps,needed_delegates,needed_programs);
//    NICKNAMES bothOptions[NUM_NICK]={SKINNING_NICK,GLOW_NICK,REFLECTION_NICK,NUM_NICK};
    for (unsigned int gl=0;gl<=3;++gl) {//which layer should glow be on
        for (unsigned int i=1;i<=3;++i) {//which layer should the brunt of textures be on
            for (unsigned int k=0;k<4;++k) {
                for (unsigned int j=0;j<3;++j) {//use bump?  no sense to use bump in uv 3
                    for (unsigned int m=0;m<12;++m) {//dirt layer //turn off overlay for now
                        if (m==4||m==8||m==12) m++;//don't want OFF but combined funny
                        for (unsigned int ms=0;ms<2;++ms) {//spec dirt layer
                            for (unsigned int p=0;p<3;++p) {//normal dirt layer
                                for (unsigned int pk=0;pk<3;++pk) {//normal dirt layer combiner, only combine for dirt with alpha <-- artists dont like it
                                    for (unsigned int n=0;n<8;++n) {//dirt cruft layer//turn off overlay for now
                                        if (n==4||n==8||n==12) n++;//don't want OFF but combined funny
                                        for (unsigned int ns=0;ns<2;++ns) {//dirt spec layer
                                            for (unsigned int skin=0;skin<3;++skin) {
                                                for (unsigned int reflect=0;reflect<1;++reflect) {
													for (unsigned int two_sided=0;two_sided<2;++two_sided) {
	                                                    caps[SKINNING_NICK]=(skin!=0?1:0);
		                                                caps[TWO_SIDED_NICK]=two_sided;
				                                        caps[SUBSURFACE_NICK]=(skin==2?1:0);
			                                            caps[AMBIENT_ENVIRONMENT_NICK]=(skin==2?1:0);
                                                    
	                                                    caps[DIFFUSE_NICK]=i;
		                                                caps[SPECULAR_NICK]=i;
			                                            caps[GLOW_NICK]=gl; 
				                                        caps[NORMAL_NICK]=k;
					                                    caps[BUMP_NICK]=j;
						                                caps[DIFFUSE_DIRT_NICK]=m;
							                            caps[SPECULAR_DIRT_NICK]=ms?m:0;
														caps[DIRT_NORMAL_NICK]=p?(((p==2&&n)?n/*if not using alpha and cruft, use Cruft*/:m)%4)+4*pk:0;
									                    caps[DIFFUSE_CRUFT_NICK]=n;
										                caps[SPECULAR_CRUFT_NICK]=ns?n:0;
											            caps[REFLECTION_NICK]=reflect;
												        genProgramSpecializations(needed_programs,needed_materials,caps,dependencies_and_write,dependencies_and_write);
													}
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }
    }    
}
namespace ProgramDefines {
void generateAllShaders() {
    std::map<std::string, std::string> needed_programs;
    std::map<std::string,DXCaps::ShaderCapabilities> needed_materials;
	::generateAllShaders(needed_programs,needed_materials,true);
}
int findMatch(int desired_capabilities[::ProgramDefines::NUM_NICK],std::string &foundName,int found_capabilities[::ProgramDefines::NUM_NICK], bool writeMaterial) {
    using namespace std;
    using namespace ProgramDefines;
    map<string,string> needed_programs;
    static map<string,DXCaps::ShaderCapabilities>needed_materials;
    int diff=(1<<30);
    DXCaps::ShaderCapabilities capstr=OptionsToCode(desired_capabilities);
	static bool generated=false;
	if (!generated) {
		generated=true;
	    map<string,string> needed_programs_tmp;

		::generateAllShaders(needed_programs_tmp,needed_materials,false);
	}
    std::string desiredName=capName(desired_capabilities,false);
    DXCaps::ShaderCapabilities finalShader;
    if (needed_materials.find(desiredName)!=needed_materials.end()){
        printf( "Perfect match %s\n",desiredName.c_str());
        foundName=desiredName;
        memcpy(&finalShader.values[0],&desired_capabilities[0],sizeof(int)*NUM_NICK);
        diff=0;
    }else {
        for (map<string,DXCaps::ShaderCapabilities>::iterator i=needed_materials.begin(),ie=needed_materials.end();i!=ie;++i) {
            int curdiff=stringdiff(capstr,i->second);
            if (curdiff<diff) {
                diff=curdiff;
                memcpy(&finalShader.values[0],&i->second.values[0],sizeof(int)*NUM_NICK);                
                foundName=i->first; 
            }
        }
    }
    memcpy(&found_capabilities[0],&finalShader.values[0],sizeof(int)*NUM_NICK);
	if (writeMaterial) {
#ifdef _WIN32
		_mkdir("materials");
		_mkdir("materials/scripts");
#else
		mkdir("materials",0777);
		mkdir("materials/scripts",0777);
#endif
		::makeMaterial(foundName,finalShader.values,needed_programs,true);
	}
    return diff;
}
}
#ifdef STANDALONE
int main() {
    using namespace std;
    using namespace ProgramDefines; 
    generateAllShaders();
    return 0;
}
#endif
#if TEST
int main () {
    using namespace ProgramDefines;
    int caps[NUM_NICK]={0};
    caps[DIFFUSE_NICK]=1;
    caps[NUL_LIGHTS_NICK]=8;
    caps[AMBIENT_NICK]=1;
    caps[AMBIENT_OCCLUSION_NICK]=1;
    caps[DIFFUSE_NICK]=2;
    caps[DIFFUSE_DIRT_NICK]=2;
    caps[DIFFUSE_CRUFT_NICK]=7;
    caps[DIRT_NORMAL_NICK]=6;
    caps[NORMAL_NICK]=1;
    caps[BUMP_NICK]=1;
    caps[GLOW_NICK]=1;
    int fcaps[NUM_NICK];
    std::string foundName;
    int diff= findMatch(caps,foundName,fcaps,false);
    printf( "Closest match is %d : %s\n",diff,foundName.c_str());

    return 0;
}

#endif
